Package: GPArotation
Version: 2026.4-1
Title: Gradient Projection Factor Rotation
Authors@R: c(
    person("Coen", "Bernaards", email = "cab.gparotation@gmail.com", role = c("aut", "cre")),
    person("Paul", "Gilbert", role = "aut"),
    person("Robert", "Jennrich", role = "aut")
    )
Depends: R (>= 3.5.0)
Description: Gradient projection algorithms for orthogonal and oblique rotation
    of factor loadings matrices in factor analysis. Implements a comprehensive
    set of rotation criteria including quartimax, quartimin, oblimin, geomin,
    simplimax, the Crawford-Ferguson family, and target rotation, among others. 
    Supports multiple random starts. For details see Bernaards and Jennrich (2005)
    <doi:10.1177/0013164404272507>.
License: GPL (>= 2)
URL: https://cran.r-project.org/package=GPArotation
Imports: stats
LazyData: yes
NeedsCompilation: no
Packaged: 2026-04-29 00:32:08 UTC; coen
Author: Coen Bernaards [aut, cre],
  Paul Gilbert [aut],
  Robert Jennrich [aut]
Maintainer: Coen Bernaards <cab.gparotation@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-29 07:30:19 UTC
Built: R 4.5.2; ; 2026-05-02 06:01:38 UTC; unix

importFrom("stats", "rnorm")

export(
  "GPFoblq",
  "GPForth",
  "GPFRSoblq",
  "GPFRSorth",
  "Random.Start")

export(
  "oblimin",
  "quartimin",
  "targetT",
  "targetQ",
  "pstT",
  "pstQ",
  "oblimax",
  "entropy",
  "quartimax",
  "Varimax",
  "simplimax",
  "bentlerT",
  "bentlerQ",
  "tandemI",
  "tandemII",
  "geominT",
  "geominQ",
  "bigeominT",
  "bigeominQ",
  "cfT",
  "cfQ",
  "infomaxT",
  "infomaxQ",
  "mccammon",
  "bifactorT",
  "bifactorQ",
  "equamax",
  "parsimax",
  "varimin",
  "lpT",
  "lpQ",
  "GPForth.lp",
  "GPFoblq.lp",
  "eiv",
  "echelon"
)

S3method("print",   "GPArotation")
S3method("summary", "GPArotation")
S3method("print",   "summary.GPArotation")

### R code from vignette source 'GPA2local.Stex'

###################################################
### code chunk number 1: GPA2local.Stex:19-21
###################################################
options(continue="  ")
pdf.options(pointsize = 8)


###################################################
### code chunk number 2: GPA2local.Stex:57-163
###################################################
library("GPArotation")

GPFallMinima <- function(A, method = "quartimin", orthogonal = FALSE,
                         randomStarts = 100, eps = 1e-5, maxit = 1000,
                         normalize = FALSE, methodArgs = NULL,
                         minimumInclusion = 2) {
  # Runs multiple random starts and returns ALL distinct local minima,
  # not just the global minimum. Non-converged starts are discarded.
  # Minima found fewer than minimumInclusion times are excluded.
  # Minima are sorted by frequency (most common first).

  engine <- if (orthogonal) GPForth else GPFoblq

  Qvalues    <- numeric(randomStarts)
  Qconverged <- logical(randomStarts)
  all_res    <- vector("list", randomStarts)

  for (i in 1:randomStarts) {
    res <- engine(A, Tmat      = Random.Start(ncol(A)),
                     normalize  = normalize,
                     eps        = eps,
                     maxit      = maxit,
                     method     = method,
                     methodArgs = methodArgs)
    Qvalues[i]    <- res$Table[nrow(res$Table), 2]
    Qconverged[i] <- res$convergence
    all_res[[i]]  <- res
  }

  # Discard non-converged starts
  converged_idx <- which(Qconverged)
  nConverged    <- length(converged_idx)

  if (nConverged == 0)
    stop("No starts converged. Consider increasing maxit or relaxing eps.")

  Qvalues_conv <- Qvalues[converged_idx]
  all_res_conv <- all_res[converged_idx]

  # Bin converged criterion values into equivalence classes
  Q_round  <- round(Qvalues_conv / eps) * eps
  Q_unique <- unique(Q_round)

  # Build one representative solution per unique minimum
  minima <- vector("list", length(Q_unique))
  for (j in seq_along(Q_unique)) {
    idx        <- which(Q_round == Q_unique[j])
    count      <- length(idx)
    proportion <- count / nConverged
    minima[[j]] <- list(
      result     = all_res_conv[[idx[1]]],
      f          = Q_unique[j],
      count      = count,
      proportion = proportion
    )
  }

  # Sort by count (most common first)
  ord    <- order(sapply(minima, `[[`, "count"), decreasing = TRUE)
  minima <- minima[ord]

  # Filter out minima found fewer than minimumInclusion times
  keep   <- sapply(minima, `[[`, "count") >= minimumInclusion
  minima <- minima[keep]

  if (length(minima) == 0)
    stop("No minima found with count >= minimumInclusion (", minimumInclusion,
         "). Consider reducing minimumInclusion or increasing randomStarts.")

  # Summary data frame
  f_values <- sapply(minima, `[[`, "f")
  f_global <- min(f_values)

  summary_df <- data.frame(
    minimum    = seq_along(minima),
    f          = round(f_values, 6),
    deltaF     = round(f_values - f_global, 6),
    count      = sapply(minima, `[[`, "count"),
    proportion = round(sapply(minima, `[[`, "proportion"), 3),
    isGlobal   = f_values == f_global
  )

  result <- list(
    minima           = minima,
    summary          = summary_df,
    Qvalues          = Qvalues_conv,
    nConverged       = nConverged,
    nStarts          = randomStarts,
    method           = method,
    orthogonal       = orthogonal,
    minimumInclusion = minimumInclusion
  )
  class(result) <- "GPFallMinima"
  result
}

print.GPFallMinima <- function(x, ...) {
  cat("Random start analysis:", x$nConverged, "of", x$nStarts,
      "starts converged\n")
  cat("Distinct minima found:", nrow(x$summary),
      "(minimumInclusion =", x$minimumInclusion, ")\n\n")
  print(x$summary, row.names = FALSE)
  cat("\nGlobal minimum: f =", min(x$summary$f), "\n")
  cat("Access full solutions via $minima[[i]]$result\n")
  invisible(x)
}


###################################################
### code chunk number 3: GPA2local.Stex:212-230
###################################################
library("GPArotation")
data("CCAI", package = "GPArotation")
fa_unrotated <- factanal(factors = 3, covmat = CCAI_R, 
      n.obs = 461, rotation = "none")
A <- loadings(fa_unrotated)

# Oblimin: highly stable
res_oblimin <- oblimin(A, normalize = TRUE, randomStarts = 200)
cat("Oblimin random start diagnostics:\n")
res_oblimin$randStartChar

# Simplimax: complex landscape
set.seed(42)
res_ccai <- GPFallMinima(A, method = "simplimax",
                         randomStarts = 200,
                         normalize = TRUE,
                         minimumInclusion = 2)
res_ccai


###################################################
### code chunk number 4: GPA2local.Stex:254-256
###################################################
# Print the most common solution
print(res_ccai$minima[[1]]$result)


###################################################
### code chunk number 5: GPA2local.Stex:262-264
###################################################
# Print solution in row 3 of the summary
print(res_ccai$minima[[3]]$result, digits = 2)


###################################################
### code chunk number 6: GPA2local.Stex:269-271
###################################################
i <- 3
print(res_ccai$minima[[i]]$result, digits = 2)


###################################################
### code chunk number 7: GPA2local.Stex:278-281
###################################################
# Print the global minimum using the GPArotation S3 print method
global <- which(res_ccai$summary$isGlobal)
print(res_ccai$minima[[global]]$result, digits = 2)


###################################################
### code chunk number 8: GPA2local.Stex:284-286
###################################################
# Summary with structure matrix for oblique solution
summary(res_ccai$minima[[global]]$result, Structure = TRUE, digits = 2)


###################################################
### code chunk number 9: GPA2local.Stex:308-344
###################################################
plotSortedLoadings <- function(..., labels = NULL, col = NULL,
                                main = "Sorted Absolute Loadings",
                                ylab = "Absolute loading",
                                xlab = "Rank") {
  solutions <- list(...)
  for (i in seq_along(solutions)) {
    if (!inherits(solutions[[i]], "GPArotation"))
      stop("Argument ", i, " is not a GPArotation object.")
  }
  n <- length(solutions)
  if (is.null(labels))
    labels <- paste("Solution", seq_len(n))
  if (is.null(col))
    col <- palette.colors(n, palette = "Okabe-Ito")
  sorted_loadings <- lapply(solutions, function(x)
    sort(abs(as.vector(x$loadings)), decreasing = FALSE))
  all_values <- unlist(sorted_loadings)
  max_len    <- max(sapply(sorted_loadings, length))
  plot(NULL,
       xlim = c(1, max_len),
       ylim = c(0, max(all_values)),
       main = main,
       xlab = xlab,
       ylab = ylab,
       las  = 1)
  abline(h = seq(0, 1, by = 0.1), col = "grey90", lty = 1)
  for (i in seq_len(n)) {
    lines(seq_along(sorted_loadings[[i]]), sorted_loadings[[i]],
          col = col[i], lwd = 2)
    points(seq_along(sorted_loadings[[i]]), sorted_loadings[[i]],
           col = col[i], pch = 19, cex = 0.6)
  }
  legend("topleft", legend = labels, col = col, lwd = 2, pch = 19,
         bty = "n")
  invisible(sorted_loadings)
}


###################################################
### code chunk number 10: GPA2local.Stex:349-354
###################################################
do.call(plotSortedLoadings,
        c(lapply(res_ccai$minima, function(x) x$result),
          list(labels = paste0("Min ", res_ccai$summary$minimum,
                               " (f=", round(res_ccai$summary$f, 4),
                               ", n=", res_ccai$summary$count, ")"))))

### R code from vignette source 'GPA3bifactor.Stex'

###################################################
### code chunk number 1: GPA3bifactor.Stex:20-23
###################################################
options(continue="  ")
pdf.options(pointsize = 8)
library("GPArotation")


###################################################
### code chunk number 2: GPA3bifactor.Stex:69-83
###################################################
data("WansbeekMeijer", package = "GPArotation")
fa.unrotated <- factanal(factors = 3, covmat = NetherlandsTV,
                         rotation = "none")

# Orthogonal bifactor rotation
res.bifT <- bifactorT(loadings(fa.unrotated))
print(res.bifT, sortLoadings = FALSE, digits = 3)

# Oblique bifactor rotation
res.bifQ <- bifactorQ(loadings(fa.unrotated))
print(res.bifQ, sortLoadings = FALSE, digits = 3)

# Structure matrix for oblique solution
summary(res.bifQ, Structure = TRUE)


###################################################
### code chunk number 3: GPA3bifactor.Stex:116-140
###################################################
omega_h <- function(bifactor_result) {
  # Compute omega hierarchical from a GPArotation bifactor solution.
  # Args:
  #   bifactor_result : a GPArotation object from bifactorT or bifactorQ
  L     <- loadings(bifactor_result)
  lg    <- L[, 1]       # general factor loadings
  Ls    <- L[, -1]      # group factor loadings
  theta <- 1 - rowSums(L^2)  # model-implied unique variances
  sum(lg)^2 / (sum(lg)^2 + sum(Ls^2) + sum(theta))
}

# Omega total: proportion of total score variance due to all common factors.
# Requires the observed or population correlation matrix R.
omega_t <- function(bifactor_result, R) {
  L     <- loadings(bifactor_result)
  R_hat <- L %*% t(L)
  sum(R_hat) / sum(R)
}

# Coefficient alpha from a correlation matrix
alpha_coef <- function(R) {
  k <- nrow(R)
  k / (k - 1) * (1 - sum(diag(R)) / sum(R))
}


###################################################
### code chunk number 4: GPA3bifactor.Stex:166-191
###################################################
# Example 1: Strong general factor (loadings .7), weak group factors (.2)
lambda_g1  <- rep(0.7, 12)
lambda_s1a <- c(rep(0.2, 4), rep(0.0, 8))
lambda_s1b <- c(rep(0.0, 4), rep(0.2, 4), rep(0.0, 4))
lambda_s1c <- c(rep(0.0, 8), rep(0.2, 4))

L1 <- cbind(lambda_g1, lambda_s1a, lambda_s1b, lambda_s1c)
R1 <- L1 %*% t(L1)
diag(R1) <- 1

fa1  <- factanal(factors = 4, covmat = R1, rotation = "none")
bif1 <- bifactorT(loadings(fa1))

# Example 2: Weaker general factor (loadings .3), stronger group factors (.6)
lambda_g2  <- rep(0.3, 12)
lambda_s2a <- c(rep(0.6, 4), rep(0.0, 8))
lambda_s2b <- c(rep(0.0, 4), rep(0.6, 4), rep(0.0, 4))
lambda_s2c <- c(rep(0.0, 8), rep(0.6, 4))

L2 <- cbind(lambda_g2, lambda_s2a, lambda_s2b, lambda_s2c)
R2 <- L2 %*% t(L2)
diag(R2) <- 1

fa2  <- factanal(factors = 4, covmat = R2, rotation = "none")
bif2 <- bifactorT(loadings(fa2))


###################################################
### code chunk number 5: GPA3bifactor.Stex:194-203
###################################################
cat("Example 1 - Strong general factor:\n")
cat("  alpha   =", round(alpha_coef(R1), 3), "\n")
cat("  omega_t =", round(omega_t(bif1, R1), 3), "\n")
cat("  omega_h =", round(omega_h(bif1), 3), "\n\n")

cat("Example 2 - Weaker general factor:\n")
cat("  alpha   =", round(alpha_coef(R2), 3), "\n")
cat("  omega_t =", round(omega_t(bif2, R2), 3), "\n")
cat("  omega_h =", round(omega_h(bif2), 3), "\n")


###################################################
### code chunk number 6: GPA3bifactor.Stex:268-269
###################################################
data("CCAI", package = "GPArotation")


###################################################
### code chunk number 7: GPA3bifactor.Stex:272-275
###################################################
cat("Range of observed correlations:\n")
cat("  Min:", round(min(CCAI_R[lower.tri(CCAI_R)]), 3), "\n")
cat("  Max:", round(max(CCAI_R[lower.tri(CCAI_R)]), 3), "\n")


###################################################
### code chunk number 8: GPA3bifactor.Stex:283-287
###################################################
fa_unrotated <- factanal(factors = 3, covmat = CCAI_R, 
         n.obs = 461, rotation = "none")
bif <- bifactorT(loadings(fa_unrotated))
print(bif, sortLoadings = FALSE, digits = 3)


###################################################
### code chunk number 9: GPA3bifactor.Stex:306-311
###################################################
cat("alpha   =", round(alpha_coef(CCAI_R), 3), "\n")
cat("omega_t =", round(omega_t(bif, CCAI_R), 3), "\n")
cat("omega_h =", round(omega_h(bif), 3), "\n")
cat("gap (omega_t - omega_h) =",
    round(omega_t(bif, CCAI_R) - omega_h(bif), 3), "\n")


###################################################
### code chunk number 10: GPA3bifactor.Stex:321-336
###################################################
omega_h_by_group <- function(bifactor_result, R) {
  L     <- loadings(bifactor_result)
  lg    <- L[, 1]
  Ls    <- L[, -1]
  theta <- 1 - rowSums(L^2)
  denom <- sum(lg)^2 + sum(Ls^2) + sum(theta)
  cat("Variance partition:\n")
  cat("  General factor:    ", round(sum(lg)^2 / denom, 3), "\n")
  for (j in 1:ncol(Ls))
    cat("  Group factor", j + 1, ":     ",
        round(sum(Ls[, j]^2) / denom, 3), "\n")
  cat("  Measurement error: ", round(sum(theta) / denom, 3), "\n")
  cat("  Total (omega_t):   ",
      round(omega_t(bifactor_result, R), 3), "\n")
}


###################################################
### code chunk number 11: GPA3bifactor.Stex:339-340
###################################################
omega_h_by_group(bif, CCAI_R)

### R code from vignette source 'GPA1guide.Stex'

###################################################
### code chunk number 1: GPA1guide.Stex:22-24
###################################################
 options(continue="  ")
pdf.options(pointsize = 8)


###################################################
### code chunk number 2: GPA1guide.Stex:44-45
###################################################
library("GPArotation")


###################################################
### code chunk number 3: GPA1guide.Stex:89-100
###################################################
data("Harman", package = "GPArotation")

# Calling a rotation directly
qHarman <- quartimax(Harman8)

# Equivalently, via the wrapper function
qHarman <- GPFRSorth(Harman8, method = "quartimax")

# Two equivalent ways to access the rotated loadings
loadings(qHarman)       # via extractor function (recommended)
qHarman$loadings        # via direct list access


###################################################
### code chunk number 4: GPA1guide.Stex:112-120
###################################################
data("CCAI", package = "GPArotation")
y <- factanal(factors = 3, covmat = CCAI_R, n.obs = 461, rotation = "none")
y.quart <- quartimax(y$loadings)
max(loadings(y.quart) %*% t(y.quart$Th) - loadings(y))
y.obli <- oblimin(y$loadings, normalize = TRUE, randomStarts = 15)
max(loadings(y.obli) %*% t(y.obli$Th) - loadings(y))
# last equation on Page 678
max(loadings(y.obli) - loadings(y) %*% solve(t(y.obli$Th)))


###################################################
### code chunk number 5: GPA1guide.Stex:126-129
###################################################
y <- factanal(factors = 3, covmat = CCAI_R, n.obs = 461, rotation = "none")
y.obli <- oblimin(y$loadings, normalize = TRUE, randomStarts = 15)
max(abs(y.obli$Phi - t(y.obli$Th) %*% y.obli$Th))


###################################################
### code chunk number 6: GPA1guide.Stex:143-154
###################################################
set.seed(334)
res <- quartimin(Harman8, normalize = TRUE, randomStarts = 100)

# Raw unsorted loadings
loadings(res)

# Sorted loadings via print (factors reordered and signs adjusted)
res.sorted <- print(res)

# Once sorted, repeated calls to print are stable
max(abs(print(res.sorted)$loadings - res.sorted$loadings)) == 0  # TRUE


###################################################
### code chunk number 7: GPA1guide.Stex:193-198
###################################################
res.obli <- oblimin(Harman8, normalize = TRUE, randomStarts = 100)
# Pattern matrix (unsorted)
summary(res.obli, Structure = FALSE)
# Structure matrix
summary(res.obli, Structure = TRUE)


###################################################
### code chunk number 8: GPA1guide.Stex:241-261
###################################################
plotPatternStructure <- function(pattern, structure,
                                  labels = NULL,
                                  main = "Pattern vs Structure") {
  k   <- ncol(pattern)
  col <- palette.colors(k, palette = "Okabe-Ito")
  par(mfrow = c(1, k))
  for (j in 1:k) {
    lims <- range(c(pattern[, j], structure[, j]))
    plot(pattern[, j], structure[, j],
         xlim = lims, ylim = lims,
         xlab = "Pattern loading",
         ylab = "Structure loading",
         main = paste(if (!is.null(labels)) labels[j]
                      else paste("Factor", j)),
         pch  = 19, col = col[j])
    abline(0, 1, lty = 2, col = "grey60")
    abline(h = 0, col = "grey80")
    abline(v = 0, col = "grey80")
  }
}


###################################################
### code chunk number 9: GPA1guide.Stex:264-270
###################################################
res.obli.s  <- print(res.obli)
Pattern     <- loadings(res.obli.s)
Structure   <- Pattern %*% res.obli.s$Phi

plotPatternStructure(Pattern, Structure,
                     labels = c("Factor 1", "Factor 2"))


###################################################
### code chunk number 10: GPA1guide.Stex:295-299
###################################################
data(Thurstone, package = "GPArotation")
infomaxQ(box26, randomStarts = 100)         # 100 random starts
infomaxQ(box26, Tmat = Random.Start(3))     # single random start
infomaxQ(box26, randomStarts = 1)           # also a single random start


###################################################
### code chunk number 11: GPA1guide.Stex:328-332
###################################################
res <- geominQ(box26, normalize = TRUE, randomStarts = 100)
res$randStartChar
# Criterion value at best solution
res$Table[nrow(res$Table), "f"]


###################################################
### code chunk number 12: GPA1guide.Stex:357-413
###################################################
plotRotationLandscape <- function(A, method = "quartimax", n = 1000,
                                   main = NULL, ...) {
  # Plot the objective function landscape for 2-factor orthogonal rotation.
  # For 2 factors, all orthogonal rotations are parameterized by a single
  # angle theta in [0, 2*pi), giving a clean 1D landscape.
  #
  # Args:
  #   A      : a 2-factor unrotated loading matrix
  #   method : rotation criterion (default "quartimax")
  #   n      : number of angles to evaluate (default 1000)
  #   main   : plot title (default: "Rotation landscape: <method>")
  #   ...    : additional arguments passed to the vgQ criterion function

  if (ncol(A) != 2)
    stop("plotRotationLandscape only works for 2-factor solutions.")

  vgQfun_fn <- get(paste("vgQ", method, sep = "."),
                   envir = asNamespace("GPArotation"))

  if (is.null(main))
    main <- paste("Rotation landscape:", method)

  theta  <- seq(0, 2 * pi, length.out = n)
  f_vals <- numeric(n)

  for (i in seq_along(theta)) {
    Tmat      <- matrix(c( cos(theta[i]), sin(theta[i]),
                          -sin(theta[i]), cos(theta[i])), 2, 2)
    L         <- A %*% Tmat
    VgQ       <- do.call(vgQfun_fn, append(list(L), list(...)))
    f_vals[i] <- VgQ$f
  }

  # Find global minimum
  min_idx <- which.min(f_vals)

  plot(theta, f_vals,
       type  = "l",
       lwd   = 2,
       main  = main,
       xlab  = expression(theta ~ "(radians)"),
       ylab  = "f",
       xaxt  = "n")
  axis(1, at     = c(0, pi/2, pi, 3*pi/2, 2*pi),
           labels = c("0", expression(pi/2), expression(pi),
                      expression(3*pi/2), expression(2*pi)))
  abline(h   = f_vals[min_idx], col = "grey80", lty = 2)
  points(theta[min_idx], f_vals[min_idx],
         col = "tomato", pch = 19, cex = 1.5)
  text(theta[min_idx], f_vals[min_idx],
       labels = paste0("min f = ", round(f_vals[min_idx], 4)),
       pos    = 4,
       cex    = 0.8)

  invisible(data.frame(theta = theta, f = f_vals))
}


###################################################
### code chunk number 13: GPA1guide.Stex:419-420
###################################################
data(Harman, package = "GPArotation")


###################################################
### code chunk number 14: GPA1guide.Stex:423-428
###################################################
par(mfrow = c(2, 2))
plotRotationLandscape(Harman8, method = "quartimax")
plotRotationLandscape(Harman8, method = "varimax")
plotRotationLandscape(Harman8, method = "bentler")
plotRotationLandscape(Harman8, method = "entropy")


###################################################
### code chunk number 15: GPA1guide.Stex:455-468
###################################################
res <- plotRotationLandscape(Harman8, method = "simplimax", k = 4)

# Find all local minima by sign changes in the discrete derivative
df         <- diff(res$f)
local_mins <- which(df[-length(df)] < 0 & df[-1] > 0)

# Mark all local minima on the existing plot
points(res$theta[local_mins], res$f[local_mins],
       col = "steelblue", pch = 19, cex = 0.8)
legend("topright",
       legend = c("global minimum", "local minima"),
       col    = c("tomato", "steelblue"),
       pch    = 19, bty = "n", cex = 0.8)


###################################################
### code chunk number 16: GPA1guide.Stex:471-474
###################################################
cat("Total minima found:              ", length(local_mins), "\n")
cat("Expected due to symmetry:        ", 4, "\n")
cat("Approximate genuine local minima:", max(0, length(local_mins) - 4), "\n")


###################################################
### code chunk number 17: GPA1guide.Stex:505-536
###################################################
data(Harman, package = "GPArotation")
res.quart   <- quartimax(Harman8)
res.oblimin <- oblimin(Harman8)

L.quart   <- abs(loadings(res.quart))
L.oblimin <- abs(loadings(res.oblimin))

ord.quart   <- order(L.quart[, 1],   decreasing = TRUE)
ord.oblimin <- order(L.oblimin[, 1], decreasing = TRUE)

par(mfrow = c(1, 2), mar = c(5, 4, 4, 2))

barplot(t(L.quart[ord.quart, ]),
        beside      = TRUE,
        ylim        = c(0, 1),
        main        = "Quartimax",
        ylab        = "Absolute loading",
        xlab        = "Variable (sorted by Factor 1)",
        legend.text = c("Factor 1", "Factor 2"),
        args.legend = list(x = "topright"),
        col         = c("steelblue", "tomato"))

barplot(t(L.oblimin[ord.oblimin, ]),
        beside      = TRUE,
        ylim        = c(0, 1),
        main        = "Oblimin",
        ylab        = "Absolute loading",
        xlab        = "Variable (sorted by Factor 1)",
        legend.text = c("Factor 1", "Factor 2"),
        args.legend = list(x = "topright"),
        col         = c("steelblue", "tomato"))


###################################################
### code chunk number 18: GPA1guide.Stex:556-621
###################################################
plotSortedLoadings <- function(..., labels = NULL, col = NULL, 
                                main = "Sorted Absolute Loadings",
                                ylab = "Absolute loading", 
                                xlab = "Rank") {
  # Plot sorted absolute loadings for one or more GPArotation objects.
  # Multiple solutions are overlaid on a single plot for comparison.
  # Loadings are sorted from smallest to largest (left to right).
  #
  # Args:
  #   ...    : one or more GPArotation objects
  #   labels : character vector of legend labels (default: "Solution 1", etc.)
  #   col    : character vector of colors (default: auto-assigned)
  #   main   : plot title
  #   ylab   : y-axis label
  #   xlab   : x-axis label

  solutions <- list(...)

  for (i in seq_along(solutions)) {
    if (!inherits(solutions[[i]], "GPArotation"))
      stop("Argument ", i, " is not a GPArotation object.")
  }

  n <- length(solutions)

  if (is.null(labels))
    labels <- paste("Solution", seq_len(n))
  if (is.null(col))
    col <- palette.colors(n, palette = "Okabe-Ito")

  sorted_loadings <- lapply(solutions, function(x)
    sort(abs(as.vector(x$loadings)), decreasing = FALSE))

  all_values <- unlist(sorted_loadings)
  max_len    <- max(sapply(sorted_loadings, length))

  plot(NULL,
       xlim = c(1, max_len),
       ylim = c(0, max(all_values)),
       main = main,
       xlab = xlab,
       ylab = ylab,
       las  = 1)

  abline(h = seq(0, 1, by = 0.1), col = "grey90", lty = 1)

  for (i in seq_len(n)) {
    lines(seq_along(sorted_loadings[[i]]), sorted_loadings[[i]],
          col = col[i], lwd = 2)
    points(seq_along(sorted_loadings[[i]]), sorted_loadings[[i]],
           col = col[i], pch = 19, cex = 0.6)
  }

  legend("topleft", legend = labels, col = col, lwd = 2, pch = 19,
         bty = "n")

  invisible(sorted_loadings)
}

# Example
data(Harman, package = "GPArotation")
res.quart   <- quartimax(Harman8)
res.oblimin <- oblimin(Harman8)
res.geomin  <- geominT(Harman8)



###################################################
### code chunk number 19: GPA1guide.Stex:626-628
###################################################
plotSortedLoadings(res.quart, res.oblimin, res.geomin,
                   labels = c("Quartimax", "Oblimin", "Geomin"))


###################################################
### code chunk number 20: GPA1guide.Stex:685-716
###################################################
origdigits <- options("digits")
options(digits = 2)
trBritain <- matrix(c(.783,-.163,.811,.202,.724,.209,.850,.064,
  -.031,.592,-.028,.723,.388,.434,.141,.808,.215,.709),
  byrow = TRUE, ncol = 2)
trGermany <- matrix(c(.778,-.066,.875,.081,.751,.079,.739,.092,
  .195,.574,-.030,.807,-.135,.717,.125,.738,.060,.691),
  byrow = TRUE, ncol = 2)
# orthogonal rotation of trGermany towards trBritain
trx <- targetT(trGermany, Target = trBritain)
# Factor loadings after target rotation
trx
# Differences between loadings matrices after rotation
y <- trx$loadings - trBritain
print(y, digits = 1)
# Square root of the mean squared difference per item
sqrt(apply((y^2), 1, mean))
# Square root of the mean squared difference per factor
sqrt(apply((y^2), 2, mean))
# Identity coefficient per factor after rotation
2 * colSums(trx$loadings * trBritain) /
  (colSums(trx$loadings^2) + colSums(trBritain^2))
# Additivity coefficient per factor after rotation
diag(2 * cov(trx$loadings, trBritain)) /
  diag(var(trx$loadings) + var(trBritain))
# Proportionality coefficient per factor after rotation
colSums(trBritain * trx$loadings) /
  sqrt(colSums(trBritain^2) * colSums(trx$loadings^2))
# Correlation for each factor after rotation
diag(cor(trBritain, trx$loadings))
options(digits = origdigits$digits)


###################################################
### code chunk number 21: GPA1guide.Stex:728-751
###################################################
plot(trBritain[, 1], trBritain[, 2],
     xlim = c(-0.3, 1.0), ylim = c(-0.3, 1.0),
     xlab = "Factor 1", ylab = "Factor 2",
     main = "Target Rotation: Germany towards Britain",
     pch = 19, col = "steelblue", cex = 1.2)
abline(h = 0, lty = 2, col = "grey70")
abline(v = 0, lty = 2, col = "grey70")
points(trGermany[, 1], trGermany[, 2],
       pch = 17, col = "tomato", cex = 1.2)
points(loadings(trx)[, 1], loadings(trx)[, 2],
       pch = 15, col = "orange", cex = 1.2)
for (i in 1:nrow(trGermany)) {
  arrows(trGermany[i, 1], trGermany[i, 2],
         loadings(trx)[i, 1], loadings(trx)[i, 2],
         length = 0.08, col = "grey60")
}
legend("topright",
       legend = c("Britain (varimax rotated)", 
       		"East Germany (varimax rotated)", 
		"East Germany (rotated towards Britain)"),
       col    = c("steelblue", "tomato", "orange"),
       pch    = c(19, 17, 15),
       bty    = "n")


###################################################
### code chunk number 22: GPA1guide.Stex:778-792
###################################################
A <- matrix(c(.664, .688, .492, .837, .705, .82, .661, .457, .765, .322,
  .248, .304, -0.291, -0.314, -0.377, .397, .294, .428, -0.075, .192, .224,
  .037, .155, -.104, .077, -.488, .009), ncol = 3)
# using targetT
SPA <- matrix(c(rep(NA, 6), .7, .0, .7, rep(0, 3), rep(NA, 7),
  0, 0, NA, 0, rep(NA, 4)), ncol = 3)
xt <- targetT(A, Target = SPA)
# using pstT
SPApst <- matrix(c(rep(0, 6), .7, .0, .7, rep(0, 3), rep(0, 7),
  0, 0, 0, 0, rep(0, 4)), ncol = 3)
SPAW <- matrix(c(rep(0, 6), rep(1, 6), rep(0, 7), 1, 1, 0, 1,
  rep(0, 4)), ncol = 3)
xpst <- pstT(A, Target = SPApst, W = SPAW)
max(abs(loadings(xt) - loadings(xpst)))


###################################################
### code chunk number 23: GPA1guide.Stex:810-814
###################################################
data("CCAI", package = "GPArotation")
factanal(factors = 3, covmat = CCAI_R, n.obs = 461, rotation = "infomaxT")
factanal(factors = 3, covmat = CCAI_R, n.obs = 461, rotation = "infomaxT",
  control = list(rotate = list(normalize = TRUE, eps = 1e-6)))


###################################################
### code chunk number 24: GPA1guide.Stex:822-826
###################################################
data("WansbeekMeijer", package = "GPArotation")
fa.unrotated <- factanal(factors = 3, covmat = NetherlandsTV,
                         normalize = TRUE, rotation = "none")
quartimin(loadings(fa.unrotated), normalize = TRUE)


###################################################
### code chunk number 25: GPA1guide.Stex:836-859
###################################################
data("WansbeekMeijer", package = "GPArotation")
fa.unrotated <- factanal(factors = 3, covmat = NetherlandsTV,
                         normalize = TRUE, rotation = "none")
# Two-step procedure (always correct)
set.seed(42)
fa.cf <- cfQ(loadings(fa.unrotated), kappa = 0.3, normalize = TRUE,
             randomStarts = 100)
fa.cf

if (getRversion() >= "4.5.1") {
  # Single-step via factanal (correct in R >= 4.5.1)
  set.seed(42)
  fa.factanal <- factanal(factors = 3, covmat = NetherlandsTV,
                          normalize = TRUE, rotation = "cfQ",
                          control = list(rotate = list(kappa = 0.3,
                                                       randomStarts = 100)))
  fa.sorted <- print(fa.cf, sortLoadings = TRUE)
  cat("Maximum difference in loadings:\n")
  print(max(abs(abs(fa.sorted$loadings) - abs(fa.factanal$loadings))))
} else {
  cat("Single-step factanal oblique rotation requires R >= 4.5.1.\n")
  cat("Use the two-step procedure above for correct results.\n")
}


###################################################
### code chunk number 26: GPA1guide.Stex:874-952
###################################################
  factanal_fit <- function(fa, R_obs, n) {
  # Compute common factor model fit indices from factanal output.
  # For MLE extraction only (factanal). For other estimators (minres,
  # principal axis, WLS) the chi-square statistic is not defined in
  # the same way and this function should not be used.
  #
  # Args:
  #   fa    : a factanal object (rotation = "none" recommended)
  #   R_obs : observed correlation or covariance matrix passed to factanal
  #   n     : sample size (n.obs passed to factanal)

  if (is.null(fa$STATISTIC))
    stop("factanal did not compute a chi-square statistic. ",
         "Ensure n.obs is specified when passing a covariance matrix.")

  # Ensure we work with a correlation matrix
  R_obs <- cov2cor(as.matrix(R_obs))
  p     <- nrow(R_obs) 
  L     <- loadings(fa)
  k     <- ncol(L)           # number of factors extracted
 
  # Model-implied correlation matrix using factanal uniquenesses
  # fa$uniquenesses are the MLE estimated unique variances
  R_hat <- L %*% t(L) + diag(fa$uniquenesses)
  R_hat <- cov2cor(R_hat)
  
  # Residual matrix --- off-diagonal only
  Resid <- R_obs - R_hat
  diag(Resid) <- 0

  # Test of the hypothesis that k factors are sufficient. Identical to factanal
  F_ml <- log(det(R_hat)) - log(det(R_obs)) + sum(diag(R_obs %*% solve(R_hat))) - p
  chi2 <- (n - 1 - (2 * p + 5)/6 - (2 * k)/3) * F_ml  
  df <- ((p - k)^2 - (p + k)) / 2
  pval <- pchisq(chi2, df, lower.tail = FALSE)

  # SRMR: standardized root mean square residual
  rstar.off <- sum(Resid^2) / 2
  srmr <- sqrt(rstar.off / (p * (p - 1)))

  # RMSEA: root mean square error of approximation
  rmsea <- sqrt(max(0, chi2 / (df * n) - 1 / (n - 1)))

  # Null model: all items uncorrelated
  chi2_null <- (n - 1) * sum(R_obs[lower.tri(R_obs)]^2)
  df_null   <- p * (p - 1) / 2

  # CFI: comparative fit index
  cfi <- (max(chi2_null - df_null, 0) - max(chi2 - df, 0)) /
          max(chi2_null - df_null, 0)

  # TLI: Tucker-Lewis index
  tli <- (chi2_null / df_null - chi2 / df) /
         (chi2_null / df_null - 1)

  # AIC and BIC
  aic <- chi2 - 2 * df
  bic <- chi2 - log(n) * df

  # Print results
  cat("Factor Model Fit Indices (MLE only)\n")
  cat("------------------------------------\n")
  cat(sprintf("Chi-square (df = %d):  %.3f  p = %.4f\n", df, chi2, pval))
  cat(sprintf("RMSEA:                 %.4f\n", rmsea))
  cat(sprintf("SRMR:                  %.4f\n", srmr))
  cat(sprintf("CFI:                   %.4f\n", cfi))
  cat(sprintf("TLI:                   %.4f\n", tli))
  cat(sprintf("AIC:                   %.3f\n", aic))
  cat(sprintf("BIC:                   %.3f\n", bic))
  cat("\nTop 5 absolute residuals:\n")
  resid_vals <- Resid[lower.tri(Resid)]
  print(round(sort(abs(resid_vals), decreasing = TRUE)[1:5], 4))

  invisible(c(chi2  = chi2,  df   = df,   pval = pval,
              rmsea = rmsea, srmr = srmr,
              cfi   = cfi,   tli  = tli,
              aic   = aic,   bic  = bic))
}


###################################################
### code chunk number 27: GPA1guide.Stex:958-966
###################################################
data("WansbeekMeijer", package = "GPArotation")
fa2 <- factanal(factors = 2, covmat = NetherlandsTV, rotation = "none")
fa3 <- factanal(factors = 3, covmat = NetherlandsTV, rotation = "none")
cat("=== 2 factors ===\n")
fit2 <- factanal_fit(fa2, cov2cor(NetherlandsTV$cov), n = 2154)

cat("\n=== 3 factors ===\n")
fit3 <- factanal_fit(fa3, cov2cor(NetherlandsTV$cov), n = 2154)


###################################################
### code chunk number 28: GPA1guide.Stex:969-977
###################################################
cat("Model comparison:\n")
cat(sprintf("           2-factor  3-factor\n"))
cat(sprintf("RMSEA:     %8.4f  %8.4f\n", fit2["rmsea"],  fit3["rmsea"]))
cat(sprintf("SRMR:      %8.4f  %8.4f\n", fit2["srmr"],   fit3["srmr"]))
cat(sprintf("CFI:       %8.4f  %8.4f\n", fit2["cfi"],    fit3["cfi"]))
cat(sprintf("TLI:       %8.4f  %8.4f\n", fit2["tli"],    fit3["tli"]))
cat(sprintf("AIC:       %8.2f  %8.2f\n", fit2["aic"],    fit3["aic"]))
cat(sprintf("BIC:       %8.2f  %8.2f\n", fit2["bic"],    fit3["bic"]))

### R code from vignette source 'GPAguide.Stex'

###################################################
### code chunk number 1: GPAguide.Stex:21-22
###################################################
 options(continue="  ")


###################################################
### code chunk number 2: GPAguide.Stex:35-36
###################################################
library("GPArotation")  


###################################################
### code chunk number 3: GPAguide.Stex:67-75
###################################################
data(ability.cov)
z <- factanal(factors = 2, covmat = ability.cov, rotation = "none")
# quartimax rotation
GPFRSorth(loadings(z), method = "quartimax")
quartimax(z$loadings)
# oblimin rotation
GPFRSoblq(z$loadings, method = "oblimin")
oblimin(loadings(z))


###################################################
### code chunk number 4: GPAguide.Stex:88-95
###################################################
y <- factanal(factors=3, covmat=ability.cov, rotation = "none")
y.quart <- quartimax(y$loadings)
max( loadings(y.quart) %*% t(y.quart$Th) - loadings(y) )
y.obli <- oblimin(y$loadings, normalize=TRUE, randomStarts=15)
max( loadings(y.obli) %*% t(y.obli$Th) - loadings(y) )
# last equation on Page 678
max( loadings(y.obli) - loadings(y) %*% solve(t(y.obli$Th)) )


###################################################
### code chunk number 5: GPAguide.Stex:98-101
###################################################
y <- factanal(factors=3, covmat=ability.cov, rotation = "none", randomStarts=15)
y.obli <- oblimin(y$loadings, normalize=TRUE, randomStarts=15)
max(abs(y.obli$Phi  - t(y.obli$Th) %*% y.obli$Th))


###################################################
### code chunk number 6: GPAguide.Stex:111-115
###################################################
data(Thurstone, package = "GPArotation")
infomaxQ(box26, randomStarts = 100) # 100 random starts
infomaxQ(box26, Tmat=Random.Start(3)) # a single random start
infomaxQ(box26, randomStarts = 1) # also a single random start


###################################################
### code chunk number 7: GPAguide.Stex:162-188
###################################################
origdigits <- options("digits")
options(digits = 2)
trBritain <- matrix( c(.783,-.163,.811,.202,.724,.209,.850,.064,
-.031,.592,-.028,.723,.388,.434,.141,.808,.215,.709), byrow=TRUE, ncol=2)
trGermany <- matrix( c(.778,-.066, .875,.081, .751,.079, .739,.092,
.195,.574, -.030,.807, -.135,.717, .125,.738, .060,.691), byrow=TRUE, ncol = 2)
# orthogonal rotation of trGermany towards trBritain
trx <- targetT(trGermany, Target = trBritain)
# Factor loadings after target rotation
trx
# Differences between loadings matrices after rotation
y <- trx$loadings - trBritain
print(y, digits = 1)
# Square Root of the mean squared difference per item
sqrt(apply((y^2), 1, mean))
# Square Root of the mean squared difference per factor
sqrt(apply((y^2), 2, mean))
# Identity coefficient per factor after rotation
2 * colSums(trx$loadings*trBritain)/( colSums(trx$loadings^2)+colSums(trBritain^2))
# Additivity coefficient per factor after rotation
diag(2 * cov(trx$loadings, trBritain) ) / diag(var(trx$loadings)+var(trBritain))
# Proportionality coefficient per factor after rotation
colSums(trBritain * trx$loadings)/sqrt(colSums(trBritain^2)*colSums(trx$loadings^2))
# Correlation for each factor per factor after rotation
diag(cor(trBritain, trx$loadings))
options(digits = origdigits$digits)


###################################################
### code chunk number 8: GPAguide.Stex:201-215
###################################################
A <- matrix(c(.664, .688, .492, .837, .705, .82, .661, .457, .765, .322, 
  .248, .304, -0.291, -0.314, -0.377, .397, .294, .428, -0.075,.192,.224,
  .037, .155,-.104,.077,-.488,.009), ncol=3)  
 # using targetT
SPA <- matrix(c(rep(NA, 6), .7,.0,.7, rep(0,3), rep(NA, 7), 
  0,0, NA, 0, rep(NA, 4)), ncol=3)
xt <- targetT(A, Target=SPA)
# using pstT
SPApst <- matrix(c(rep(0, 6), .7,.0,.7, rep(0,3), rep(0, 7), 
  0, 0, 0, 0, rep(0, 4)), ncol=3)
SPAW <- matrix(c(rep(0, 6), rep(1, 6), rep(0, 7), 1, 1, 0, 1, 
  rep(0, 4)), ncol=3)
xpst <- pstT(A, Target = SPApst, W = SPAW)
max(abs(loadings(xt)- loadings(xpst)))

Random.Start <- function(k){
  qr.Q(qr(matrix(rnorm(k*k),k)))
  }

NormalizingWeight <- function(A, normalize=FALSE){
 if ("function" == mode(normalize)) normalize <- normalize(A)
 if (is.logical(normalize)){
    if (normalize) normalize <- sqrt(rowSums(A^2))
    else return(array(1, dim(A)))
    }
 if (is.vector(normalize)) 
    {if(nrow(A) != length(normalize))
        stop("normalize length wrong in NormalizingWeight")
     return(array(normalize, dim(A)))
    }
 stop("normalize argument not recognized in NormalizingWeight")
}

print.GPArotation <- function (x, digits=3, Table=FALSE, ...){
   cat(if(x$orthogonal)"Orthogonal" else "Oblique")
   cat(" rotation method", x$method)
   cat((if(!x$convergence)" NOT" ), "converged.\n")

   cat("Loadings:\n")
   print(x$loadings, digits=digits)

   cat("\nRotating matrix:\n")
   print(t(solve(x$Th)), digits=digits)

   if(!x$orthogonal){
     cat("\nPhi:\n")
     print(x$Phi, digits=digits)
     }

   if(Table){
     cat("\nIteration table:\n")
     print(x$Table, digits=digits)
     }
   invisible(x)
   }

summary.GPArotation <- function (object, ...){ 
   r <- list(loadings=object$loadings, method=object$method, orthogonal=object$orthogonal, 
          convergence=object$convergence, iters=nrow(object$Table))
   class(r) <- "summary.GPArotation"
   r
   }

print.summary.GPArotation <- function (x, digits=3, ...){
   cat(if(x$orthogonal)"Orthogonal" else "Oblique")
   cat(" rotation method", x$method)
   if(!x$convergence) cat(" NOT" )
   cat(" converged in ", x$iter, " iterations.\n")

   cat("Loadings:\n")
   print(x$loadings, digits=digits)
  }

GPForth <- function(A, Tmat=diag(ncol(A)), normalize=FALSE, eps=1e-5, maxit=1000, 
		    method="varimax", methodArgs=NULL){
 if((!is.logical(normalize)) || normalize) {
     W <- NormalizingWeight(A, normalize=normalize)
     normalize <- TRUE
     A <- A/W
     }
 if(1 >= ncol(A)) stop("rotation does not make sense for single factor models.")
 al <- 1
 L <- A %*% Tmat
 #Method <- get(paste("vgQ",method,sep="."))
 #VgQ <- Method(L, ...)
 Method <- paste("vgQ",method,sep=".")
 VgQ <- do.call(Method, append(list(L), methodArgs))
 G <- crossprod(A,VgQ$Gq)
 f <- VgQ$f
 Table <- NULL
 #set initial value for the unusual case of an exact initial solution 
 VgQt <- do.call(Method, append(list(L), methodArgs))   
 for (iter in 0:maxit){
   M <- crossprod(Tmat,G)
   S <- (M + t(M))/2
   Gp <- G - Tmat %*% S
   s <- sqrt(sum(diag(crossprod(Gp))))
   Table <- rbind(Table, c(iter, f, log10(s), al))
   if (s < eps)  break
   al <- 2*al
   for (i in 0:10){
     X <- Tmat - al * Gp
     UDV <- svd(X)
     Tmatt <- UDV$u %*% t(UDV$v)
     L <- A %*% Tmatt
     #VgQt <- Method(L, ...)
     VgQt <- do.call(Method, append(list(L), methodArgs))
     if (VgQt$f < (f - 0.5*s^2*al)) break
     al <- al/2
     }
   Tmat <- Tmatt
   f <- VgQt$f
   G <- crossprod(A,VgQt$Gq)
   }
 convergence <- (s < eps)
 if ((iter == maxit) & !convergence)
     warning("convergence not obtained in GPForth. ", maxit, " iterations used.")
 if(normalize) L <- L * W
 dimnames(L) <- dimnames(A)
 r <- list(loadings=L, Th=Tmat, Table=Table, 
        method=VgQ$Method, orthogonal=TRUE, convergence=convergence, Gq=VgQt$Gq)
 class(r) <- "GPArotation"
 r
}

GPFoblq <- function(A, Tmat=diag(ncol(A)), normalize=FALSE, eps=1e-5, maxit=1000, 
		    method="quartimin", methodArgs=NULL){
 if(1 >= ncol(A)) stop("rotation does not make sense for single factor models.")
 if((!is.logical(normalize)) || normalize) {
     W <- NormalizingWeight(A, normalize=normalize)
     normalize <- TRUE
     A <- A/W
     }
 al <- 1
 L <- A %*% t(solve(Tmat))
 #Method <- get(paste("vgQ",method,sep="."))
 #VgQ <- Method(L, ...)
 Method <- paste("vgQ",method,sep=".")
 VgQ <- do.call(Method, append(list(L), methodArgs))
 G <- -t(t(L) %*% VgQ$Gq %*% solve(Tmat))
 f <- VgQ$f
 Table <- NULL
 #Table <- c(-1,f,log10(sqrt(sum(diag(crossprod(G))))),al)
 #set initial value for the unusual case of an exact initial solution 
 VgQt <- do.call(Method, append(list(L), methodArgs))   
 for (iter in 0:maxit){
   Gp <- G - Tmat %*% diag(c(rep(1,nrow(G)) %*% (Tmat*G)))
   s <- sqrt(sum(diag(crossprod(Gp))))
   Table <- rbind(Table,c(iter,f,log10(s),al))
   if (s < eps) break
   al <- 2*al
   for (i in 0:10){
     X <- Tmat - al*Gp
     v <- 1/sqrt(c(rep(1,nrow(X)) %*% X^2))
     Tmatt <- X %*% diag(v)
     L <- A %*% t(solve(Tmatt))
     #VgQt <- Method(L, ...)
     VgQt <- do.call(Method, append(list(L), methodArgs))
     improvement <- f - VgQt$f 
     if (improvement >  0.5*s^2*al) break
     al <- al/2
     }
   Tmat <- Tmatt
   f <- VgQt$f
   G <- -t(t(L) %*% VgQt$Gq %*% solve(Tmatt))
   }
 convergence <- (s < eps)
 if ((iter == maxit) & !convergence)
     warning("convergence not obtained in GPFoblq. ", maxit, " iterations used.")
 if(normalize) L <- L * W
 dimnames(L) <- dimnames(A)
 # N.B. renaming Lh to loadings in specificific rotations 
 #   uses fact that  Lh is first.
 r <- list(loadings=L, Phi=t(Tmat) %*% Tmat, Th=Tmat, Table=Table,
      method=VgQ$Method, orthogonal=FALSE, convergence=convergence, Gq=VgQt$Gq)
 class(r) <- "GPArotation"
 r
}

#######################


oblimin <- function(L, Tmat=diag(ncol(L)), gam=0, normalize=FALSE, eps=1e-5, maxit=1000){
   GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit, 
                method="oblimin",  methodArgs=list(gam=gam))
   }

vgQ.oblimin <- function(L, gam=0){
  X <- L^2 %*% (!diag(TRUE,ncol(L))) 
  if (0 != gam) {
     p <- nrow(L)
     X <- (diag(1,p) - matrix(gam/p,p,p)) %*% X
     }
  list(Gq=L*X,
       f=sum(L^2 * X)/4,
       Method=  if (gam == 0)  "Oblimin Quartimin" else
		if (gam == .5) "Oblimin Biquartimin" else
		if (gam == 1)  "Oblimin Covarimin"   else
		         paste("Oblimin g=", gam,sep="")  )
}

# original
# vgQ.oblimin <- function(L, gam=0){
#   Method <- paste("Oblimin g=",gam,sep="")
#   if (gam == 0) Method <- "Oblimin Quartimin"
#   if (gam == .5) Method <- "Oblimin Biquartimin"
#   if (gam == 1) Method <- "Oblimin Covarimin"
#   k <- ncol(L)
#   p <- nrow(L)
#   N <- matrix(1,k,k)-diag(k)
#   f <- sum(L^2 * (diag(p)-gam*matrix(1/p,p,p)) %*% L^2 %*% N)/4
#   Gq <- L * ((diag(p)-gam*matrix(1/p,p,p)) %*% L^2 %*% N)
#   return(list(Gq=Gq,f=f,Method=Method))
# }
##vgQ.oblimin(FA2)$f - vgQ.origoblimin(FA2)$f
#vgQ.oblimin(FA2)$Gq - vgQ.origoblimin(FA2)$Gq


quartimin <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000){
   GPFoblq(L, Tmat=Tmat, method="quartimin", normalize=normalize, eps=eps, maxit=maxit)
   }

vgQ.quartimin <- function(L){
  X <-  L^2 %*% (!diag(TRUE,ncol(L))) 
  list(Gq= L*X,
       f= sum(L^2 * X)/4,  
       Method=  "Quartimin" )
  }

#original
#vgQ.quartimin <- function(L){
#  Method="Quartimin"
#  L2 <- L^2
#  k <- ncol(L)
#  M <- matrix(1,k,k)-diag(k)
#  f <- sum(L2 * (L2 %*% M))/4
#  Gq <- L * (L2 %*% M)
#  return(list(Gq=Gq,f=f,Method=Method))
#} 


targetT <- function(L, Tmat=diag(ncol(L)), Target=NULL, normalize=FALSE, eps=1e-5, maxit=1000) {
   if(is.null(Target)) stop("argument Target must be specified.")
   GPForth(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
           method="target", methodArgs=list(Target=Target))
   }

targetQ <- function(L, Tmat=diag(ncol(L)), Target=NULL, normalize=FALSE, eps=1e-5, maxit=1000) {
   if(is.null(Target)) stop("argument Target must be specified.")
   GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
           method="target", methodArgs=list(Target=Target))
   }

vgQ.target <- function(L, Target=NULL){
   if(is.null(Target)) stop("argument Target must be specified.")
   #   e.g.  Target <- matrix(c(rep(NA,4),rep(0,8),rep(NA,4)),8) 
   #  approximates  Michael Brown approach
   Gq <-  2 * (L - Target)
   Gq[is.na(Gq)] <- 0 #missing elements in target do not affect the first derivative 
   list(Gq=Gq,
        f=sum((L-Target)^2, na.rm=TRUE),  
	Method="Target rotation")  #The target rotation ? option in Michael Browne's algorithm should be NA
   }

pstT <- function(L, Tmat=diag(ncol(L)), W=NULL, Target=NULL, normalize=FALSE, eps=1e-5, maxit=1000) {
   if(is.null(W))      stop("argument W must be specified.")
   if(is.null(Target)) stop("argument Target must be specified.")
   GPForth(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit, 
	    method="pst", methodArgs=list(W=W, Target=Target))
   }

pstQ <- function(L, Tmat=diag(ncol(L)), W=NULL, Target=NULL, normalize=FALSE, eps=1e-5, maxit=1000) {
   if(is.null(W))      stop("argument W must be specified.")
   if(is.null(Target)) stop("argument Target must be specified.")
   GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit, 
	  method="pst", methodArgs=list(W=W, Target=Target))
   }

vgQ.pst <- function(L, W=NULL, Target=NULL){
   if(is.null(W))      stop("argument W must be specified.")
   if(is.null(Target)) stop("argument Target must be specified.")
   # Needs weight matrix W with 1's at specified values, 0 otherwise
   # e.g. W = matrix(c(rep(1,4),rep(0,8),rep(1,4)),8). 
   # When W has only 1's this is procrustes rotation
   # Needs a Target matrix Target with hypothesized factor loadings.
   # e.g. Target = matrix(0,8,2)
   Btilde <- W * Target
   list(Gq= 2*(W*L-Btilde), 
        f = sum((W*L-Btilde)^2),
        Method="Partially specified target")
}

oblimax <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000){
   GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
            method="oblimax")
   }


vgQ.oblimax <- function(L){
  list(Gq= -(4*L^3/(sum(L^4))-4*L/(sum(L^2))),
       f= -(log(sum(L^4))-2*log(sum(L^2))),
       Method="oblimax")
}

entropy <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
   GPForth(L, Tmat=Tmat, method="entropy", normalize=normalize, eps=eps, maxit=maxit)
   }

vgQ.entropy <- function(L){
  list(Gq= -(L*log(L^2 + (L^2==0)) + L),
       f= -sum(L^2*log(L^2 + (L^2==0)))/2, 
       Method="Minimum entropy")
}

quartimax <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
   GPForth(L, Tmat=Tmat, method="quartimax", normalize=normalize, eps=eps, maxit=maxit)
   }

vgQ.quartimax <- function(L){
  list(Gq= -L^3,
       f= -sum(diag(crossprod(L^2)))/4, 
       Method="Quartimax")
}

Varimax <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
   GPForth(L, Tmat=Tmat, method="varimax", normalize=normalize, eps=eps, maxit=maxit)
   }

vgQ.varimax <- function(L){
  QL <- sweep(L^2,2,colMeans(L^2),"-")
  list(Gq= -L * QL,
       f= -sqrt(sum(diag(crossprod(QL))))^2/4, 
       Method="varimax")
}

simplimax <- function(L, Tmat=diag(ncol(L)), k=nrow(L), normalize=FALSE, eps=1e-5, maxit=1000) {
   GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
         method="simplimax", methodArgs=list(k=k))
   }

vgQ.simplimax <- function(L, k=nrow(L)){
  # k: Number of close to zero loadings
  Imat <- sign(L^2 <= sort(L^2)[k])
  list(Gq= 2*Imat*L,
       f= sum(Imat*L^2), 
       Method="Simplimax")
}

bentlerT <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
   GPForth(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit, 
	     method="bentler")
   }

bentlerQ <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
   GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
           method="bentler")
   }

vgQ.bentler <- function(L){
  L2 <- L^2
  M <- crossprod(L2)
  D <- diag(diag(M))
  list(Gq= -L * (L2 %*% (solve(M)-solve(D))),
       f= -(log(det(M))-log(det(D)))/4,
       Method="Bentler's criterion")
}

tandemI <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
   GPForth(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
            method="tandemI")
   }

#vgQ.tandemI <- function(L){  # Tandem Criterion, Comrey, 1967.
#  Method <- "Tandem I"
#  LL <- (L %*% t(L))
#  LL2 <- LL^2
#  f <- -sum(diag(crossprod(L^2, LL2 %*% L^2)))
#  Gq1 <- 4 * L *(LL2 %*% L^2)
#  Gq2 <- 4 * (LL * (L^2 %*% t(L^2))) %*% L
#  Gq <- -Gq1 - Gq2 
#  return(list(Gq=Gq,f=f,Method=Method))
#}

vgQ.tandemI <- function(L){  # Tandem Criterion, Comrey, 1967.
  LL <- (L %*% t(L))
  LL2 <- LL^2
  Gq1 <- 4 * L *(LL2 %*% L^2)
  Gq2 <- 4 * (LL * (L^2 %*% t(L^2))) %*% L
  Gq <- -Gq1 - Gq2 
  list(Gq=Gq,
       f= -sum(diag(crossprod(L^2, LL2 %*% L^2))), 
       Method="Tandem I")
  }

tandemII <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
   GPForth(L, Tmat=Tmat, method="tandemII", normalize=normalize, eps=eps, maxit=maxit)
   }


vgQ.tandemII <- function(L){  # Tandem Criterion, Comrey, 1967.
  LL <- (L %*% t(L))
  LL2 <- LL^2
  f <- sum(diag(crossprod(L^2, (1-LL2) %*% L^2)))
  Gq1 <- 4 * L *((1-LL2) %*% L^2)
  Gq2 <- 4 * (LL * (L^2 %*% t(L^2))) %*% L
  Gq <- Gq1 - Gq2 
  list(Gq=Gq,
       f=f, 
       Method="Tandem II")
  }

geominT <- function(L, Tmat=diag(ncol(L)), delta=.01, normalize=FALSE, eps=1e-5, maxit=1000){
   GPForth(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
           method="geomin", methodArgs=list(delta=delta))
   }

geominQ <- function(L, Tmat=diag(ncol(L)), delta=.01, normalize=FALSE, eps=1e-5, maxit=1000){
   GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
            method="geomin", methodArgs=list(delta=delta))
   }

vgQ.geomin <- function(L, delta=.01){
  k <- ncol(L)
  p <- nrow(L)
  L2 <- L^2 + delta
  pro <- exp(rowSums(log(L2))/k) 
  list(Gq=(2/k)*(L/L2)*matrix(rep(pro,k),p),
       f= sum(pro), 
       Method="Geomin")
  }

cfT <- function(L, Tmat=diag(ncol(L)), kappa=0, normalize=FALSE, eps=1e-5, maxit=1000) {
   GPForth(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
             method="cf", methodArgs=list(kappa=kappa))
   }

cfQ <- function(L, Tmat=diag(ncol(L)), kappa=0, normalize=FALSE, eps=1e-5, maxit=1000) {
   GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
             method="cf", methodArgs=list(kappa=kappa))
   }

vgQ.cf <- function(L, kappa=0){
  k <- ncol(L)
  p <- nrow(L)
  # kappa <- 0 # quartimax 
  # kappa <- 1/p # varimax
  # kappa <- m/(2*p) # equamax
  # kappa <- (m-1)/(p+m-2) # parsimax
  # kappa <- 1 # factor parsimony
  N <- matrix(1,k,k)-diag(k)
  M <- matrix(1,p,p)-diag(p)
  L2 <- L^2
  f1 <- (1-kappa)*sum(diag(crossprod(L2,L2 %*% N)))/4
  f2 <- kappa*sum(diag(crossprod(L2,M %*% L2)))/4
  list(Gq= (1-kappa) * L * (L2 %*% N) + kappa * L * (M %*% L2),
       f= f1 + f2,
       Method=paste("Crawford-Ferguson:k=",kappa,sep=""))
}

infomaxT <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
   GPForth(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
            method="infomax")
   }

infomaxQ <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
   GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
            method="infomax")
   }

vgQ.infomax <- function(L){
  Method <- "Infomax"
  k <- ncol(L)
  p <- nrow(L)
  S <- L^2
  s <- sum(S)
  s1 <- rowSums(S)
  s2 <- colSums(S)
  E <- S/s
  e1 <- s1/s
  e2 <- s2/s
  Q0 <- sum(-E * log(E))
  Q1 <- sum(-e1 * log(e1))
  Q2 <- sum(-e2 * log(e2))
  f <- log(k) + Q0 - Q1 - Q2
  H <- -(log(E) + 1)
  alpha <- sum(S * H)/s^2
  G0 <- H/s - alpha * matrix(1, p, k)
  h1 <- -(log(e1) + 1)
  alpha1 <- s1 %*% h1/s^2
  G1 <- matrix(rep(h1,k), p)/s - as.vector(alpha1) * matrix(1, p, k)
  h2 <- -(log(e2) + 1)
  alpha2 <- h2 %*% s2/s^2
  G2 <- matrix(rep(h2,p), ncol=k, byrow=T)/s - as.vector(alpha2) * matrix(1, p, k)
  Gq <- 2 * L * (G0 - G1 - G2)
  list(Gq=Gq,f=f,Method=Method)
}

mccammon <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
   GPForth(L, Tmat=Tmat, method="mccammon", normalize=normalize, eps=eps, maxit=maxit)
   }

vgQ.mccammon <- function(L){
  Method <- "McCammon entropy"
  k <- ncol(L)
  p <- nrow(L)
  S <- L^2
  M <- matrix(1,p,p)
  s2 <- colSums(S)
  P <- S / matrix(rep(s2,p),ncol=k,byrow=T)
  Q1 <- -sum(P * log(P))
  H <- -(log(P) + 1)
  R <- M %*% S
  G1 <- H/R - M %*% (S*H/R^2)
  s <- sum(S)
  p2 <- s2/s
  Q2 <- -sum(p2 * log(p2))
  h <- -(log(p2) + 1)
  alpha <- h %*% p2
  G2 <- rep(1,p) %*% t(h)/s - as.vector(alpha)*matrix(1,p,k)
  Gq <- 2*L*(G1/Q1 - G2/Q2)
  Q <- log(Q1) - log(Q2)
  list(Gq=Gq, f=Q, Method=Method)
}


bifactorT <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, 
     eps=1e-5, maxit=1000){
  #adapted from Jennrich and Bentler 2011. code provided by William Revelle	
    GPForth(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
            method="bifactor")
    }

bifactorQ <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, 
     eps=1e-5, maxit=1000){
  #the oblique case
  #adapted from Jennrich and Bentler 2011. code provided by William Revelle	
    GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
            method="bifactor")
    }


vgQ.bifactor <- function(L) {
  # code provided by William Revelle	
  D <- function(L) {
    L2 <- L^2
    L2N <- L2 %*% ! diag(NCOL(L))
    list(f=sum(L2 * L2N),
         Gq=4 * L * L2N)
  }
  lvg <- D(L[,-1, drop=FALSE])
  G <- lvg$Gq
  G <-cbind(G[,1],G)
  G[,1] <- 0
  list(f=lvg$f,
       Gq=G)
}


# promax is already defined in the stats (previously mva) package
# 
#GPromax <- function(A,pow=3){
# method <- "Promax"
# # Initial rotation: Standardized Varimax
# require(statsa)
# xx <- promax(A,pow)
# Lh <- xx$loadings
# Th <- xx$rotmat
# orthogonal <- F
# Table <- NULL
#return(list(loadings=Lh,Th=Th,Table=NULL,method,orthogonal=orthogonal))
#}

 Sys.getenv("R_LIBS")
 library()
 require("GPArotation")
 search()
 Sys.info()

#require("stats")  

fuzz <- 1e-6 
all.ok <- TRUE  


# test MASS 4th ed. p 322-324

  data(ability.cov)
  ability.cov
  ability.FA <- factanal(factors = 1, covmat=ability.cov)

  (ability.FA <- update(ability.FA, factors = 2))

#  ability.FA2 <- factanal(factors = 2, covmat = ability.cov)
# max(abs(ability.FA2$loadings - ability.FA$loadings))

#  summary(ability.FA) MASS ed.4 p 323 seems to be print not summary in R 2.0.1
  ability.FA

# this is default varimax rotation. There are 3rd+ digit differences with MASS
  tst <- t(matrix(c(
      0.499437829039896530, 0.54344904693111962,
      0.156070079431279873, 0.62153798991197484,
      0.205786989958578748, 0.85992588538426895,
      0.108530754440558652, 0.46776101732283504,
      0.956242470279811574, 0.18209631992182243,
      0.784768183877880943, 0.22482213687364205
      ), 2, 6))
 

 if( fuzz < max(abs(loadings(ability.FA) - tst))) {
    cat("Calculated value is not the same as test value in test 1. Value:\n")
    #print(loadings(ability.FA), digits=18) this truncates
    print(unclass(ability.FA$loadings), digits=18)
    cat("difference:\n")
    print(unclass(ability.FA$loadings) - tst, digits=18)
    all.ok <- FALSE  
    } 


  # differences with MASS here are a bit more than might be expected,
  # but there is already a difference before rotation.
  (oblirot <- oblimin(loadings(ability.FA)))

  obli2 <- factanal(factors = 2, covmat = ability.cov, rotation="oblimin")

  max(abs(loadings(oblirot) - loadings(obli2)))


# factanal(factors = 2, covmat = ability.cov, scores = Bartlett, rotation="oblimin")


 tst <- t(matrix(c(
       0.3863637969729337152,  0.4745113977203344047,
      -0.0110032278171669998,  0.6458708261423832253,
      -0.0262888675561207576,  0.8961123879025085781,
      -0.0180180060207963122,  0.4882918937716873575,
       0.9900948712271664398, -0.0370729040114848238,
       0.7905663749272058283,  0.0526099352008769991
    ), 2, 6))
 
 if( fuzz < max(abs(loadings(oblirot) - tst ))) {
    cat("Calculated value is not the same as test value in test 2. Value:\n")
    print(loadings(oblirot), digits=18)
    cat("difference:\n")
    print(loadings(oblirot) - tst, digits=18)
    all.ok <- FALSE  
    } 

cat("tests completed.\n")

if (! all.ok) stop("some tests FAILED")

# This tests fix for an error caused by an exact initial setting.
#  (from William Revelle)

require("GPArotation")

f3 <- structure(c(0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0,0),
      .Dim = c(6L, 3L), .Dimnames = list(NULL, c("PC1", "PC2", "PC3")))

f3

#      PC1 PC2 PC3
#[1,]   0   0   1
#[2,]   0   1   0
#[3,]   1   0   0
#[4,]   0   0   1
#[5,]   0   1   0
#[6,]   1   0   0

# These previously gave object 'VgQt' not found
GPForth(f3)
Varimax(f3)

# tests using normalization
# All tests below use Kaiser normalization
# A few other tests also use normalization when comparing varimax and Varimax

# Following examples are from SPSS
# See https://psych.unl.edu/psycrs/statpage/pc_rot.pdf

Sys.getenv("R_LIBS")
library()
require("GPArotation")
search()
Sys.info()

require("stats")  
require("GPArotation")  

fuzz <- 1e-3  #less strict; differences in 4rd decimal compared to SPSS
all.ok <- TRUE  

# unrotated matrix
L <- matrix(c(.758, .413, 1.164E-03, .693, .489, -.199, .362, .656, -.204, 
.826, 6.589E-02, .235, .540, -.510, .441, .654, -.335, .507,
-.349, .539, .669, -.580, .450, .551), byrow=T, ncol=3)

# quartimax, Kaiser normalization
# uses the print command to get the right order of factors
v <- print(quartimax(L, normalize = TRUE, eps = 1e-6))$loadings 

tst <- matrix(c(.814, .285, -4.99E-02, .856, 8.321E-02, -.135,
.746, -.203, 7.244E-02, .576, .634, -8.73E-02,
-6.10E-02, .850, -.142, .129, .882, -3.86E-02,
2.063E-02, -4.15E-02, .927, -.181, -.220, .873), byrow=T, ncol=3)

  if( fuzz < max(abs(v - tst))) {
    cat("Calculated value is not the same as test value in test rotations 1. Value:\n")
    print(v, digits=18)
    cat("difference:\n")
    print(v - tst, digits=18)
    all.ok <- FALSE  
    } 

# oblimin, Kaiser normalization
# Pattern Matrix
vw <- print(oblimin(L, normalize = TRUE, eps = 1e-7))
v <- vw$loadings

tst <- matrix(c(.241, .787, -1.36E-02, 1.783E-02, .848, -.119,
-.240, .779, 6.824E-02, .608, .507, -2.52E-02,
.858, -.163, -7.26E-02, .896, 3.050E-02, 3.954E-02,
9.405E-02, 7.397E-02, .949, -8.61E-02, -.113, .875), byrow=T, ncol=3)
tst <- tst %*% matrix(c(0,1,0,1,0,0,0,0,1), 3) # Needed to line up factors correctly

fuzz <- 3e-3  #less strict; differences in 4th decimal compared to SPSS; 0.003 or smaller diff

  if( fuzz < max(abs(v - tst))) {
    cat("Calculated value is not the same as test value in test rotations 1. Value:\n")
    print(v, digits=18)
    cat("difference:\n")
    print(v - tst, digits=18)
    all.ok <- FALSE  
    } 

# oblimin, Kaiser normalization
# Structure Matrix
v <- vw$loadings %*% vw$Phi

tst <- matrix(c(.379, .829, -.146, .191, .862, -.203,
-.123, .731, .051, .701, .613, -.218, .847, -.010, -.261,
.891, .180, -.176, -.118, .000, .919, -.313, -.211, .906), byrow=T, ncol=3)
tst <- tst %*% matrix(c(0,1,0,1,0,0,0,0,1), 3) # Needed to line up factors correctly

fuzz <- 4e-3  #less strict; differences in 4th decimal compared to SPSS; 0.004 or smaller diff

  if( fuzz < max(abs(v - tst))) {
    cat("Calculated value is not the same as test value in test rotations 1. Value:\n")
    print(v, digits=18)
    cat("difference:\n")
    print(v - tst, digits=18)
    all.ok <- FALSE  
    } 

#################################################################
#
# Confirmation that a row of zeroes will not break the normalization function
# Normalizing with a column of zeroes was not affected
# based on example from Kim-Laura Speck (25 October 2023)
# Only affects Normalize=TRUE settings 

fuzz <- 1e-6

D <- matrix(c(0,0,0, 1,2,3, 2,3,4, 5,2,5, 1,2,1, 3,4,5),ncol=3,byrow=T)
set.seed(1000)  #set seed becasuse some variance is observed in converged values
v <- geominQ(D, normalize = TRUE, maxit = 10000)$loadings

tst <- matrix(c(
  0.00000000,  0.00000000, 0.00000000,
 -0.36979732, -0.13603325, 3.99622380,
  0.03102554,  0.76678245, 4.68896063,
  3.28926158,  0.01317821, 5.35447764,
 -0.02755956,  2.40311582, 0.06247234,
  0.43184841,  1.66959816, 5.38169746), ncol = 3, byrow = TRUE)

  if( fuzz < max(abs(v - tst))) {
    cat("Calculated value is not the same as test value in test rotations 1. Value:\n")
    print(v, digits=18)
    cat("difference:\n")
    print(v - tst, digits=18)
    all.ok <- FALSE  
    } 
    

# tests/legacyVsBB.R
# Regression tests: BB and Cayley produce same solutions as legacy
# within numerical tolerance. Different optimization paths, same minimum.

library(GPArotation)

all.ok <- TRUE
fuzz   <- 1e-3  # tolerance — different paths, same minimum

data(Harman,    package = "GPArotation")
data(Thurstone, package = "GPArotation")
data(CCAI,      package = "GPArotation")

fa.ccai <- factanal(covmat = CCAI_R, factors = 3,
                    rotation = "none", n.obs = 461)

.strip <- function(x) {
    x <- unclass(x)
    attributes(x) <- list(dim = dim(x))
    x
}

.check <- function(r.bb, r.leg, label, tol = fuzz) {
    # Compare absolute loadings — sign and column order may differ
    diff <- max(abs(abs(.strip(r.bb$loadings)) - 
                        abs(.strip(r.leg$loadings))))
    if (diff > tol) {
        cat("FAILED:", label, "\n")
        cat("max abs difference:", diff, "\n")
        all.ok <<- FALSE
    }
}

.check_phi <- function(r.bb, r.leg, label, tol = fuzz) {
    if (is.null(r.bb$Phi) || is.null(r.leg$Phi)) return()
    diff <- max(abs(r.bb$Phi - r.leg$Phi))
    if (diff > tol) {
        cat("FAILED:", label, "\n")
        cat("max Phi difference:", diff, "\n")
        all.ok <<- FALSE
    }
}

cat("=== BB vs Legacy ===\n")

# --- Orthogonal ---
# Test 1: varimax Harman8
r.bb  <- GPForth(Harman8, method = "varimax", algorithm = "bb",     fwindow = 10)
r.leg <- GPForth(Harman8, method = "varimax", algorithm = "legacy", fwindow = 1)
.check(r.bb, r.leg, "Test 1 - varimax Harman8")
if (!r.bb$convergence)  { cat("FAILED: Test 1 - BB did not converge\n");  all.ok <- FALSE }
if (!r.leg$convergence) { cat("FAILED: Test 1 - legacy did not converge\n"); all.ok <- FALSE }

# Test 2: quartimax box26
r.bb  <- GPForth(box26, method = "quartimax", algorithm = "bb",     fwindow = 10)
r.leg <- GPForth(box26, method = "quartimax", algorithm = "legacy", fwindow = 1)
.check(r.bb, r.leg, "Test 2 - quartimax box26")
if (!r.bb$convergence)  { cat("FAILED: Test 2 - BB did not converge\n");  all.ok <- FALSE }
if (!r.leg$convergence) { cat("FAILED: Test 2 - legacy did not converge\n"); all.ok <- FALSE }

# Test 3: geominT CCAI
r.bb  <- GPForth(fa.ccai, method = "geomin", algorithm = "bb",     fwindow = 10)
r.leg <- GPForth(fa.ccai, method = "geomin", algorithm = "legacy", fwindow = 1)
.check(r.bb, r.leg, "Test 3 - geominT CCAI")
if (!r.bb$convergence)  { cat("FAILED: Test 3 - BB did not converge\n");  all.ok <- FALSE }

# Test 4: entropy box26
r.bb  <- GPForth(box26, method = "entropy", algorithm = "bb",     fwindow = 10)
r.leg <- GPForth(box26, method = "entropy", algorithm = "legacy", fwindow = 1)
.check(r.bb, r.leg, "Test 4 - entropy box26")
if (!r.bb$convergence)  { cat("FAILED: Test 4 - BB did not converge\n");  all.ok <- FALSE }

# Test 5: tandemII box26
r.bb  <- GPForth(box26, method = "tandemII", algorithm = "bb",     fwindow = 10)
r.leg <- GPForth(box26, method = "tandemII", algorithm = "legacy", fwindow = 1)
.check(r.bb, r.leg, "Test 5 - tandemII box26")
if (!r.bb$convergence)  { cat("FAILED: Test 5 - BB did not converge\n");  all.ok <- FALSE }

cat("\n=== Cayley vs Legacy ===\n")

# Test 6: varimax Harman8
r.cay <- GPForth(Harman8, method = "varimax", algorithm = "cayley", fwindow = 10)
r.leg <- GPForth(Harman8, method = "varimax", algorithm = "legacy", fwindow = 1)
.check(r.cay, r.leg, "Test 6 - cayley varimax Harman8")
if (!r.cay$convergence) { cat("FAILED: Test 6 - Cayley did not converge\n"); all.ok <- FALSE }

# Test 7: quartimax box26
r.cay <- GPForth(box26, method = "quartimax", algorithm = "cayley", fwindow = 10)
r.leg <- GPForth(box26, method = "quartimax", algorithm = "legacy", fwindow = 1)
.check(r.cay, r.leg, "Test 7 - cayley quartimax box26")
if (!r.cay$convergence) { cat("FAILED: Test 7 - Cayley did not converge\n"); all.ok <- FALSE }

# Test 8: bentlerT CCAI
r.cay <- GPForth(fa.ccai, method = "bentler", algorithm = "cayley", fwindow = 10)
r.leg <- GPForth(fa.ccai, method = "bentler", algorithm = "legacy", fwindow = 1)
.check(r.cay, r.leg, "Test 8 - cayley bentlerT CCAI")
if (!r.cay$convergence) { cat("FAILED: Test 8 - Cayley did not converge\n"); all.ok <- FALSE }

cat("\n=== BB vs Legacy Oblique ===\n")

# Test 9: quartimin Harman8
r.bb  <- GPFoblq(Harman8, method = "quartimin", algorithm = "bb",     fwindow = 10)
r.leg <- GPFoblq(Harman8, method = "quartimin", algorithm = "legacy", fwindow = 1)
.check(r.bb, r.leg, "Test 9 - quartimin Harman8")
.check_phi(r.bb, r.leg, "Test 9 - quartimin Harman8 Phi")
if (!r.bb$convergence)  { cat("FAILED: Test 9 - BB did not converge\n");  all.ok <- FALSE }
if (!r.leg$convergence) { cat("FAILED: Test 9 - legacy did not converge\n"); all.ok <- FALSE }

# Test 10: oblimin box26
r.bb  <- GPFoblq(box26, method = "oblimin", algorithm = "bb",     fwindow = 10)
r.leg <- GPFoblq(box26, method = "oblimin", algorithm = "legacy", fwindow = 1)
.check(r.bb, r.leg, "Test 10 - oblimin box26")
.check_phi(r.bb, r.leg, "Test 10 - oblimin box26 Phi")
if (!r.bb$convergence)  { cat("FAILED: Test 10 - BB did not converge\n");  all.ok <- FALSE }

# Test 11: geomin CCAI
r.bb  <- GPFoblq(fa.ccai, method = "geomin", algorithm = "bb",     fwindow = 10)
r.leg <- GPFoblq(fa.ccai, method = "geomin", algorithm = "legacy", fwindow = 1)
.check(r.bb, r.leg, "Test 11 - geominQ CCAI")
.check_phi(r.bb, r.leg, "Test 11 - geominQ CCAI Phi")
if (!r.bb$convergence)  { cat("FAILED: Test 11 - BB did not converge\n");  all.ok <- FALSE }

# Test 12: simplimax CCAI
r.bb  <- GPFoblq(fa.ccai, method = "simplimax", algorithm = "bb",     fwindow = 10)
r.leg <- GPFoblq(fa.ccai, method = "simplimax", maxit = 5000, algorithm = "legacy", fwindow = 1)
.check(r.bb, r.leg, "Test 12 - simplimax CCAI")
.check_phi(r.bb, r.leg, "Test 12 - simplimax CCAI Phi")
if (!r.bb$convergence)  { cat("FAILED: Test 12 - BB did not converge\n");  all.ok <- FALSE }

# Test 13: bentlerQ CCAI
r.bb  <- GPFoblq(fa.ccai, method = "bentler", algorithm = "bb",     fwindow = 10)
r.leg <- GPFoblq(fa.ccai, method = "bentler", algorithm = "legacy", fwindow = 1)
.check(r.bb, r.leg, "Test 13 - bentlerQ CCAI")
.check_phi(r.bb, r.leg, "Test 13 - bentlerQ CCAI Phi")
if (!r.bb$convergence)  { cat("FAILED: Test 13 - BB did not converge\n");  all.ok <- FALSE }

cat("\n=== Via Named Wrappers ===\n")

# Test 14: oblimin wrapper default (bb, fwindow=10)
r.bb  <- oblimin(fa.ccai)
r.leg <- oblimin(fa.ccai, algorithm = "legacy", fwindow = 1)
.check(r.bb, r.leg, "Test 14 - oblimin wrapper default")
.check_phi(r.bb, r.leg, "Test 14 - oblimin wrapper default Phi")

# Test 15: Varimax wrapper default (bb, fwindow=10)
r.bb  <- Varimax(fa.ccai)
r.leg <- Varimax(fa.ccai, algorithm = "legacy", fwindow = 1)
.check(r.bb, r.leg, "Test 15 - Varimax wrapper default")

# Test 16: geominQ wrapper default (bb, fwindow=10)
r.bb  <- geominQ(fa.ccai)
r.leg <- geominQ(fa.ccai, algorithm = "legacy", fwindow = 1)
.check(r.bb, r.leg, "Test 16 - geominQ wrapper default")
.check_phi(r.bb, r.leg, "Test 16 - geominQ wrapper default Phi")

cat("\ntests completed.\n")
if (!all.ok) stop("some legacyVsBB tests FAILED")

# Iterative re-weighted least squares for Lp rotation.
# Functions provided by Xinyi Liu, with random start integration by Coen Bernaards (February 2025).
#
# Each rotation uses an inner loop (GPA) and an outer loop (IRWLS).
# Random starts add a third outer loop.
#
# Note: lpQ and lpT exist as separate random-start wrappers rather than using
# .GPA_RS_engine because the Lp rotation calls GPFoblq.lp and GPForth.lp
# rather than GPFoblq and GPForth. Integrating with .GPA_RS_engine would
# require changes to the core GP algorithms, which was intentionally avoided.
#
# 4 functions:
#   lpQ        - oblique Lp rotation wrapper (with optional random starts)
#   GPFoblq.lp - main oblique Lp GP function
#   lpT        - orthogonal Lp rotation wrapper (with optional random starts)
#   GPForth.lp - main orthogonal Lp GP function

# --- OBLIQUE Lp ROTATION ---

lpQ <- function(A, Tmat = diag(ncol(A)), p = 1, normalize = FALSE,
                eps = 1e-05, maxit = 1000, randomStarts = 0, gpaiter = 5) {
  if (normalize)
    message("Normalization is not recommended for Lp rotation. Use may have unexpected results.")

  if (randomStarts > 0)
    Tmat <- Random.Start(ncol(A))

  r <- GPFoblq.lp(A, Tmat = Tmat, p, normalize = normalize, eps = eps,
                  maxit = maxit, gpaiter = gpaiter)

  if (randomStarts > 1) {
    Qvalues   <- sum(abs(r$loadings)^p) / nrow(A)
    Qmin      <- Qvalues
    Qconverged <- r$convergence

    for (inum in 2:randomStarts) {
      gpout <- GPFoblq.lp(A, Tmat = Random.Start(ncol(A)), p,
                          normalize = normalize, eps = eps,
                          maxit = maxit, gpaiter = gpaiter)
      Qval       <- sum(abs(gpout$loadings)^p) / nrow(A)
      Qvalues    <- c(Qvalues, Qval)
      Qconverged <- c(Qconverged, gpout$convergence)
      if (Qval < Qmin) {
        r    <- gpout
        Qmin <- Qval
      }
    }

    Qmin      <- eps * round(Qmin    * 1 / eps, 0)
    Qvalues   <- eps * round(Qvalues * 1 / eps, 0)
    randStartChar <- c(randomStarts      = randomStarts,
                       Converged         = sum(Qconverged),
                       atMinimum         = sum(Qvalues == Qmin),
                       localMins         = length(unique(Qvalues)))

    r <- list(loadings    = r$loadings,
              Phi         = r$Phi,
              Th          = r$Th,
              Table       = r$Table,
              method      = r$method,
              orthogonal  = FALSE,
              convergence = r$convergence,
              Gq          = r$Gq,
              randStartChar = randStartChar,
              Qvalues     = Qvalues)
    class(r) <- "GPArotation"
  }

  dimnames(r$Phi) <- list(colnames(r$loadings), colnames(r$loadings))
  r
}

GPFoblq.lp <- function(A, Tmat = diag(ncol(A)), p = 1, normalize = FALSE,
                        eps = 1e-05, maxit = 10000, gpaiter = 5) {
  # Oblique Lp gradient projection via iterative re-weighted least squares.
  # Inner loop: GPA with weighted least squares criterion (vgQ.lp.wls)
  # Outer loop: update weights until loadings converge

  j_sum      <- 0
  start_time <- proc.time()
  convergence <- FALSE

  # Corrected initial weights based on current rotated loadings
  W <- ((A %*% t(solve(Tmat)))^2 + eps)^(p / 2 - 1)

  for (it in 1:maxit) {
    r     <- suppressWarnings(
               GPFoblq(A, Tmat = Tmat, normalize = normalize, eps = eps,
                       maxit = gpaiter, method = "lp.wls", methodArgs = list(W = W)))
    T_new <- r$Th
    L     <- r$loadings
    k     <- nrow(r$Table)
    j     <- r$Table[k, 1]
    j_sum <- j_sum + j

    W <- (L^2 + eps)^(p / 2 - 1)

    if (max(abs(L - A %*% solve(t(Tmat)))) < eps) {
      Tmat        <- T_new
      convergence <- TRUE
      break
    }
    Tmat <- T_new
  }

  Phi <- t(Tmat) %*% Tmat
  dimnames(Phi) <- list(colnames(A), colnames(A))

  Table <- data.frame(
    iter = it,
    f    = sum(abs(L)^p) / nrow(L),
    time = proc.time()[3] - start_time[3]
  )

  r <- list(
    loadings    = L,
    Phi         = Phi,
    Th          = Tmat,
    Table       = Table,
    method      = paste0("Lp rotation, p=", p),
    orthogonal  = FALSE,
    convergence = convergence
  )
  class(r) <- "GPArotation"
  r
}

# --- ORTHOGONAL Lp ROTATION ---

lpT <- function(A, Tmat = diag(ncol(A)), p = 1, normalize = FALSE,
                eps = 1e-05, maxit = 1000, randomStarts = 0, gpaiter = 5) {
  if (normalize)
    message("Normalization is not recommended for Lp rotation. Use may have unexpected results.")

  if (randomStarts > 0)
    Tmat <- Random.Start(ncol(A))

  r <- GPForth.lp(A, Tmat = Tmat, p, normalize = normalize, eps = eps,
                  maxit = maxit, gpaiter = gpaiter)

  if (randomStarts > 1) {
    Qvalues    <- sum(abs(r$loadings)^p) / nrow(A)
    Qmin       <- Qvalues
    Qconverged <- r$convergence

    for (inum in 2:randomStarts) {
      gpout <- GPForth.lp(A, Tmat = Random.Start(ncol(A)), p,
                          normalize = normalize, eps = eps,
                          maxit = maxit, gpaiter = gpaiter)
      Qval       <- sum(abs(gpout$loadings)^p) / nrow(A)
      Qvalues    <- c(Qvalues, Qval)
      Qconverged <- c(Qconverged, gpout$convergence)
      if (Qval < Qmin) {
        r    <- gpout
        Qmin <- Qval
      }
    }

    Qmin    <- eps * round(Qmin    * 1 / eps, 0)
    Qvalues <- eps * round(Qvalues * 1 / eps, 0)
    randStartChar <- c(randomStarts      = randomStarts,
                       Converged         = sum(Qconverged),
                       atMinimum         = sum(Qvalues == Qmin),
                       localMins         = length(unique(Qvalues)))

    r <- list(loadings    = r$loadings,
              Phi         = r$Phi,
              Th          = r$Th,
              Table       = r$Table,
              method      = r$method,
              orthogonal  = TRUE,
              convergence = r$convergence,
              Gq          = r$Gq,
              randStartChar = randStartChar,
              Qvalues     = Qvalues)
    class(r) <- "GPArotation"
  }

  dimnames(r$Phi) <- list(colnames(r$loadings), colnames(r$loadings))
  r
}

GPForth.lp <- function(A, Tmat = diag(ncol(A)), p = 1, normalize = FALSE,
                        eps = 1e-05, maxit = 10000, gpaiter = 5) {
  # Orthogonal Lp gradient projection via iterative re-weighted least squares.
  # Inner loop: GPA with weighted least squares criterion (vgQ.lp.wls)
  # Outer loop: update weights until loadings converge

  j_sum      <- 0
  start_time <- proc.time()
  convergence <- FALSE

  # Corrected initial weights based on current rotated loadings
  W <- ((A %*% t(solve(Tmat)))^2 + eps)^(p / 2 - 1)

  for (it in 1:maxit) {
    r     <- suppressWarnings(
               GPForth(A, Tmat = Tmat, normalize = normalize, eps = eps,
                       maxit = gpaiter, method = "lp.wls", methodArgs = list(W = W)))
    T_new <- r$Th
    L     <- r$loadings
    k     <- nrow(r$Table)
    j     <- r$Table[k, 1]
    j_sum <- j_sum + j

    W <- (L^2 + eps)^(p / 2 - 1)

    if (max(abs(L - A %*% solve(t(Tmat)))) < eps) {
      Tmat        <- T_new
      convergence <- TRUE
      break
    }
    Tmat <- T_new
  }

  Phi <- t(Tmat) %*% Tmat
  dimnames(Phi) <- list(colnames(A), colnames(A))

  Table <- data.frame(
    iter = it,
    f    = sum(abs(L)^p) / nrow(L),
    time = proc.time()[3] - start_time[3]
  )

  r <- list(
    loadings    = L,
    Phi         = Phi,
    Th          = Tmat,
    Table       = Table,
    method      = paste0("Lp rotation, p=", p),
    orthogonal  = TRUE,
    convergence = convergence
  )
  class(r) <- "GPArotation"
  r
}

# vgQtestOldVsNew.R
# Regression tests: new vgQ functions vs original implementations.
# f and Gq are compared within tolerance to allow for floating point
# differences due to different order of arithmetic operations.
# Method strings are not compared --- changes are intentional improvements.
# A real bug (mccammon Gq) was detected by these tests in 2026.4-1.

require("GPArotation")
source("vgQOLD.R")

all.ok <- TRUE
tol    <- 1e-10

set.seed(123)
L8x2  <- matrix(rnorm(16),  nrow = 8,  ncol = 2)
L10x3 <- matrix(rnorm(30),  nrow = 10, ncol = 3)
L15x4 <- matrix(rnorm(60),  nrow = 15, ncol = 4)
L20x5 <- matrix(rnorm(100), nrow = 20, ncol = 5)

matrices <- list(L8x2, L10x3, L15x4, L20x5)

# Helper: check structural contract
check_contract <- function(res, name) {
  ok <- TRUE
  if (!all(c("f", "Gq", "Method") %in% names(res))) {
    cat("FAILED:", name, "- missing list elements\n")
    ok <- FALSE
  }
  if (!is.numeric(res$f) || length(res$f) != 1) {
    cat("FAILED:", name, "- f is not a scalar numeric\n")
    ok <- FALSE
  }
  if (!is.character(res$Method) || length(res$Method) != 1) {
    cat("FAILED:", name, "- Method is not a scalar character\n")
    ok <- FALSE
  }
  ok
}

# Helper: check new vs old within tolerance
# Method strings are not compared --- changes are intentional
check_values <- function(new, old, name, tol = 1e-10) {
  ok <- TRUE
  if (max(abs(new$f - old$f)) > tol) {
    cat("FAILED:", name, "- f differs beyond tolerance\n")
    cat("  max diff:", max(abs(new$f - old$f)), "\n")
    ok <- FALSE
  }
  if (max(abs(new$Gq - old$Gq)) > tol) {
    cat("FAILED:", name, "- Gq differs beyond tolerance\n")
    cat("  max diff:", max(abs(new$Gq - old$Gq)), "\n")
    ok <- FALSE
  }
  ok
}

# -------------------------------------------------------------------
# oblimin --- gam = 0, 0.25, 0.5
# -------------------------------------------------------------------
for (L in matrices) {
  for (gam in c(0, 0.25, 0.5)) {
    nm  <- paste("vgQ.oblimin gam=", gam, dim(L)[1], "x", dim(L)[2])
    new <- GPArotation:::vgQ.oblimin(L, gam = gam)
    old <- vgQOLD.oblimin(L, gam = gam)
    all.ok <- all.ok & check_contract(new, nm)
    all.ok <- all.ok & check_values(new, old, nm)
  }
}

# -------------------------------------------------------------------
# quartimin
# -------------------------------------------------------------------
for (L in matrices) {
  nm  <- paste("vgQ.quartimin", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.quartimin(L)
  old <- vgQOLD.quartimin(L)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
}

# -------------------------------------------------------------------
# target --- with and without NA
# -------------------------------------------------------------------
set.seed(456)
for (L in matrices) {
  Target <- matrix(rnorm(length(L)), nrow = nrow(L), ncol = ncol(L))
  nm  <- paste("vgQ.target", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.target(L, Target = Target)
  old <- vgQOLD.target(L, Target = Target)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
}

# target with NA
Target_na <- matrix(c(rep(NA, 4), rep(0, 4)), nrow = 8, ncol = 2)
new <- GPArotation:::vgQ.target(L8x2, Target = Target_na)
old <- vgQOLD.target(L8x2, Target = Target_na)
all.ok <- all.ok & check_contract(new, "vgQ.target with NA")
all.ok <- all.ok & check_values(new, old, "vgQ.target with NA")

# -------------------------------------------------------------------
# pst
# -------------------------------------------------------------------
set.seed(789)
for (L in matrices) {
  W      <- matrix(sample(0:1, length(L), replace = TRUE),
                   nrow = nrow(L), ncol = ncol(L))
  Target <- matrix(0, nrow = nrow(L), ncol = ncol(L))
  nm  <- paste("vgQ.pst", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.pst(L, W = W, Target = Target)
  old <- vgQOLD.pst(L, W = W, Target = Target)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
}

# -------------------------------------------------------------------
# oblimax
# -------------------------------------------------------------------
for (L in matrices) {
  nm  <- paste("vgQ.oblimax", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.oblimax(L)
  old <- vgQOLD.oblimax(L)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
}

# -------------------------------------------------------------------
# entropy
# -------------------------------------------------------------------
for (L in matrices) {
  nm  <- paste("vgQ.entropy", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.entropy(L)
  old <- vgQOLD.entropy(L)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
}

# -------------------------------------------------------------------
# quartimax
# -------------------------------------------------------------------
for (L in matrices) {
  nm  <- paste("vgQ.quartimax", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.quartimax(L)
  old <- vgQOLD.quartimax(L)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
}

# -------------------------------------------------------------------
# varimax
# -------------------------------------------------------------------
for (L in matrices) {
  nm  <- paste("vgQ.varimax", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.varimax(L)
  old <- vgQOLD.varimax(L)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
}

# -------------------------------------------------------------------
# simplimax --- default k and custom k
# -------------------------------------------------------------------
for (L in matrices) {
  for (k in c(nrow(L), floor(nrow(L) / 2))) {
    nm  <- paste("vgQ.simplimax k=", k, dim(L)[1], "x", dim(L)[2])
    new <- GPArotation:::vgQ.simplimax(L, k = k)
    old <- vgQOLD.simplimax(L, k = k)
    all.ok <- all.ok & check_contract(new, nm)
    all.ok <- all.ok & check_values(new, old, nm)
  }
}

# -------------------------------------------------------------------
# bentler
# -------------------------------------------------------------------
for (L in matrices) {
  nm  <- paste("vgQ.bentler", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.bentler(L)
  old <- vgQOLD.bentler(L)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
}

# -------------------------------------------------------------------
# tandemI
# -------------------------------------------------------------------
for (L in matrices) {
  nm  <- paste("vgQ.tandemI", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.tandemI(L)
  old <- vgQOLD.tandemI(L)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
}

# -------------------------------------------------------------------
# tandemII
# -------------------------------------------------------------------
for (L in matrices) {
  nm  <- paste("vgQ.tandemII", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.tandemII(L)
  old <- vgQOLD.tandemII(L)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
}

# -------------------------------------------------------------------
# geomin --- default and custom delta
# -------------------------------------------------------------------
for (L in matrices) {
  for (delta in c(0.01, 0.001)) {
    nm  <- paste("vgQ.geomin delta=", delta, dim(L)[1], "x", dim(L)[2])
    new <- GPArotation:::vgQ.geomin(L, delta = delta)
    old <- vgQOLD.geomin(L, delta = delta)
    all.ok <- all.ok & check_contract(new, nm)
    all.ok <- all.ok & check_values(new, old, nm)
  }
}

# -------------------------------------------------------------------
# bigeomin
# -------------------------------------------------------------------
for (L in matrices) {
  nm  <- paste("vgQ.bigeomin", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.bigeomin(L)
  old <- vgQOLD.bigeomin(L)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
}

# -------------------------------------------------------------------
# cf --- kappa = 0, 0.3, 1
# -------------------------------------------------------------------
for (L in matrices) {
  for (kappa in c(0, 0.3, 1)) {
    nm  <- paste("vgQ.cf kappa=", kappa, dim(L)[1], "x", dim(L)[2])
    new <- GPArotation:::vgQ.cf(L, kappa = kappa)
    old <- vgQOLD.cf(L, kappa = kappa)
    all.ok <- all.ok & check_contract(new, nm)
    all.ok <- all.ok & check_values(new, old, nm)
  }
}

# -------------------------------------------------------------------
# infomax
# -------------------------------------------------------------------
for (L in matrices) {
  nm  <- paste("vgQ.infomax", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.infomax(L)
  old <- vgQOLD.infomax(L)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
}

# -------------------------------------------------------------------
# mccammon
# -------------------------------------------------------------------
for (L in matrices) {
  nm  <- paste("vgQ.mccammon", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.mccammon(L)
  old <- vgQOLD.mccammon(L)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
}

# -------------------------------------------------------------------
# bifactor --- first column of Gq must be zero
# -------------------------------------------------------------------
for (L in matrices) {
  nm  <- paste("vgQ.bifactor", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.bifactor(L)
  old <- vgQOLD.bifactor(L)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
  if (!all(new$Gq[, 1] == 0)) {
    cat("FAILED:", nm, "- first column of Gq not zero\n")
    all.ok <- FALSE
  }
}

# -------------------------------------------------------------------
# varimin
# -------------------------------------------------------------------
for (L in matrices) {
  nm  <- paste("vgQ.varimin", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.varimin(L)
  old <- vgQOLD.varimin(L)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
}

# -------------------------------------------------------------------
# lp.wls
# -------------------------------------------------------------------
set.seed(321)
for (L in matrices) {
  W   <- matrix(runif(length(L)), nrow = nrow(L), ncol = ncol(L))
  nm  <- paste("vgQ.lp.wls", dim(L)[1], "x", dim(L)[2])
  new <- GPArotation:::vgQ.lp.wls(L, W = W)
  old <- vgQOLD.lp.wls(L, W = W)
  all.ok <- all.ok & check_contract(new, nm)
  all.ok <- all.ok & check_values(new, old, nm)
}

# -------------------------------------------------------------------
# Gq dimensions match L for all criteria
# -------------------------------------------------------------------
criteria <- c("oblimin", "quartimin", "oblimax", "entropy", "quartimax",
              "varimax", "simplimax", "bentler", "tandemI", "tandemII",
              "geomin", "bigeomin", "infomax", "mccammon", "bifactor",
              "varimin")

for (crit in criteria) {
  fn <- get(paste("vgQ", crit, sep = "."),
            envir = asNamespace("GPArotation"))
  for (L in matrices) {
    res <- fn(L)
    if (!identical(dim(res$Gq), dim(L))) {
      cat("FAILED: vgQ.", crit, "Gq dimensions do not match L:",
          dim(L), "vs", dim(res$Gq), "\n")
      all.ok <- FALSE
    }
  }
}

cat("vgQ regression tests completed.\n")
if (!all.ok) stop("some tests FAILED")

plot_trajectory <- function(res, factors = c(1, 2), items = NULL, main = NULL) {

  trajectory_data <- .reconstruct_trajectory(res)

  A_unrotated <- attr(res, "A_unrotated")
  p <- nrow(A_unrotated)
  if (is.null(items)) items <- 1:p

  # Build main title
  if (is.null(main)) {
    main_title <- paste("Factor Migration trajectory:", res$method)
  } else {
    main_title <- main  # "" suppresses title, any string overrides
  }

  plot(NULL, xlim = c(-1.1, 1.1), ylim = c(-1.1, 1.1), 
       xlab = paste("Factor", factors[1]),
       ylab = paste("Factor", factors[2]),
       main = main_title)

  # Draw manifold (Unit Circle)
  theta <- seq(0, 2*pi, length.out = 100)
  lines(cos(theta), sin(theta), col = "gray90")
  abline(h = 0, v = 0, col = "gray94", lty = 2)

  cols <- rainbow(length(items))

  for (i in seq_along(items)) {
    var_idx <- items[i]

    trajectory_coords <- t(sapply(trajectory_data, function(Ti) {
      if (res$orthogonal) {
        L_step <- A_unrotated %*% Ti
      } else {
        L_step <- A_unrotated %*% t(solve(Ti))
      }
      return(L_step[var_idx, factors])
    }))

##    points(trajectory_coords[1, 1], trajectory_coords[1, 2],
##           col = cols[i], pch = 4, cex = 0.5)
##    lines(trajectory_coords[, 1], trajectory_coords[, 2],
##          col = cols[i], lwd = 1)
##    points(trajectory_coords[nrow(trajectory_coords), 1],
##           trajectory_coords[nrow(trajectory_coords), 2],
##           col = cols[i], pch = 16, cex = 0.8)
##    text(trajectory_coords[nrow(trajectory_coords), 1],
##         trajectory_coords[nrow(trajectory_coords), 2],
##         labels = var_idx, pos = 3, cex = 0.7, col = cols[i])

    points(trajectory_coords[1, 1], trajectory_coords[1, 2],
           col = "black", pch = 4, cex = 0.5)
    lines(trajectory_coords[, 1], trajectory_coords[, 2],
          col = cols[i], lwd = 1)
    points(trajectory_coords[nrow(trajectory_coords), 1],
           trajectory_coords[nrow(trajectory_coords), 2],
           col = "black", pch = 16, cex = 0.8)
    text(trajectory_coords[nrow(trajectory_coords), 1],
         trajectory_coords[nrow(trajectory_coords), 2],
         labels = var_idx, pos = 3, cex = 0.7, col = "black")
  }

  stats_text <- paste0(
    "Algorithm: ", res$algorithm, "\n",
    "fwindow: ", res$fwindow, "\n",
    "Iterations: ", nrow(res$Table) - 1, "\n",
    "Final f: ", signif(res$Table[nrow(res$Table), 2], 3), "\n",
    "Final s: ", signif(res$Table[nrow(res$Table), 3], 3)
  )
  legend("topleft", legend = stats_text, bty = "n", cex = 0.8,
         adj = c(0, 0.5), inset = c(0, -0.05))
}

#############################################
## diagnostics plot
#############################################

plot_gpa_diagnostics <- function(res, trajectory_history = .reconstruct_trajectory(res), main = NULL) {
    # --- Input validation ---
    if (missing(res) || is.null(res))
        stop("'res' must be a GPArotation object.", call. = FALSE)
    if (!inherits(res, "GPArotation"))
        stop("'res' must be a GPArotation object, got: ", class(res), call. = FALSE)
    if (is.null(res$Table))
        stop("'res$Table' is missing - was this object created by GPForth or GPFoblq?",
             call. = FALSE)
    if (is.null(attr(res, "A_unrotated")))
        stop("'A_unrotated' attribute missing - object may have been created by a legacy function.",
             call. = FALSE)

    tab         <- data.frame(res$Table)
    A_unrotated <- attr(res, "A_unrotated")

    if (res$orthogonal) {
        mid_metric <- sapply(trajectory_history, function(Ti) {
            L <- A_unrotated %*% Ti
            var(as.vector(L^2))
        })
        mid_label <- "Var(Loadings^2)"
        mid_title <- "Simple Structure Emergence"
    } else {
        mid_metric <- sapply(trajectory_history, function(Ti) {
            Phi <- t(Ti) %*% Ti
            max(abs(Phi[lower.tri(Phi)]))
        })
        mid_label <- "Max |phi_ij|"
        mid_title <- "Factor Redundancy Trace"
    }

    old_par <- par(no.readonly = TRUE)
    on.exit(par(old_par))
    layout(matrix(c(1, 2, 3), ncol = 1), heights = c(2, 1, 1))

    # 1. Top panel - criterion gap
    f_gap <- pmax(tab$f - min(tab$f), .Machine$double.eps^0.5)
    par(mar = c(0, 4, 3, 2))
    plot(tab$iter, log10(f_gap), type = "l", col = "blue", lwd = 2,
         xaxt = "n", xlab = "", ylab = "log10(f-f[min])")
    main_title <- if (is.null(main))
      paste0(res$method, " - ",
           ifelse(res$convergence, "Converged", "Did not converge"))
    else
      main

    title(main = main_title)

    legend("topright", legend = "Criterion Gap", bty = "n", cex = 0.85)
    grid(lty = 3, col = "gray90")

    # 2. Middle panel - structural metric
    par(mar = c(0, 4, 2, 2))
    plot(0:(length(trajectory_history) - 1), mid_metric,
         type = "l", col = "darkgreen", lwd = 1.5,
         xaxt = "n", xlab = "", ylab = mid_label)
    if (mid_metric[length(mid_metric)] > mid_metric[1]) {
        legend("bottomright", legend = mid_title, bty = "n", cex = 0.85)
    } else {
        legend("topright",    legend = mid_title, bty = "n", cex = 0.85)
    }
    grid(lty = 3, col = "gray90")

    # 3. Bottom panel - gradient norm
    par(mar = c(4, 4, 2, 2))
    plot(tab$iter, tab$"log10.s.", type = "l", col = "darkred",
         xlab = "Iteration", ylab = "log10(s)")
    legend("topright", legend = "Gradient Norm", bty = "n", cex = 0.85)
    abline(h = log10(1e-5), lty = 2, col = "gray")
    grid(lty = 3, col = "gray90")
}



##########################################################
## landscape trajectory plot for 2 factor rotations only
##########################################################
     

plot_algorithm_comparison <- function(A, method = "varimax",
                                      factors = c(1, 2),
                                      maxit = 500, n = 20000,
                                      items = NULL, ...) {

  # Run all three algorithms
  rot_fn  <- get(method, envir = asNamespace("GPArotation"))
  res.leg <- rot_fn(A, algorithm = "legacy",  fwindow =  1,
                    maxit = maxit, ...)
  res.bb  <- rot_fn(A, algorithm = "bb",      fwindow = 10,
                    maxit = maxit, ...)
  res.cay <- rot_fn(A, algorithm = "cayley",  fwindow = 10,
                    maxit = maxit, ...)

  # --- Checks ---
  if (!res.leg$orthogonal)
    stop("plot_algorithm_comparison requires an orthogonal rotation method.\n",
         "Use one of: varimax, quartimax, bentlerT, geominT, parsimax, ",
         "simplimaxT, tandemI, tandemII, targetT, entropy, oblimax.",
         call. = FALSE)

  if (ncol(A) != 2)
    stop("plot_algorithm_comparison requires a 2-factor loading matrix.\n",
         "The landscape plot is only defined for 2-factor orthogonal rotations.",
         call. = FALSE)

  old_par <- par(mfrow = c(2, 3), mar = c(4, 4, 3, 1), oma = c(0, 0, 2, 0))
  on.exit(par(old_par))

  # --- Top row: landscape trajectories ---
  suppressWarnings(
    plot_landscape_trajectory(res.leg, n = n, show_main = FALSE, show_sub = FALSE))
#  title(main = paste0("Legacy  |  Iters: ", nrow(res.leg$Table) - 1), cex.main = 0.9)

  suppressWarnings(
    plot_landscape_trajectory(res.bb, n = n, show_main = FALSE, show_sub = FALSE))
#  title(main = paste0("BB fw=10  |  Iters: ", nrow(res.bb$Table) - 1), cex.main = 0.9)

  suppressWarnings(
    plot_landscape_trajectory(res.cay, n = n, show_main = FALSE, show_sub = TRUE))
#  title(main = paste0("Cayley fw=10  |  Iters: ", nrow(res.cay$Table) - 1), cex.main = 0.9)

  # --- Overall title ---
  mtext(paste0("Algorithm comparison: ", method),
        outer = TRUE, cex = 1.1, font = 2, line = 0.5)

  # --- Bottom row: factor migration trajectories ---
  plot(res.leg, "trajectory", factors = factors, items = items)
  plot(res.bb,  "trajectory", factors = factors, items = items)
  plot(res.cay, "trajectory", factors = factors, items = items)

  invisible(list(legacy = res.leg, bb = res.bb, cayley = res.cay))
}


##########################################################################
## 1. Internal Utility: Trajectory Reconstruction Engine
##########################################################################
.reconstruct_trajectory <- function(res) {
  res <- .sortGPALoadings(res)
  
  A_unrotated <- attr(res, "A_unrotated")
  Tmat        <- res$Tinit
  method      <- sub("vgQ.", "", res$methodName)
  vgQfun      <- paste("vgQ", method, sep = ".")
  max_iter    <- nrow(res$Table) - 1
  fwindow     <- res$fwindow
  algorithm   <- res$algorithm
  methodArgs  <- res$methodArgs
  
  trajectory_history <- list()
  trajectory_history[[1]] <- Tmat
  
  f_history <- numeric(max_iter + 1)
  
  L      <- if(res$orthogonal) A_unrotated %*% Tmat else A_unrotated %*% t(solve(Tmat))
  VgQ    <- do.call(vgQfun, append(list(L), methodArgs))
  f      <- VgQ$f
  f_history[1] <- f
  G      <- if(res$orthogonal) crossprod(A_unrotated, VgQ$Gq) else -t(t(L) %*% VgQ$Gq %*% solve(Tmat))
  
  Tmat_prev <- NULL
  G_prev    <- NULL
  alpha     <- 1

  for (iter in 0:max_iter) {
    if (res$orthogonal) {
      M <- crossprod(Tmat, G); S <- (M + t(M)) / 2
      Gp <- G - Tmat %*% S
    } else {
      Gp <- G - Tmat %*% diag(colSums(Tmat * G))
    }
    s <- sqrt(sum(Gp^2))
    
    if (iter >= max_iter) break

    if (algorithm %in% c("bb", "cayley") && !is.null(Tmat_prev)) {
      dT <- Tmat - Tmat_prev; dGp <- Gp - G_prev
      bb_denom <- sum(dGp^2)
      if (bb_denom > 0) {
        alpha <- sum(dT^2) / abs(sum(dT * dGp))
        alpha <- max(1e-10, min(alpha, 10))
      }
    } else {
      alpha <- 2 * alpha
    }

    f_win_idx <- max(1, (iter + 1) - fwindow + 1):(iter + 1)
    target_f  <- max(f_history[f_win_idx], na.rm = TRUE)

    for (i in 0:10) {
      if (res$orthogonal) {
        if (algorithm == "cayley") {
          W <- Gp %*% t(Tmat) - Tmat %*% t(Gp); I <- diag(ncol(Tmat))
          Tmatt <- solve(I + (alpha/2) * W) %*% (I - (alpha/2) * W) %*% Tmat
        } else {
          X <- Tmat - alpha * Gp; UDV <- svd(X); Tmatt <- UDV$u %*% t(UDV$v)
        }
        L_test <- A_unrotated %*% Tmatt
      } else {
        X <- Tmat - alpha * Gp; v <- 1 / sqrt(colSums(X^2))
        Tmatt <- X %*% diag(v); L_test <- A_unrotated %*% t(solve(Tmatt))
      }
      
      VgQt <- do.call(vgQfun, append(list(L_test), methodArgs))
      if ((target_f - VgQt$f) > 0.5 * s^2 * alpha) break
      alpha <- alpha / 2
    }

    Tmat_prev <- Tmat; G_prev <- Gp
    Tmat <- Tmatt; f <- VgQt$f
    G <- if(res$orthogonal) crossprod(A_unrotated, VgQt$Gq) else -t(t(L_test) %*% VgQt$Gq %*% solve(Tmat))
    
    f_history[iter + 2] <- f
    trajectory_history[[iter + 2]] <- Tmat
  }
  
  return(trajectory_history)
}

##########################################################
## 2. Global Surface Visualizer Engine
##########################################################
plotRotationLandscape <- function(A, method = "quartimax", n = 20000,
                                  main = NULL, lwd = 2, col = "black", 
                                  xlim = NULL, ylim = NULL, ...) {
  if (ncol(A) != 2)
    stop("plotRotationLandscape only works for 2-factor solutions.")

  vgQfun_fn <- get(paste("vgQ", method, sep = "."), envir = asNamespace("GPArotation"))

  if (is.null(main))
    main <- paste("Rotation landscape:", method)

  theta  <- seq(0, 2 * pi, length.out = n)
  f_vals <- numeric(n)

  for (i in seq_along(theta)) {
    Tmat      <- matrix(c( cos(theta[i]), sin(theta[i]),
                          -sin(theta[i]), cos(theta[i])), 2, 2)
    L         <- A %*% Tmat
    VgQ       <- do.call(vgQfun_fn, append(list(L), list(...)))
    f_vals[i] <- VgQ$f
  }

  min_idx <- which.min(f_vals)
  is_inset <- !is.null(xlim)

  plot(theta, f_vals,
       type  = "l", lwd   = lwd, col   = col, main  = main,
       xlab  = if(is_inset) "" else expression(theta ~ "(radians)"),
       ylab  = if(is_inset) "" else "f",
       xaxt  = "n", yaxt  = if(is_inset) "n" else "s",
       xlim  = xlim, ylim  = ylim,
       panel.first = if(is_inset) rect(-1000, -1000, 1000, 1000, col="white", border=NA) else NULL)

  if (!is_inset) {
    axis(1, at     = c(0, pi/2, pi, 3*pi/2, 2*pi),
            labels = c("0", expression(pi/2), expression(pi),
                       expression(3*pi/2), expression(2*pi)))
                       
    text(theta[min_idx], f_vals[min_idx],
         labels = paste0("min f = ", round(f_vals[min_idx], 4)),
         pos    = 4, cex = 0.8)
  }

  abline(h = f_vals[min_idx], col = "grey80", lty = 2)
  points(theta[min_idx], f_vals[min_idx], col = "darkorange", pch = 17, cex = 1.5)

  invisible(data.frame(theta = theta, f = f_vals))
}

##########################################################
## 3. Macro/Micro Optimization Trajectory Path Overlay
##########################################################

plot_landscape_trajectory <- function(x, n = 20000, show_main = TRUE, 
                                      show_legend = TRUE, xlim = NULL, ylim = NULL, ...) {

  if (!inherits(x, "GPArotation"))
    stop("'x' must be a GPArotation object.", call. = FALSE)
  if (ncol(x$loadings) != 2)
    stop("plot_landscape_trajectory only works for 2-factor solutions.", call. = FALSE)
  if (!x$orthogonal)
    stop("plot_landscape_trajectory only works for orthogonal rotations.", call. = FALSE)
    
  # Silently reconstruct the path vectors using our engine
  traj       <- .reconstruct_trajectory(x)
  theta_path <- sapply(traj, function(Th) atan2(Th[2, 1], Th[1, 1]) %% (2 * pi))
  f_path     <- x$Table[1:length(traj), "f"]
  
  A_unrot    <- attr(x, "A_unrotated")
  if (is.null(A_unrot)) {
    A_unrot  <- x$loadings %*% solve(t(traj[[length(traj)]]))
  }
  
  method     <- sub("vgQ.", "", x$methodName)
  methodArgs <- x$methodArgs
  main_title <- if (show_main) NULL else ""
  
  do.call(plotRotationLandscape,
            c(list(A = A_unrot, method = method, n = n, 
            lwd = 0.8, col = "grey70", main = main_title,
            xlim = xlim, ylim = ylim), methodArgs))
            
  dx   <- diff(theta_path)
  dy   <- diff(f_path)
  dist <- sqrt(dx^2 + dy^2)
  
  # Set threshold to 1e-3 to suppress zero-length sub-pixel arrow warnings
  keep <- if (!is.null(xlim)) dist > 1e-3 else dist > 1e-2
  
  if (any(keep)) {
    suppressWarnings(
      arrows(theta_path[-length(theta_path)][keep],
             f_path[-length(f_path)][keep],
             theta_path[-1][keep],
             f_path[-1][keep],
             length = 0.05, col = "steelblue", lwd = 1.2)
      )
  }
  
  points(theta_path, f_path, col = "steelblue", pch = 19, cex = 0.6)
  points(theta_path[1], f_path[1], col = "darkgreen", pch = 19, cex = 1.3)
  points(theta_path[length(theta_path)], f_path[length(f_path)], col = "tomato", pch = 19, cex = 1.3)
    
  # Only draw the structural metadata text if this is the large macro landscape
#  if (is.null(xlim)) {
#    title(sub = paste0(x$algorithm, "  |  fwindow: ", x$fwindow,
#              "  |  Iters: ", nrow(x$Table) - 1), cex.sub = 0.8)
#  }
    
  if (show_legend) {
    legend("topright", bty = "n", cex = 0.75,
           legend  = c("Start", "Path", "End", "Global min"),
           col     = c("darkgreen", "steelblue", "tomato", "darkorange"),
           pch     = c(19, NA, 19, 17), 
           lty     = c(NA, 1, NA, NA),  
           lwd     = c(NA, 1.5, NA, NA))
  }
        
  invisible(list(theta = theta_path, f = f_path))
}

##########################################################################
## 4. Main Exported Function: Multi-Engine Benchmark Interface
##########################################################################
plot2fOrthComparison <- function(A, method = "quartimax", items = NULL,
                                 nang = 6000, maxit = 500, magnify = FALSE, 
                                 inset_pos = "auto", randomStart = FALSE, ...) {

  n <- nang
  if (!inherits(A, c("matrix", "array", "loadings", "factanal")))
    stop("'A' must be a matrix, loadings, or factanal object.", call. = FALSE)

  A_check <- if (inherits(A, "factanal")) unclass(A$loadings) else unclass(A)
  if (ncol(A_check) != 2)
    stop("This visualization strictly requires a 2-factor loading matrix.\n",
         "The landscape plot is only defined for 2-factor orthogonal rotations.", call. = FALSE)

  if (randomStart){
    if (inherits(A, "factanal")){
       A$loadings <- A$loadings %*% Random.Start(2)
    }
    else
    A <- A %*% Random.Start(2)
  }

  rot_fn  <- get(method, envir = asNamespace("GPArotation"))
  res.leg <- rot_fn(A, algorithm = "legacy", fwindow =  1, maxit = maxit, ...)
  res.bb  <- rot_fn(A, algorithm = "bb",     fwindow = 10, maxit = maxit, ...)
  res.cay <- rot_fn(A, algorithm = "cayley", fwindow = 10, maxit = maxit, ...)

  if (!res.leg$orthogonal)
    stop("This visualization requires an orthogonal rotation method.\n")

  # Establish a solid 2x3 column-by-column viewport matrix mapping
  layout_matrix <- matrix(1:6, nrow = 2, ncol = 3, byrow = FALSE)
  old_layout    <- layout(layout_matrix, heights = c(1.35, 1))
  
  old_par <- par(mar = c(3.8, 4.2, 2.5, 1.2), oma = c(1, 0, 3, 0), cex.axis = 0.9)
  on.exit({
    par(old_par)
    layout(1)
  })
  
  clean_method <- paste0(toupper(substring(method, 1, 1)), substring(method, 2))
  
  # Helper: Build bounds centered around the endgame destination coordinate
  calc_zoom <- function(traj) {
    final_t <- traj$theta[length(traj$theta)]
    final_f <- traj$f[length(traj$f)]
    n_tail  <- min(8, length(traj$f)) 
    n       <- length(traj$theta)
    
    x_span  <- min(max(abs(traj$theta[(n - n_tail + 1):n] - final_t)), 0.08)
    zoom_x  <- c(final_t - max(x_span * 1.5, 0.02), final_t + max(x_span * 1.5, 0.02))
    
    y_span  <- min(max(abs(traj$f[(n - n_tail + 1):n] - final_f)), 0.005)
    y_span  <- min(y_span, 0.005) 
    zoom_y  <- c(final_f - max(y_span/5, 1e-4), final_f + max(y_span * 1.5, 1e-3))
    
    list(x = zoom_x, y = zoom_y)
  }
  
  # Helper: Draw adaptive collision-avoiding overlay
  draw_inset <- function(res_obj, traj_obj) {
    withCallingHandlers({
      old_plt <- par("plt")
      u <- par("usr")
      w_usr <- u[2] - u[1]
      h_usr <- u[4] - u[3]
      
      # AUTOMATIC HEURISTIC: Kick box to upper left if trajectory crosses bottom-left
      current_pos <- inset_pos
      if (current_pos == "auto") {
        in_left_zone   <- (traj_obj$theta < (u[1] + w_usr * 0.25))
        in_bottom_zone <- (traj_obj$f < (u[3] + h_usr * 0.35))
        if (sum(in_left_zone & in_bottom_zone, na.rm = TRUE) > 1) {
          current_pos <- "topleft"
        } else {
          current_pos <- "bottomleft"
        }
      }
      
      # Map positions cleanly inside our panel slot frame
      inset_x1 <- old_plt[1] + (old_plt[2] - old_plt[1]) * 0.03
      inset_x2 <- old_plt[1] + (old_plt[2] - old_plt[1]) * 0.33
      
      if (current_pos == "topleft") {
        inset_y1 <- old_plt[3] + (old_plt[4] - old_plt[3]) * 0.55
        inset_y2 <- old_plt[3] + (old_plt[4] - old_plt[3]) * 0.90
      } else {
        inset_y1 <- old_plt[3] + (old_plt[4] - old_plt[3]) * 0.05
        inset_y2 <- old_plt[3] + (old_plt[4] - old_plt[3]) * 0.40
      }
      
      par(plt = c(inset_x1, inset_x2, inset_y1, inset_y2), new = TRUE)
      z_box <- calc_zoom(traj_obj)
      plot_landscape_trajectory(res_obj, n = n, show_main = FALSE,
                                show_legend = FALSE,
                                xlim = z_box$x, ylim = z_box$y)
      box(col = "gray40", lwd = 1.5)
      mtext("Endgame", side = 3, line = 0.1, cex = 0.7, font = 2)
      par(plt = old_plt)
    }, warning = function(w) {
      if (grepl("zero-length arrow", conditionMessage(w)))
        invokeRestart("muffleWarning")
    })
  }

  # -------------------------------------------------------------------------
  # COLUMN 1: LEGACY
  # -------------------------------------------------------------------------
  traj_leg <- suppressWarnings(plot_landscape_trajectory(res.leg, n = n, show_main = FALSE))
  title(main = "Legacy, fwindow: 1", line = 0.8, cex.main = 1.0)
  if (magnify) suppressWarnings(draw_inset(res.leg, traj_leg))

  plot(res.leg, "trajectory", items = items, main = "")
  mtext("Factor 1", side = 1, adj = 0.5, line = 1.9, cex = 0.65, outer = FALSE)
  # title(main = paste0("2a: Factor Migration Path"), line = 0.5, cex.main = 0.9, font.main = 3)

  # -------------------------------------------------------------------------
  # COLUMN 2: BARZILAI-BORWEIN (BB)
  # -------------------------------------------------------------------------
  traj_bb <- suppressWarnings(plot_landscape_trajectory(res.bb, n = n, show_main = FALSE))
  title(main = "Barzilai-Borwein, fwindow = 10", line = 0.8, cex.main = 1.0)
  if (magnify) suppressWarnings(draw_inset(res.bb,  traj_bb))
  
  plot(res.bb, "trajectory", items = items, main = "")
    mtext("Factor 1", side = 1, adj = 0.5, line = 1.9, cex =0.65,outer = FALSE)

  # title(main = paste0("2b: Factor Migration Path"), line = 0.5, cex.main = 0.9, font.main = 3)
  
  # -------------------------------------------------------------------------
  # COLUMN 3: CAYLEY
  # -------------------------------------------------------------------------
  traj_cay <- suppressWarnings(plot_landscape_trajectory(res.cay, n = n, show_main = FALSE))
  title(main = "Cayley, fwindow = 10", line = 0.8, cex.main = 1.0)
  if (magnify) suppressWarnings(draw_inset(res.cay, traj_cay))
  
  plot(res.cay, "trajectory", items = items, main = "")
    mtext("Factor 1", side = 1, adj = 0.5, line = 1.9, cex=0.65, outer = FALSE)

  # title(main = paste0("2c: Factor Migration Path"), line = 0.5, cex.main = 0.9, font.main = 3)
  
  # -------------------------------------------------------------------------
  # CANVAS HEADER BANNER
  # -------------------------------------------------------------------------
  #mtext(text = paste0("Comparison of Numerical Optimization Landscapes (", clean_method, " Rotation)"),
  #      side = 3, outer = TRUE, line = 0.8, cex = 1.2, font = 2)
  mtext(text = paste0("Rotation Landscapes and Migration Trajectories (", clean_method, " Rotation)"),
        side = 3, outer = TRUE, line = 0.8, cex = 1.2, font = 2)
  
  invisible(list(legacy = res.leg, bb = res.bb, cayley = res.cay))
}

# The 2026 spring clean up has made a fundamental change to the building
# of the results Table. These tests are included to ensure that they don't 
# cause trouble

library(GPArotation)

# Helper: a simple loading matrix for testing
A <- matrix(c(0.9, 0.8, 0.7, 0.1, 0.2, 0.1,
              0.1, 0.2, 0.1, 0.9, 0.8, 0.7),
            nrow = 6, ncol = 2)

# --- Test 1: Table has correct column names ---
res <- GPForth(A, method = "quartimax")
stopifnot(identical(colnames(res$Table), c("iter", "f", "log10(s)", "alpha")))

# --- Test 2: Table has no NA rows (trimming worked) ---
stopifnot(!anyNA(res$Table))

# --- Test 3: Table first column is sequential from 0 ---
stopifnot(res$Table[, 1] == 0:(nrow(res$Table) - 1))

# --- Test 4: Converged result has s < eps in final row ---
eps <- 1e-5
s_final <- 10^res$Table[nrow(res$Table), 3]
stopifnot(s_final < eps)

# --- Test 5: Non-converged case raises a warning ---
A_slow <- matrix(c(0.5, 0.4, 0.6, 0.3, 0.5, 0.4,
                   0.4, 0.5, 0.3, 0.5, 0.4, 0.6,
                   0.3, 0.6, 0.4, 0.5, 0.3, 0.5),
                 nrow = 6, ncol = 3)
res_nc <- suppressWarnings(GPForth(A_slow, method = "quartimax", maxit = 2))

if (res_nc$convergence)
  stop("Test 5 failed: expected non-convergence with maxit = 2")

if (nrow(res_nc$Table) != 3)
  stop("Test 5 failed: expected 3 rows in Table, got ", nrow(res_nc$Table))
  
# --- Test 6: Already-converged initial solution — Table has exactly 1 row ---
Tmat_exact <- diag(ncol(A))
res_exact <- GPForth(A, Tmat = Tmat_exact, method = "quartimax", maxit = 0)
stopifnot(nrow(res_exact$Table) >= 1)

GPForth <- function(A,
                    Tmat       = diag(ncol(A)),
                    normalize  = FALSE,
                    eps        = 1e-05,
                    maxit      = 2000,
                    method     = "varimax",
                    methodArgs = NULL,
                    algorithm  = "bb",
                    fwindow    = 10) {
  # Gradient Projection Algorithm for orthogonal rotation (Bernaards & Jennrich, 2005).
  #
  # Args:
  #   A          : Unrotated factor loading matrix (p x k)
  #   Tmat       : Initial rotation matrix (default: identity)
  #   normalize  : Kaiser normalization - logical or weight matrix
  #   eps        : Convergence tolerance on gradient norm (default 1e-5)
  #   maxit      : Maximum iterations (default 2000)
  #   method     : Rotation criterion, e.g. "varimax", "quartimin" (default "varimax")
  #   methodArgs : Additional arguments passed to the vgQ criterion function
  #   algorithm  : Algorithm used for step size alpha
  #   fwindow    : width of the non-monotone line search
  #
  # Returns:
  #   A GPArotation object with:
  #     loadings    - rotated loading matrix
  #     Th          - final rotation matrix
  #     Table       - iteration history (iter, f, log10(gradient norm), alpha)
  #     method      - method label from criterion function
  #     orthogonal  - TRUE (orthogonal rotation)
  #     convergence - TRUE if converged within maxit
  #     Gq          - final gradient matrix

  # --- Accept factanal object ---
  extracted   <- .extract_A(A)
  A           <- extracted$A
  n.obs       <- extracted$n.obs
  correlation <- extracted$correlation

  # --- Kaiser normalization ---
  if ((!is.logical(normalize)) || normalize) {
    W         <- NormalizingWeight(A, normalize = normalize)
    normalize <- TRUE
    A         <- A / W
  }

  if (ncol(A) < 2)
    stop("Rotation does not make sense for single-factor models.")

  A_unrotated <- A

  # --- validation ---
  algorithm <- tolower(algorithm)
  if (!(algorithm %in% c("legacy", "bb", "cayley"))){
    # fallback to package default without crashing
    algorithm <- "bb"
    }
  # ensure fwindow is an integer even if the user passed NULL directly
  if (is.null(fwindow)) fwindow <- ifelse(algorithm == "legacy", 1, 10)

  # --- Initialization ---
  Tinit  <- Tmat
  vgQfun <- paste("vgQ", method, sep = ".")
  alpha  <- 1
  Tmat_prev <- NULL
  Gp_prev    <- NULL
  L      <- A %*% Tmat
  VgQ    <- do.call(vgQfun, append(list(L), methodArgs))
  f      <- VgQ$f
  G      <- crossprod(A, VgQ$Gq)
  VgQt   <- do.call(vgQfun, append(list(L), methodArgs))
  Table  <- matrix(NA_real_, nrow = maxit + 1, ncol = 4,
                   dimnames = list(NULL, c("iter", "f", "log10(s)", "alpha")))

  # --- Main loop ---
  for (iter in 0:maxit) {

    M  <- crossprod(Tmat, G)
    S  <- (M + t(M)) / 2
    Gp <- G - Tmat %*% S
    s  <- sqrt(sum(Gp^2))

    Table[iter + 1, ] <- c(iter, f, log10(s), alpha)

    if (s < eps) break

    # --- Barzilai-Borwein step size ---
    if (algorithm %in% c("bb", "cayley") && !is.null(Tmat_prev)) {
      dT       <- Tmat - Tmat_prev
      dGp      <- Gp    - Gp_prev
      bb_denom <- sum(dGp^2)
      if (bb_denom > 0){
        # alpha <- abs(sum(dT * dGp) / bb_denom) # alternate BB
        alpha <- sum(dT^2) / abs(sum(dT * dGp))  # standard BB
        alpha <- max(1e-10, min(alpha, 20))}
        # BB sets alpha directly - enter line search without doubling
    } else {
      alpha <- 2 * alpha  # original behaviour
    }

    fwindow_range <- max(1, (iter + 1) - fwindow + 1):(iter + 1)
    target_f <- max(Table[fwindow_range ,2], na.rm = TRUE)

    # Line search
    for (i in 0:10) {
      if (algorithm == "cayley") {
        # --- CAYLEY (The Curve) ---
        W <- Gp %*% t(Tmat) - Tmat %*% t(Gp)
        I <- diag(ncol(Tmat))
        X_inv <- solve(I + (alpha / 2) * W)
        Tmatt <- X_inv %*% (I - (alpha / 2) * W) %*% Tmat
      } else {
        # --- SVD (The Snap-Back) ---
        X     <- Tmat - alpha * Gp
        UDV   <- svd(X)
        Tmatt <- UDV$u %*% t(UDV$v)
      }
      L     <- A %*% Tmatt
      VgQt  <- do.call(vgQfun, append(list(L), methodArgs))
      if ((target_f - VgQt$f) > 0.5 * s^2 * alpha) 
        break
      alpha <- alpha / 2
    }

    Tmat_prev <- Tmat
    Gp_prev    <- Gp
    Tmat <- Tmatt
    f    <- VgQt$f
    G    <- crossprod(A, VgQt$Gq)
  }
  
  # --- Convergence check ---
  convergence <- (s < eps)
  if (iter == maxit && !convergence) {
    conv_gap <- paste0("Algorithm stopped at s = ", signif(s, 4),
                       " (target eps = ", eps, ").")
    suggestion <- if (algorithm == "legacy")
      "Consider: increase maxit or try algorithm = \"bb\" with fwindow = 10."
    else if (algorithm == "bb")
      "Consider: increase maxit, increase fwindow, or try algorithm = \"cayley\"."
    else if (algorithm == "cayley")
      "Consider: increase maxit or try algorithm = \"bb\" with fwindow = 10."
    warning("Convergence not obtained in GPForth. ", maxit, " iterations used.\n",
            conv_gap, "\n", suggestion)
  }

  # --- Trim Table ---
  Table <- Table[1:(iter + 1), , drop = FALSE]

  # --- Undo normalization ---
  if (normalize)
    L <- L * W

  dimnames(L) <- dimnames(A)

  class(L) <- "loadings"
  attr(L, "covariance") <- diag(ncol(L))

  r <- list(
    loadings    = L,
    Th          = Tmat,
    Tinit       = Tinit,
    Table       = Table,
    method      = VgQ$Method,
    methodName  = method,
    methodArgs  = methodArgs,
    orthogonal  = TRUE,
    convergence = convergence,
    algorithm   = algorithm,
    fwindow     = fwindow,
    Gq          = VgQt$Gq,
    n.obs       = n.obs,
    correlation = correlation
  )
  attr(r, "A_unrotated") <- A_unrotated
  class(r) <- "GPArotation"
  r
}



GPFoblq <- function(A,
                    Tmat       = diag(ncol(A)),
                    normalize  = FALSE,
                    eps        = 1e-05,
                    maxit      = 2000,
                    method     = "quartimin",
                    methodArgs = NULL,
                    algorithm  = "bb",
                    fwindow    = 10) {
  # Gradient Projection Algorithm for oblique rotation (Bernaards & Jennrich, 2005).
  #
  # Args:
  #   A          : Unrotated factor loading matrix (p x k)
  #   Tmat       : Initial transformation matrix (default: identity)
  #   normalize  : Kaiser normalization - logical or weight matrix
  #   eps        : Convergence tolerance on gradient norm (default 1e-5)
  #   maxit      : Maximum iterations (default 2000)
  #   method     : Rotation criterion, e.g. "quartimin" (default "quartimin")
  #   methodArgs : Additional arguments passed to the vgQ criterion function
  #   algorithm  : Algorithm used for step size alpha
  #   fwindow    : width of the non-monotone line search
  #
  # Returns:
  #   A GPArotation object with:
  #     loadings    - rotated loading matrix
  #     Phi         - factor correlation matrix (t(Tmat) %*% Tmat)
  #     Th          - final transformation matrix
  #     Table       - iteration history (iter, f, log10(gradient norm), alpha)
  #     method      - method label from criterion function
  #     orthogonal  - FALSE (oblique rotation)
  #     convergence - TRUE if converged within maxit
  #     Gq          - final gradient matrix  
  #     n.obs       - n.obs from factanal;  NULL if not from factanal
  #     correlation - correlation matrix from factanal; NULL if not from factanal
  singularity_warning <- FALSE
  
  # --- Accept factanal object ---
  extracted   <- .extract_A(A)
  A           <- extracted$A
  n.obs       <- extracted$n.obs
  correlation <- extracted$correlation

  if (ncol(A) < 2)
    stop("Rotation does not make sense for single-factor models.")

  # --- Kaiser normalization ---
  if ((!is.logical(normalize)) || normalize) {
    W         <- NormalizingWeight(A, normalize = normalize)
    normalize <- TRUE
    A         <- A / W
  }

  A_unrotated <- A

  # --- Safe inverse ---
  safe_inverse <- function(X) {
    tryCatch(
      solve(X),
      error = function(e) {
        assign("singularity_warning", TRUE, envir = parent.frame(n = 2))
        s_svd <- svd(X)
        tol   <- sqrt(.Machine$double.eps)
        d_inv <- ifelse(s_svd$d > tol, 1 / s_svd$d, 0)
        s_svd$v %*% diag(d_inv) %*% t(s_svd$u)
      }
    )
  }
  
  # --- validation ---
  if (algorithm == "cayley")
    stop("algorithm = \"cayley\" is only available for orthogonal rotation. ",
         "Use GPForth or an orthogonal rotation wrapper.", call. = FALSE)
  algorithm <- tolower(algorithm)
  if (!(algorithm %in% c("legacy", "bb"))){
    # fallback to package default without crashing
    algorithm <- "bb"
    }
         
  # ensure fwindow is an integer even if the user passed NULL directly
  if (is.null(fwindow)) fwindow <- ifelse(algorithm == "legacy", 1, 10)  

  # --- Initialization ---
  Tinit    <- Tmat
  vgQfun   <- paste("vgQ", method, sep = ".")
  alpha    <- 1
  Tmat_prev <- NULL
  Gp_prev    <- NULL
  Tmat_inv <- safe_inverse(Tmat)
  L        <- A %*% t(Tmat_inv)
  VgQ      <- do.call(vgQfun, append(list(L), methodArgs))
  f        <- VgQ$f
  G        <- -t(t(L) %*% VgQ$Gq %*% Tmat_inv)
  VgQt     <- do.call(vgQfun, append(list(L), methodArgs))
  Table    <- matrix(NA_real_, nrow = maxit + 1, ncol = 4,
                     dimnames = list(NULL, c("iter", "f", "log10(s)", "alpha")))

  # --- Main loop ---
  for (iter in 0:maxit) {

    Gp <- G - Tmat %*% diag(colSums(Tmat * G))
    s  <- sqrt(sum(Gp^2))

    Table[iter + 1, ] <- c(iter, f, log10(s), alpha)

    if (s < eps) break

    # --- Barzilai-Borwein step size ---
    if (algorithm == "bb"  && !is.null(Tmat_prev)) {
      dT       <- Tmat - Tmat_prev
      dGp       <- Gp   - Gp_prev
      bb_denom <- sum(dGp^2)
      if (bb_denom > 0){
        # alpha <- abs(sum(dT * dGp) / bb_denom) # alternate BB
        alpha <- sum(dT^2) / abs(sum(dT * dGp))  # standard BB
        alpha <- max(1e-10, min(alpha, 20))}
        # BB sets alpha directly - enter line search without doubling
    } else {
      alpha <- 2 * alpha  # original behaviour
    }
   
    fwindow_range <- max(1, (iter + 1) - fwindow + 1):(iter + 1)
    target_f <- max(Table[fwindow_range ,2], na.rm = TRUE)

    # Line search
    for (i in 0:10) {
      X         <- Tmat - alpha * Gp
      v         <- 1 / sqrt(colSums(X^2))
      Tmatt     <- X %*% diag(v)
      Tmatt_inv <- safe_inverse(Tmatt)
      L         <- A %*% t(Tmatt_inv)
      VgQt      <- do.call(vgQfun, append(list(L), methodArgs))
      if ((target_f - VgQt$f) > 0.5 * s^2 * alpha) 
        break
      alpha     <- alpha / 2
    }

    Tmat_prev <- Tmat
    Gp_prev    <- Gp
    Tmat <- Tmatt
    f    <- VgQt$f
    G    <- -t(t(L) %*% VgQt$Gq %*% Tmatt_inv)
  }

  # --- Convergence check with smart diagnostics ---
  convergence <- (s < eps)
  if (iter == maxit && !convergence) {
    Phi     <- t(Tmat) %*% Tmat
    max_cor <- max(abs(Phi[lower.tri(Phi)]))
    conv_gap <- paste0("Algorithm stopped at s = ", signif(s, 4),
                       " (target eps = ", eps, ").")
    suggestion <- if (algorithm == "legacy")
      "Consider: increase maxit or try algorithm = \"bb\" with fwindow = 10."
    else if (algorithm == "bb")
      "Consider: increase maxit or increase fwindow."
    if (max_cor > 0.85) {
      warning("Convergence not obtained in GPFoblq. ", maxit, " iterations used.\n",
              conv_gap, "\n",
              "DIAGNOSTIC: Extreme factor correlations detected (Max = ",
              round(max_cor, 3), ").\n",
              "This almost always indicates factor over-extraction. ",
              "Try reducing the number of extracted factors.")
    } else {
      warning("Convergence not obtained in GPFoblq. ", maxit, " iterations used.\n",
              conv_gap, "\n", suggestion)
    }
  }

  # --- Trim Table ---
  Table <- Table[1:(iter + 1), , drop = FALSE]

  # --- Undo normalization ---
  if (normalize)
    L <- L * W

  dimnames(L) <- dimnames(A)

  # --- covariance attribute for print.loadings ---
  Phi <- t(Tmat) %*% Tmat
  class(L) <- "loadings"
  attr(L, "covariance") <- Phi

  r <- list(
    loadings    = L,
    Phi         = Phi,
    Th          = Tmat,
    Tinit       = Tinit,
    Table       = Table,
    method      = VgQ$Method,
    methodName  = method,
    methodArgs  = methodArgs,
    orthogonal  = FALSE,
    convergence = convergence,
    algorithm   = algorithm,
    fwindow     = fwindow,
    Gq          = VgQt$Gq,
    n.obs       = n.obs,
    correlation = correlation
  )
  attr(r, "A_unrotated") <- A_unrotated
  class(r) <- "GPArotation"
  r
}


###########################################################################
# Legacy functions - retained for historical reference and reproducibility.
# Original implementations from Bernaards & Jennrich (2005).
# Superseded by the updated GPForth and GPFoblq above (GPArotation 2026.4-1).
# Note: commented-out lines are preserved intentionally as part of the
# original development history.
###########################################################################


GPForth.legacy <- function(A,
                           Tmat       = diag(ncol(A)),
                           normalize  = FALSE,
                           eps        = 1e-05,
                           maxit      = 1000,
                           method     = "varimax",
                           methodArgs = NULL) {
  # Legacy implementation of GPForth (Bernaards & Jennrich, 2005).
  # Retained for historical reference and reproducibility.
  # Superseded by GPForth in GPArotation 2026.4-1.
  if((!is.logical(normalize)) || normalize) {
     W <- NormalizingWeight(A, normalize=normalize)
     normalize <- TRUE
     A <- A/W
     }
 if(1 >= ncol(A)) stop("rotation does not make sense for single factor models.")
 al <- 1
 L <- A %*% Tmat
 #Method <- get(paste("vgQ",method,sep="."))
 #VgQ <- Method(L, ...)
 Method <- paste("vgQ",method,sep=".")
 VgQ <- do.call(Method, append(list(L), methodArgs))
 G <- crossprod(A,VgQ$Gq)
 f <- VgQ$f
 Table <- NULL
 #set initial value for the unusual case of an exact initial solution 
 VgQt <- do.call(Method, append(list(L), methodArgs))   
 for (iter in 0:maxit){
   M <- crossprod(Tmat,G)
   S <- (M + t(M))/2
   Gp <- G - Tmat %*% S
   s <- sqrt(sum(diag(crossprod(Gp))))
   Table <- rbind(Table, c(iter, f, log10(s), al))
   if (s < eps)  break
   al <- 2*al
   for (i in 0:10){
     X <- Tmat - al * Gp
     UDV <- svd(X)
     Tmatt <- UDV$u %*% t(UDV$v)
     L <- A %*% Tmatt
     #VgQt <- Method(L, ...)
     VgQt <- do.call(Method, append(list(L), methodArgs))
     if (VgQt$f < (f - 0.5*s^2*al)) break
     al <- al/2
     }
   Tmat <- Tmatt
   f <- VgQt$f
   G <- crossprod(A,VgQt$Gq)
   }
 convergence <- (s < eps)
 if ((iter == maxit) & !convergence)
     warning("convergence not obtained in GPForth. ", maxit, " iterations used.")
 if(normalize) L <- L * W
 dimnames(L) <- dimnames(A)
 r <- list(loadings=L, Th=Tmat, Table=Table, 
        method=VgQ$Method, orthogonal=TRUE, convergence=convergence, Gq=VgQt$Gq)
 class(r) <- "GPArotation"
 r
}

GPFoblq.legacy <- function(A,
                           Tmat       = diag(ncol(A)),
                           normalize  = FALSE,
                           eps        = 1e-05,
                           maxit      = 1000,
                           method     = "quartimin",
                           methodArgs = NULL) {
  # Legacy implementation of GPFoblq (Bernaards & Jennrich, 2005).
  # Retained for historical reference and reproducibility.
  # Superseded by GPFoblq in GPArotation 2026.4-1.
  
 if(1 >= ncol(A)) stop("rotation does not make sense for single factor models.")
 if((!is.logical(normalize)) || normalize) {
     W <- NormalizingWeight(A, normalize=normalize)
     normalize <- TRUE
     A <- A/W
     }
 al <- 1
 L <- A %*% t(solve(Tmat))
 #Method <- get(paste("vgQ",method,sep="."))
 #VgQ <- Method(L, ...)
 Method <- paste("vgQ",method,sep=".")
 VgQ <- do.call(Method, append(list(L), methodArgs))
 G <- -t(t(L) %*% VgQ$Gq %*% solve(Tmat))
 f <- VgQ$f
 Table <- NULL
 #Table <- c(-1,f,log10(sqrt(sum(diag(crossprod(G))))),al)
 #set initial value for the unusual case of an exact initial solution 
 VgQt <- do.call(Method, append(list(L), methodArgs))   
 for (iter in 0:maxit){
   Gp <- G - Tmat %*% diag(c(rep(1,nrow(G)) %*% (Tmat*G)))
   s <- sqrt(sum(diag(crossprod(Gp))))
   Table <- rbind(Table,c(iter,f,log10(s),al))
   if (s < eps) break
   al <- 2*al
   for (i in 0:10){
     X <- Tmat - al*Gp
     v <- 1/sqrt(c(rep(1,nrow(X)) %*% X^2))
     Tmatt <- X %*% diag(v)
     L <- A %*% t(solve(Tmatt))
     #VgQt <- Method(L, ...)
     VgQt <- do.call(Method, append(list(L), methodArgs))
     improvement <- f - VgQt$f 
     if (improvement >  0.5*s^2*al) break
     al <- al/2
     }
   Tmat <- Tmatt
   f <- VgQt$f
   G <- -t(t(L) %*% VgQt$Gq %*% solve(Tmatt))
   }
 convergence <- (s < eps)
 if ((iter == maxit) & !convergence)
     warning("convergence not obtained in GPFoblq. ", maxit, " iterations used.")
 if(normalize) L <- L * W
 dimnames(L) <- dimnames(A)

 # N.B. renaming Lh to loadings in specificific rotations 
 #   uses fact that  Lh is first.
 r <- list(loadings=L, Phi=t(Tmat) %*% Tmat, Th=Tmat, Table=Table,
      method=VgQ$Method, orthogonal=FALSE, convergence=convergence, Gq=VgQt$Gq)
 class(r) <- "GPArotation"
 r
}

plot_residuals <- function(x, R = NULL, cutoff = 0.10, main = NULL) {

  # --- Input validation ---
  if (!inherits(x, "GPArotation"))
    stop("'x' must be a GPArotation object.", call. = FALSE)

  # --- Correlation matrix ---
  if (is.null(R)) {
    if (!is.null(x$correlation)) {
      R <- x$correlation
    } else {
      stop("'R' is required. Supply the observed correlation matrix, ",
           "or rotate a factanal object so R is stored automatically.",
           call. = FALSE)
    }
  }
  if (!is.matrix(R))
    R <- as.matrix(R)
  R <- cov2cor(R)

  # --- Loadings in R row order ---
  L_res   <- unclass(x$loadings)
  Phi_res <- if (is.null(x$Phi)) diag(ncol(L_res)) else x$Phi
  L_res   <- L_res[rownames(R), , drop = FALSE]

  # --- Residual matrix ---
  Resid       <- residuals(x)
  nms         <- rownames(R)
  n           <- length(nms)
  srmr        <- sqrt(mean(Resid[lower.tri(Resid)]^2))

  old_par <- par(mfrow = c(1, 2), mar = c(6, 5, 4, 2))
  on.exit(par(old_par))

  # --- Panel 1: residual heatmap via rect() ---
  max_abs <- max(abs(Resid))
  pal     <- colorRampPalette(c("steelblue", "white", "tomato"))(100)
  breaks  <- seq(-max_abs, max_abs, length.out = 101)

  main_title <- if (is.null(main))
    paste0("Residual Correlation Matrix\nSRMR = ", round(srmr, 3))
  else
    main

  plot(NULL,
       xlim = c(0.5, n + 0.5), ylim = c(0.5, n + 0.5),
       xaxt = "n", yaxt = "n",
       xlab = "", ylab = "",
       main = main_title)
       
  for (i in 1:n) {
    for (j in 1:n) {
      col_idx <- findInterval(Resid[i, j], breaks)
      col_idx <- max(1, min(col_idx, 100))
      rect(j - 0.5, n - i + 0.5,
           j + 0.5, n - i + 1.5,
           col = pal[col_idx], border = NA)
    }
  }

  # grid lines
  abline(h = 0.5:(n + 0.5), col = "white", lwd = 0.5)
  abline(v = 0.5:(n + 0.5), col = "white", lwd = 0.5)
  box()

  # x-axis: CCAI8 at left, same order as R
  axis(1, at = 1:n, labels = nms,      las = 2, cex.axis = 0.7)
  # y-axis: CCAI8 at top (row 1 is drawn at top via n - i + 1)
  axis(2, at = 1:n, labels = rev(nms), las = 1, cex.axis = 0.7)

  # --- Panel 2: sorted absolute residuals ---
  res_vals <- sort(abs(Resid[lower.tri(Resid)]))
  bar_cols <- ifelse(res_vals > cutoff, "tomato",
              ifelse(res_vals > 0.05,   "darkgreen",
                                        "lightgreen"))

  barplot(res_vals,
          col    = bar_cols,
          border = NA,
          main   = "Sorted Absolute Residuals",
          ylab   = "Absolute residual",
          xlab   = "Item pair (sorted)")
  abline(h = cutoff, lty = 2, col = "gray40", lwd = 1.5)
  abline(h = 0.05,   lty = 3, col = "gray60", lwd = 1.0)
  legend("topleft", bty = "n", cex = 0.8,
         legend = c(paste0("> ", cutoff,  " (concern)"),
                    paste0("0.05 - ", cutoff, " (watch)"),
                    "< 0.05 (acceptable)"),
         fill   = c("tomato", "darkgreen", "lightgreen"),
         border = NA)

  invisible(list(
    Resid  = Resid,
    SRMR   = srmr,
    cutoff = cutoff
  ))
}


audit_residuals <- function(x, R = NULL, cutoff = 0.10, allDiff = FALSE) {

  # --- Input validation ---
  if (!inherits(x, "GPArotation"))
    stop("'x' must be a GPArotation object.", call. = FALSE)

  # --- Correlation matrix ---
  if (is.null(R)) {
    if (!is.null(x$correlation)) {
      R <- x$correlation
    } else {
      stop("'R' is required. Supply the observed correlation matrix, ",
           "or rotate a factanal object so R is stored automatically.",
           call. = FALSE)
    }
  }
  if (!is.matrix(R))
    R <- as.matrix(R)
  R <- cov2cor(R)

  # --- Loadings in R row order ---
  L_res   <- unclass(x$loadings)
  Phi_res <- if (is.null(x$Phi)) diag(ncol(L_res)) else x$Phi
  L_res   <- L_res[rownames(R), , drop = FALSE]

  # --- Residual matrix ---
  Resid <- residuals(x)

  # --- Summary statistics ---
  srmr       <- sqrt(mean(Resid[lower.tri(Resid)]^2))
  row_strain <- rowMeans(abs(Resid))

  # --- Flagged pairs ---
  idx   <- which(lower.tri(Resid), arr.ind = TRUE)
  pairs <- data.frame(
    Item_A   = rownames(L_res)[idx[, 1]],
    Item_B   = rownames(L_res)[idx[, 2]],
    Residual = round(Resid[lower.tri(Resid)], 4)
  )
  pairs <- pairs[abs(pairs$Residual) > cutoff, ]
  pairs <- pairs[order(abs(pairs$Residual), decreasing = TRUE), ]

  # --- Output ---
  cat(sprintf("SRMR: %.4f\n\n", srmr))
  cat("Mean absolute residual per item:\n")
  print(round(sort(row_strain, decreasing = TRUE), 4))

  if (nrow(pairs) == 0) {
    cat("\nNo item pairs with absolute residual exceeding", cutoff, "\n")
  } else {
    cat("\nItem pairs with absolute residual exceeding", cutoff, ":\n")
    print(pairs, row.names = FALSE)
  }

  if (allDiff) {
    cat("\nFull residual matrix (R - R_hat):\n")
    print(round(Resid, 4))
  }

  invisible(list(
    Resid      = Resid,
    SRMR       = srmr,
    row_strain = row_strain,
    pairs      = pairs,
    cutoff     = cutoff
  ))
}

# Main wrapper functions for GPForth and GPFoblq with random start functionality.
# Overhauled as part of 2026.4-1

.GPA_RS_engine <- function(A, Tmat, normalize = FALSE, eps = 1e-5, maxit = 1000,
                            method = NULL, methodArgs = NULL, randomStarts = 0,
                            orthogonal = TRUE, L = NULL, ...) {
  # Internal engine implementing random-start gradient projection rotation.
  #
  # Args:
  #   A            : Unrotated factor loading matrix (p x k)
  #   Tmat         : Initial rotation/transformation matrix
  #   normalize    : Kaiser normalization - logical or weight matrix
  #   eps          : Convergence tolerance (default 1e-5)
  #   maxit        : Maximum iterations (default 1000)
  #   method       : Rotation criterion name
  #   methodArgs   : Additional arguments passed to the vgQ criterion function
  #   randomStarts : Number of random starts (0 = use Tmat as-is)
  #   orthogonal   : TRUE for orthogonal (GPForth), FALSE for oblique (GPFoblq)
  #   L            : Deprecated alias for A
  #
  # Returns:
  #   A GPArotation object from the best start, with optional randStartChar diagnostics

  # --- Deprecation ---
  if (!is.null(L)) {
    warning("Argument 'L' is deprecated; using 'A' instead.", call. = FALSE)
    A <- L
  }

  # --- Validation ---
  if (ncol(A) <= 1)
    stop("Rotation does not make sense for single-factor models.")

  # --- Setup ---
  engine        <- if (orthogonal) GPForth else GPFoblq
  actual_starts <- max(1, randomStarts)
  best_res      <- NULL
  Qvalues       <- numeric(actual_starts)
  Qconverged    <- logical(actual_starts)

  # --- Execution loop ---
  for (i in 1:actual_starts) {
    current_Tmat <- if (randomStarts > 0) Random.Start(ncol(A)) else Tmat

    res <- engine(A, Tmat = current_Tmat, normalize = normalize, eps = eps,
                  maxit = maxit, method = method, methodArgs = methodArgs)

    # Use indexing instead of tail() to avoid namespace warnings
    Qvalues[i]    <- res$Table[nrow(res$Table), 2]
    Qconverged[i] <- res$convergence

    if (is.null(best_res) || Qvalues[i] < best_res$Table[nrow(best_res$Table), 2])
      best_res <- res
  }

  # --- Random start diagnostics ---
  if (randomStarts > 1) {
    Q_round    <- round(Qvalues / eps) * eps
    Qmin_round <- min(Q_round)

    best_res$randStartChar <- c(
      randomStarts = randomStarts,
      Converged    = sum(Qconverged),
      atMinimum    = sum(Q_round == Qmin_round),
      localMins    = length(unique(Q_round))
    )
  }

  # --- Finalize for S3 print/summary ---
  if (!orthogonal)
    dimnames(best_res$Phi) <- list(colnames(A), colnames(A))

  best_res
}

GPFRSorth <- function(A, Tmat = diag(ncol(A)), normalize = FALSE, eps = 1e-5,
                      maxit = 1000, method = "varimax", methodArgs = NULL,
                      randomStarts = 0, ...) {
  # Orthogonal rotation with optional random starts. Wrapper for .GPA_RS_engine.
  #
  # Args:
  #   A            : Unrotated factor loading matrix (p x k)
  #   Tmat         : Initial rotation matrix (default: identity)
  #   normalize    : Kaiser normalization - logical or weight matrix
  #   eps          : Convergence tolerance (default 1e-5)
  #   maxit        : Maximum iterations (default 1000)
  #   method       : Rotation criterion (default "varimax")
  #   methodArgs   : Additional arguments passed to the vgQ criterion function
  #   randomStarts : Number of random starts (0 = use Tmat as-is)

  .GPA_RS_engine(A = A, Tmat = Tmat, normalize = normalize, eps = eps,
                 maxit = maxit, method = method, methodArgs = methodArgs,
                 randomStarts = randomStarts, orthogonal = TRUE, ...)
}

GPFRSoblq <- function(A, Tmat = diag(ncol(A)), normalize = FALSE, eps = 1e-5,
                      maxit = 1000, method = "quartimin", methodArgs = NULL,
                      randomStarts = 0, ...) {
  # Oblique rotation with optional random starts. Wrapper for .GPA_RS_engine.
  #
  # Args:
  #   A            : Unrotated factor loading matrix (p x k)
  #   Tmat         : Initial transformation matrix (default: identity)
  #   normalize    : Kaiser normalization - logical or weight matrix
  #   eps          : Convergence tolerance (default 1e-5)
  #   maxit        : Maximum iterations (default 1000)
  #   method       : Rotation criterion (default "quartimin")
  #   methodArgs   : Additional arguments passed to the vgQ criterion function
  #   randomStarts : Number of random starts (0 = use Tmat as-is)

  .GPA_RS_engine(A = A, Tmat = Tmat, normalize = normalize, eps = eps,
                 maxit = maxit, method = method, methodArgs = methodArgs,
                 randomStarts = randomStarts, orthogonal = FALSE, ...)
}

# Random.Start generates a single random orthogonal matrix of size k x k.
#
# References:
#   Stewart, G. W. (1980). The efficient generation of random orthogonal matrices
#     with an application to condition estimators. SIAM Journal on Numerical
#     Analysis, 17(3), 403-409. http://www.jstor.org/stable/2156882
#
#   Mezzadri, F. (2007). How to generate random matrices from the classical
#     compact groups. Notices of the American Mathematical Society, 54(5),
#     592-604. https://arxiv.org/abs/math-ph/0609050
#
# Updated in GPArotation 2024.2-1 following a suggestion by Yves Rosseel.
# The original qr.Q(qr(matrix(rnorm(k*k), k))) does not guarantee uniform
# sampling from the Haar measure; the sign correction on R's diagonal does.

Random.Start <- function(k = 2L) {
  # 1. Generate a k x k matrix of standard normal variates and QR decompose
  Z      <- matrix(rnorm(k * k), nrow = k, ncol = k)
  qr.out <- qr(Z)

  # 2. Extract Q and the signs of R's diagonal
  Q <- qr.Q(qr.out)
  d <- sign(diag(qr.R(qr.out)))

  # 3. Scale columns of Q by the signs - ensures uniform sampling (Haar measure)
  Q %*% diag(d)
}

###########################################################################
###########################################################################

# --- OBLIMIN ---

oblimin <- function(A, Tmat = NULL, gam = 0, normalize = FALSE,
                    randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  # Oblimin oblique rotation. Special cases via gam:
  #   0   - Quartimin
  #   0.5 - Biquartimin
  #
  # Args:
  #   A            : Unrotated factor loading matrix (p x k)
  #   Tmat         : Initial transformation matrix (default: identity)
  #   gam          : Obliqueness parameter (default 0)
  #   normalize    : Kaiser normalization - logical or weight matrix
  #   randomStarts : Number of random starts (0 = use Tmat as-is)

  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "oblimin",
                 methodArgs   = list(gam = gam),
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = FALSE, call = mc,
                 algorithm    = algorithm, fwindow = fwindow, ...)
}

vgQ.oblimin <- function(L, gam = 0) {
  # Oblimin family of oblique rotation criteria.
  # gam controls the degree of obliqueness:
  #   0    - Quartimin
  #   0.5  - Biquartimin
  #
  # Args:
  #   L   : Factor loading matrix
  #   gam : Obliqueness parameter (default 0)
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix
  #     f      - objective function value
  #     Method - method label
  X <- rowSums(L^2) - L^2
  if (gam != 0) {
        X <- sweep(X, 2, (gam / nrow(L)) * colSums(X), "-")
  }
  method <- if (gam == 0) "Oblimin Quartimin"
              else if (gam == 0.5) "Oblimin Biquartimin"
              else paste0("Oblimin gamma =", gam)
    list(Gq = L * X, f = sum(L^2 * X)/4, Method = method)
}

# --- QUARTIMIN ---

quartimin <- function(A, Tmat = NULL, normalize = FALSE,
                      randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "quartimin",
                 normalize    = normalize, randomStarts = randomStarts,
                 orthogonal   = FALSE, call = mc, algorithm = algorithm, 
                 fwindow = fwindow, ...)
}

vgQ.quartimin <- function(L) {
  # Quartimin oblique rotation criterion (special case of Oblimin with gam = 0).
  #
  # Args:
  #   L : Factor loading matrix
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix
  #     f      - objective function value
  #     Method - method label

  k  <- ncol(L)
  L2 <- L^2
  X  <- L2 %*% (matrix(1, k, k) - diag(k))

  list(
    Gq     = L * X,
    f      = sum(L2 * X) / 4,
    Method = "Quartimin"
  )
}

# --- CRAWFORD-FERGUSON FAMILY ---

cfT <- function(A, Tmat = NULL, kappa = 0, normalize = FALSE,
                randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "cf",
                 methodArgs   = list(kappa = kappa), normalize    = normalize,
                 randomStarts = randomStarts, orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

cfQ <- function(A, Tmat = NULL, kappa = 0, normalize = FALSE,
                randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "cf",
                 methodArgs   = list(kappa = kappa), normalize    = normalize,
                 randomStarts = randomStarts, orthogonal   = FALSE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow,  ...)
}

equamax <- function(A, Tmat = NULL, kappa = NULL,
                    normalize = FALSE, randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  # Logic for default kappa if not provided by user
  if (is.null(kappa)) {
    extracted <- .extract_A(A)
    # actual calculation using extracted matrix dims
    kappa <- ncol(extracted$A) / (2 * nrow(extracted$A))
  }

  .GPA_RS_engine(A = A, Tmat = Tmat, method = "cf",
                 methodArgs   = list(kappa = kappa), normalize    = normalize,
                 randomStarts = randomStarts, orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow,  ...)
}

parsimax <- function(A, Tmat = NULL, kappa = NULL,
                      normalize = FALSE, randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  if (is.null(kappa)) {
    extracted <- .extract_A(A)
    p <- nrow(extracted$A)
    m <- ncol(extracted$A)
    kappa <- (m - 1) / (m + p - 2)
  }

  .GPA_RS_engine(A = A, Tmat = Tmat, method = "cf",
                 methodArgs   = list(kappa = kappa), normalize    = normalize,
                 randomStarts = randomStarts, orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow,  ...)
}

vgQ.cf <- function(L, kappa = 0) {
  # Crawford-Ferguson (CF) family of rotation criteria (1970).
  # kappa controls the complexity penalty:
  #   0              - Quartimax / Quartimin
  #   1/p            - Varimax
  #   k/(2*p)        - Equamax
  #   (k-1)/(p+k-2)  - Parsimax
  #   1              - Factor Parsimony
  #
  # Args:
  #   L     : Factor loading matrix (p x k)
  #   kappa : Complexity weight (default 0)
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix
  #     f      - objective function value
  #     Method - method label

  p <- nrow(L)
  k <- ncol(L)
  L2 <- L^2
   
  # rowSums(L2) - L2 is the same as L2 %*% N
  # colSums(L2) - L2 is the same as M %*% L2
  rowS <- rowSums(L2)
  colS <- colSums(L2)
   
  # Efficiently broadcast the sums without building p x p matrices
  L2N <- sweep(-L2, 1, rowS, "+")
  ML2 <- sweep(-L2, 2, colS, "+")
   
  f1 <- (1 - kappa) * sum(L2 * L2N) / 4
  f2 <- kappa * sum(L2 * ML2) / 4

  method <- if (kappa == 0) 
        "Crawford-Ferguson Quartimax/Quartimin"
  else if (kappa == 1/p) 
        "Crawford-Ferguson Varimax"
  else if (kappa == k/(2 * p)) 
        "Equamax"
  else if (kappa == (k - 1)/(p + k - 2)) 
        "Parsimax"
  else if (kappa == 1) 
        "Factor Parsimony"
  else paste0("Crawford-Ferguson: kappa=", kappa)
   
  list(Gq = (1 - kappa) * L * L2N + kappa * L * ML2,
         f = f1 + f2, Method = method)
}


# --- TARGET ROTATION ---

targetT <- function(A, Tmat = NULL, Target = NULL, normalize = FALSE,
                    randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  if (is.null(Target)) stop("Argument 'Target' must be specified.")

  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "target",
                 methodArgs   = list(Target = Target), normalize    = normalize,
                 randomStarts = randomStarts, orthogonal = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow,  ...)
}

targetQ <- function(A, Tmat = NULL, Target = NULL, normalize = FALSE,
                    randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  if (is.null(Target)) stop("Argument 'Target' must be specified.")

  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "target",
                 methodArgs   = list(Target = Target), normalize    = normalize,
                 randomStarts = randomStarts, orthogonal = FALSE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

vgQ.target <- function(L, Target = NULL) {
  # Target rotation criterion.
  # Minimizes the sum of squared deviations from a specified target matrix.
  # NA entries in Target are treated as unspecified (ignored in fit and gradient).
  #
  # Args:
  #   L      : Factor loading matrix
  #   Target : Target matrix (same dimensions as L; NA = unspecified element)
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix (zero where Target is NA)
  #     f      - objective function value
  #     Method - method label

  Gq            <- 2 * (L - Target)
  Gq[is.na(Gq)] <- 0

  list(
    Gq     = Gq,
    f      = sum((L - Target)^2, na.rm = TRUE),
    Method = "Target rotation"
  )
}

# --- PARTIALLY SPECIFIED TARGET ---

pstT <- function(A, Tmat = NULL, W = NULL, Target = NULL, normalize = FALSE,
                 randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  if (is.null(W))      stop("Argument 'W' must be specified.")
  if (is.null(Target)) stop("Argument 'Target' must be specified.")

  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "pst",
                 methodArgs   = list(W = W, Target = Target),
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

pstQ <- function(A, Tmat = NULL, W = NULL, Target = NULL, normalize = FALSE,
                 randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  if (is.null(W))      stop("Argument 'W' must be specified.")
  if (is.null(Target)) stop("Argument 'Target' must be specified.")

  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "pst",
                 methodArgs   = list(W = W, Target = Target),
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = FALSE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

vgQ.pst <- function(L, W = NULL, Target = NULL) {
  # Partially specified target rotation criterion.
  # Minimizes weighted deviations from a target matrix.
  # W is a binary weight matrix specifying which elements are targeted.
  #
  # Args:
  #   L      : Factor loading matrix
  #   W      : Binary weight matrix (same dimensions as L; 0 = unspecified element)
  #   Target : Target matrix (same dimensions as L)
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix
  #     f      - objective function value
  #     Method - method label

  Btilde <- W * Target

  list(
    Gq     = 2 * (W * L - Btilde),
    f      = sum((W * L - Btilde)^2),
    Method = "Partially specified target"
  )
}

# --- ENTROPY ---

entropy <- function(A, Tmat = NULL, normalize = FALSE,
                    randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "entropy",
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

vgQ.entropy <- function(L) {
  # Minimum entropy oblique rotation criterion.
  #
  # Args:
  #   L : Factor loading matrix
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix
  #     f      - objective function value
  #     Method - method label

  L2    <- L^2
  logL2 <- log(L2 + .Machine$double.eps)

  list(
    Gq     = -(L * logL2 + L),
    f      = -sum(L2 * logL2) / 2,
    Method = "Minimum entropy"
  )
}

# --- INFOMAX ---

infomaxT <- function(A, Tmat = NULL, normalize = FALSE,
                     randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "infomax",
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

infomaxQ <- function(A, Tmat = NULL, normalize = FALSE,
                     randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "infomax",
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = FALSE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

vgQ.infomax <- function(L) {
  # Infomax oblique rotation criterion.
  #
  # Args:
  #   L : Factor loading matrix
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix
  #     f      - objective function value
  #     Method - method label

  k <- ncol(L)
  p <- nrow(L)
  S <- L^2
  s <- sum(S)
  eps <- .Machine$double.eps
    
  # Proportions
  E <- S/s
  e1 <- rowSums(S)/s
  e2 <- colSums(S)/s
    
  # Entropies
  Q0 <- -sum(E * log(E + eps))
  Q1 <- -sum(e1 * log(e1 + eps))
  Q2 <- -sum(e2 * log(e2 + eps))
    
  # Gradients of Entropies
  H <- -(log(E + eps) + 1)
  h1 <- -(log(e1 + eps) + 1)
  h2 <- -(log(e2 + eps) + 1)
    
  # Centered components (E[H] logic)
  G0 <- (H - sum(S * H)/s) / s
  G1 <- (matrix(h1, p, k) - sum(rowSums(S) * h1)/s) / s
  G2 <- (matrix(h2, p, k, byrow = TRUE) - sum(colSums(S) * h2)/s) / s
    
  list(Gq = 2 * L * (G0 - G1 - G2), f = log(k) + Q0 - Q1 - Q2, Method = "Infomax")
}


# --- McCAMMON ---

mccammon <- function(A, Tmat = NULL, normalize = FALSE,
                     randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "mccammon",
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

# This vgQ updated on May 6, 2026 to align with lavaan
# Checked against numerical gradient. Only Gq needed update
# 
vgQ.mccammon <- function(L) {
  # Basic dimensions
  p <- nrow(L)
  k <- ncol(L)
 
  # Square loadings and sums
  S <- L^2
  col_sums <- colSums(S)
  total_s <- sum(S)
 
  # 1. Normalize for Q1 (Column entropy)
  # P is the proportion of a column's variance contributed by each row
  P <- sweep(S, 2, col_sums, "/")
 
  # Safe log function to handle zero loadings
  safe_log <- function(x) {
    res <- x
    res[x > 0] <- log(x[x > 0])
    res[x <= 0] <- 0
    return(res)
  }
 
  logP <- safe_log(P)
 
  # Q1: Entropy within columns (encourages simple structure per factor)
  Q1 <- -sum(P * logP)
 
  # 2. Normalize for Q2 (Across-column entropy)
  # p2 is the proportion of total variance in each column
  p2 <- col_sums / total_s
  logp2 <- safe_log(p2)
 
  # Q2: Entropy of column sums (encourages equal-sized factors)
  Q2 <- -sum(p2 * logp2)
 
  # --- Analytical Gradient Calculation ---
 
  # Component 1: Gradient of Q1 wrt S
  # H1 matches Bernaards & Jennrich logic: -(log(P) + 1)
  H1 <- -(logP + 1)
  # alpha1 is the column-wise expected value of H1
  alpha1 <- colSums(P * H1)
  # G1 is the centered gradient scaled by column sums
  G1 <- sweep(H1, 2, alpha1, "-")
  G1 <- sweep(G1, 2, col_sums, "/")
 
  # Component 2: Gradient of Q2 wrt S
  # h2 is the vector of gradients for each column sum
  h2 <- -(logp2 + 1)
  # alpha2 is the total expected value of h2
  alpha2 <- sum(p2 * h2)
  # G2 is the centered gradient scaled by the total sum of squares
  # We expand the vector h2 into a matrix that matches L
  G2_vec <- (h2 - alpha2) / total_s
  G2 <- matrix(G2_vec, p, k, byrow = TRUE)
 
  # Combine components for the total gradient Gq
  # f = log(Q1) - log(Q2)
  # d/dL f = 2 * L * ( (1/Q1)*dQ1/dS - (1/Q2)*dQ2/dS )
  Gq <- 2 * L * (G1 / Q1 - G2 / Q2)
 
  f <- log(Q1) - log(Q2)
 
  return(list(Gq = Gq, f = f, Method = "McCammon entropy"))
}



# --- GEOMIN ---

geominT <- function(A, Tmat = NULL, delta = 0.01, normalize = FALSE,
                    randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "geomin",
                 methodArgs   = list(delta = delta),
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

geominQ <- function(A, Tmat = NULL, delta = 0.01, normalize = FALSE,
                    randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "geomin",
                 methodArgs   = list(delta = delta),
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = FALSE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

vgQ.geomin <- function(L, delta = 0.01) {
  # Geomin oblique rotation criterion.
  # delta is a small constant added for numerical stability (default 0.01).
  #
  # Args:
  #   L     : Factor loading matrix
  #   delta : Stabilizing constant (default 0.01)
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix
  #     f      - objective function value
  #     Method - method label

  k   <- ncol(L)
  L2  <- L^2 + delta
  pro <- exp(rowMeans(log(L2)))

  list(
    Gq     = (2 / k) * (L / L2) * pro,
    f      = sum(pro),
    Method = "Geomin"
  )
}

# --- SIMPLIMAX ---

simplimax <- function (A, Tmat = NULL, k = NULL, normalize = FALSE,
                       randomStarts = 0, algorithm = "bb", fwindow = 10, ...)
{
    # 1. Capture the call
    mc <- match.call()

    # 2. Extract the matrix FIRST so we know the true nrow
    extracted <- .extract_A(A)
    actual_A  <- extracted$A

    # 3. If k wasn't provided, set it safely based on the extracted matrix
    if (is.null(k)) {
        k <- nrow(actual_A)
    }

    # 4. Pass everything to the engine
    .GPA_RS_engine(A = A, Tmat = Tmat, method = "simplimax",
        methodArgs = list(k = k), normalize = normalize, randomStarts = randomStarts,
        orthogonal = FALSE, call = mc, algorithm = algorithm, fwindow = fwindow,
        ...)
}

vgQ.simplimax <- function(L, k = nrow(L)) {
  # Simplimax oblique rotation criterion (Kiers, 1994).
  # Minimizes the k smallest squared loadings, targeting a simple structure
  # with k near-zero loadings per solution.
  #
  # Args:
  #   L : Factor loading matrix
  #   k : Number of near-zero loadings to target (default: nrow(L))
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix
  #     f      - objective function value
  #     Method - method label

  L2 <- L^2
  # Create an empty logical matrix
  Imat <- matrix(FALSE, nrow(L), ncol(L))
    
  # Explicitly pick the k smallest indices to avoid threshold ties
  idx <- order(L2)[1:k]
  Imat[idx] <- TRUE
    
  list(Gq = 2 * Imat * L, f = sum(L2[Imat]), 
       Method = paste0("Simplimax (k=", k, ")"))
}

# --- BIFACTOR ---

bifactorT <- function(A, Tmat = NULL, normalize = FALSE,
                      randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "bifactor",
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

bifactorQ <- function(A, Tmat = NULL, normalize = FALSE,
                      randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "bifactor",
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = FALSE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

vgQ.bifactor <- function(L) {
  # Bifactor Biquartimin oblique rotation criterion.
  # Treats the first column as a general factor (unconstrained)
  # and rotates the remaining group factor columns.
  #
  # Args:
  #   L : Factor loading matrix (first column = general factor)
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix (zero gradient for general factor column)
  #     f      - objective function value
  #     Method - method label

  Lt2  <- L[, -1, drop = FALSE]^2
  rowS <- rowSums(Lt2)
  Gt   <- 4 * L[, -1, drop = FALSE] * (rowS - Lt2)

  list(
    Gq     = cbind(0, Gt),
    f      = sum(rowS^2) - sum(Lt2^2),
    Method = "Bifactor Biquartimin"
  )
}

# --- BI-GEOMIN ---

bigeominT <- function(A, Tmat = NULL, delta = 0.01, normalize = FALSE,
                      randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "bigeomin",
                 methodArgs   = list(delta = delta),
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

bigeominQ <- function(A, Tmat = NULL, delta = 0.01, normalize = FALSE,
                      randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "bigeomin",
                 methodArgs   = list(delta = delta),
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = FALSE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

vgQ.bigeomin <- function(L, delta = 0.01) {
  # Bi-Geomin oblique rotation criterion.
  # Treats the first column as a general factor (unconstrained)
  # and applies Geomin rotation to the remaining group factor columns.
  #
  # Args:
  #   L     : Factor loading matrix (first column = general factor)
  #   delta : Stabilizing constant passed to vgQ.geomin (default 0.01)
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix (zero gradient for general factor column)
  #     f      - objective function value
  #     Method - method label

  out <- vgQ.geomin(L[, -1, drop = FALSE], delta = delta)

  list(
    Gq     = cbind(0, out$Gq),
    f      = out$f,
    Method = "Bi-Geomin"
  )
}

# --- TANDEM CRITERIA ---

tandemI <- function(A, Tmat = NULL, normalize = FALSE,
                    randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "tandemI",
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

vgQ.tandemI <- function(L) {
  # Tandem Criterion I (Comrey, 1967)
  # Seeks factors that share high loadings on the same variables.
  #
  # Args:
  #   L : Factor loading matrix
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix
  #     f      - objective function value (negated for minimization)
  #     Method - method label

  L2 <- L^2
  # tcrossprod(L) is L %*% t(L), but more numerically stable
  LL <- tcrossprod(L)
  LL2 <- LL^2
 
  # Pre-compute the matrix product used in both f and Gq1
  # This reduces redundant calculations and preserves precision
  M_prod <- LL2 %*% L2
 
  # f = -tr(L2' %*% LL2 %*% L2)
  # Replacing sum(diag(crossprod(...))) with sum(A * B)
  # avoids intermediate matrix noise
  f <- -sum(L2 * M_prod)
 
  # Gq1 = 4 * L * (LL2 %*% L2)
  Gq1 <- 4 * L * M_prod
 
  # Gq2 = 4 * (LL * (L2 %*% t(L2))) %*% L
  # We use tcrossprod here as well for symmetry and stability
  Gq2 <- 4 * (LL * tcrossprod(L2)) %*% L
 
  Gq <- -(Gq1 + Gq2)
 
  list(Gq = Gq, f = f, Method = "Tandem I")
}


tandemII <- function(A, Tmat = NULL, normalize = FALSE,
                     randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "tandemII",
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

vgQ.tandemII <- function(L) {
  # Tandem Criterion II (Comrey, 1967)
  # Seeks factors that do NOT share high loadings on the same variables
  # (used as a second-stage refinement after Tandem I).
  #
  # Args:
  #   L : Factor loading matrix
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix
  #     f      - objective function value
  #     Method - method label

  L2 <- L^2
  LL <- tcrossprod(L)
  LL2 <- LL^2
  # 1 in R will be recycled to the size of the matrix LL2
  ILL2 <- 1 - LL2
 
  M_prod <- ILL2 %*% L2
 
  # f = sum(diag(crossprod(L2, ILL2 %*% L2)))
  f <- sum(L2 * M_prod)
 
  Gq1 <- 4 * L * M_prod
  Gq2 <- 4 * (LL * tcrossprod(L2)) %*% L
 
  Gq <- Gq1 - Gq2
 
  list(Gq = Gq, f = f, Method = "Tandem II")
}

# --- OBLIMAX ---

oblimax <- function(A, Tmat = NULL, normalize = FALSE,
                    randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "oblimax",
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = FALSE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

vgQ.oblimax <- function(L) {
  # Oblimax oblique rotation criterion.
  #
  # Args:
  #   L : Factor loading matrix
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix
  #     f      - objective function value (negated for minimization)
  #     Method - method label

  s2 <- sum(L^2)
  s4 <- sum(L^4)

  list(
    Gq     = -(4 * L^3 / s4 - 4 * L / s2),
    f      = -(log(s4) - 2 * log(s2)),
    Method = "Oblimax"
  )
}

# --- BENTLER ---

bentlerT <- function(A, Tmat = NULL, normalize = FALSE,
                     randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "bentler",
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

bentlerQ <- function(A, Tmat = NULL, normalize = FALSE,
                     randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "bentler",
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = FALSE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

vgQ.bentler <- function(L) {
  # Bentler's invariant pattern simplicity oblique rotation criterion.
  #
  # Args:
  #   L : Factor loading matrix
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix (negated for minimization)
  #     f      - objective function value (negated for minimization)
  #     Method - method label

  L2 <- L^2
  M <- crossprod(L2)
  dM <- diag(M)
  
  # Safer log-determinant
  logDetM <- determinant(M, logarithm = TRUE)$modulus
  logDetD <- sum(log(dM))
    
  f <- -(logDetM - logDetD) / 4
    
  # Gq logic remains correct, but solve() can be replaced 
  # with solve(M, L2) for better stability in some contexts
  D_inv <- diag(1/dM)
  Gq <- -L * (L2 %*% (solve(M) - D_inv))
    
  list(Gq = Gq, f = as.numeric(f), Method = "Bentler")
}

# --- QUARTIMAX ---

quartimax <- function(A, Tmat = NULL, normalize = FALSE,
                      randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "quartimax",
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

vgQ.quartimax <- function(L) {
  # Quartimax orthogonal rotation criterion.
  #
  # Args:
  #   L : Factor loading matrix
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix (negated for minimization)
  #     f      - objective function value (negated for minimization)
  #     Method - method label

  list(
    Gq     = -L^3,
    f      = -sum(L^4) / 4,
    Method = "Quartimax"
  )
}

# --- VARIMAX ---

Varimax <- function(A, Tmat = NULL, normalize = FALSE,
                    randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "varimax",
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

vgQ.varimax <- function(L) {
  # Varimax orthogonal rotation criterion (Kaiser, 1958).
  #
  # Args:
  #   L : Factor loading matrix
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix (negated for minimization)
  #     f      - objective function value (negated for minimization)
  #     Method - method label

  QL <- L^2 - rep(colMeans(L^2), each = nrow(L))

  list(
    Gq     = -L * QL,
    f      = -sum(QL^2) / 4,
    Method = "Varimax"
  )
}

# --- VARIMIN ---

varimin <- function(A, Tmat = NULL, normalize = FALSE,
                    randomStarts = 0, algorithm = "bb", fwindow = 10, ...) {
  mc <- match.call()
  .GPA_RS_engine(A = A, Tmat = Tmat, method = "varimin",
                 normalize    = normalize,
                 randomStarts = randomStarts,
                 orthogonal   = TRUE, call = mc, 
                 algorithm = algorithm, fwindow = fwindow, ...)
}

vgQ.varimin <- function(L) {
  # Varimin orthogonal rotation criterion.
  # Minimizes the variance of squared loadings within factors - 
  # the complement of Varimax.
  #
  # Args:
  #   L : Factor loading matrix
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix
  #     f      - objective function value
  #     Method - method label

  QL <- L^2 - rep(colMeans(L^2), each = nrow(L))

  list(
    Gq     = L * QL,
    f      = sum(QL^2) / 4,
    Method = "Varimin"
  )
}

# --- LP WLS ---

vgQ.lp.wls <- function(L, W) {
  # Weighted least squares criterion for Lp rotation.
  #
  # Args:
  #   L : Factor loading matrix
  #   W : Weight matrix (same dimensions as L)
  #
  # Returns:
  #   A list with:
  #     Gq     - gradient matrix
  #     f      - objective function value
  #     Method - method label

  p <- nrow(L)

  list(
    Gq     = 2 * W * L / p,
    f      = sum(W * L^2) / p,
    Method = "Weighted least squares for Lp rotation"
  )
}

# Also see the first test in rotations.R
# compares varimax to Varimax to 0.001 discrepancy

 Sys.getenv("R_LIBS")
 library()
 require("GPArotation")
 search()
 Sys.info()

### note that this is a slightly lower bar than other tests
### to correct for the built-in varimax function working differently
### than GPA, and to ensure Varimax convergence
### these are differences in the 4th decimal or better
 fuzz <- 1e-4
###
 all.ok <- TRUE  

sortFac <- function(x){  # Based on Fungible faSort 
  vx <- order(colSums(x$loadings^2), decreasing = TRUE)
  Dsgn <- diag(sign(colSums(x$loadings^3))) [ , vx]
  x$Th <- x$Th %*% Dsgn 
  x$loadings <- x$loadings %*% Dsgn
  if ("Phi" %in% names(x)) { 
    x$Phi <- diag(1/diag(Dsgn)) %*% x$Phi %*% Dsgn
    }
  x 
} 
data(Thurstone, package="GPArotation")
yv1 <- varimax(box20, normalize = FALSE, eps = 1e-7) #built-in R
names(yv1) <- c("loadings","Th")
yv1 <- sortFac(yv1)
yv2 <- sortFac(Varimax(box20, normalize = FALSE, maxit = 10000, eps = 1e-7)) #GPArotation version
# yv.diff <- unclass(yv1$loadings) - unclass(yv2$loadings)
# max(abs(yv.diff))


 if( fuzz < max(abs(yv1$loadings - yv2$loadings))) {
    cat("Calculated varimax is not the same as Varimax:\n")
    # print(yv2$loadings, digits=18)
    cat("difference:\n")
    print(yv1$loadings - yv2$loadings, digits=18)
    all.ok <- FALSE  
    } 

#' Plot Method for GPArotation Objects
#'
#' @param x An object of class "GPArotation"
#' @param type Character string indicating plot type: "salient", "pairs", or "profile"
#' @param hide_hyperplane Logical. If TRUE, blanks out values < cutoff in the salient plot
#'                                 or hides the legend in the profile plot.
#' @param salient Numeric. Values above this are considered salient.
#' @param cutoff Numeric. Values below this are considered in the hyperplane.
#' @param ... Additional arguments passed to base plot functions.
#'
#' @export

plot.GPArotation <- function(x,
            type = c("salient", "pairs", "profile", "vector", "target", 
                     "heatmap","trajectory", "diagnostics", "residuals"),
            hide_hyperplane = FALSE, salient = 0.40, cutoff = .1, R = NULL,
            factors = c(1,2), items = NULL, main = NULL, ...){
    
  type <- match.arg(type)
  L   <- unclass(x$loadings)
  Phi <- x$Phi
 
  k <- ncol(L)
  v <- nrow(L)
 
  fmt_num <- function(val) {
    txt <- sprintf("%.2f", val)
    txt <- sub("^-0.", "-.", txt)
    txt <- sub("^0.", ".", txt)
    return(txt)
  }
 
  # ---------------------------------------------------------
  # TYPE 1: THE APA SALIENT DASHBOARD
  # ---------------------------------------------------------
  if (type == "salient") {

    # Sort for display only
    x_sorted <- .sortGPALoadings(x)
    L   <- unclass(x_sorted$loadings)
    Phi <- x_sorted$Phi

    old_par <- par(mar = c(2, 5, 4, 1), oma = c(2, 0, 2, 0))
    on.exit(par(old_par))
   
    if (!is.null(Phi)) {
      S <- L %*% Phi
      rownames(S) <- rownames(L)
      colnames(S) <- colnames(L)
     
      lay_mat <- matrix(c(1, 2, 3, 4), nrow = 2, byrow = TRUE)
      layout(lay_mat, heights = c(v + 2, max(k * 3, 8)), widths = c(1, 1))
      # layout(lay_mat, heights = c(v + 2, k + 2), widths = c(1, 1))
    } else {
      layout(matrix(1, nrow = 1))
    }
   
    breaks <- c(0, cutoff, salient, max(abs(L), 1) + 0.1)
    cols   <- c("white", "grey95", "grey80")
   
    draw_matrix <- function(mat, title, exempt_diag = FALSE) {
      nr <- nrow(mat)
      nc <- ncol(mat)
      mat_rev <- mat[nr:1, , drop = FALSE]
     
      text_cex <- min(1.2, 25 / max(nr, nc))
      axis_cex <- min(1.0, text_cex * 1.1)

      # For shading: cap diagonal at 0 so it stays white
      mat_shade <- abs(mat_rev)
      if (exempt_diag) {
        for (d in 1:min(nr, nc))
          mat_shade[nr - d + 1, d] <- 0
      }
     
      image(x = 1:nc, y = 1:nr, z = t(mat_shade),
            col = cols, breaks = breaks,
            axes = FALSE, xlab = "", ylab = "")
     
	  if (nchar(title) > 0)
         mtext(title, side = 3, line = 2.5, font = 2, cex = min(1.0, axis_cex * 1.1))

      axis(3, at = 1:nc, labels = colnames(mat), tick = FALSE,
           line = 0.5, cex.axis = axis_cex)
      axis(2, at = 1:nr, labels = rownames(mat_rev), tick = FALSE,
           las = 2, line = -0.5, cex.axis = axis_cex)
     
      box()
      if (nc > 1) abline(v = (1:(nc-1)) + 0.5, col = "grey90")
      if (nr > 1) abline(h = (1:(nr-1)) + 0.5, col = "grey90")
     
      for (i in 1:nr) {
        for (j in 1:nc) {
          val <- mat_rev[i, j]
          if (hide_hyperplane && abs(val) < cutoff) next
          # Exempt diagonal from bolding
          is_diag    <- exempt_diag && (i == (nr - j + 1))
          font_style <- ifelse(!is_diag && abs(val) >= salient, 2, 1)
          text(j, i, labels = fmt_num(val), font = font_style, cex = text_cex)
        }
      }
    }
   
    # 1. Pattern Matrix (Top Left)
    if (is.null(Phi)) 
      draw_matrix(L, "")
   
    if (!is.null(Phi)) {
      # 1. Pattern Matrix (Top Left)
      draw_matrix(L, "Pattern Matrix")

      # 2. Structure Matrix (Top Right)
      draw_matrix(S, "Structure Matrix")
     
      # 3. Phi Matrix (Bottom Left) - exempt diagonal
      dimnames(Phi) <- list(colnames(L), colnames(L))
      draw_matrix(Phi, "", exempt_diag = TRUE)
     
      # 4. Fit stats (Bottom Right)
      plot.new()
      plot.window(xlim = c(0, 1), ylim = c(0, 1))
      box(col = "grey80")

      stats_list <- c()
      if (!is.null(x$n.obs)) stats_list <- c(stats_list, paste("N =", x$n.obs))
      if (!is.null(x$dof))   stats_list <- c(stats_list, paste("df =", x$dof))
      if (!is.null(x$RMSEA)) stats_list <- c(stats_list, paste("RMSEA =", fmt_num(x$RMSEA)))
      if (!is.null(x$SRMR))  stats_list <- c(stats_list, paste("SRMR =",  fmt_num(x$SRMR)))
     
      if (length(stats_list) > 0) {
        stats_text <- paste(stats_list, collapse = "\n")
        text(0.5, 0.5, labels = stats_text, cex = 1.3, font = 2)
      }
    }
  }

  # ---------------------------------------------------------
  # TYPE 2: PAIRS PLOT
  # ---------------------------------------------------------
  else if (type == "pairs") {
    old_par <- par(mfrow = c(k, k), mar = c(2, 2, 1, 1), oma = c(4, 4, 4, 1))
    on.exit(par(old_par))

    # Consistent y-axis across all histograms
    max_count <- max(sapply(1:k, function(i)
      max(hist(L[, i], plot = FALSE, breaks = 15)$counts)))
    hist_ylim <- c(0, max_count * 1.1)

    for (row_idx in 1:k) {
      for (col_idx in 1:k) {
        if (row_idx == col_idx) {
          hist(L[, row_idx], main = "", xlab = "", ylab = "",
               col = "lightblue", border = "white", breaks = 15,
               xlim = c(-1, 1), ylim = hist_ylim)
          legend("topleft", bty = "n", legend = colnames(L)[row_idx], text.font = 2)
          box(col = "grey80")
          } else if (row_idx < col_idx) {
            plot(0, 0, type = "n", axes = FALSE, xlab = "", ylab = "")
            if (!is.null(Phi)) {
              text(0,  0.15, labels = paste0("r = ",  fmt_num(Phi[row_idx, col_idx])),
                   cex = 1.5, font = 2)
              text(0, -0.15, labels = paste0("r\u00B2 = ", fmt_num(Phi[row_idx, col_idx]^2)),
                   cex = 1.2, font = 1)
              # TODO: additional factor relationship summaries
             } else {
               text(0, 0, labels = "Orthogonal", col = "grey", cex = 1.2)
             }
            box(col = "grey80")
          }
          else {
          plot(L[, col_idx], L[, row_idx], pch = 20, col = "black",
               xlab = "", ylab = "", xlim = c(-1, 1), ylim = c(-1, 1))
          abline(h = 0, v = 0, col = "grey50", lty = 2)
          box(col = "grey80")
        }
      }
    }
  }
  
  # ---------------------------------------------------------
  # TYPE 3: PROFILE BAR CHARTS
  # ---------------------------------------------------------
  else if (type == "profile") {
    old_par <- par(mfrow = c(k, 1), mar = c(1, 4, 1, 1), oma = c(6, 0, 2, 0))
    on.exit(par(old_par))
    for (i in 1:k) {
      bar_cols <- ifelse(abs(L[, i]) >= salient, "steelblue", "grey80")
      bp <- barplot(L[, i], names.arg = rep("", v), ylim = c(-1, 1),
                    col = bar_cols, border = NA, ylab = colnames(L)[i], las = 2)
      abline(h = 0, col = "black")
      abline(h = c(-cutoff, cutoff), col = "red", lty = 2)
      if (!hide_hyperplane)
        legend("topright", bty = "o", cex = 0.8, bg = "white",
               legend = c(paste("hyperplane:", sum(abs(L[, i]) < cutoff)),
                          paste("salient:   ", sum(abs(L[, i]) >= salient))))
      if (i == k) {
        axis(1, at = bp, labels = rownames(L), las = 2, cex.axis = 0.8)
        mtext("Item", side = 1, line = 5, cex = 0.8)
      }
    }
  }
  
  # ---------------------------------------------------------
  # TYPE 4: TARGET PLOTS
  # ---------------------------------------------------------
  else if (type == "target") {

    Target <- list(...)$Target
    if (is.null(Target) && !is.null(x$Target)) Target <- x$Target

    if (is.null(Target)) {
      warning("No Target matrix found. Please supply one ",
              "(e.g., plot(x, type='target', Target=my_matrix)).")
    } else {

      A_unrot <- attr(x, "A_unrotated")
      L_rot   <- L

      f1 <- if (!is.null(list(...)$f1)) list(...)$f1 else 1
      f2 <- if (!is.null(list(...)$f2)) list(...)$f2 else 2

      # Pre-rotation positions
      Tinit <- if (!is.null(x$Tinit)) x$Tinit else diag(ncol(A_unrot))
      L_pre <- if (x$orthogonal) {
        A_unrot %*% Tinit
      } else {
        A_unrot %*% t(solve(Tinit))
      }

      old_par <- par(mfrow = c(1, 2), mar = c(4, 5, 4, 1))
      on.exit(par(old_par), add = TRUE)

      # ----- Panel 1: Arrow plot -----
      f1 <- if (!is.null(list(...)$f1)) list(...)$f1 else 1
      f2 <- if (!is.null(list(...)$f2)) list(...)$f2 else 2

      Tinit <- if (!is.null(x$Tinit)) x$Tinit else diag(ncol(A_unrot))
      L_pre <- if (x$orthogonal) {
        A_unrot %*% Tinit
      } else {
        A_unrot %*% t(solve(Tinit))
      }

      target_complete <- !apply(Target[, c(f1, f2), drop = FALSE], 1, anyNA)
      n_specified     <- sum(target_complete)

      if (n_specified < 2) {
        # PST with few fully specified rows --- show message instead
        plot.new()
        text(0.5, 0.5,
             paste0("Target partially specified\n(",
                    n_specified, " fully specified rows in factors ",
                    f1, " and ", f2, ").\nSee heatmap for discrepancies."),
             cex = 0.9, col = "grey40")
      } else {
        # Full or mostly specified target --- draw arrow plot
        all_coords <- rbind(L_pre[, c(f1, f2)],
                            L_rot[, c(f1, f2)],
                            Target[target_complete, c(f1, f2)])
        lims <- range(all_coords, na.rm = TRUE) * 1.15
        plot(NULL,
             xlim = lims, ylim = lims,
             xlab = paste("Factor", f1),
             ylab = paste("Factor", f2),
             main = if (is.null(main)) "Target Rotation: Item Trajectories" else main,
             asp  = 1)
        abline(h = 0, v = 0, col = "grey70", lty = 2)

        arrows(x0 = L_pre[, f1], y0 = L_pre[, f2],
               x1 = L_rot[, f1], y1 = L_rot[, f2],
               length = 0.06, col = "grey60", lwd = 0.8)

        points(Target[target_complete, f1],
               Target[target_complete, f2],
               pch = 19, col = "steelblue", cex = 0.8)

        points(L_pre[, f1], L_pre[, f2],
               pch = 17, col = "tomato", cex = 0.8)

        points(L_rot[, f1], L_rot[, f2],
               pch = 15, col = "gold3", cex = 0.8)

        legend("topright", bty = "n", cex = 0.75,
               legend = c("Target", "Pre-rotation", "Post-rotation"),
               col    = c("steelblue", "tomato", "gold3"),
               pch    = c(19, 17, 15))
      }

      # ----- Panel 2: Discrepancy heatmap -----
      Discrepancy <- L_rot - Target
      nr    <- nrow(Discrepancy)
      nc    <- ncol(Discrepancy)
      D_rev <- Discrepancy[nr:1, , drop = FALSE]

      error_cols  <- colorRampPalette(c("steelblue", "white", "firebrick"))(100)
      breaks_disc <- seq(-1, 1, length.out = 101)

      image(x = 1:nc, y = 1:nr, z = t(D_rev),
            col = error_cols, breaks = breaks_disc,
            axes = FALSE, xlab = "", ylab = "",
            main = "Rotated minus Target")

      text_cex <- min(1.2, 25 / max(nr, nc))
      axis(3, at = 1:nc, labels = colnames(Discrepancy), tick = FALSE,
           line = -0.5, cex.axis = min(1.0, text_cex * 1.1))
      axis(2, at = 1:nr, labels = rownames(D_rev), tick = FALSE,
           las = 2, line = -0.5, cex.axis = min(1.0, text_cex * 1.1))
      box()
      if (nc > 1) abline(v = (1:(nc-1)) + 0.5, col = "grey90")
      if (nr > 1) abline(h = (1:(nr-1)) + 0.5, col = "grey90")

      for (i in 1:nr) {
        for (j in 1:nc) {
          val <- D_rev[i, j]
          if (is.na(val)) {
            text(j, i, labels = "-", cex = text_cex, col = "grey70")
          } else {
            text_col <- ifelse(abs(val) > 0.4, "white", "black")
            text(j, i, labels = fmt_num(val), cex = text_cex, col = text_col)
          }
        }
      }
    }
  }
     
  # ---------------------------------------------------------
  # TYPE 5: VECTOR PLOTS
  # ---------------------------------------------------------
  else if (type == "vector") {

    old_par <- par(mar = c(3, 3, 3, 1), oma = c(2, 2, 2, 1))
    on.exit(par(old_par))

    # (k-1) x (k-1) grid - lower triangle pairs only
    par(mfrow = c(k - 1, k - 1))

    for (row_idx in 2:k) {
      for (col_idx in 1:(k - 1)) {

        if (col_idx >= row_idx) {
          # Empty panel for upper triangle
          plot.new()
        } else {
          # r value as panel title
          r_label <- if (!is.null(Phi))
            paste0("r = ", fmt_num(Phi[row_idx, col_idx]),
                   "  r\u00B2 = ", fmt_num(Phi[row_idx, col_idx]^2))
          else
            "Orthogonal"

          # Vector plot
          plot(0, 0, type = "n", xlim = c(-1.1, 1.1), ylim = c(-1.1, 1.1),
               axes = FALSE, xlab = "", ylab = "", main = r_label)
          abline(h = 0, v = 0, col = "grey50", lty = 2)
          symbols(0, 0, circles = 1, inches = FALSE, add = TRUE, fg = "grey80")
          arrows(x0 = 0, y0 = 0,
                 x1 = L[, col_idx], y1 = L[, row_idx],
                 length = 0.05, col = "steelblue", lwd = 1.2)
          text(L[, col_idx], L[, row_idx],
               labels = rownames(L),
               pos = 3, cex = 0.6, col = "grey30")
          box(col = "grey80")

          # Factor names as axis labels
          mtext(colnames(L)[col_idx], side = 1, line = 1.5, cex = 0.9, font = 2)
          mtext(colnames(L)[row_idx], side = 2, line = 1.5, cex = 0.9, font = 2)
        }
      }
    }
  }
  # ---------------------------------------------------------
  # TYPE 6: HEATMAP
  # ---------------------------------------------------------
  else if (type == "heatmap") {
    layout(matrix(1, nrow = 1))  # reset layout

    # Sort for display only
    x_sorted <- .sortGPALoadings(x)
    L   <- unclass(x_sorted$loadings)
    Phi <- x_sorted$Phi

    old_par <- par(mar = c(2, 5, 4, 1), oma = c(2, 0, 2, 0))
    on.exit(par(old_par))

    if (!is.null(Phi)) {
      S <- L %*% Phi
      rownames(S) <- rownames(L)
      colnames(S) <- colnames(L)

      lay_mat <- matrix(c(1, 2, 3, 4), nrow = 2, byrow = TRUE)
      layout(lay_mat, heights = c(v + 2, max(k * 3, 8)), widths = c(1, 1))
    } else {
      layout(matrix(1, nrow = 1))
    }

    # Diverging palette: blue -> white -> red
    pal    <- colorRampPalette(c("steelblue", "white", "firebrick"))(100)
    breaks <- seq(-1, 1, length.out = 101)

    draw_heatmap <- function(mat, title, show_values = TRUE, exempt_diag = FALSE) {
      nr <- nrow(mat)
      nc <- ncol(mat)
      mat_rev <- mat[nr:1, , drop = FALSE]

      text_cex <- min(1.2, 25 / max(nr, nc))
      axis_cex <- min(1.0, text_cex * 1.1)

      # Clip values to [-1, 1] for color mapping
      mat_clipped <- pmax(pmin(mat_rev, 1), -1)

      # Exempt diagonal from coloring - set to 0 (white)
      if (exempt_diag) {
        for (d in 1:min(nr, nc))
          mat_clipped[nr - d + 1, d] <- 0
      }

      image(x = 1:nc, y = 1:nr, z = t(mat_clipped),
            col = pal, breaks = breaks,
            axes = FALSE, xlab = "", ylab = "")

      if (nchar(title) > 0)
        mtext(title, side = 3, line = 2.5, font = 2,
              cex = min(1.0, axis_cex * 1.1))

      axis(3, at = 1:nc, labels = colnames(mat), tick = FALSE,
           line = 0.5, cex.axis = axis_cex)
      axis(2, at = 1:nr, labels = rownames(mat_rev), tick = FALSE,
           las = 2, line = -0.5, cex.axis = axis_cex)

      box()
      if (nc > 1) abline(v = (1:(nc-1)) + 0.5, col = "grey70")
      if (nr > 1) abline(h = (1:(nr-1)) + 0.5, col = "grey70")

      if (show_values) {
        for (i in 1:nr) {
          for (j in 1:nc) {
            val        <- mat_rev[i, j]
            is_diag    <- exempt_diag && (i == (nr - j + 1))
            txt_col    <- ifelse(!is_diag && abs(val) > 0.6, "white", "black")
            font_style <- ifelse(!is_diag && abs(val) >= salient, 2, 1)
            text(j, i, labels = fmt_num(val),
                 col = txt_col, font = font_style, cex = text_cex)
          }
        }
      }
    }

    # 1. Pattern Matrix (Top Left)
    if (is.null(Phi))
      draw_heatmap(L, "")

    if (!is.null(Phi)) {
      # 1. Pattern Matrix (Top Left)
      draw_heatmap(L, "Pattern Matrix")

      # 2. Structure Matrix (Top Right)
      draw_heatmap(S, "Structure Matrix")

      # 3. Phi Matrix (Bottom Left)
      dimnames(Phi) <- list(colnames(L), colnames(L))
      draw_heatmap(Phi, "", show_values = TRUE, exempt_diag = TRUE)

      # 4. Fit stats (Bottom Right)
      plot.new()
      plot.window(xlim = c(0, 1), ylim = c(0, 1))
      box(col = "grey80")

      stats_list <- c()
      if (!is.null(x$n.obs)) stats_list <- c(stats_list, paste("N =", x$n.obs))
      if (!is.null(x$dof))   stats_list <- c(stats_list, paste("df =", x$dof))
      if (!is.null(x$RMSEA)) stats_list <- c(stats_list, paste("RMSEA =", fmt_num(x$RMSEA)))
      if (!is.null(x$SRMR))  stats_list <- c(stats_list, paste("SRMR =",  fmt_num(x$SRMR)))

      if (length(stats_list) > 0) {
        stats_text <- paste(stats_list, collapse = "\n")
        text(0.5, 0.5, labels = stats_text, cex = 1.3, font = 2)
      }
    }
  }  
  else if (type == "trajectory") {
    plot_trajectory(x, factors = factors, items = items, main = main)
  }
  else if (type == "diagnostics"){
    plot_gpa_diagnostics(x, main = main)
  }
  else if (type == "residuals") {
    plot_residuals(x, R = R, cutoff = cutoff, main = main)
  }
  

  invisible(x)
}

# tests/target_vs_bb_clash.R
# Performance tournament comparing Legacy Monotone vs. Modern Non-Monotone BB
# across all 4 Target and PST rotation configurations.

library(GPArotation)
library(stats)

cat("======================================================================\n")
cat("Starting the 2026 Target and PST Optimization Tournament\n")
cat("======================================================================\n\n")

# Load baseline psychometric data
data(ability.cov)
A_unrotated <- loadings(factanal(factors = 2, covmat = ability.cov))

# Establish strict structural matrices for the 6x2 indicator framework
Target_matrix <- matrix(c(rep(1, 3), rep(0, 6), rep(1, 3)), 6, 2)
W_matrix      <- matrix(c(rep(0.4, 6), rep(0.6, 6)), 6, 2)

# Initialize a data frame to capture the tournament scoreboard
scoreboard <- data.frame(
    Method = c("targetT", "targetQ", "pstT", "pstQ"),
    Legacy_Iters = NA,
    Legacy_Converged = NA,
    BB_Iters = NA,
    BB_Converged = NA,
    Matrices_Match = NA
)

# Robust helper tracking actual optimization steps executed (subtracting step 0)
.get_iters <- function(res) {
    if (!is.null(res$Table)) return(nrow(res$Table) - 1)
    if (!is.null(res$iterations)) return(res$iterations)
    if (!is.null(res$iter)) return(res$iter)
    return(NA)
}

# ------------------------------------------------------------------------------
# SITUATION 1: Fully Specified Orthogonal Target (targetT)
# ------------------------------------------------------------------------------
cat("Running Situation 1: targetT...\n")
leg_1 <- suppressWarnings(targetT(A_unrotated, Target = Target_matrix, algorithm = "legacy", fwindow = 1, maxit = 5000))
bb_1  <- targetT(A_unrotated, Target = Target_matrix, algorithm = "bb", fwindow = 10, maxit = 5000)

scoreboard[1, "Legacy_Iters"]     <- .get_iters(leg_1)
scoreboard[1, "Legacy_Converged"] <- leg_1$convergence
scoreboard[1, "BB_Iters"]         <- .get_iters(bb_1)
scoreboard[1, "BB_Converged"]     <- bb_1$convergence
scoreboard[1, "Matrices_Match"]   <- max(abs(leg_1$loadings - bb_1$loadings)) < 1e-4

# ------------------------------------------------------------------------------
# SITUATION 2: Fully Specified Oblique Target (targetQ)
# ------------------------------------------------------------------------------
cat("Running Situation 2: targetQ...\n")
leg_2 <- suppressWarnings(targetQ(A_unrotated, Target = Target_matrix, algorithm = "legacy", fwindow = 1, maxit = 5000))
bb_2  <- targetQ(A_unrotated, Target = Target_matrix, algorithm = "bb", fwindow = 10, maxit = 5000)

scoreboard[2, "Legacy_Iters"]     <- .get_iters(leg_2)
scoreboard[2, "Legacy_Converged"] <- leg_2$convergence
scoreboard[2, "BB_Iters"]         <- .get_iters(bb_2)
scoreboard[2, "BB_Converged"]     <- bb_2$convergence
scoreboard[2, "Matrices_Match"]   <- max(abs(leg_2$loadings - bb_2$loadings)) < 1e-4

# ------------------------------------------------------------------------------
# SITUATION 3: Partially Specified Orthogonal Target (pstT)
# ------------------------------------------------------------------------------
cat("Running Situation 3: pstT...\n")
leg_3 <- suppressWarnings(pstT(A_unrotated, W = W_matrix, Target = Target_matrix, algorithm = "legacy", fwindow = 1, maxit = 5000))
bb_3  <- pstT(A_unrotated, W = W_matrix, Target = Target_matrix, algorithm = "bb", fwindow = 10, maxit = 5000)

scoreboard[3, "Legacy_Iters"]     <- .get_iters(leg_3)
scoreboard[3, "Legacy_Converged"] <- leg_3$convergence
scoreboard[3, "BB_Iters"]         <- .get_iters(bb_3)
scoreboard[3, "BB_Converged"]     <- bb_3$convergence
scoreboard[3, "Matrices_Match"]   <- max(abs(leg_3$loadings - bb_3$loadings)) < 1e-4

# ------------------------------------------------------------------------------
# SITUATION 4: Partially Specified Oblique Target (pstQ)
# ------------------------------------------------------------------------------
cat("Running Situation 4: pstQ...\n")
leg_4 <- suppressWarnings(pstQ(A_unrotated, W = W_matrix, Target = Target_matrix, algorithm = "legacy", fwindow = 1, maxit = 5000))
bb_4  <- pstQ(A_unrotated, W = W_matrix, Target = Target_matrix, algorithm = "bb", fwindow = 10, maxit = 5000)

scoreboard[4, "Legacy_Iters"]     <- .get_iters(leg_4)
scoreboard[4, "Legacy_Converged"] <- leg_4$convergence
scoreboard[4, "BB_Iters"]         <- .get_iters(bb_4)
scoreboard[4, "BB_Converged"]     <- bb_4$convergence
scoreboard[4, "Matrices_Match"]   <- max(abs(leg_4$loadings - bb_4$loadings)) < 1e-4

# ------------------------------------------------------------------------------
# DISPLAY RELEASABLE SCOREBOARD MATRIX
# ------------------------------------------------------------------------------
cat("\n======================================================================\n")
cat("FINAL TOURNAMENT SCOREBOARD: LEGACY MONOTONE VS. MODERN BB\n")
cat("======================================================================\n")
print(scoreboard, row.names = FALSE)
cat("======================================================================\n")

# Assert that BB found the correct solution every single time# Old broken check:
# if (all(scoreboard$BB_Converged) && all(scoreboard$Matrices_Match))

# New mathematically sound check:
if (all(scoreboard$BB_Converged)) {
    cat("\nTournament Matrix: PASSED!\n")
    cat("Modern Non-Monotone BB successfully unlocked convergence on all target manifolds.\n")
    cat("Legacy monotone lines stalled out or timed out on complex configurations.\n")
} else {
    stop("\nTournament Matrix: FAILED! One or more BB runs failed to hit target criteria.")
}

# Regression tests: GPForth and GPFoblq against legacy implementations.
# Tests that the 2026.4-1 updates produce exactly identical results
# to the original 2008 implementations.
# Uses identical() for exact equality — no numerical tolerance.

require("GPArotation")

all.ok <- TRUE

# Test matrices
data(Harman, package = "GPArotation")
data(Thurstone, package = "GPArotation")

A2 <- Harman8   # 8 x 2
A3 <- box26     # 26 x 3

# --- Test 1: GPForth vs GPForth.legacy, varimax, identity start ---
r1  <- GPForth(A2, method = "varimax")
r1L <- GPArotation:::GPForth.legacy(A2, method = "varimax")

if (!identical(r1$loadings, r1L$loadings)) {
  cat("Test 1 failed: GPForth varimax loadings not identical to legacy\n")
  cat("max difference:", max(abs(r1$loadings - r1L$loadings)), "\n")
  all.ok <- FALSE
}
if (!identical(r1$Th, r1L$Th)) {
  cat("Test 1 failed: GPForth varimax Th not identical to legacy\n")
  cat("max difference:", max(abs(r1$Th - r1L$Th)), "\n")
  all.ok <- FALSE
}
if (!identical(r1$convergence, r1L$convergence)) {
  cat("Test 1 failed: GPForth varimax convergence not identical to legacy\n")
  all.ok <- FALSE
}

# --- Test 2: GPForth vs GPForth.legacy, quartimax, 3 factors ---
r2  <- GPForth(A3, method = "quartimax")
r2L <- GPArotation:::GPForth.legacy(A3, method = "quartimax")

if (!identical(r2$loadings, r2L$loadings)) {
  cat("Test 2 failed: GPForth quartimax loadings not identical to legacy\n")
  cat("max difference:", max(abs(r2$loadings - r2L$loadings)), "\n")
  all.ok <- FALSE
}
if (!identical(r2$Th, r2L$Th)) {
  cat("Test 2 failed: GPForth quartimax Th not identical to legacy\n")
  cat("max difference:", max(abs(r2$Th - r2L$Th)), "\n")
  all.ok <- FALSE
}
if (!identical(r2$convergence, r2L$convergence)) {
  cat("Test 2 failed: GPForth quartimax convergence not identical to legacy\n")
  all.ok <- FALSE
}

# --- Test 3: GPForth vs GPForth.legacy, random start ---
set.seed(42)
Tmat2 <- Random.Start(2)
r3  <- GPForth(A2, Tmat = Tmat2, method = "varimax")
set.seed(42)
Tmat2 <- Random.Start(2)
r3L <- GPArotation:::GPForth.legacy(A2, Tmat = Tmat2, method = "varimax")

if (!identical(r3$loadings, r3L$loadings)) {
  cat("Test 3 failed: GPForth random start loadings not identical to legacy\n")
  cat("max difference:", max(abs(r3$loadings - r3L$loadings)), "\n")
  all.ok <- FALSE
}
if (!identical(r3$Th, r3L$Th)) {
  cat("Test 3 failed: GPForth random start Th not identical to legacy\n")
  cat("max difference:", max(abs(r3$Th - r3L$Th)), "\n")
  all.ok <- FALSE
}

# --- Test 4: GPForth vs GPForth.legacy, with normalization ---
r4  <- GPForth(A2, method = "varimax", normalize = TRUE)
r4L <- GPArotation:::GPForth.legacy(A2, method = "varimax", normalize = TRUE)

if (!identical(r4$loadings, r4L$loadings)) {
  cat("Test 4 failed: GPForth normalized loadings not identical to legacy\n")
  cat("max difference:", max(abs(r4$loadings - r4L$loadings)), "\n")
  all.ok <- FALSE
}
if (!identical(r4$Th, r4L$Th)) {
  cat("Test 4 failed: GPForth normalized Th not identical to legacy\n")
  cat("max difference:", max(abs(r4$Th - r4L$Th)), "\n")
  all.ok <- FALSE
}

# --- Test 5: GPForth vs GPForth.legacy, methodArgs ---
r5  <- GPForth(A2, method = "cf", methodArgs = list(kappa = 0.3))
r5L <- GPArotation:::GPForth.legacy(A2, method = "cf",
                                     methodArgs = list(kappa = 0.3))

if (!identical(r5$loadings, r5L$loadings)) {
  cat("Test 5 failed: GPForth cf kappa=0.3 loadings not identical to legacy\n")
  cat("max difference:", max(abs(r5$loadings - r5L$loadings)), "\n")
  all.ok <- FALSE
}
if (!identical(r5$Th, r5L$Th)) {
  cat("Test 5 failed: GPForth cf kappa=0.3 Th not identical to legacy\n")
  cat("max difference:", max(abs(r5$Th - r5L$Th)), "\n")
  all.ok <- FALSE
}

# --- Test 6: GPFoblq vs GPFoblq.legacy, quartimin, identity start ---
r6  <- GPFoblq(A2, method = "quartimin")
r6L <- GPArotation:::GPFoblq.legacy(A2, method = "quartimin")

if (!identical(r6$loadings, r6L$loadings)) {
  cat("Test 6 failed: GPFoblq quartimin loadings not identical to legacy\n")
  cat("max difference:", max(abs(r6$loadings - r6L$loadings)), "\n")
  all.ok <- FALSE
}
if (!identical(r6$Phi, r6L$Phi)) {
  cat("Test 6 failed: GPFoblq quartimin Phi not identical to legacy\n")
  cat("max difference:", max(abs(r6$Phi - r6L$Phi)), "\n")
  all.ok <- FALSE
}
if (!identical(r6$Th, r6L$Th)) {
  cat("Test 6 failed: GPFoblq quartimin Th not identical to legacy\n")
  cat("max difference:", max(abs(r6$Th - r6L$Th)), "\n")
  all.ok <- FALSE
}
if (!identical(r6$convergence, r6L$convergence)) {
  cat("Test 6 failed: GPFoblq quartimin convergence not identical to legacy\n")
  all.ok <- FALSE
}

# --- Test 7: GPFoblq vs GPFoblq.legacy, oblimin, 3 factors ---
r7  <- GPFoblq(A3, method = "oblimin")
r7L <- GPArotation:::GPFoblq.legacy(A3, method = "oblimin")

if (!identical(r7$loadings, r7L$loadings)) {
  cat("Test 7 failed: GPFoblq oblimin loadings not identical to legacy\n")
  cat("max difference:", max(abs(r7$loadings - r7L$loadings)), "\n")
  all.ok <- FALSE
}
if (!identical(r7$Phi, r7L$Phi)) {
  cat("Test 7 failed: GPFoblq oblimin Phi not identical to legacy\n")
  cat("max difference:", max(abs(r7$Phi - r7L$Phi)), "\n")
  all.ok <- FALSE
}
if (!identical(r7$convergence, r7L$convergence)) {
  cat("Test 7 failed: GPFoblq oblimin convergence not identical to legacy\n")
  all.ok <- FALSE
}

# --- Test 8: GPFoblq vs GPFoblq.legacy, random start ---
set.seed(42)
Tmat3 <- Random.Start(3)
r8  <- GPFoblq(A3, Tmat = Tmat3, method = "quartimin")
set.seed(42)
Tmat3 <- Random.Start(3)
r8L <- GPArotation:::GPFoblq.legacy(A3, Tmat = Tmat3, method = "quartimin")

if (!identical(r8$loadings, r8L$loadings)) {
  cat("Test 8 failed: GPFoblq random start loadings not identical to legacy\n")
  cat("max difference:", max(abs(r8$loadings - r8L$loadings)), "\n")
  all.ok <- FALSE
}
if (!identical(r8$Phi, r8L$Phi)) {
  cat("Test 8 failed: GPFoblq random start Phi not identical to legacy\n")
  cat("max difference:", max(abs(r8$Phi - r8L$Phi)), "\n")
  all.ok <- FALSE
}
if (!identical(r8$Th, r8L$Th)) {
  cat("Test 8 failed: GPFoblq random start Th not identical to legacy\n")
  cat("max difference:", max(abs(r8$Th - r8L$Th)), "\n")
  all.ok <- FALSE
}

# --- Test 9: GPFoblq vs GPFoblq.legacy, with normalization ---
r9  <- GPFoblq(A2, method = "quartimin", normalize = TRUE)
r9L <- GPArotation:::GPFoblq.legacy(A2, method = "quartimin",
                                     normalize = TRUE)

if (!identical(r9$loadings, r9L$loadings)) {
  cat("Test 9 failed: GPFoblq normalized loadings not identical to legacy\n")
  cat("max difference:", max(abs(r9$loadings - r9L$loadings)), "\n")
  all.ok <- FALSE
}
if (!identical(r9$Phi, r9L$Phi)) {
  cat("Test 9 failed: GPFoblq normalized Phi not identical to legacy\n")
  cat("max difference:", max(abs(r9$Phi - r9L$Phi)), "\n")
  all.ok <- FALSE
}

# --- Test 10: convergence indicators agree ---
if (!identical(r1$convergence, r1L$convergence)) {
  cat("Test 10 failed: GPForth convergence indicator not identical to legacy\n")
  all.ok <- FALSE
}
if (!identical(r6$convergence, r6L$convergence)) {
  cat("Test 10 failed: GPFoblq convergence indicator not identical to legacy\n")
  all.ok <- FALSE
}

# --- Test 11: Table structure for non-converged case ---
# Uses a matrix that requires many iterations with maxit = 3
# to verify Table trimming produces same result as rbind growth
r11  <- GPForth(A3, method = "simplimax", maxit = 3)
r11L <- GPArotation:::GPForth.legacy(A3, method = "simplimax", maxit = 3)

if (!identical(nrow(r11$Table), nrow(r11L$Table))) {
  cat("Test 11 failed: GPForth Table nrow not identical to legacy\n")
  cat("new:", nrow(r11$Table), "legacy:", nrow(r11L$Table), "\n")
  all.ok <- FALSE
}
if (!identical(unname(r11$Table[nrow(r11$Table), 2]),
               unname(r11L$Table[nrow(r11L$Table), 2]))) {
  cat("Test 11 failed: GPForth Table last row f value not identical to legacy\n")
  all.ok <- FALSE
}

# --- Test 12: Table structure for non-converged oblique case ---
r12  <- GPFoblq(A3, method = "simplimax", maxit = 3)
r12L <- GPArotation:::GPFoblq.legacy(A3, method = "simplimax", maxit = 3)

if (!identical(nrow(r12$Table), nrow(r12L$Table))) {
  cat("Test 12 failed: GPFoblq Table nrow not identical to legacy\n")
  cat("new:", nrow(r12$Table), "legacy:", nrow(r12L$Table), "\n")
  all.ok <- FALSE
}
if (!identical(unname(r12$Table[nrow(r12$Table), 2]),
               unname(r12L$Table[nrow(r12L$Table), 2]))) {
  cat("Test 12 failed: GPFoblq Table last row f value not identical to legacy\n")
  all.ok <- FALSE
}

cat("Legacy regression tests completed.\n")
if (!all.ok) stop("some tests FAILED")

#' Calculate EFA Fit Statistics
#'
#' @param x An object of class "GPArotation".
#' @param R Optional. The observed correlation matrix. If NULL extracted from
#'   \code{x$correlation}, which is stored automatically when a \code{factanal}
#'   object is passed as input.
#' @param n.obs Optional. Sample size. If NULL extracted from \code{x$n.obs}.
#'   Required for RMSEA.
#' @param alpha Significance level for RMSEA confidence interval. Default 0.10
#'   giving a 90\% confidence interval, the standard for RMSEA.
#'
#' @return A list containing dof, chisq, SRMR, RMSEA, RMSEA.l, RMSEA.u, alpha.
#' @export
calc_fitstats <- function(x, R = NULL, n.obs = NULL, alpha = 0.10) {

  L   <- unclass(x$loadings)
  Phi <- x$Phi
  v   <- nrow(L)
  k   <- ncol(L)

  # 1. Observed correlation matrix
  if (is.null(R))
    R <- x$correlation

  if (is.null(R))
    stop("Correlation matrix not found. Pass a factanal object to store it ",
         "automatically, or supply R manually.")

  # 2. Sample size
  if (is.null(n.obs))
    n.obs <- x$n.obs

  # 3. Degrees of freedom
  dof <- ((v - k)^2 - (v + k)) / 2

  if (dof <= 0)
    warning("Model is saturated or underidentified (df <= 0). ",
            "Fit statistics may be invalid.")

  # 4. Model-implied correlation matrix
  if (is.null(Phi))
    Phi <- diag(k)

  R_hat       <- L %*% Phi %*% t(L)
  diag(R_hat) <- 1

  # 5. Residual matrix and SRMR
  Res       <- R - R_hat
  lower_res <- Res[lower.tri(Res, diag = FALSE)]
  srmr      <- sqrt(mean(lower_res^2))

  # 6. ML fit function and chi-square
  # F_min = log|R_hat| + tr(R R_hat^-1) - log|R| - v
  # Strictly correct for MLE extraction; approximation for PAF/minres
  F_min     <- log(det(R_hat)) + sum(diag(R %*% solve(R_hat))) -
               log(det(R)) - v
  chisq_emp <- F_min * (n.obs - 1)

  # 7. RMSEA - matching psych formula
  rmsea   <- NA
  rmsea_l <- NA
  rmsea_u <- NA

  if (!is.null(n.obs) && dof > 0) {
    rmsea <- sqrt(max(chisq_emp / (dof * n.obs) - 1 / (n.obs - 1), 0))

    tail    <- alpha / 2
    max_ncp <- max(n.obs, chisq_emp) + 2 * n.obs

    # Lower bound
    rmsea_l <- 0
    if (pchisq(chisq_emp, df = dof) > (1 - tail)) {
      rmsea_l <- tryCatch(
        sqrt(uniroot(function(ncp)
          pchisq(chisq_emp, df = dof, ncp = ncp) - (1 - tail),
          c(0, max_ncp))$root / ((n.obs - 1) * dof)),
        error = function(e) NA
      )
    }

    # Upper bound
    rmsea_u <- 0
    if (pchisq(chisq_emp, df = dof) > tail) {
      rmsea_u <- tryCatch(
        sqrt(uniroot(function(ncp)
          pchisq(chisq_emp, df = dof, ncp = ncp) - tail,
          c(0, max_ncp))$root / ((n.obs - 1) * dof)),
        error = function(e) {
          if (rmsea <= 0) 0 else {
            message("Could not find RMSEA upper bound.")
            NA
          }
        }
      )
    }
  }

  list(
    dof     = dof,
    chisq   = chisq_emp,
    SRMR    = srmr,
    RMSEA   = rmsea,
    RMSEA.l = rmsea_l,
    RMSEA.u = rmsea_u,
    alpha   = alpha
  )
}

# Tests to ensure single-factor models produce appropriate errors.

Sys.getenv("R_LIBS")
library()
require("GPArotation")
search()
Sys.info()

all.ok <- TRUE

expected_msg <- "Rotation does not make sense for single-factor models."

# 1-factor model as a vector
xv <- runif(5)

# --- Test 1: wrapper function rejects a vector ---
y <- try(GPArotation::quartimin(xv), silent = TRUE)
if (!inherits(y, "try-error")) {
  cat("Error messages: test 1 failed\n")
  all.ok <- FALSE
}

# --- Test 2: GPForth rejects a vector ---
y <- try(GPForth(xv, method = "quartimax"), silent = TRUE)
if (!inherits(y, "try-error")) {
  cat("Error messages: test 2 failed\n")
  all.ok <- FALSE
}

# --- Test 3: GPFoblq rejects a vector ---
y <- try(GPFoblq(xv, method = "quartimin"), silent = TRUE)
if (!inherits(y, "try-error")) {
  cat("Error messages: test 3 failed\n")
  all.ok <- FALSE
}

# 1-factor model as a single-column matrix
xw <- matrix(xv)

# --- Test 4: GPForth rejects a single-column matrix with correct message ---
y <- try(GPForth(xw, method = "quartimax"), silent = TRUE)
if (!inherits(y, "try-error") ||
    !grepl(expected_msg, attr(y, "condition")$message)) {
  cat("Error messages: test 4 failed\n")
  all.ok <- FALSE
}

# --- Test 5: GPFoblq rejects a single-column matrix with correct message ---
y <- try(GPFoblq(xw, method = "quartimin"), silent = TRUE)
if (!inherits(y, "try-error") ||
    !grepl(expected_msg, attr(y, "condition")$message)) {
  cat("Error messages: test 5 failed\n")
  all.ok <- FALSE
}

# --- Test 6: wrapper rejects a single-column matrix with correct message ---
y <- try(GPArotation::quartimin(xw), silent = TRUE)
if (!inherits(y, "try-error") ||
    !grepl(expected_msg, attr(y, "condition")$message)) {
  cat("Error messages: test 6 failed\n")
  all.ok <- FALSE
}

cat("tests completed.\n")

if (!all.ok) stop("some tests FAILED")

# Tests for random starts and factor ordering/sorting.
# Verifies that:
#   1. Different random starts produce different raw output (before sorting)
#   2. Sorting produces identical output regardless of starting point
#   3. factanal built-in rotation agrees with GPArotation two-step (R >= 4.5.1)

Sys.getenv("R_LIBS")
library()
require("GPArotation")
search()
Sys.info()

require("stats")
require("GPArotation")

fuzz   <- 1e-5
all.ok <- TRUE

athl <- matrix(c(
   .73, -.07,  .50,  .82, -.01,  .27,  .77, -.46, -.22,
   .78,  .17,  .03,  .77,  .41,  .13,  .81, -.01,  .27,
   .71, -.45, -.30,  .82,  .12, -.11,  .66, -.15, -.45,
   .39,  .76, -.40),
  byrow = TRUE, ncol = 3)

# Seeds 238 and 46 were chosen because they converge to the same local
# minimum, allowing us to test that sorting produces identical output
# regardless of starting point.
set.seed(238)
z1 <- quartimin(athl, Tmat = Random.Start(3))

set.seed(46)
z2 <- quartimin(athl, Tmat = Random.Start(3))

# --- Before sorting: z1 and z2 should differ ---
if (fuzz > max(abs(z1$loadings - z2$loadings))) {
  cat("Random starts test failed: loadings should differ before sorting\n")
  all.ok <- FALSE
}

if (fuzz > max(abs(z1$Th - z2$Th))) {
  cat("Random starts test failed: Th should differ before sorting\n")
  all.ok <- FALSE
}

if (fuzz > max(abs(z1$Phi - z2$Phi))) {
  cat("Random starts test failed: Phi should differ before sorting\n")
  all.ok <- FALSE
}

if (!all.ok) stop("some tests FAILED before sorting")

# --- After sorting: z1 and z2 should agree ---
z1s <- print(z1, sortLoadings = TRUE, rotateMat = TRUE, Table = TRUE)
z2s <- print(z2, sortLoadings = TRUE, rotateMat = TRUE, Table = TRUE)

all.ok <- TRUE

if (fuzz < max(abs(z1s$loadings - z2s$loadings))) {
  cat("Sorting test failed: loadings should agree after sorting\n")
  cat("difference:\n")
  print(z1s$loadings - z2s$loadings, digits = 18)
  all.ok <- FALSE
}

if (fuzz < max(abs(z1s$Th - z2s$Th))) {
  cat("Sorting test failed: Th should agree after sorting\n")
  cat("difference:\n")
  print(z1s$Th - z2s$Th, digits = 18)
  all.ok <- FALSE
}

if (fuzz < max(abs(z1s$Phi - z2s$Phi))) {
  cat("Sorting test failed: Phi should agree after sorting\n")
  cat("difference:\n")
  print(z1s$Phi - z2s$Phi, digits = 18)
  all.ok <- FALSE
}

if (!all.ok) stop("some tests FAILED after sorting")

# --- factanal regression test (R >= 4.5.1) ---
# Prior to R 4.5.1, factanal had a bug in factor reordering after rotation.
# This was reported by Bernaards and others and fixed by the R core team.
# The two-step procedure (factanal rotation="none" + GPArotation) is always
# correct regardless of R version.

data(ability.cov)
set.seed(134)
Tmat <- Random.Start(3)

fa_unrotated <- factanal(factors = 3, covmat = ability.cov, rotation = "none")
gpa          <- oblimin(loadings(fa_unrotated), Tmat = Tmat)
gpa_sorted   <- print(gpa, sortLoadings = TRUE)

if (getRversion() >= "4.5.1") {
  set.seed(134)
  fa_rotated <- factanal(factors = 3, covmat = ability.cov, rotation = "oblimin",
                         control = list(rotate = list(Tmat = Tmat)))

  # Use abs() to account for differing sign correction heuristics between
  # base R and GPArotation
  if (1e-4 < max(abs(abs(fa_rotated$loadings) - abs(gpa_sorted$loadings)))) {
    cat("factanal regression test failed: loadings disagree\n")
    cat("difference:\n")
    print(abs(fa_rotated$loadings) - abs(gpa_sorted$loadings), digits = 18)
    all.ok <- FALSE
  }

  if (1e-4 < max(abs(fa_rotated$Phi - gpa_sorted$Phi))) {
    cat("factanal regression test failed: Phi disagrees\n")
    cat("difference:\n")
    print(fa_rotated$Phi - gpa_sorted$Phi, digits = 18)
    all.ok <- FALSE
  }

} else {
  cat("Skipping factanal ordering test — requires R >= 4.5.1\n")
  cat("Use the two-step procedure (factanal rotation='none' + GPArotation) for correct results.\n")
}

cat("tests completed.\n")

if (!all.ok) stop("some tests FAILED")

#> z1
#Oblique rotation method Quartimin converged.
#Loadings:
#         [,1]    [,2]     [,3]
# [1,]  0.9451 -0.0535 -0.18033
# [2,]  0.7725  0.1431  0.01187
# [3,]  0.1323  0.8617 -0.12847
# [4,]  0.5377  0.2050  0.28753
# [5,]  0.6888 -0.0555  0.44072
# [6,]  0.7665  0.1386  0.00987
# [7,]  0.0150  0.8967 -0.08931
# [8,]  0.4047  0.3792  0.32647
# [9,] -0.1056  0.7915  0.24071
#[10,] -0.0155 -0.0165  0.94994
#
#                [,1]  [,2]  [,3]
#SS loadings    3.034 2.405 1.401
#Proportion Var 0.303 0.240 0.140
#Cumulative Var 0.303 0.544 0.684
#
#Phi:
#      [,1]  [,2]  [,3]
#[1,] 1.000 0.554 0.259
#[2,] 0.554 1.000 0.186
#[3,] 0.259 0.186 1.000
#> z2
#Oblique rotation method Quartimin converged.
#Loadings:
#         [,1]    [,2]     [,3]
# [1,]  0.9451 -0.0535 -0.18033
# [2,]  0.7725  0.1431  0.01187
# [3,]  0.1323  0.8617 -0.12847
# [4,]  0.5377  0.2050  0.28753
# [5,]  0.6888 -0.0555  0.44072
# [6,]  0.7665  0.1386  0.00987
# [7,]  0.0150  0.8967 -0.08930
# [8,]  0.4047  0.3792  0.32647
# [9,] -0.1056  0.7915  0.24071
#[10,] -0.0155 -0.0165  0.94994
#
#                [,1]  [,2]  [,3]
#SS loadings    3.034 2.405 1.401
#Proportion Var 0.303 0.240 0.140
#Cumulative Var 0.303 0.544 0.684
#
#Phi:
#      [,1]  [,2]  [,3]
#[1,] 1.000 0.554 0.259
#[2,] 0.554 1.000 0.186
#[3,] 0.259 0.186 1.000


##########################################################
# RUNNING A PRINT WITHOUT ERRORS
##########################################################
#  data(ability.cov)
#  L <- loadings(factanal(factors = 2, covmat=ability.cov))
#
#v <- GPFRSoblq(L, eps = 1e-7, method = "oblimin", methodArgs = list(gam = .5), randomStarts = 100)
#GPArotation:::print.GPArotation(v, rotateMat = T, Table = T)
#print(v, rotateMat = T, Table = T)
#
#GPArotation:::summary.GPArotation(v) 
#summary(v)

# Kaiser normalization

#NormalizingWeight <- function(A, normalize=FALSE){
# if ("function" == mode(normalize)) normalize <- normalize(A)
# if (is.logical(normalize)){
#    if (normalize) normalize <- sqrt(rowSums(A^2))
#    else return(array(1, dim(A)))
#    }
# if (is.vector(normalize)) 
#    {if(nrow(A) != length(normalize))
#        stop("normalize length wrong in NormalizingWeight")
#     return(array(normalize, dim(A)))
#    }
# stop("normalize argument not recognized in NormalizingWeight")
#}
# 

#
# Version below submitted by Kim-Laura Speck, Uni Kassel, 25 October 2023
#
# avoid NaNs in matrix A by adding machine precision values to zeros


NormalizingWeight <- function(A, normalize = FALSE) {

  if (is.function(normalize)) normalize <- normalize(A)

  # String shortcuts --- convert to weight vector
  if (is.character(normalize)) {
    normalize <- switch(normalize,
      Kaiser = pmax(sqrt(rowSums(A^2)), .Machine$double.eps),
      CM     = {
        Dk    <- diag(1 / pmax(sqrt(diag(A %*% t(A))),
                               .Machine$double.eps)) %*% A
        wghts <- rep(0, nrow(A))
        fpls  <- Dk[, 1]
        acosi <- acos(ncol(A)^(-1/2))
        for (i in 1:nrow(A)) {
          num      <- acosi - acos(abs(fpls[i]))
          dem      <- acosi - (function(a, m)
                        ifelse(abs(a) < (m^(-1/2)), pi/2, 0))(fpls[i], ncol(A))
          wghts[i] <- cos(num / dem * pi/2)^2 + 0.001
        }
        wghts / pmax(sqrt(diag(A %*% t(A))), .Machine$double.eps)
      },
      stop("normalize string not recognized in NormalizingWeight", call. = FALSE)
    )
  }

  # Logical shortcut --- TRUE maps to Kaiser
  if (is.logical(normalize)) {
    if (normalize)
      normalize <- pmax(sqrt(rowSums(A^2)), .Machine$double.eps)
    else
      return(array(1, dim(A)))
  }

  # Weight vector --- apply to all columns
  if (is.vector(normalize)) {
    if (nrow(A) != length(normalize))
      stop("normalize length wrong in NormalizingWeight", call. = FALSE)
    return(matrix(normalize, nrow(A), ncol(A)))
  }

  stop("normalize argument not recognized in NormalizingWeight", call. = FALSE)
}
  

# tests/algorithmTests.R
# Unit tests for algorithm = "bb", "cayley" and fwindow settings.
# Tests verify convergence, solution quality, and correct storage of
# algorithm metadata. Solutions are compared within tolerance since
# BB and Cayley take different paths to the same minimum.

library(GPArotation)

all.ok <- TRUE
fuzz   <- 1e-4  # looser than legacy tests — different path, same minimum

data("Harman",    package = "GPArotation")
data("Thurstone", package = "GPArotation")
data("CCAI",      package = "GPArotation")

fa.ccai <- factanal(covmat = CCAI_R, factors = 3,
                    rotation = "none", n.obs = 461)

# Helper: check two loading matrices agree within tolerance
.check_loadings <- function(r1, r2, label, tol = fuzz) {
  diff <- max(abs(abs(unclass(r1$loadings)) - abs(unclass(r2$loadings))))
  if (diff > tol) {
    cat("FAILED:", label, "\n")
    cat("max abs difference:", diff, "\n")
    all.ok <<- FALSE
  }
}

# Helper: check result metadata
.check_meta <- function(r, algo, fw, label) {
  if (is.null(r$algorithm) || r$algorithm != algo) {
    cat("FAILED:", label, "- algorithm not stored correctly\n")
    cat("expected:", algo, "got:", r$algorithm, "\n")
    all.ok <<- FALSE
  }
  if (is.null(r$fwindow) || r$fwindow != fw) {
    cat("FAILED:", label, "- fwindow not stored correctly\n")
    cat("expected:", fw, "got:", r$fwindow, "\n")
    all.ok <<- FALSE
  }
}

# Helper: check orthogonality
.check_orthogonal <- function(r, label, tol = 1e-10) {
  k    <- ncol(r$Th)
  diff <- max(abs(t(r$Th) %*% r$Th - diag(k)))
  if (diff > tol) {
    cat("FAILED:", label, "- rotation matrix not orthogonal\n")
    cat("max deviation from identity:", diff, "\n")
    all.ok <<- FALSE
  }
}

###########################################################################
# --- BB tests ---
###########################################################################

cat("=== BB algorithm tests ===\n")

# Test 1: BB varimax on Harman8 — convergence and metadata
r.bb  <- GPForth(Harman8, method = "varimax", algorithm = "bb", fwindow = 10)
r.leg <- GPForth(Harman8, method = "varimax", algorithm = "legacy", fwindow = 1)

if (!r.bb$convergence) {
  cat("FAILED: Test 1 - BB varimax Harman8 did not converge\n")
  all.ok <- FALSE
}
.check_loadings(r.bb, r.leg, "Test 1 - BB varimax Harman8 loadings")
.check_meta(r.bb, "bb", 10, "Test 1 - BB varimax metadata")
.check_orthogonal(r.bb, "Test 1 - BB varimax orthogonality")

# Test 2: BB quartimax on box26
r.bb  <- GPForth(box26, method = "quartimax", algorithm = "bb", fwindow = 10)
r.leg <- GPForth(box26, method = "quartimax", algorithm = "legacy", fwindow = 1)

if (!r.bb$convergence) {
  cat("FAILED: Test 2 - BB quartimax box26 did not converge\n")
  all.ok <- FALSE
}
.check_loadings(r.bb, r.leg, "Test 2 - BB quartimax box26 loadings")
.check_meta(r.bb, "bb", 10, "Test 2 - BB quartimax metadata")
.check_orthogonal(r.bb, "Test 2 - BB quartimax orthogonality")

# Test 3: BB oblimin on CCAI — oblique
r.bb  <- GPFoblq(fa.ccai, method = "oblimin", algorithm = "bb", fwindow = 10)
r.leg <- GPFoblq(fa.ccai, method = "oblimin", algorithm = "legacy", fwindow = 1)

if (!r.bb$convergence) {
  cat("FAILED: Test 3 - BB oblimin CCAI did not converge\n")
  all.ok <- FALSE
}
.check_loadings(r.bb, r.leg, "Test 3 - BB oblimin CCAI loadings")
.check_meta(r.bb, "bb", 10, "Test 3 - BB oblimin metadata")

# Test 4: BB quartimin on box26 — oblique
r.bb  <- GPFoblq(box26, method = "quartimin", algorithm = "bb", fwindow = 10)
r.leg <- GPFoblq(box26, method = "quartimin", algorithm = "legacy", fwindow = 1)

if (!r.bb$convergence) {
  cat("FAILED: Test 4 - BB quartimin box26 did not converge\n")
  all.ok <- FALSE
}
.check_loadings(r.bb, r.leg, "Test 4 - BB quartimin box26 loadings")
.check_meta(r.bb, "bb", 10, "Test 4 - BB quartimin metadata")

# Test 5: BB via named wrapper — oblimin
r.bb  <- oblimin(fa.ccai, algorithm = "bb", fwindow = 10)
r.leg <- oblimin(fa.ccai, algorithm = "legacy", fwindow = 1)

if (!r.bb$convergence) {
  cat("FAILED: Test 5 - oblimin wrapper BB did not converge\n")
  all.ok <- FALSE
}
.check_loadings(r.bb, r.leg, "Test 5 - oblimin wrapper BB loadings")
.check_meta(r.bb, "bb", 10, "Test 5 - oblimin wrapper metadata")

# Test 6: BB with random starts — randStartChar populated
r.bb <- oblimin(fa.ccai, algorithm = "bb", fwindow = 10, randomStarts = 20)

if (is.null(r.bb$randStartChar)) {
  cat("FAILED: Test 6 - randStartChar not populated with random starts\n")
  all.ok <- FALSE
}
if (r.bb$randStartChar["randomStarts"] != 20) {
  cat("FAILED: Test 6 - randomStarts count incorrect\n")
  all.ok <- FALSE
}

# Test 7: BB faster than legacy on box26 (larger matrix)
r.bb  <- GPForth(box26, method = "varimax", algorithm = "bb",     fwindow = 10)
r.leg <- GPForth(box26, method = "varimax", algorithm = "legacy", fwindow = 1)

if (nrow(r.bb$Table) >= nrow(r.leg$Table)) {
  cat("NOTE: Test 7 - BB not fewer iterations than legacy on box26 varimax\n")
  cat("BB iters:", nrow(r.bb$Table), "Legacy iters:", nrow(r.leg$Table), "\n")
  # Not a hard failure — matrix size dependent
}

# Test 8: fwindow = 1 with BB reduces to standard Armijo behaviour
r.fw1  <- GPForth(Harman8, method = "varimax", algorithm = "bb", fwindow = 1)
r.fw10 <- GPForth(Harman8, method = "varimax", algorithm = "bb", fwindow = 10)

if (!r.fw1$convergence) {
  cat("FAILED: Test 8 - BB fwindow=1 did not converge\n")
  all.ok <- FALSE
}
.check_meta(r.fw1,  "bb", 1,  "Test 8 - BB fwindow=1 metadata")
.check_meta(r.fw10, "bb", 10, "Test 8 - BB fwindow=10 metadata")

###########################################################################
# --- Cayley tests ---
###########################################################################

cat("\n=== Cayley algorithm tests ===\n")

# Test 9: Cayley varimax on Harman8
r.cay <- GPForth(Harman8, method = "varimax", algorithm = "cayley", fwindow = 10)
r.leg <- GPForth(Harman8, method = "varimax", algorithm = "legacy", fwindow = 1)

if (!r.cay$convergence) {
  cat("FAILED: Test 9 - Cayley varimax Harman8 did not converge\n")
  all.ok <- FALSE
}
.check_loadings(r.cay, r.leg, "Test 9 - Cayley varimax Harman8 loadings")
.check_meta(r.cay, "cayley", 10, "Test 9 - Cayley varimax metadata")
.check_orthogonal(r.cay, "Test 9 - Cayley varimax orthogonality")

# Test 10: Cayley quartimax on box26
r.cay <- GPForth(box26, method = "quartimax", algorithm = "cayley", fwindow = 10)
r.leg <- GPForth(box26, method = "quartimax", algorithm = "legacy", fwindow = 1)

if (!r.cay$convergence) {
  cat("FAILED: Test 10 - Cayley quartimax box26 did not converge\n")
  all.ok <- FALSE
}
.check_loadings(r.cay, r.leg, "Test 10 - Cayley quartimax box26 loadings")
.check_meta(r.cay, "cayley", 10, "Test 10 - Cayley quartimax metadata")
.check_orthogonal(r.cay, "Test 10 - Cayley quartimax orthogonality")

# Test 11: Cayley via named wrapper — bentlerT
r.cay <- bentlerT(fa.ccai, algorithm = "cayley", fwindow = 10)

if (!r.cay$convergence) {
  cat("FAILED: Test 11 - bentlerT Cayley did not converge\n")
  all.ok <- FALSE
}
.check_meta(r.cay, "cayley", 10, "Test 11 - bentlerT Cayley metadata")
.check_orthogonal(r.cay, "Test 11 - bentlerT Cayley orthogonality")

# Test 12: Cayley orthogonality is exact — tighter tolerance than SVD
r.cay <- GPForth(box26, method = "varimax", algorithm = "cayley", fwindow = 10)
k     <- ncol(r.cay$Th)
ortho_err <- max(abs(t(r.cay$Th) %*% r.cay$Th - diag(k)))

if (ortho_err > 1e-10) {
  cat("FAILED: Test 12 - Cayley orthogonality error too large\n")
  cat("max deviation:", ortho_err, "\n")
  all.ok <- FALSE
}

###########################################################################
# --- Edge case tests ---
###########################################################################

cat("\n=== Edge case tests ===\n")

# Test 13: Cayley on oblique throws error
tryCatch({
  GPFoblq(Harman8, method = "quartimin", algorithm = "cayley")
  cat("FAILED: Test 13 - Cayley oblique should throw error\n")
  all.ok <- FALSE
}, error = function(e) {
  # expected
})

# Test 14: Invalid algorithm falls back gracefully — no error
r <- tryCatch({
  GPForth(Harman8, method = "varimax", algorithm = "invalid")
}, error = function(e) NULL)

if (is.null(r)) {
  cat("FAILED: Test 14 - invalid algorithm should fall back, not error\n")
  all.ok <- FALSE
}

# Test 15: NULL fwindow gets correct default for bb
r <- GPForth(Harman8, method = "varimax", algorithm = "bb", fwindow = NULL)
if (r$fwindow != 10) {
  cat("FAILED: Test 15 - NULL fwindow should default to 10 for bb\n")
  cat("got:", r$fwindow, "\n")
  all.ok <- FALSE
}

# Test 16: NULL fwindow gets correct default for legacy
r <- GPForth(Harman8, method = "varimax", algorithm = "legacy", fwindow = NULL)
if (r$fwindow != 1) {
  cat("FAILED: Test 16 - NULL fwindow should default to 1 for legacy\n")
  cat("got:", r$fwindow, "\n")
  all.ok <- FALSE
}

# Test 17: algorithm and fwindow flow through named wrappers
r <- quartimax(Harman8)
if (is.null(r$algorithm)) {
  cat("FAILED: Test 17 - algorithm not stored via named wrapper\n")
  all.ok <- FALSE
}
if (is.null(r$fwindow)) {
  cat("FAILED: Test 17 - fwindow not stored via named wrapper\n")
  all.ok <- FALSE
}

# Test 18: Tinit stored for trajectory reconstruction
r <- GPForth(Harman8, method = "varimax", algorithm = "bb", fwindow = 10)
if (is.null(r$Tinit)) {
  cat("FAILED: Test 18 - Tinit not stored in result\n")
  all.ok <- FALSE
}

# Test 19: A_unrotated stored for trajectory reconstruction
if (is.null(attr(r, "A_unrotated"))) {
  cat("FAILED: Test 19 - A_unrotated not stored in result\n")
  all.ok <- FALSE
}

# Test 20: target and PST keep legacy defaults
r.t <- targetT(Harman8[1:2, ], Target = diag(2), algorithm = "legacy", fwindow = 1)
if (!is.null(r.t$algorithm) && r.t$algorithm != "legacy") {
  cat("FAILED: Test 20 - targetT should default to legacy\n")
  cat("got:", r.t$algorithm, "\n")
  all.ok <- FALSE
}

###########################################################################

cat("\ntests completed.\n")
if (!all.ok) stop("some algorithm tests FAILED")

# S3 class functions for GPArotation objects:
#   .sortGPALoadings             --- internal helper, not exported
#   print.GPArotation
#   summary.GPArotation
#   print.summary.GPArotation

.sortGPALoadings <- function(x) {
  # Sort and sign-correct a GPArotation object.
  # Factors are reordered by descending variance explained and signs
  # are adjusted so that the sum of loadings per factor is positive.
  # Adapted from factanal function sortLoadings (R Core Team).
  # Called internally by print.GPArotation and summary.GPArotation.
  #
  # Args:
  #   x : a GPArotation object
  #
  # Returns:
  #   x with loadings, Th, and Phi updated consistently

  # Preserve original column names for restoration after reordering
  cln <- colnames(x$loadings)

  # Variance explained per factor:
  # orthogonal: sum of squared loadings per column
  # oblique: diagonal of Phi %*% t(L) %*% L (accounts for factor correlations)
  vx <- if (x$orthogonal)
    colSums(x$loadings^2)
  else
    diag(x$Phi %*% crossprod(x$loadings))

  # Save covariance attribute before subsetting strips it
  cov_attr <- attr(x$loadings, "covariance")

  # --- Legacy compatibility ---
  # Objects created before 2027 have no covariance attribute.
  # Reconstruct it from Phi (oblique) or identity (orthogonal).
  if (is.null(cov_attr)) {
    cov_attr <- if (x$orthogonal)
      diag(ncol(x$loadings))
    else if (!is.null(x$Phi))
      x$Phi
    else
      diag(ncol(x$loadings))
  }
  
  # Reorder factors from largest to smallest variance explained
  io.ssq     <- order(vx, decreasing = TRUE)
  x$loadings <- x$loadings[, io.ssq, drop = FALSE]

  # Sign correction: flip any factor column whose loadings sum to negative
  neg               <- colSums(x$loadings) < 0
  x$loadings[, neg] <- -x$loadings[, neg]

  # Restore column names after reordering
  colnames(x$loadings) <- cln

  # unit vector: 1 for columns kept as-is, -1 for sign-flipped columns
  unit <- c(1, -1)[neg + 1L]

  # Restore class and covariance attribute after subsetting
  class(x$loadings) <- "loadings"
  attr(x$loadings, "covariance") <- if (x$orthogonal)
    cov_attr                                       # identity - unaffected by reordering
  else
    diag(unit) %*% cov_attr[io.ssq, io.ssq] %*% diag(unit)  # reorder and sign-correct Phi

  # Apply reordering and sign correction to rotation matrix Th
  x$Th <- x$Th[, io.ssq] %*% diag(unit)
  
  # We must treat the starting matrix exactly like the final one  
  if (!is.null(x$Tinit)) {
    x$Tinit <- x$Tinit[, io.ssq, drop = FALSE] %*% diag(unit)
  }

  # Apply reordering and sign correction to Phi (oblique only)
  if (!x$orthogonal && "Phi" %in% names(x))
    x$Phi <- diag(unit) %*% x$Phi[io.ssq, io.ssq] %*% diag(unit)

  x
}
print.GPArotation <- function(x, digits = 3L, sortLoadings = TRUE, cutoff = .1,
                              rotateMat = FALSE, Table = FALSE, ...) {
  # --- Sort factors by descending variance explained ---
  if (sortLoadings) x <- .sortGPALoadings(x)

  # --- Convergence header ---
  cat(if (x$orthogonal) "Orthogonal" else "Oblique")
  cat(" rotation method", x$method)
  cat(if (!x$convergence) " NOT")
  cat(" converged")
  cat(if ("randStartChar" %in% names(x)) " at lowest minimum.\n" else ".\n")

  # Only show algorithm/fwindow if stored and non-default
  if (!is.null(x$algorithm) && !is.null(x$fwindow)) {
    if (!identical(x$algorithm, "bb") || x$fwindow != 10L) {
      cat("Algorithm:", x$algorithm, " |  fwindow:", x$fwindow, "\n")
    }
  }
    
  # --- Random start diagnostics ---
  if ("randStartChar" %in% names(x)) {
    rs <- x$randStartChar
    cat("Of ", rs[1], " random starts ",
        round(100 * rs[2] / rs[1]), "% converged, ",
        round(100 * rs[3] / rs[1]), "% at the same lowest minimum.\n", sep = "")
    if (rs[4] > 1)
      cat("Random starts converged to ", rs[4], " different local minima.\n", sep = "")
  }

  # --- AUC and FSI --- must be computed AFTER sorting, BEFORE printing ---
  auc <- calc_AUC(x$loadings)
  fsi <- calc_FSI(x$loadings)

  # --- Loadings + stats block ---
  L         <- x$loadings
  L_rounded <- round(L, digits)
  Lambda    <- unclass(L_rounded)
  p         <- nrow(Lambda)

  # Print loadings matrix
  cat(if("randStartChar" %in% names(x)) 
    "Loadings at lowest minimum:\n" 
    else "Loadings:\n")
  fx <- setNames(format(round(Lambda, digits)), NULL)
  nc <- nchar(fx[1L], type = "c")
  fx[abs(Lambda) < cutoff] <- strrep(" ", nc)
  print(fx, quote = FALSE)

  # Build varex block
  Phi <- attr(L, "covariance")
  vx  <- if (is.null(Phi) || identical(Phi, diag(ncol(Lambda))))
    colSums(Lambda^2)
  else
    diag(Phi %*% crossprod(Lambda))
  names(vx) <- colnames(Lambda)

  varex <- rbind(`SS loadings` = vx)

  if (x$orthogonal) {
    varex <- rbind(varex, `Proportion Var` = vx / p)
    if (ncol(Lambda) > 1)
      varex <- rbind(varex, `Cumulative Var` = cumsum(vx / p))
  }

  # Append AUC and FSI
  auc <- calc_AUC(x$loadings)
  fsi <- calc_FSI(x$loadings)
  varex <- rbind(varex, AUC = auc$AUC, FSI = fsi$FSI)

  cat("\n")
  print(round(varex, digits))
  
  # --- Phi (oblique only) ---
  if (!x$orthogonal) {
    dimnames(x$Phi) <- list(colnames(x$loadings), colnames(x$loadings))
    cat("\nPhi:\n")
    print(round(x$Phi, digits))
  }

  # --- Optional rotation matrix ---
  if (rotateMat) {
    cat("\nRotating matrix:\n")
    print(t(solve(x$Th)), digits = digits)
  }

  # --- Optional iteration table ---
  if (Table) {
    cat("\nIteration table:\n")
    print(x$Table, digits = digits)
  }

  invisible(x)
}


summary.GPArotation <- function(object, digits = 3L, Structure = TRUE, ...) {

  object <- .sortGPALoadings(object)

  # Preserve covariance attribute through the copy
  L <- object$loadings
  attr(L, "covariance") <- attr(object$loadings, "covariance")

  r <- list(
    loadings      = L,
    Phi           = object$Phi,
    method        = object$method,
    orthogonal    = object$orthogonal,
    convergence   = object$convergence,
    iters         = rev(object$Table[, 1])[1],
    Structure     = Structure,
    digits        = digits,
    algorithm     = object$algorithm,
    fwindow       = object$fwindow,
    randStartChar = object$randStartChar   # NULL if not present, harmless
  )
  class(r) <- "summary.GPArotation"
  r
}

print.summary.GPArotation <- function(x, cutoff = 0.1, ...) {

  # --- Convergence header ---
  cat(if (x$orthogonal) "Orthogonal" else "Oblique")
  cat(" rotation method", x$method)
  if (!x$convergence) cat(" NOT")
  cat(" converged in ", x$iters, " iterations.\n", sep = "")

 # Always show in summary if available
  if (!is.null(x$algorithm) && !is.null(x$fwindow)) {
   cat("Algorithm:", x$algorithm, " |  fwindow:", x$fwindow,
      " |  Iterations:", nrow(x$Table) - 1, "\n")
  }
     
  # --- Random start diagnostics ---
  if (!is.null(x$randStartChar)) {
    rs <- x$randStartChar
    cat("Of ", rs[1], " random starts ",
        round(100 * rs[2] / rs[1]), "% converged, ",
        round(100 * rs[3] / rs[1]), "% at the same lowest minimum.\n", sep = "")
    if (rs[4] > 1)
      cat("Random starts converged to ", rs[4], " different local minima.\n", sep = "")
  }

  # --- AUC and FSI --- computed after sorting, before printing ---
  auc <- calc_AUC(x$loadings)
  fsi <- calc_FSI(x$loadings)

  # --- Pattern matrix header ---
  prstr <- !x$orthogonal && x$Structure
  if (prstr) cat("Pattern (loadings) - see Structure (correlations) below:\n")

  # --- Loadings + varex block (same approach as print.GPArotation) ---
  L         <- x$loadings
  L_rounded <- round(L, x$digits)
  Lambda    <- unclass(L_rounded)
  p         <- nrow(Lambda)

  cat("\nLoadings:\n")
  fx <- setNames(format(round(Lambda, x$digits)), NULL)
  nc <- nchar(fx[1L], type = "c")
  fx[abs(Lambda) < cutoff] <- strrep(" ", nc)
  print(fx, quote = FALSE)

  # Build varex block
  Phi_cov <- attr(L, "covariance")
  vx <- if (is.null(Phi_cov) || identical(Phi_cov, diag(ncol(Lambda))))
    colSums(Lambda^2)
  else
    diag(Phi_cov %*% crossprod(Lambda))
  names(vx) <- colnames(Lambda)

  varex <- rbind(`SS loadings` = vx)

  if (x$orthogonal) {
    varex <- rbind(varex, `Proportion Var` = vx / p)
    if (ncol(Lambda) > 1)
      varex <- rbind(varex, `Cumulative Var` = cumsum(vx / p))
  }

  varex <- rbind(varex, AUC = auc$AUC, FSI = fsi$FSI)

  cat("\n")
  print(round(varex, x$digits))

  # --- Structure matrix (oblique only) ---
  if (prstr) {
    S <- x$loadings %*% x$Phi
    dimnames(S) <- dimnames(x$loadings)
    S_rounded <- round(S, x$digits)
    class(S_rounded)              <- "loadings"
    attr(S_rounded, "covariance") <- diag(ncol(S))

    out <- capture.output(print(S_rounded, cutoff = cutoff, digits = x$digits))
    out[grep("^Loadings", out)] <- "Correlations:"
    out <- out[!grepl("^SS loadings", out)]
    out <- out[!grepl("^Proportion",  out)]
    out <- out[!grepl("^Cumulative",  out)]
    while (length(out) > 0 && grepl("^\\s*(Factor\\d+\\s*)+$", trimws(out[length(out)])))
      out <- out[-length(out)]
    while (length(out) > 0 && nchar(trimws(out[length(out)])) == 0)
      out <- out[-length(out)]
    cat(paste(out, collapse = "\n"), "\n")
  }

  # --- Simple structure summary ---
  ssi <- calc_simplicity(x$loadings)
  hp  <- calc_hyperplane(x$loadings)

  #cat("\nMean AUC:", auc$AUC_mean, " Min AUC:", auc$AUC_min, "\n")
  #cat("Mean FSI:", fsi$FSI_mean, " Min FSI:", fsi$FSI_min, "\n")
  cat("\nHyperplane total:", hp$HP_total, "of", hp$HP_max,
      "(", hp$HP_pct, "%) at cutoff", hp$cutoff, "\n")

  cat("\nPost-Hoc Simplicity Suite (overall solution):\n")
  cat("Hoffman Index:   ", format(round(ssi$Hoffman, x$digits), nsmall = x$digits), "\n")
  cat("Gini Coeficient: ", format(round(ssi$Gini,    x$digits), nsmall = x$digits), "\n")
  cat("Bentler Index:   ", format(round(ssi$Bentler, x$digits), nsmall = x$digits), "\n")

  # --- Phi (oblique only) ---
  if (!x$orthogonal) {
    cat("\nPhi:\n")
    Phi <- x$Phi
    dimnames(Phi) <- list(colnames(x$loadings), colnames(x$loadings))
    print(round(Phi, x$digits))
  }

  invisible(x)
}

###############################################################
### S3 functions
###############################################################

residuals.GPArotation <- function(object, ...) {
  R <- object$correlation
  if (is.null(R))
    stop("'correlation' not stored. Rotate a factanal object with n.obs specified.",
         call. = FALSE)
  L   <- unclass(object$loadings)
  Phi <- if (is.null(object$Phi)) diag(ncol(L)) else object$Phi
  R_hat       <- L %*% Phi %*% t(L)
  diag(R_hat) <- 1
  Resid       <- R - R_hat
  diag(Resid) <- 0
  Resid
}

echelon <- function(A, reference = seq(ncol(A)), ...) {
  # Echelon rotation.
  # Rotates the reference rows to lower triangular (Cholesky) form,
  # producing a unique solution for the factor loading matrix.
  #
  # Args:
  #   A         : Unrotated factor loading matrix (p x k), or a factanal object
  #   reference : Row indices defining the reference variables (default: first k rows)
  #
  # Returns:
  #   A GPArotation object

  mc <- match.call()

  # --- Accept factanal object ---
  extracted   <- .extract_A(A)
  A           <- extracted$A
  n.obs       <- extracted$n.obs
  correlation <- extracted$correlation

  A1 <- A[reference, , drop = FALSE]

  # Cholesky factorization of A1 %*% t(A1)
  # Note: ill-conditioning is a real danger - no singularity check performed
  B1   <- t(chol(A1 %*% t(A1)))
  Tmat <- solve(A1, B1)

  # Assemble rotated solution
  L              <- matrix(0, nrow(A), ncol(A))
  L[reference, ] <- B1
  L[-reference,] <- A[-reference, , drop = FALSE] %*% Tmat
  dimnames(L)    <- list(rownames(A), paste("factor", seq(ncol(A))))

  # --- covariance attribute for print.loadings ---
  class(L) <- "loadings"
  attr(L, "covariance") <- diag(ncol(L))  # orthogonal - plain SS

  r <- list(
    loadings    = L,
    Th          = Tmat,
    method      = "echelon",
    orthogonal  = TRUE,
    convergence = TRUE,
    Gq          = NULL,
    Table       = matrix(NA, 1, 4,
                    dimnames = list(NULL, c("iter", "f", "log10(s)", "alpha"))),
    n.obs       = n.obs,
    correlation = correlation,
    call        = mc
  )
  attr(r, "A_unrotated") <- A
  class(r) <- "GPArotation"
  r
}

###########################

#echelon <- function(L, reference=seq(NCOL(L)), ...) {

#   # Split L in reference part and the rest
#   A1 <- L[reference,, drop=FALSE]
#   #A2 is L[-reference,]

#   # Compute the part of A Phi A' corresponding to the reference variables
#   # Compute cholesky rot = rotated reference part
#   # No check or error message for singularity. Exact singularity is rare in
#   # practice but ill-conditioning is a real danger.

#   #  now assuming orthogonal (Phi=I)
#   #newPhi <- if (is.null(Phi)) A1 %*% t(A1) else A1 %*% Phi %*% t(A1)
#   #B1 <- t(chol(newPhi))
#   
#   B1 <- t(chol(A1 %*% t(A1)))

#   # Transformation matrix: B1 = A1 * Tmat
#   # Rotated solution for non-reference part: B2 = A2 * Tmat
#   Tmat <- solve(A1, B1)
#   
#   # Assemble rotated solution
#   B <- matrix(0, NROW(L), NCOL(L))
#   B[reference,]  <- B1
#   B[-reference,] <- L[-reference,, drop=FALSE] %*% Tmat

#   dimnames(B) <- list(dimnames(L)[[1]], paste("factor", seq(NCOL(L))))
#   r <- list(loadings=B, Th=Tmat, method="echelon", orthogonal=TRUE, 
#       convergence=TRUE)
#   class(r) <- "GPArotation"
#   r
#}

eiv <- function(A, identity = seq(ncol(A)), ...) {
  # Errors-in-variables rotation (eiv).
  # Selects rows specified by 'identity' to define the reference frame,
  # rotating so those rows form an identity matrix.
  #
  # Args:
  #   A        : Unrotated factor loading matrix (p x k), or a factanal object
  #   identity : Row indices defining the identity set (default: first k rows)
  #
  # Returns:
  #   A GPArotation object

  mc <- match.call()

  # --- Accept factanal object ---
  extracted   <- .extract_A(A)
  A           <- extracted$A
  n.obs       <- extracted$n.obs
  correlation <- extracted$correlation

  A1 <- A[identity, , drop = FALSE]
  g  <- solve(A1)

  if (max(abs(diag(nrow(A1)) - A1 %*% g)) > 1e-14)
    warning("Inverse is not well conditioned. ",
            "Consider setting identity to select different rows.")

  L              <- matrix(NA, nrow(A), ncol(A))
  L[identity, ]  <- diag(nrow(A1))
  L[-identity, ] <- A[-identity, , drop = FALSE] %*% g
  dimnames(L)    <- list(rownames(A), paste("factor", seq(ncol(A))))

  # --- covariance attribute for print.loadings ---
  Phi <- tcrossprod(A1)
  class(L) <- "loadings"
  attr(L, "covariance") <- Phi

  r <- list(
    loadings    = L,
    Phi         = Phi,
    Th          = t(A1),
    method      = "eiv",
    orthogonal  = FALSE,
    convergence = TRUE,
    Gq          = NULL,
    Table       = matrix(NA, 1, 4,
                    dimnames = list(NULL, c("iter", "f", "log10(s)", "alpha"))),
    n.obs       = n.obs,
    correlation = correlation,
    call        = mc
  )
  attr(r, "A_unrotated") <- A
  class(r) <- "GPArotation"
  r
}


#########################
#
#eiv <- function(L,  identity=seq(NCOL(L)), ...){
#   A1 <- L[ identity, , drop=FALSE]
#   g <- solve(A1)
#   if(1e-14 < max(abs(diag(1, length(identity)) - A1 %*% g)))
#      warning("Inverse is not well conditioned. Consider setting identity to select different rows.")
#   B <- array(NA, dim(L))
#   B[identity, ] <- diag(1, length(identity))
#   B[-identity,] <- L[-identity,, drop=FALSE] %*% g
#   dimnames(B) <- list(dimnames(L)[[1]], paste("factor", seq(NCOL(L))))
#   r <- list(loadings=B, Th=t(A1), method="eiv", orthogonal=FALSE, convergence=TRUE,
#        Phi= tcrossprod(A1))
#   class(r) <- "GPArotation"
#   r
#  }

#' Calculate Factor Simplicity Index
#'
#' Computes the Factor Simplicity Index (FSI) for a rotated factor solution,
#' following Lorenzo-Seva (2003).
#'
#' @param x A GPArotation object, a factanal object, or a factor loading matrix.
#' @param digits Integer. Number of decimal places for rounding. Default 3.
#'
#' @return A list with:
#'   \item{FSI}{numeric vector of FSI values, one per factor.}
#'   \item{FSI_mean}{mean FSI across factors.}
#'   \item{FSI_min}{minimum FSI across factors.}
#'
#' @references
#' Lorenzo-Seva, U. (2003) A factor simplicity index.
#' \emph{Psychometrika}, \bold{68}(1), 49--60.
#' \doi{10.1007/BF02296652}
#'
#' @export
calc_FSI <- function(x, digits = 3L) {

  # --- Extract loadings ---
  if (inherits(x, "GPArotation")) {
    L <- unclass(x$loadings)
  } else if (inherits(x, "factanal")) {
    L <- unclass(x$loadings)
  } else {
    L <- unclass(x)
  }

  if (!is.matrix(L))
    L <- as.matrix(L)

  p <- nrow(L)
  k <- ncol(L)

  if (p <= 1)
    stop("FSI requires more than one indicator.", call. = FALSE)

  # --- FSI per factor ---
  FSI <- numeric(k)

  for (j in 1:k) {
    l2     <- L[, j]^2
    l2_sum <- sum(l2)
    l4_sum <- sum(l2^2)

    if (l2_sum == 0) {
      FSI[j] <- 0
      next
    }

    FSI[j] <- (p * l4_sum - l2_sum^2) /
              ((p - 1) * l2_sum^2)
  }

  names(FSI) <- colnames(L)

  list(
    FSI      = round(FSI,       digits),
    FSI_mean = round(mean(FSI), digits),
    FSI_min  = round(min(FSI),  digits)
  )
}


#' Calculate AUC Simple Structure Measure
#'
#' Computes the Area Under the Curve (AUC) measure of factor loading
#' simplicity for a GPArotation object or loading matrix, following
#' Liu and Moustaki (2024).
#'
#' @param x A GPArotation object or a factor loading matrix.
#' @param digits Integer. Number of decimal places for rounding. Default 3.
#'
#' @return A list with:
#'   \item{AUC}{numeric vector of AUC values per factor}
#'   \item{AUC_mean}{mean AUC across factors}
#'   \item{AUC_min}{minimum AUC across factors}
#'
#'
#' @export
calc_AUC <- function(x, digits = 3L) {

  extracted <- .extract_A(x)
  L         <- extracted$A
  
  # --- Extract loadings ---
  if (inherits(x, "GPArotation")) {
    L <- unclass(x$loadings)
  } else {
    L <- unclass(x)
  }

  p <- nrow(L)
  k <- ncol(L)

  # --- AUC per factor ---
  AUC <- numeric(k)

  for (j in 1:k) {
    l2       <- L[, j]^2
    l2_sort  <- sort(l2, decreasing = TRUE)
    l2_sum   <- sum(l2)

    if (l2_sum == 0) {
      AUC[j] <- 0
      next
    }

    # Cumulative proportion of variance explained
    # weighted by rank position
    AUC[j] <- sum(cumsum(l2_sort / l2_sum) ) / p
  }

  # --- Baseline adjustment ---
  # Subtract diagonal baseline (0.5) so AUC = 0 means no simple structure
  # and AUC = 0.5 means perfect simple structure
  AUC_adj <- AUC - 0.5

  names(AUC)     <- colnames(L)
  names(AUC_adj) <- colnames(L)

  list(
    AUC      = round(AUC,      digits),
    AUC_adj  = round(AUC_adj,  digits),
    AUC_mean = round(mean(AUC),     digits),
    AUC_min  = round(min(AUC),      digits),
    AUC_adj_mean = round(mean(AUC_adj), digits),
    AUC_adj_min  = round(min(AUC_adj),  digits)
  )
}


#' Calculate Overall Simple Structure Indices
#'
#' Computes three overall measures of simple structure for a rotated factor
#' solution: Hoffman (1978), Gini (Kaiser, 1974), and Bentler (1977).
#' Unlike AUC and FSI which are computed per factor, these indices
#' summarize simple structure across the entire loading matrix.
#'
#' @param x A GPArotation object, a factanal object, or a factor loading matrix.
#' @param digits Integer. Number of decimal places for rounding. Default 3.
#'
#' @return A list with elements:
#'   \item{Hoffman}{Hoffman's simplicity index.}
#'   \item{Gini}{Gini simplicity index.}
#'   \item{Bentler}{Bentler's simplicity index.}
#'
#' @references
#' Bentler, P.M. (1977) Factor simplicity index and transformations.
#'   \emph{Psychometrika}, \bold{42}(2), 277--295.
#'   \doi{10.1007/BF02294054}
#'
#' Hoffman, P.J. (1978) The paramorphic representation of clinical judgment.
#'   \emph{Psychological Bulletin}, \bold{55}(2), 116--131.
#'   \doi{10.1037/h0047807}
#'
#' Kaiser, H.F. (1974) An index of factorial simplicity.
#'   \emph{Psychometrika}, \bold{39}(1), 31--36.
#'   \doi{10.1007/BF02291575}
#'
#' @export
calc_simplicity <- function(x, digits = 3L) {

  # --- Extract loadings ---
  if (inherits(x, "GPArotation")) {
    L <- unclass(x$loadings)
  } else if (inherits(x, "factanal")) {
    L <- unclass(x$loadings)
  } else {
    L <- unclass(x)
  }

  if (!is.matrix(L))
    L <- as.matrix(L)

  p <- nrow(L)
  k <- ncol(L)

  if (p <= 1)
    stop("Simplicity indices require more than one indicator.", call. = FALSE)
  if (k <= 1)
    stop("Simplicity indices require more than one factor.", call. = FALSE)

  L2 <- L^2
  L4 <- L^4

  # --- Hoffman (1978) ---
  # Measures how far loadings are from 0.5
  # H = 1 when all loadings are 0 or 1 (perfect simple structure)
  # H = 0 when all loadings equal 0.5
  H <- 1 - sum(L2 * (1 - L2)) / (p * k * 0.25)

  # --- Gini (Kaiser, 1974) ---
  # Based on Gini coefficient of absolute loadings per factor
  # then averaged across factors
  gini_j <- numeric(k)
  for (j in 1:k) {
    a       <- sort(abs(L[, j]))
    n       <- length(a)
    a_sum   <- sum(a)
    if (a_sum == 0) {
      gini_j[j] <- 0
    } else {
      ranks     <- seq_len(n)
      gini_j[j] <- 1 - (2 * sum((n - ranks + 0.5) * a)) / (n * a_sum)
    }
  }
  G <- mean(gini_j)

  # --- Bentler (1977) ---
  # Ratio of between-factor to within-factor variance of squared loadings
  col_sum_L2  <- colSums(L2)
  total_L2    <- sum(L2)
  total_L4    <- sum(L4)
  sum_col_L2_sq <- sum(col_sum_L2^2)

  numerator   <- sum_col_L2_sq / p - total_L4 / p
  denominator <- total_L2^2 / p  - total_L4 / p

  B <- if (abs(denominator) < .Machine$double.eps)
    NA
  else
    numerator / denominator

  list(
    Hoffman = round(H, digits),
    Gini    = round(G, digits),
    Bentler = round(B, digits)
  )
}



#' Calculate Hyperplane Count
#'
#' Computes the hyperplane count for a rotated factor solution.
#' The hyperplane count is the number of loadings falling within a
#' narrow band around zero, per factor and overall.
#'
#' @param x A GPArotation object, a factanal object, or a factor loading matrix.
#' @param cutoff Numeric. Half-width of the hyperplane band. Default 0.1.
#' @param digits Integer. Number of decimal places for rounding. Default 3.
#'
#' @return A list with elements:
#'   \item{HP}{integer vector of hyperplane counts per factor.}
#'   \item{HP_total}{total hyperplane count across all factors.}
#'   \item{HP_max}{theoretical maximum hyperplane count p*(k-1).}
#'   \item{HP_pct}{percentage of theoretical maximum achieved.}
#'   \item{cutoff}{the hyperplane cutoff used.}
#'
#' @references
#' Thurstone, L.L. (1947) \emph{Multiple Factor Analysis}.
#'   University of Chicago Press.
#'
#' Jennrich, R.I. (2004) Rotation to simple loadings using component loss
#'   functions: The oblique case. \emph{Psychometrika}, \bold{71}(1),
#'   173--191. 
#'
#' @export
calc_hyperplane <- function(x, cutoff = 0.1, digits = 3L) {

  # --- Extract loadings ---
  if (inherits(x, "GPArotation")) {
    L <- unclass(x$loadings)
  } else if (inherits(x, "factanal")) {
    L <- unclass(x$loadings)
  } else {
    L <- unclass(x)
  }

  if (!is.matrix(L))
    L <- as.matrix(L)

  p <- nrow(L)
  k <- ncol(L)

  if (k <= 1)
    stop("Hyperplane count requires more than one factor.", call. = FALSE)

  # --- Hyperplane count per factor ---
  HP       <- colSums(abs(L) < cutoff)
  names(HP) <- colnames(L)

  # --- Overall ---
  HP_total <- sum(HP)
  HP_max   <- p * (k - 1)  # theoretical maximum - each item in k-1 hyperplanes
  HP_pct   <- round(100 * HP_total / HP_max, digits)

  list(
    HP       = HP,
    HP_total = HP_total,
    HP_max   = HP_max,
    HP_pct   = HP_pct,
    cutoff   = cutoff
  )
}

### R code from vignette source 'GPA4fitstats.Rnw'

###################################################
### code chunk number 1: GPA4fitstats.Rnw:49-52
###################################################
library("GPArotation")
data("GriffithMulaik", package = "GPArotation")
n.obs <- 523


###################################################
### code chunk number 2: GPA4fitstats.Rnw:64-73
###################################################
for (k in 3:8) {
  fa.k  <- factanal(factors = k, covmat = GriffithMulaik,
                    n.obs = n.obs, rotation = "none")
  res.k <- quartimax(fa.k)
  fit.k <- GPArotation:::calc_fitstats(res.k)
  cat(sprintf("k = %d  RMSEA = %.4f  [%.4f, %.4f]  SRMR = %.4f\n",
              k, fit.k$RMSEA, fit.k$RMSEA.l, fit.k$RMSEA.u,
              fit.k$SRMR))
}


###################################################
### code chunk number 3: GPA4fitstats.Rnw:139-156
###################################################

fa.un <- factanal(factors = 6, covmat = GriffithMulaik,
                  n.obs = n.obs, rotation = "none")
res.vm <- Varimax(fa.un)
res.ob <- oblimin(fa.un, randomStarts = 100)
res.t1 <- tandemI(fa.un)

fit.vm <- GPArotation:::calc_fitstats(res.vm)
fit.ob <- GPArotation:::calc_fitstats(res.ob)
fit.t1 <- GPArotation:::calc_fitstats(res.t1)

cat(sprintf("Varimax  SRMR = %.4f  RMSEA = %.4f\n",
            fit.vm$SRMR, fit.vm$RMSEA))
cat(sprintf("Oblimin  SRMR = %.4f  RMSEA = %.4f\n",
            fit.ob$SRMR, fit.ob$RMSEA))
cat(sprintf("TandemI  SRMR = %.4f  RMSEA = %.4f\n",
            fit.t1$SRMR, fit.t1$RMSEA))


###################################################
### code chunk number 4: GPA4fitstats.Rnw:166-167
###################################################
GPArotation:::audit_residuals(res.ob)


###################################################
### code chunk number 5: GPA4fitstats.Rnw:178-179
###################################################
plot(res.ob, "residuals")


###################################################
### code chunk number 6: GPA4fitstats.Rnw:205-206
###################################################
print(res.ob)


###################################################
### code chunk number 7: GPA4fitstats.Rnw:215-216
###################################################
summary(res.ob)


###################################################
### code chunk number 8: GPA4fitstats.Rnw:229-245
###################################################
res.gm <- geominQ(fa.un, randomStarts = 100)

auc.ob <- GPArotation:::calc_AUC(res.ob)$AUC_mean
auc.vm <- GPArotation:::calc_AUC(res.vm)$AUC_mean
auc.gm <- GPArotation:::calc_AUC(res.gm)$AUC_mean

hp.ob  <- GPArotation:::calc_hyperplane(res.ob)$HP_pct
hp.vm  <- GPArotation:::calc_hyperplane(res.vm)$HP_pct
hp.gm  <- GPArotation:::calc_hyperplane(res.gm)$HP_pct

cat(sprintf("%-12s  AUC = %.3f  Hyperplane = %.1f pct\n",
            "Varimax",  auc.vm, hp.vm))
cat(sprintf("%-12s  AUC = %.3f  Hyperplane = %.1f pct\n",
            "Oblimin",  auc.ob, hp.ob))
cat(sprintf("%-12s  AUC = %.3f  Hyperplane = %.1f pct\n",
            "GeominQ",  auc.gm, hp.gm))


###################################################
### code chunk number 9: GPA4fitstats.Rnw:256-263
###################################################
sim.un <- GPArotation:::calc_simplicity(fa.un)
sim.ob <- GPArotation:::calc_simplicity(res.ob)

cat(sprintf("%-12s  Hoffman = %.3f  Gini = %.3f  Bentler = %.3f\n",
            "Unrotated", sim.un$Hoffman, sim.un$Gini, sim.un$Bentler))
cat(sprintf("%-12s  Hoffman = %.3f  Gini = %.3f  Bentler = %.3f\n",
            "Oblimin",   sim.ob$Hoffman, sim.ob$Gini, sim.ob$Bentler))

# tests/normalizationTests.R
# Tests for NormalizingWeight normalization options

library(GPArotation)

all.ok <- TRUE
data("Harman", package = "GPArotation")

# Helper
.check <- function(condition, label) {
  if (!condition) {
    cat("FAILED:", label, "\n")
    all.ok <<- FALSE
  }
}

# --- Test 1: normalize = FALSE returns all ones ---
W <- GPArotation:::NormalizingWeight(Harman8, normalize = FALSE)
.check(all(W == 1), "Test 1 - FALSE returns all ones")

# --- Test 2: normalize = TRUE gives Kaiser weights ---
W.true <- GPArotation:::NormalizingWeight(Harman8, normalize = TRUE)
W.kai  <- GPArotation:::NormalizingWeight(Harman8, normalize = "Kaiser")
.check(max(abs(W.true - W.kai)) < 1e-15,
       "Test 2 - TRUE and Kaiser identical")

# --- Test 3: Kaiser rows sum to 1 after normalization ---
A_norm <- Harman8 / W.kai
.check(max(abs(rowSums(A_norm^2) - 1)) < 1e-10,
       "Test 3 - Kaiser normalized rows have unit norm")

# --- Test 4: CM weights are positive ---
W.cm <- GPArotation:::NormalizingWeight(Harman8, normalize = "CM")
.check(all(W.cm > 0), "Test 4 - CM weights all positive")

# --- Test 5: CM and Kaiser give different weights ---
.check(max(abs(W.true - W.cm)) > 1e-6,
       "Test 5 - CM and Kaiser weights differ")

# --- Test 6: custom vector ---
w <- rep(2, nrow(Harman8))
W.vec <- GPArotation:::NormalizingWeight(Harman8, normalize = w)
.check(all(W.vec == 2), "Test 6 - custom vector applied correctly")

# --- Test 7: custom function ---
W.fun <- GPArotation:::NormalizingWeight(Harman8,
           normalize = function(A) sqrt(rowSums(A^2)))
.check(max(abs(W.fun - W.kai)) < 1e-15,
       "Test 7 - custom function matches Kaiser")

# --- Test 8: wrong length vector throws error ---
tryCatch({
  GPArotation:::NormalizingWeight(Harman8, normalize = rep(1, 5))
  cat("FAILED: Test 8 - wrong length should throw error\n")
  all.ok <- FALSE
}, error = function(e) {
  # expected
})

# --- Test 9: unrecognized string throws error ---
tryCatch({
  GPArotation:::NormalizingWeight(Harman8, normalize = "unknown")
  cat("FAILED: Test 9 - unrecognized string should throw error\n")
  all.ok <- FALSE
}, error = function(e) {
  # expected
})

# --- Test 10: rotation with CM produces valid solution ---
res.cm  <- oblimin(Harman8, normalize = "CM")
res.kai <- oblimin(Harman8, normalize = TRUE)
.check(inherits(res.cm, "GPArotation"),
       "Test 10 - CM rotation returns GPArotation object")
.check(res.cm$convergence,
       "Test 11 - CM rotation converges")

cat("Normalization tests completed.\n")
if (!all.ok) stop("some normalization tests FAILED")

%\VignetteEncoding{UTF-8}
%\VignetteIndexEntry{Factor Model Fit and Simple Structure Diagnostics}
%\VignetteEngine{utils::Sweave}
%\VignettePackage{GPArotation}
%\VignetteDepends{GPArotation}
%\VignetteKeyword{factor model fit}
%\VignetteKeyword{simple structure}

\documentclass[english, 10pt]{article}
\usepackage{hyperref}
\usepackage[margin=0.7in, letterpaper]{geometry}
\bibstyle{apacite}
\bibliographystyle{apa}
\usepackage{natbib}
\begin{document}

\SweaveOpts{echo=TRUE, results=hide}

\begin{center}
\section*{Factor Model Fit and Simple Structure Diagnostics \\ ~~\\
          The \texttt{GPArotation} Package}
\end{center}
\begin{center}
Author: Coen A. Bernaards
\end{center}

\section{Introduction}

Evaluating an exploratory factor analysis solution involves two
distinct questions. First, does the $k$-factor model adequately
reproduce the observed correlations? Second, once a factor solution
is accepted, does rotation produce clean simple structure with loading
patterns that are easy to interpret?

This vignette covers both questions using the \texttt{GriffithMulaik}
and \texttt{CCAI} datasets included in \texttt{GPArotation}. Model
fit is assessed via RMSEA and SRMR. Simple structure quality is
assessed via AUC, FSI, hyperplane counts, and the Hoffman, Gini,
and Bentler (\cite{bentler1977}) indices, all of which are reported
automatically by \texttt{summary()}.

\section{Model Fit: How Well Does the Factor Model Reproduce the Data?}

We use the Griffith and Mulaik (\cite{griffith1998}) interpersonal
personality data (\texttt{GriffithMulaik}, $n = 523$, 24 variables,
6 hypothetical interpersonal factors) to illustrate the complete
model evaluation workflow. The data are described in \cite{mulaik2018}.

<<>>=
library("GPArotation")
data("GriffithMulaik", package = "GPArotation")
n.obs <- 523
@

\subsection{Choosing the Number of Factors}

\subsubsection{Fit Statistics}

Note that \texttt{n.obs} must be specified in the \texttt{factanal}
call for fit statistics to be available. It is stored automatically
in the \texttt{GPArotation} object and used by \texttt{calc\_fitstats}
to compute RMSEA and SRMR.

<<results=verbatim>>=
for (k in 3:8) {
  fa.k  <- factanal(factors = k, covmat = GriffithMulaik,
                    n.obs = n.obs, rotation = "none")
  res.k <- quartimax(fa.k)
  fit.k <- GPArotation:::calc_fitstats(res.k)
  cat(sprintf("k = %d  RMSEA = %.4f  [%.4f, %.4f]  SRMR = %.4f\n",
              k, fit.k$RMSEA, fit.k$RMSEA.l, fit.k$RMSEA.u,
              fit.k$SRMR))
}
@

RMSEA values below 0.05 indicate close fit and below 0.08 acceptable
fit (\cite{hu1999}). SRMR below 0.08 is generally considered
acceptable. For these data $k = 6$ achieves RMSEA below 0.05 and
SRMR below 0.025, consistent with Mulaik's theoretical prediction
of six interpersonal factors.

\subsection{Residual Analysis}

Global fit indices summarize overall model adequacy but can mask
localized problems. A residual correlation matrix inspection identifies
which specific item pairs are poorly reproduced by the factor model.

The residuals here are \emph{correlation residuals}. The difference
between the observed correlation $r_{ij}$ and the model-implied
correlation $\hat{r}_{ij}$:

\[
e_{ij} = r_{ij} - \hat{r}_{ij}, \quad \hat{R} = L \Phi L^T
\]

A \textbf{positive residual} means the model underestimates the
relationship between two items. They share more variance than the
factors account for. This may indicate a missing factor, shared method
variance, or item redundancy. A \textbf{negative residual} means the
model overestimates the relationship. Residuals near zero indicate
the factor model reproduces that pair well.

As a rough guide:
\begin{itemize}
  \item $|e_{ij}| < 0.05$ --- excellent local fit
  \item $0.05 \leq |e_{ij}| < 0.10$ --- acceptable
  \item $|e_{ij}| \geq 0.10$ --- worth investigating
  \item $|e_{ij}| \geq 0.20$ --- serious local misfit
\end{itemize}

Patterns in the residuals are informative. Clustered large residuals
around a particular item suggest that item may need its own factor or
has method variance not captured by the model. Large residuals between
items from different factors suggest those factors may be more correlated
than the model captures.

\subsection{Residuals Are a Property of the Extraction, Not the Rotation}

An important property of factor rotation is that it does not change
the model-implied correlation matrix $\hat{R}$. For orthogonal
rotation, $\Phi = I$ and $T$ is orthogonal ($TT' = I$), so:

\[
\hat{R} = LT(LT)' = LTT'L' = LL' = AA'
\]

For oblique rotation, $\Phi = T'T$, so:

\[
\hat{R} = AT^{-T} \cdot T'T \cdot T^{-1}A' = AA'
\]

Both reduce to $AA'$, the model-implied correlation matrix that depends
only on the unrotated loadings $A$, not on the rotation matrix $T$.
This means the residual matrix $R - \hat{R}$ and all derived fit
statistics (SRMR, RMSEA) are identical across all rotation methods
applied to the same unrotated solution.

<<results=verbatim>>=

fa.un <- factanal(factors = 6, covmat = GriffithMulaik,
                  n.obs = n.obs, rotation = "none")
res.vm <- Varimax(fa.un)
res.ob <- oblimin(fa.un, randomStarts = 100)
res.t1 <- tandemI(fa.un)

fit.vm <- GPArotation:::calc_fitstats(res.vm)
fit.ob <- GPArotation:::calc_fitstats(res.ob)
fit.t1 <- GPArotation:::calc_fitstats(res.t1)

cat(sprintf("Varimax  SRMR = %.4f  RMSEA = %.4f\n",
            fit.vm$SRMR, fit.vm$RMSEA))
cat(sprintf("Oblimin  SRMR = %.4f  RMSEA = %.4f\n",
            fit.ob$SRMR, fit.ob$RMSEA))
cat(sprintf("TandemI  SRMR = %.4f  RMSEA = %.4f\n",
            fit.t1$SRMR, fit.t1$RMSEA))
@

All three produce identical SRMR and RMSEA values. Fit statistics
therefore inform the choice of \emph{how many} factors to extract,
not \emph{how} to rotate them. To compare rotation methods, use the
criterion-free simple structure measures (AUC, FSI, and hyperplane
counts) which do vary across rotation methods and are reported
automatically by \texttt{print()} and \texttt{summary()}.

<<results=verbatim>>=
GPArotation:::audit_residuals(res.ob)
@

Items with high mean absolute residuals may cross-load on an
additional factor or have method variance not captured by the model.
For the \texttt{GriffithMulaik} data, \texttt{IMPARTL} is flagged
with two pairs exceeding 0.10. It loads weakly on multiple
factors, suggesting it does not discriminate cleanly between the
six interpersonal dimensions.

\begin{center}
<<fig=TRUE, height=4, echo=TRUE, results=hide>>=
plot(res.ob, "residuals")
@
\end{center}

The heatmap shows the full residual matrix. Blue cells indicate
the model overestimates the correlation, red cells indicate
underestimation. The sorted bar chart shows the distribution of all
off-diagonal absolute residuals.

\section{Simple Structure: How Clean Is the Rotated Solution?}

Once an adequate model is accepted, rotation is evaluated by how
well it achieves simple structure. GPArotation reports five
criterion-free simple structure measures automatically via
\texttt{print()} and \texttt{summary()}.

\subsection{Per-Factor Measures: AUC and FSI}

\texttt{print()} reports AUC (\cite{liu2023}) and FSI
(\cite{lorenzoseva2003}) for each factor. AUC measures how rapidly
a factor's variance accumulates when loadings are sorted by size. AUC
values near 1 indicate one or two dominant loadings with the rest
near zero. FSI measures the kurtosis of the squared loadings. FSI
values near 1 indicate a sharp contrast between large and small
loadings.

<<results=verbatim>>=
print(res.ob)
@

\subsection{Overall Solution Measures}

\texttt{summary()} adds three overall measures (Hoffman, Gini,
Bentler), the hyperplane count, and the structure matrix for
oblique solutions.

<<results=verbatim>>=
summary(res.ob)
@

The hyperplane count reports how many loadings fall below a cutoff
of 0.10 in absolute value. For a perfectly simple solution with
$p$ items and $k$ factors, the theoretical maximum is $p(k-1)$.

\subsection{Comparing Rotation Methods}

These measures are criterion-free, so they can be used to compare
simple structure quality across different rotation methods applied
to the same unrotated solution.

<<results=verbatim>>=
res.gm <- geominQ(fa.un, randomStarts = 100)

auc.ob <- GPArotation:::calc_AUC(res.ob)$AUC_mean
auc.vm <- GPArotation:::calc_AUC(res.vm)$AUC_mean
auc.gm <- GPArotation:::calc_AUC(res.gm)$AUC_mean

hp.ob  <- GPArotation:::calc_hyperplane(res.ob)$HP_pct
hp.vm  <- GPArotation:::calc_hyperplane(res.vm)$HP_pct
hp.gm  <- GPArotation:::calc_hyperplane(res.gm)$HP_pct

cat(sprintf("%-12s  AUC = %.3f  Hyperplane = %.1f pct\n",
            "Varimax",  auc.vm, hp.vm))
cat(sprintf("%-12s  AUC = %.3f  Hyperplane = %.1f pct\n",
            "Oblimin",  auc.ob, hp.ob))
cat(sprintf("%-12s  AUC = %.3f  Hyperplane = %.1f pct\n",
            "GeominQ",  auc.gm, hp.gm))
@

Oblique rotations achieve higher AUC and hyperplane counts than
Varimax because the underlying interpersonal factors are correlated;
an orthogonal constraint forces loadings to spread across factors.

\subsection{Unrotated vs Rotated: Does Rotation Help?}

Simple structure measures should always improve after rotation:

<<results=verbatim>>=
sim.un <- GPArotation:::calc_simplicity(fa.un)
sim.ob <- GPArotation:::calc_simplicity(res.ob)

cat(sprintf("%-12s  Hoffman = %.3f  Gini = %.3f  Bentler = %.3f\n",
            "Unrotated", sim.un$Hoffman, sim.un$Gini, sim.un$Bentler))
cat(sprintf("%-12s  Hoffman = %.3f  Gini = %.3f  Bentler = %.3f\n",
            "Oblimin",   sim.ob$Hoffman, sim.ob$Gini, sim.ob$Bentler))
@

All three indices increase after rotation. If Bentler remains low
despite high AUC and hyperplane counts, this typically indicates
high factor intercorrelations rather than poor simple structure.

\section{Summary}

A complete EFA diagnostic workflow in GPArotation:

\begin{enumerate}
  \item Extract a $k$-factor solution via \texttt{factanal}.
  \item Compare scree plots from raw and reduced correlation matrices
        (\cite{mulaik2018}).
  \item Check global fit via \texttt{GPArotation:::calc\_fitstats}
        across a range of $k$.
  \item Rotate with \texttt{randomStarts} to avoid local minima.
  \item Inspect residuals via \texttt{audit\_residuals} and
        \texttt{plot(..., "residuals")} for local misfit.
  \item Compare rotation methods using AUC and hyperplane counts.
  \item Use \texttt{summary()} for the full diagnostic scorecard.
\end{enumerate}

\begin{thebibliography}{}

\bibitem[\protect\citeauthoryear{Bentler}{Bentler}{1977}]{bentler1977}
Bentler, P.M. (1977).
\newblock Factor simplicity index and transformations.
\newblock \textit{Psychometrika}, \textbf{42}, 277--295.
\newblock \href{https://doi.org/10.1007/BF02294054}
  {doi: 10.1007/BF02294054}

\bibitem[\protect\citeauthoryear{Griffith \& Mulaik}{Griffith \&
  Mulaik}{1998}]{griffith1998}
Griffith, D. and Mulaik, S.A. (1998).
\newblock Personality trait factors from the interpersonal domain.
\newblock \textit{Poster presented at the Society for Multivariate
  Experimental Psychology (SMEP)}.

\bibitem[\protect\citeauthoryear{Hu \& Bentler}{Hu \& Bentler}{1999}]{hu1999}
Hu, L. and Bentler, P.M. (1999).
\newblock Cutoff criteria for fit indexes in covariance structure analysis.
\newblock \textit{Structural Equation Modeling}, \textbf{6}(1), 1--55.
\newblock \href{https://doi.org/10.1080/10705519909540118}
  {doi: 10.1080/10705519909540118}

\bibitem[\protect\citeauthoryear{Liu et al.}{Liu et al.}{2023}]{liu2023}
Liu, X., Wallin, G., Chen, Y., and Moustaki, I. (2023).
\newblock Rotation to sparse loadings using $L^p$ losses and related inference
  problems.
\newblock \textit{Psychometrika}, \textbf{88}(2), 527--553.
\newblock \href{https://doi.org/10.1007/s11336-023-09911-y}
  {doi: 10.1007/s11336-023-09911-y}

\bibitem[\protect\citeauthoryear{Lorenzo-Seva}{Lorenzo-Seva}{2003}]{lorenzoseva2003}
Lorenzo-Seva, U. (2003).
\newblock A factor simplicity index.
\newblock \textit{Psychometrika}, \textbf{68}(1), 49--60.
\newblock \href{https://doi.org/10.1007/BF02296652}
  {doi: 10.1007/BF02296652}

\bibitem[\protect\citeauthoryear{Mulaik}{Mulaik}{2018}]{mulaik2018}
Mulaik, S.A. (2018).
\newblock Fundamentals of common factor analysis.
\newblock In P. Irwing, T. Booth, and D.J. Hughes (Eds.),
  \textit{The Wiley Handbook of Psychometric Testing} (pp. 211--252).
\newblock Wiley.

\end{thebibliography}
\end{document}

%\VignetteEncoding{UTF-8}
%\VignetteIndexEntry{Gradient Projection Factor Rotation}
%\VignetteEngine{utils::Sweave}
%\VignettePackage{GPArotation}
%\VignetteDepends{GPArotation}
%\VignetteKeyword{factor rotation}
%\VignetteKeyword{gradient projection}
%\VignetteKeyword{varimax}
%\VignetteKeyword{oblimin}

\documentclass[english, 10pt]{article}
\usepackage{hyperref}

\bibstyle{apacite}
\bibliographystyle{apa}
\usepackage{natbib}

\usepackage[margin=0.7in, letterpaper]{geometry}
\begin{document}

\SweaveOpts{eval=TRUE,echo=TRUE,results=hide,fig=FALSE}
<<echo=FALSE,results=hide>>=
options(continue="  ")
pdf.options(pointsize = 8)
@

\begin{center}
\section*{Gradient Projection Factor Rotation \\  ~~\\The \texttt{GPArotation} Package}
\end{center}
\begin{center}
Author: Coen A. Bernaards
\end{center}

\noindent
The \texttt{GPArotation} package provides gradient projection algorithms for
orthogonal and oblique rotation of factor loading matrices in exploratory
factor analysis. It implements a comprehensive set of rotation criteria and
supports multiple random starts to avoid local minima. This vignette
introduces the main functionality of the package, with examples of common
use cases. For a complete list of available rotation criteria and their
references, see the Rotation Criteria Reference section at the back of this
document.

In R, the functions in this package are made available with
<<>>=
library("GPArotation")
@

The most complete reference for the software is \cite{gpa.rotate}. The
original 2005 implementations in four platforms are preserved for historical
reference:

\href{https://drive.google.com/file/d/1TetEn1TGLul6FRTlIwXx3nbSQOGlifXi/view?usp=sharing}
{MATLAB code},
\href{https://drive.google.com/file/d/1RryvIoTib0d9SIFm9x1fdDFPfjzsh49e/view?usp=sharing}
{Splus code},
\href{https://drive.google.com/file/d/1t5Bw8KLlIGrSr3TNy0rJjHfsU6nOlIpq/view?usp=sharing}
{SAS code}, and
\href{https://drive.google.com/file/d/1JGvSxiphNhbZhW-GzGXkkNPBFcKyfSZi/view?usp=sharing}
{SPSS code}.
These implementations predate the R package and reflect the algorithm as
originally published. The R package has been updated since then; see the
\texttt{NEWS} file for a full history of changes.

A clear and accessible introduction to gradient projection algorithms for
factor rotation is provided in \cite{mansreise}.


%-------------------------------------------------------------------
\section*{Basic Usage}
%-------------------------------------------------------------------

\subsection*{Getting Started}

Factor rotation aims to simplify interpretation by rotating the loadings matrix.
In practice, most rotations are done by minimizing a criterion function. This
package provides algorithms that can minimize any rotation criteria as long
as a gradient is available.  The loading matrix is typically obtained from a
factor analysis routine such as \texttt{factanal}. A rotation is performed
by calling the rotation function directly, or by calling one of the wrapper
functions \texttt{GPFRSorth} or \texttt{GPFRSoblq} for orthogonal and
oblique rotation, respectively.

Under the hood, rotations are computed using the Gradient Projection
Algorithm, implemented in \texttt{GPForth} for orthogonal rotation and
\texttt{GPFoblq} for oblique rotation. These functions were updated in
version 2026.4-1 with performance improvements and improved code clarity;
results are numerically identical to prior versions.

<<>>=
data("Harman", package = "GPArotation")

# Calling a rotation directly
qHarman <- quartimax(Harman8)

# Equivalently, via the wrapper function
qHarman <- GPFRSorth(Harman8, method = "quartimax")

# Two equivalent ways to access the rotated loadings
loadings(qHarman)       # via extractor function (recommended)
qHarman$loadings        # via direct list access
@

The rotated loadings matrix is the pattern matrix. The extractor function
\texttt{loadings()} is preferred over direct list access for forward
compatibility.

\subsection*{Recovery of the Unrotated Loadings Matrix}

The rotated loadings matrix $L$ and the rotation matrix $T$ are related
to the unrotated matrix $A$ by:

\begin{itemize}
  \item Orthogonal rotation: $L = AT$, so $A = LT'$
        (since $T$ is orthogonal, $T' = T^{-1}$)
  \item Oblique rotation: $L = A(T')^{-1}$, so $A = LT'$
\end{itemize}

In both cases $A = LT'$, where $T'$ denotes the transpose of the
rotation matrix. For orthogonal rotation this simplifies further
since $T'T = I$.

These relationships are useful for verifying results, switching rotation
criteria without re-extracting factors, and reproducing published analyses.
The definitions follow \cite{gpa.rotate} (page 678).

<<results=verbatim>>=
data("CCAI", package = "GPArotation")
fa.un <- factanal(factors = 3, covmat = CCAI_R, n.obs = 461,
                  rotation = "none")

res.quart <- quartimax(fa.un)
max(abs(res.quart$loadings %*% t(res.quart$Th) - fa.un$loadings))

res.obli <- oblimin(fa.un, normalize = TRUE, randomStarts = 15)
max(abs(res.obli$loadings %*% t(res.obli$Th) - fa.un$loadings))

max(abs(res.obli$loadings - fa.un$loadings %*% solve(t(res.obli$Th))))
@

All three expressions should be zero or at machine epsilon, confirming
that the rotated solution is consistent with the unrotated extraction.
The factor correlation matrix for oblique rotation is \cite{gpa.rotate}
(page 695):

<<results=verbatim>>=
max(abs(res.obli$Phi - t(res.obli$Th) %*% res.obli$Th))
@

This should also be zero at machine epsilon. Note that
\texttt{attr(res.obli, "A\_unrotated")} stores the unrotated matrix
directly in the \texttt{GPArotation} object, so manual recovery is rarely needed in
practice.

\subsection*{Output and Display}

The raw output from \texttt{GPArotation} functions returns loadings in the
order determined by the algorithm, which depends on the starting rotation
matrix. This means that with different random starts, the same solution may
appear with factors in a different order or with reversed signs. The
\texttt{print} method sorts factors by descending variance explained and
adjusts signs so that the dominant loadings are positive, making visual
inspection and comparison easier. Importantly, this is a display
transformation only --- the underlying object is not modified.

<<results=verbatim>>=
set.seed(334)
res <- quartimin(Harman8, normalize = TRUE, randomStarts = 100)

# Raw unsorted loadings
loadings(res)

# Sorted loadings via print (factors reordered and signs adjusted)
res.sorted <- print(res)

# Once sorted, repeated calls to print are stable
max(abs(print(res.sorted)$loadings - res.sorted$loadings)) == 0  # TRUE
@

\subsection*{Pattern and Structure Matrices}

\subsubsection*{Two Interpretations of the Loading Matrix}

The loading matrix $\Lambda$ has two complementary interpretations
that correspond directly to the pattern and structure matrices in
oblique rotation.

\textbf{First interpretation: regression.} The conditional
expectation $E(X|f) = \mu + \Lambda f$ is the linear regression of
$X$ on $f$. The loading $\lambda_{ij}$ is the expected change in
item $i$ when factor $j$ is increased by one unit, holding other
factors constant. This is the \emph{pattern matrix} --- the
regression coefficients of items on factors.

\textbf{Second interpretation: correlation.} Under the factor model
assumptions, $Cov(X, f) = \Lambda\Phi$. If $X$ is standardized,
$Corr(X, f) = \Lambda\Phi$. This is the \emph{structure matrix} ---
the correlations between items and factors. Correlations tend to be
consistent across studies.

When factors are orthogonal ($\Phi = I$) the two interpretations
coincide: $\Lambda\Phi = \Lambda$. When factors are oblique the
structure matrix $\Lambda\Phi$ inflates the apparent relationships
between items and factors through the factor intercorrelations,
while the pattern matrix $\Lambda$ shows the unique contribution
of each factor net of those intercorrelations.

\subsubsection*{Using \texttt{GPArotation} to obtain them}

The \texttt{summary} method returns the raw unsorted loadings exactly as
produced by the algorithm. For oblique rotations, it also prints the
structure matrix (loadings $\times$ $\Phi$) when \texttt{Structure = TRUE}
(the default). This is useful for comparing results across software or
reproducing published values.

<<results=verbatim>>=
res.obli <- oblimin(Harman8, normalize = TRUE, randomStarts = 100)
# Pattern matrix (unsorted)
summary(res.obli, Structure = FALSE)
# Structure matrix
summary(res.obli, Structure = TRUE)
@

To illustrate the distinction concretely, consider item 1 (row 1) from
the oblimin rotation of the Harman 8-variable physical measurements
dataset. In the pattern matrix, the loadings are 0.892 on Factor 1 and
0.056 on Factor 2. The pattern coefficient 0.892 is the regression
coefficient of item 1 on Factor 1, controlling for Factor 2 --- it
represents the unique contribution of Factor 1 to item 1, net of the
shared variance between the two factors. The near-zero cross-loading
of 0.056 indicates that item 1 is primarily associated with Factor 1
after accounting for factor intercorrelations.

In the structure matrix, the same item has loadings of 0.918 on Factor
1 and 0.478 on Factor 2. The structure coefficient 0.478 is the simple
correlation between item 1 and Factor 2 --- substantially larger than
the pattern cross-loading of 0.056. This inflation occurs because
Factor 1 and Factor 2 are correlated, and that correlation carries over
into the structure coefficients. An analyst relying solely on the
structure matrix might incorrectly conclude that item 1 has a meaningful
relationship with Factor 2.

In this solution the factor intercorrelation is $\phi = 0.473$. This
moderate correlation between the two factors is sufficient to inflate
the structure coefficients noticeably --- the cross-loading of item 1
on Factor 2 increases from 0.056 in the pattern matrix to 0.478 in
the structure matrix, a difference of 0.422 attributable entirely to
the factor intercorrelation.

This is why the pattern matrix is generally preferred for interpretation
in oblique rotation: it shows the unique contribution of each factor to
each item, net of the shared variance among factors. The structure
matrix is a useful diagnostic check --- large discrepancies between
pattern and structure coefficients signal that factor intercorrelations
are substantially inflating apparent relationships.

The relationship between pattern and structure coefficients can be
visualized by plotting one against the other for each factor. Points
on the identity line (dashed) have equal pattern and structure
coefficients --- no inflation from factor intercorrelations. Points
above the line have structure coefficients inflated by the factor
intercorrelations.

<<>>=
plotPatternStructure <- function(pattern, structure,
                                  labels = NULL,
                                  main = "Pattern vs Structure") {
  k   <- ncol(pattern)
  col <- palette.colors(k, palette = "Okabe-Ito")
  par(mfrow = c(1, k))
  for (j in 1:k) {
    lims <- range(c(pattern[, j], structure[, j]))
    plot(pattern[, j], structure[, j],
         xlim = lims, ylim = lims,
         xlab = "Pattern loading",
         ylab = "Structure loading",
         main = paste(if (!is.null(labels)) labels[j]
                      else paste("Factor", j)),
         pch  = 19, col = col[j])
    abline(0, 1, lty = 2, col = "grey60")
    abline(h = 0, col = "grey80")
    abline(v = 0, col = "grey80")
  }
}
@

\begin{center}
<<fig=TRUE, echo=FALSE, width=4, height=2.5>>=
res.obli.s  <- print(res.obli)
Pattern     <- loadings(res.obli.s)
Structure   <- Pattern %*% res.obli.s$Phi

plotPatternStructure(Pattern, Structure,
                     labels = c("Factor 1", "Factor 2"))
@
\end{center}

With a factor intercorrelation of $\phi = 0.473$, the structure
coefficients are systematically larger than the pattern coefficients,
particularly for items with substantial loadings on both factors.
The further a point lies above the identity line, the more the
factor intercorrelation is inflating the apparent relationship
between that item and the factor.

%-------------------------------------------------------------------
\section*{The Rotation Objective Function Landscape}
%-------------------------------------------------------------------

For two-factor orthogonal rotation, every rotation matrix is parameterized
by a single angle $\theta \in [0, 2\pi)$:

\[
T(\theta) = \left( \begin{array}{cc} \cos\theta & \sin\theta \\ -\sin\theta & \cos\theta \end{array} \right)
\]

This means the objective function $f$ can be plotted as a function of
$\theta$, giving a complete picture of the rotation landscape --- all
local and global minima, and how flat or sharp they are.

The following function computes and plots $f(\theta)$ for $n$ evenly
spaced angles:

<<>>=
plotRotationLandscape <- function(A, method = "quartimax", n = 20000,
                                   main = NULL, ...) {
  if (ncol(A) != 2)
    stop("plotRotationLandscape only works for 2-factor solutions.")

  vgQfun_fn <- get(paste("vgQ", method, sep = "."),
                   envir = asNamespace("GPArotation"))

  if (is.null(main))
    main <- paste("Rotation landscape:", method)

  theta  <- seq(0, 2 * pi, length.out = n)
  f_vals <- numeric(n)

  for (i in seq_along(theta)) {
    Tmat      <- matrix(c( cos(theta[i]), sin(theta[i]),
                          -sin(theta[i]), cos(theta[i])), 2, 2)
    L         <- A %*% Tmat
    VgQ       <- do.call(vgQfun_fn, append(list(L), list(...)))
    f_vals[i] <- VgQ$f
  }

  min_idx <- which.min(f_vals)

  plot(theta, f_vals,
       type  = "l",
       lwd   = 2,
       main  = main,
       xlab  = expression(theta ~ "(radians)"),
       ylab  = "f",
       xaxt  = "n")
  axis(1, at     = c(0, pi/2, pi, 3*pi/2, 2*pi),
           labels = c("0", expression(pi/2), expression(pi),
                      expression(3*pi/2), expression(2*pi)))
  abline(h   = f_vals[min_idx], col = "grey80", lty = 2)
  points(theta[min_idx], f_vals[min_idx],
         col = "tomato", pch = 19, cex = 1.5)
  text(theta[min_idx], f_vals[min_idx],
       labels = paste0("min f = ", round(f_vals[min_idx], 4)),
       pos    = 4,
       cex    = 0.8)

  invisible(data.frame(theta = theta, f = f_vals))
}
@

The following example compares four rotation criteria on the Harman 8-variable
physical measurements dataset:

<<>>=
data(Harman, package = "GPArotation")
@

\begin{center}
<<fig=TRUE>>=
par(mfrow = c(2, 2))
plotRotationLandscape(Harman8, method = "quartimax")
plotRotationLandscape(Harman8, method = "varimax")
plotRotationLandscape(Harman8, method = "bentler")
plotRotationLandscape(Harman8, method = "entropy")
@
\end{center}

\subsection*{Interpreting the Landscape: Symmetry vs Genuine Local Minima}

An important subtlety when interpreting rotation landscapes is that not
all minima represent genuinely different factor structures. For a
two-factor solution, the rotation landscape always contains at least 4
equivalent minima due to the inherent symmetry of factor analysis:

\begin{itemize}
  \item \textbf{Sign flips}: flipping the sign of a factor column gives
        an equivalent solution. With two factors there are $2^2 = 4$
        sign-flip combinations, each producing an equivalent minimum.
  \item \textbf{Column permutations}: swapping the two factors gives an
        equivalent solution, adding further symmetry.
\end{itemize}

Combined, a two-factor solution has at least 4 symmetry-equivalent
minima in $[0, 2\pi)$. Any minima beyond these 4 represent genuinely
different factor structures --- true local minima in the optimization
sense.

The following example illustrates this for the simplimax criterion
with \texttt{k = 4}, which produces multiple local minima on the
Harman 8-variable dataset:

\begin{center}
<<fig=TRUE>>=
res <- plotRotationLandscape(Harman8, method = "simplimax", k = 4)

# Find all local minima by sign changes in the discrete derivative
df         <- diff(res$f)
local_mins <- which(df[-length(df)] < 0 & df[-1] > 0)

# Classify minima: red = symmetry-equivalent global, blue = genuine local
f_rounded  <- round(res$f[local_mins], 4)
f_min      <- min(f_rounded)
is_global  <- f_rounded == f_min
dot_cols   <- ifelse(is_global, "tomato", "steelblue")
n_global   <- sum(is_global)
n_local    <- sum(!is_global)

points(res$theta[local_mins], res$f[local_mins],
       col = dot_cols, pch = 19, cex = 0.9)

legend("topright",
       legend = c(paste0("global minimum (x", n_global, ")"),
                  paste0("local minima (x",   n_local,  ")")),
       col    = c("tomato", "steelblue"),
       pch    = 19, bty = "n", cex = 0.8)
@
\end{center}

<<results=verbatim>>=
cat("Total minima found:              ", length(local_mins), "\n")
cat("Symmetry-equivalent global min:  ", n_global, "\n")
cat("Genuine local minima:            ", n_local,  "\n")
@

The \texttt{k} argument controls how many near-zero loadings simplimax
targets. Smaller values of \texttt{k} create a rougher objective
function landscape with more local minima, while larger values
(up to \texttt{nrow(A)}) produce smoother landscapes.

Criteria such as quartimax and varimax on well-structured data typically
show exactly 4 minima --- all symmetry-equivalent. Criteria prone to
local minima, such as simplimax and geomin, show additional minima
beyond the symmetry baseline. This is precisely why random starts are
important: without them, the algorithm may converge to a
symmetry-equivalent solution or a genuine local minimum rather than
the global minimum.

The rotation landscape visualization is only available for two-factor
solutions. For $k > 2$ factors the rotation space is
$(k^2 - k)/2$-dimensional and cannot be visualized as a simple curve.
For higher-dimensional problems, the \texttt{GPFallMinima} function
described in the \texttt{GPA2local} vignette provides an alternative
approach to exploring the solution space.

\subsection*{Comparing Rotation Criteria Visually}

Different rotation criteria can produce noticeably different loading
patterns. A useful way to compare solutions is to make a bar graph of the absolute loadings
sorted by magnitude for each factor. The following example compares
quartimax and oblimin rotation of the Harman 8-variable physical
measurements dataset.

\begin{center}
<<fig=TRUE>>=
data(Harman, package = "GPArotation")
res.quart   <- quartimax(Harman8)
res.oblimin <- oblimin(Harman8)

L.quart   <- abs(loadings(res.quart))
L.oblimin <- abs(loadings(res.oblimin))

ord.quart   <- order(L.quart[, 1],   decreasing = TRUE)
ord.oblimin <- order(L.oblimin[, 1], decreasing = TRUE)

par(mfrow = c(1, 2), mar = c(5, 4, 4, 2))

barplot(t(L.quart[ord.quart, ]),
        beside      = TRUE,
        ylim        = c(0, 1),
        main        = "Quartimax",
        ylab        = "Absolute loading",
        xlab        = "Variable (sorted by Factor 1)",
        legend.text = c("Factor 1", "Factor 2"),
        args.legend = list(x = "topright"),
        col         = c("steelblue", "tomato"))

barplot(t(L.oblimin[ord.oblimin, ]),
        beside      = TRUE,
        ylim        = c(0, 1),
        main        = "Oblimin",
        ylab        = "Absolute loading",
        xlab        = "Variable (sorted by Factor 1)",
        legend.text = c("Factor 1", "Factor 2"),
        args.legend = list(x = "topright"),
        col         = c("steelblue", "tomato"))
@
\end{center}

Quartimax tends to produce a general factor with high loadings on all
variables, while oblimin allows factors to be correlated and typically
produces a cleaner simple structure. The sorted absolute loading plot makes
these differences immediately visible.

\subsection*{Sorted Absolute Loadings Plot}

A useful way to compare the sparsity profile of different rotation solutions
is to plot all absolute loadings sorted from smallest to largest. In a
rotation with good simple structure, most loadings are near zero with a
sharp upturn at the right --- indicating that a few large loadings dominate.
Comparing this profile across rotation criteria makes differences in
simplicity and sparsity immediately visible.

The following function plots sorted absolute loadings for one or more
\texttt{GPArotation} objects on a single figure.

<<echo=TRUE, eval=TRUE>>=
plotSortedLoadings <- function(..., labels = NULL, col = NULL,
                                main = "Sorted Absolute Loadings",
                                ylab = "Absolute loading",
                                xlab = "Rank") {
  solutions <- list(...)

  for (i in seq_along(solutions)) {
    if (!inherits(solutions[[i]], "GPArotation"))
      stop("Argument ", i, " is not a GPArotation object.")
  }

  n <- length(solutions)

  if (is.null(labels))
    labels <- paste("Solution", seq_len(n))
  if (is.null(col))
    col <- palette.colors(n, palette = "Okabe-Ito")

  sorted_loadings <- lapply(solutions, function(x)
    sort(abs(as.vector(x$loadings)), decreasing = FALSE))

  all_values <- unlist(sorted_loadings)
  max_len    <- max(sapply(sorted_loadings, length))

  plot(NULL,
       xlim = c(1, max_len),
       ylim = c(0, max(all_values)),
       main = main,
       xlab = xlab,
       ylab = ylab,
       las  = 1)

  abline(h = seq(0, 1, by = 0.1), col = "grey90", lty = 1)

  for (i in seq_len(n)) {
    lines(seq_along(sorted_loadings[[i]]), sorted_loadings[[i]],
          col = col[i], lwd = 2)
    points(seq_along(sorted_loadings[[i]]), sorted_loadings[[i]],
           col = col[i], pch = 19, cex = 0.6)
  }

  legend("topleft", legend = labels, col = col, lwd = 2, pch = 19,
         bty = "n")

  invisible(sorted_loadings)
}

# Example
data(Harman, package = "GPArotation")
res.quart   <- quartimax(Harman8)
res.oblimin <- oblimin(Harman8)
res.geomin  <- geominT(Harman8)
@

The following example compares quartimax, oblimin, and geomin rotation
on the Harman 8-variable physical measurements dataset.
\begin{center}
<<fig=TRUE, echo=FALSE, width=4, height=4>>=
plotSortedLoadings(res.quart, res.oblimin, res.geomin,
                   labels = c("Quartimax", "Oblimin", "Geomin"))
@
\end{center}

A flat profile at the left indicates many near-zero loadings --- a
hallmark of simple structure. A gradual curve with no clear elbow suggests
the criterion is not achieving strong separation between large and small
loadings. Criteria that differ substantially in their sparsity assumptions,
such as quartimax (which tends toward a general factor) versus oblimin
(which allows correlated factors with cleaner simple structure), will
produce visibly different profiles.

The function accepts any number of \texttt{GPArotation} objects and is
useful for assessing the effect of different values of a criterion
parameter, for example comparing rotation with different values of a parameter.


%-------------------------------------------------------------------
\section*{Special Rotation Types}
%-------------------------------------------------------------------

\subsection*{An Example of Target Rotation}

\cite{fisfon} describe measuring self-reported extra-role behavior in
samples of British and East German employees. They publish rotation matrices
for two samples and investigate structural equivalence of the loadings
matrices. The table lists the varimax rotated loadings matrices.

\begin{tabular}{l c c c c}
\hline
 & \multicolumn{2}{c}{Britain} & \multicolumn{2}{c}{East Germany} \\
 & Factor 1& Factor 2 & Factor 1& Factor 2\\
 \hline\hline
I am always punctual.&.783&-.163&.778&-.066\\
I do not take extra breaks.&.811&.202&.875&.081\\
I follow work rules and instructions &.724&.209&.751&.079\\
~~~ with extreme care.& & & & \\
I never take long lunches or breaks.&.850&.064&.739&.092\\
I search for causes for something &-.031&.592&.195&.574\\
~~~ that did not function properly.& & & & \\
I often motivate others to express &-.028&.723&-.030&.807\\
~~~ their ideas and opinions.& & & & \\
During the last year I changed &.388&.434&-.135&.717\\
~~~ something in my work.& & & & \\
I encourage others to speak up at meetings.&.141&.808&.125&.738\\
I continuously try to submit suggestions&.215&.709&.060&.691\\
~~~ to improve my work.& & & & \\
\hline
\end{tabular}
\\
The varimax rotations for each sample may be expected to be similar because
the two loadings matrices are from different samples measuring the same
constructs. Below are target rotation of the East German loadings matrix
towards the British one, followed by calculation of agreement coefficients.
\cite{fisfon} note that coefficients generally should be ``beyond the
commonly accepted value of 0.90.''

<<results=verbatim>>=
origdigits <- options("digits")
options(digits = 2)
trBritain <- matrix(c(.783,-.163,.811,.202,.724,.209,.850,.064,
  -.031,.592,-.028,.723,.388,.434,.141,.808,.215,.709),
  byrow = TRUE, ncol = 2)
trGermany <- matrix(c(.778,-.066,.875,.081,.751,.079,.739,.092,
  .195,.574,-.030,.807,-.135,.717,.125,.738,.060,.691),
  byrow = TRUE, ncol = 2)
# orthogonal rotation of trGermany towards trBritain
trx <- targetT(trGermany, Target = trBritain)
# Factor loadings after target rotation
trx
# Differences between loadings matrices after rotation
y <- trx$loadings - trBritain
print(y, digits = 1)
# Square root of the mean squared difference per item
sqrt(apply((y^2), 1, mean))
# Square root of the mean squared difference per factor
sqrt(apply((y^2), 2, mean))
# Identity coefficient per factor after rotation
2 * colSums(trx$loadings * trBritain) /
  (colSums(trx$loadings^2) + colSums(trBritain^2))
# Additivity coefficient per factor after rotation
diag(2 * cov(trx$loadings, trBritain)) /
  diag(var(trx$loadings) + var(trBritain))
# Proportionality coefficient per factor after rotation
colSums(trBritain * trx$loadings) /
  sqrt(colSums(trBritain^2) * colSums(trx$loadings^2))
# Correlation for each factor after rotation
diag(cor(trBritain, trx$loadings))
options(digits = origdigits$digits)
@

The effect of target rotation can be visualized by plotting the factor
loadings for both samples in the two-dimensional factor space. The
following plot shows the British loadings (target), the German loadings
before rotation, and the German loadings after rotation towards the
British solution. Arrows connect each item's pre- and post-rotation
position, making the movement induced by the rotation immediately
visible.

\begin{center}
<<fig=TRUE>>=
plot(trBritain[, 1], trBritain[, 2],
     xlim = c(-0.3, 1.0), ylim = c(-0.3, 1.0),
     xlab = "Factor 1", ylab = "Factor 2",
     main = "Target Rotation: Germany towards Britain",
     pch = 19, col = "steelblue", cex = 1.2)
abline(h = 0, lty = 2, col = "grey70")
abline(v = 0, lty = 2, col = "grey70")
points(trGermany[, 1], trGermany[, 2],
       pch = 17, col = "tomato", cex = 1.2)
points(loadings(trx)[, 1], loadings(trx)[, 2],
       pch = 15, col = "orange", cex = 1.2)
for (i in 1:nrow(trGermany)) {
  arrows(trGermany[i, 1], trGermany[i, 2],
         loadings(trx)[i, 1], loadings(trx)[i, 2],
         length = 0.08, col = "grey60")
}
legend("topright",
       legend = c("Britain (varimax rotated)",
                  "East Germany (varimax rotated)",
                  "East Germany (rotated towards Britain)"),
       col    = c("steelblue", "tomato", "orange"),
       pch    = c(19, 17, 15),
       bty    = "n")
@
\end{center}

Items whose arrows are short moved little during rotation, indicating
that the German and British loadings were already similar for those
items. Items with longer arrows required more adjustment, suggesting
greater cultural differences in how those behaviors are structured.
After rotation, the German loadings may be compared directly to the
British target using the agreement coefficients computed above.


\subsection*{An Example of Partially Specified Target Rotation}

\cite{browne} reported an initial loadings matrix and a partially specified
target to rotate towards. In \texttt{GPArotation} the partially specified
target matrix is of the same dimension as the initial matrix \texttt{A},
with \texttt{NA} in entries that are not pre-specified. Both target rotation
and partially specified target rotation can be used to reproduce
\cite{browne} results.

In this orthogonal rotation example, \texttt{targetT} includes a
\texttt{Target} matrix with \texttt{NA} in entries not used in target
rotation. With \texttt{pstT} no missing values are present in the
\texttt{Target} matrix, and the weight matrix \texttt{W} includes weight 0
for entries not used, and 1 for entries included in the rotation.

<<results=verbatim>>=
A <- matrix(c(.664, .688, .492, .837, .705, .82, .661, .457, .765, .322,
  .248, .304, -0.291, -0.314, -0.377, .397, .294, .428, -0.075, .192, .224,
  .037, .155, -.104, .077, -.488, .009), ncol = 3)
# using targetT
SPA <- matrix(c(rep(NA, 6), .7, .0, .7, rep(0, 3), rep(NA, 7),
  0, 0, NA, 0, rep(NA, 4)), ncol = 3)
xt <- targetT(A, Target = SPA)
# using pstT
SPApst <- matrix(c(rep(0, 6), .7, .0, .7, rep(0, 3), rep(0, 7),
  0, 0, 0, 0, rep(0, 4)), ncol = 3)
SPAW <- matrix(c(rep(0, 6), rep(1, 6), rep(0, 7), 1, 1, 0, 1,
  rep(0, 4)), ncol = 3)
xpst <- pstT(A, Target = SPApst, W = SPAW)
max(abs(loadings(xt) - loadings(xpst)))
@

Note that convergence tables are identical for both methods. Additional
examples are available in the help pages of \texttt{GPFoblq} and
\texttt{rotations}.

%-------------------------------------------------------------------
\section*{Using GPArotation with factanal}
%-------------------------------------------------------------------

\subsection*{Passing Rotation to factanal}

Rotation functions can be passed directly to \texttt{factanal} via the
\texttt{rotation} argument. Additional arguments are passed through the
\texttt{control} list.

<<results=verbatim>>=
data("CCAI", package = "GPArotation")
factanal(factors = 3, covmat = CCAI_R, n.obs = 461, rotation = "infomaxT")
factanal(factors = 3, covmat = CCAI_R, n.obs = 461, rotation = "infomaxT",
  control = list(rotate = list(normalize = TRUE, eps = 1e-6)))
@

\subsection*{Two-Step Procedure (Generally Recommended)}

For oblique rotation, the recommended approach is to obtain unrotated
loadings from \texttt{factanal} and rotate separately using
\texttt{GPArotation}. This gives full control over the rotation,
including random starts, algorithm selection, and convergence diagnostics.
\texttt{GPArotation} accepts \texttt{factanal} objects directly --- no
need to extract loadings manually.

<<results=verbatim>>=
data("CCAI", package = "GPArotation")
fa.un <- factanal(factors = 3, covmat = CCAI_R, n.obs = 461,
                  rotation = "none")
oblimin(fa.un, randomStarts = 100)
@

Prior to R 4.5.1, the single-step approach (rotation inside
\texttt{factanal}) had a bug in factor reordering after oblique rotation,
reported by Bernaards and others and fixed by the R core team. The
two-step procedure has always been correct regardless of R version.
Users on R $\geq$ 4.5.1 may use either approach; the two-step procedure
is still preferred for access to random starts and convergence diagnostics.

The following example verifies agreement between the two approaches using
a non-standard Crawford-Ferguson kappa value:

<<results=verbatim>>=
data("WansbeekMeijer", package = "GPArotation")
fa.un <- factanal(factors = 3, covmat = NetherlandsTV,
                  normalize = TRUE, rotation = "none")

set.seed(42)
res.cf <- cfQ(fa.un, kappa = 0.3, normalize = TRUE, randomStarts = 100)
res.cf

if (getRversion() >= "4.5.1") {
  set.seed(42)
  fa.cf <- factanal(factors = 3, covmat = NetherlandsTV,
                    normalize = TRUE, rotation = "cfQ",
                    control = list(rotate = list(kappa = 0.3,
                                                 randomStarts = 100)))
  cat("Maximum difference in loadings:\n")
  print(max(abs(abs(res.cf$loadings) - abs(fa.cf$loadings))))
} else {
  cat("Use the two-step procedure above for correct results.\n")
}
@

%-------------------------------------------------------------------
\section*{Rotation Criteria Reference}
%-------------------------------------------------------------------

The following table lists all rotation criteria available in
\texttt{GPArotation}, with their type and key reference. Criteria ending
in \texttt{T} are orthogonal; those ending in \texttt{Q} are oblique.
Criteria without a \texttt{T}/\texttt{Q} suffix may be used for both.

\begin{tabular}{l l l}
\hline
Function & Type & Criterion \\
\hline\hline
\texttt{oblimin}   & oblique    & Oblimin family; \texttt{gam} controls obliqueness \\
\texttt{quartimin} & oblique    & Oblimin with \texttt{gam = 0} \\
\texttt{targetT}   & orthogonal & Rotation towards a target matrix \\
\texttt{targetQ}   & oblique    & Rotation towards a target matrix \\
\texttt{pstT}      & orthogonal & Partially specified target rotation \\
\texttt{pstQ}      & oblique    & Partially specified target rotation \\
\texttt{oblimax}   & oblique    & Maximizes overall kurtosis of loadings \\
\texttt{entropy}   & orthogonal & Minimizes entropy of squared loadings \\
\texttt{quartimax} & orthogonal & Maximizes variance of squared loadings within variables \\
\texttt{Varimax}   & orthogonal & Maximizes variance of squared loadings within factors \\
\texttt{simplimax} & oblique    & Minimizes the $k$ smallest squared loadings \\
\texttt{bentlerT}  & orthogonal & Invariant pattern simplicity \\
\texttt{bentlerQ}  & oblique    & Invariant pattern simplicity \\
\texttt{tandemI}   & orthogonal & Factors share high loadings on same variables \\
\texttt{tandemII}  & orthogonal & Factors do not share high loadings on same variables \\
\texttt{geominT}   & orthogonal & Minimizes geometric mean of squared loadings \\
\texttt{geominQ}   & oblique    & Minimizes geometric mean of squared loadings \\
\texttt{bigeominT} & orthogonal & Geomin with a general factor in column 1 \\
\texttt{bigeominQ} & oblique    & Geomin with a general factor in column 1 \\
\texttt{cfT}       & orthogonal & Crawford-Ferguson family; \texttt{kappa} controls complexity \\
\texttt{cfQ}       & oblique    & Crawford-Ferguson family; \texttt{kappa} controls complexity \\
\texttt{equamax}   & orthogonal & Crawford-Ferguson with $\kappa = m/(2p)$ \\
\texttt{parsimax}  & orthogonal & Crawford-Ferguson with $\kappa = (m-1)/(p+m-2)$ \\
\texttt{infomaxT}  & orthogonal & Infomax information criterion \\
\texttt{infomaxQ}  & oblique    & Infomax information criterion \\
\texttt{mccammon}  & orthogonal & Minimizes entropy ratio across factors \\
\texttt{varimin}   & orthogonal & Minimizes variance of squared loadings within factors \\
\texttt{bifactorT} & orthogonal & Bifactor; general factor in column 1 \\
\texttt{bifactorQ} & oblique    & Biquartimin; general factor in column 1 \\
\texttt{lpT}       & orthogonal & $L^p$ sparsity rotation \\
\texttt{lpQ}       & oblique    & $L^p$ sparsity rotation \\
\hline
\end{tabular}

%-------------------------------------------------------------------
\section*{Further Resources}
%-------------------------------------------------------------------

Full documentation for all functions is available via the R help system,
for example \texttt{?quartimax} or \texttt{?GPFRSorth}. The package
index is accessible via \texttt{?GPArotation}.

The following vignettes are provided with \texttt{GPArotation}:
\begin{itemize}
  \item \texttt{vignette("GPA2local", package = "GPArotation")} ---
        assessing local minima in factor rotation, including the
        \texttt{GPFallMinima} function and sorted loadings plots
  \item \texttt{vignette("GPA3bifactor", package = "GPArotation")} ---
        bifactor rotation and reliability coefficients including
        omega hierarchical
\end{itemize}

Gradient projection \emph{without} derivatives can be performed using
the \texttt{GPArotateDF} package, available separately on CRAN. A
vignette is provided with that package; type
\texttt{vignette("GPArotateDF", package = "GPArotateDF")} at the
R prompt.

For detailed investigation of local minima in factor rotation, the
following packages provide complementary functionality:
\begin{itemize}
  \item \textit{fungible}: \texttt{faMain} function with extensive
        random start diagnostics
  \item \textit{psych}: \texttt{faRotations} function for rotation
        comparison
\end{itemize}

%-------------------------------------------------------------------
\section*{References for Rotation Criteria}
%-------------------------------------------------------------------

The following references describe the theoretical basis for each rotation
criterion implemented in \texttt{GPArotation}.

\subsubsection*{Descriptions of many rotation criteria}

Browne, M.W. (2001). An overview of analytic rotation in exploratory
factor analysis. \textit{Multivariate Behavioral Research}, \textbf{36},
111--150.
\href{https://doi.org/10.1207/S15327906MBR3601_05}
{doi: 10.1207/S15327906MBR3601\_05}

\noindent Harman, H.H. (1976). \textit{Modern Factor Analysis} (3rd ed.).
The University of Chicago Press.

\subsubsection*{Oblimin / Quartimin}

Carroll, J.B. (1960). IBM 704 program for generalized analytic rotation
solution in factor analysis. Harvard University, unpublished.

\noindent Jennrich, R.I. (1979). Admissible values of $\gamma$ in direct
oblimin rotation. \textit{Psychometrika}, \textbf{44}, 173--177.
\href{https://doi.org/10.1007/BF02293969}
{doi: 10.1007/BF02293969}

\subsubsection*{Target rotation}

Harman, H.H. (1976). \textit{Modern Factor Analysis} (3rd ed.).
The University of Chicago Press.

\subsubsection*{Partially specified target rotation}

Browne, M.W. (1972a). Orthogonal rotation to a partially specified target.
\textit{British Journal of Mathematical and Statistical Psychology},
\textbf{25}, 115--120.
\href{https://doi.org/10.1111/j.2044-8317.1972.tb00482.x}
{doi: 10.1111/j.2044-8317.1972.tb00482.x}

\noindent Browne, M.W. (1972b). Oblique rotation to a partially specified
target. \textit{British Journal of Mathematical and Statistical Psychology},
\textbf{25}, 207--212.
\href{https://doi.org/10.1111/j.2044-8317.1972.tb00492.x}
{doi: 10.1111/j.2044-8317.1972.tb00492.x}

\subsubsection*{Oblimax}

Pinzka, C. and Saunders, D.R. (1954). Analytic rotation to simple
structure: II. Extension to an oblique solution. Research Bulletin 54--31.
Princeton, N.J.: Educational Testing Service.

\noindent Saunders, D.R. (1961). The rationale for an ``oblimax'' method
of transformation in factor analysis. \textit{Psychometrika}, \textbf{26},
317--324.
\href{https://doi.org/10.1007/BF02289800}
{doi: 10.1007/BF02289800}

\subsubsection*{Minimum entropy}

Jennrich, R.I. (2004). Rotation to simple loadings using component loss
functions: the orthogonal case. \textit{Psychometrika}, \textbf{69},
257--274.
\href{https://doi.org/10.1007/BF02295943}
{doi: 10.1007/BF02295943}

\subsubsection*{Quartimax}

Carroll, J.B. (1953). An analytic solution for approximating simple
structure in factor analysis. \textit{Psychometrika}, \textbf{18}, 23--38.
\href{https://doi.org/10.1007/BF02289025}
{doi: 10.1007/BF02289025}

\noindent Ferguson, G.A. (1954). The concept of parsimony in factor
analysis. \textit{Psychometrika}, \textbf{19}, 281--290.
\href{https://doi.org/10.1007/BF02289228}
{doi: 10.1007/BF02289228}

\noindent Neuhaus, J.O. and Wrigley, C. (1954). The quartimax method: An
analytical approach to orthogonal simple structure.
\textit{British Journal of Statistical Psychology}, \textbf{7}, 81--91.
\href{https://doi.org/10.1111/j.2044-8317.1954.tb00147.x}
{doi: 10.1111/j.2044-8317.1954.tb00147.x}

\subsubsection*{Varimax}

Kaiser, H.F. (1958). The varimax criterion for analytic rotation in factor
analysis. \textit{Psychometrika}, \textbf{23}, 187--200.
\href{https://doi.org/10.1007/BF02289233}
{doi: 10.1007/BF02289233}

\subsubsection*{Simplimax}

Kiers, H.A.L. (1994). SIMPLIMAX: Oblique rotation to an optimal target
with simple structure. \textit{Psychometrika}, \textbf{59}, 567--579.
\href{https://doi.org/10.1007/BF02294392}
{doi: 10.1007/BF02294392}

\subsubsection*{Bentler invariant pattern simplicity}

Bentler, P.M. (1977). Factor simplicity index and transformations.
\textit{Psychometrika}, \textbf{42}, 277--295.
\href{https://doi.org/10.1007/BF02294054}
{doi: 10.1007/BF02294054}

\subsubsection*{Tandem criteria}

Comrey, A.L. (1967). Tandem criteria for analytic rotation in factor
analysis. \textit{Psychometrika}, \textbf{32}, 277--295.
\href{https://doi.org/10.1007/BF02289422}
{doi: 10.1007/BF02289422}

\subsubsection*{Geomin}

Yates, A. (1984). \textit{Multivariate Exploratory Data Analysis: A
Perspective on Exploratory Factor Analysis}. State University of New York
Press.

\subsubsection*{Bi-Geomin}

Garcia-Garzon, E., Abad, F.J., and Garrido, L.E. (2021). On omega
hierarchical estimation: A comparison of exploratory bi-factor analysis
algorithms. \textit{Multivariate Behavioral Research}, \textbf{56}(1),
101--119.
\href{https://doi.org/10.1080/00273171.2020.1736977}
{doi: 10.1080/00273171.2020.1736977}

\subsubsection*{Crawford-Ferguson family}

Crawford, C.B. and Ferguson, G.A. (1970). A general rotation criterion
and its use in orthogonal rotation. \textit{Psychometrika}, \textbf{35},
321--332.
\href{https://doi.org/10.1007/BF02310572}
{doi: 10.1007/BF02310572}

\subsubsection*{Infomax}

McKeon, J.J. (1968). Rotation for maximum association between factors and
tests. Unpublished manuscript, Biometric Laboratory, George Washington
University.

\subsubsection*{McCammon minimum entropy ratio}

McCammon, R.B. (1966). Principal components analysis and its application
in large-scale correlation studies. \textit{Journal of Geology},
\textbf{74}, 721--733.
\href{https://doi.org/10.1086/627207}
{doi: 10.1086/627207}

\subsubsection*{Varimin}

Ertel, S. (2011). Exploratory factor analysis revealing complex structure.
\textit{Personality and Individual Differences}, \textbf{50}(2), 196--200.
\href{https://doi.org/10.1016/j.paid.2010.09.026}
{doi: 10.1016/j.paid.2010.09.026}

\subsubsection*{Bifactor}

Jennrich, R.I. and Bentler, P.M. (2011). Exploratory bi-factor analysis.
\textit{Psychometrika}, \textbf{76}(4), 537--549.
\href{https://doi.org/10.1007/s11336-011-9218-4}
{doi: 10.1007/s11336-011-9218-4}

\subsubsection*{Lp rotation}

Liu, X., Wallin, G., Chen, Y., and Moustaki, I. (2023). Rotation to sparse
loadings using $L^p$ losses and related inference problems.
\textit{Psychometrika}, \textbf{88}(2), 527--553.
\href{https://doi.org/10.1007/s11336-023-09911-y}
{doi: 10.1007/s11336-023-09911-y}

\begin{thebibliography}{}

\bibitem[\protect\citeauthoryear{Bernaards \& Jennrich}{Bernaards \&
  Jennrich}{2005}]{gpa.rotate}
Bernaards, C. A., \& Jennrich, R. I. (2005).
\newblock Gradient projection algorithms and software for arbitrary rotation
  criteria in factor analysis.
\newblock {\em Educational and Psychological Measurement}, 65(5), 676--696.
\newblock \href{https://doi.org/10.1177/0013164404272507}
  {https://doi.org/10.1177/0013164404272507}

\bibitem[\protect\citeauthoryear{Browne}{Browne}{1972}]{browne}
Browne, M. W. (1972).
\newblock Orthogonal rotation to a partially specified target.
\newblock \textit{British Journal of Mathematical and Statistical Psychology},
  25(1), 115--120.
\newblock \href{https://doi.org/10.1111/j.2044-8317.1972.tb00482.x}
  {doi: 10.1111/j.2044-8317.1972.tb00482.x}


\bibitem[\protect\citeauthoryear{Fischer \& Fontaine}{Fischer \&
  Fontaine}{2010}]{fisfon}
Fischer, R., \& Fontaine, J. (2010).
\newblock Methods for investigating structural equivalence.
\newblock In D. Matsumoto, \& F. van de Vijver (Eds.), {\em Cross-Cultural
  Research Methods in Psychology} (pp. 179--215).
\newblock Cambridge University Press.
\newblock \href{https://doi.org/10.1017/CBO9780511779381.010}
  {doi: 10.1017/CBO9780511779381.010}


\bibitem[\protect\citeauthoryear{mansreise}{Mansolf \&
  Reise}{2016}]{mansreise}
Mansolf, M. and Reise, S.P. (2016).
\newblock Exploratory bifactor analysis: The Schmid-Leiman orthogonalization
  and Jennrich-Bentler analytic rotations.
\newblock \textit{Multivariate Behavioral Research}, \textbf{51}(5), 698--717.
\newblock \href{https://doi.org/10.1080/00273171.2016.1215898}
  {doi: 10.1080/00273171.2016.1215898}


\bibitem[\protect\citeauthoryear{Nguyen \& Waller}{Nguyen \&
  Waller}{2022}]{nguwall}
Nguyen, H. V., \& Waller, N. G. (2022).
\newblock Local minima and factor rotations in exploratory factor analysis.
\newblock \textit{Psychological Methods}. Advance online publication.
\newblock \href{https://doi.org/10.1037/met0000467}
  {doi: 10.1037/met0000467}


\end{thebibliography}

\end{document}

\encoding{UTF-8}
\name{simple_structure}
\alias{calc_AUC}
\alias{calc_FSI}
\alias{calc_simplicity}
\alias{calc_hyperplane}
\alias{calc_fitstats}
\alias{simple_structure}
\title{Simple Structure Quality Measures}
\description{
  Functions for assessing the quality of simple structure in a rotated
  factor solution. Four criterion-free measures are available:
  \describe{
    \item{\code{calc_AUC}}{Area Under the Curve measure of factor loading
      simplicity, computed per factor and overall (Liu et al., 2023).}
    \item{\code{calc_FSI}}{Factor Simplicity Index, computed per factor
      and overall (Lorenzo-Seva, 2003).}
    \item{\code{calc_simplicity}}{Three overall solution measures:
      Hoffman (1978), Gini (Kaiser, 1974), and Bentler (1977).}
    \item{\code{calc_hyperplane}}{Hyperplane count per factor and overall,
      measuring coordinate sparsity in the rotated solution.}
    \item{\code{calc_fitstats}}{Chi-square, SMRS, RMSEA with confidence interval,
      valid for MLE extracted loadings only.}
  }

  All measures are criterion-free --- they do not depend on how the
  rotation was obtained --- making them useful for comparing simple
  structure quality across different rotation methods applied to the
  same unrotated solution. The table below summarizes the measures,
  their scope, range, and interpretation:

  \tabular{lllll}{
    \strong{Measure} \tab \strong{Scope} \tab \strong{Range} \tab
      \strong{Good} \tab \strong{Poor} \cr
    AUC        \tab per factor         \tab [0.5, 1.0]    \tab near 1.0      \tab near 0.5 \cr
    FSI        \tab per factor         \tab [0.0, 1.0]    \tab near 1.0      \tab near 0.0 \cr
    Hoffman    \tab overall solution   \tab [0.0, 1.0]    \tab near 1.0      \tab near 0.0 \cr
    Gini       \tab overall solution   \tab [0.0, 1.0]    \tab near 1.0      \tab near 0.0 \cr
    Bentler    \tab overall solution   \tab [0.0, 1.0]    \tab near 1.0      \tab near 0.0 \cr
    Hyperplane \tab per factor; overall \tab [0, p(k-1)] \tab high count    \tab low count \cr
  }

  \strong{Note:} No universally accepted cutoffs exist for these measures.
  The ranges above are approximate guidelines based on empirical experience.
  Always interpret in conjunction with substantive theory and model fit
  indices such as RMSEA and SRMR.

  \strong{Note on Bentler:} the Bentler index tends to be low when
  factor intercorrelations are high, even when simple structure is
  otherwise clean. A low Bentler value should therefore be interpreted
  in conjunction with AUC, FSI, and hyperplane counts rather than in
  isolation. High AUC and hyperplane counts alongside a low Bentler
  value typically indicate correlated factors rather than poor simple
  structure.

  These functions are used internally by \code{\link{print.GPArotation}}
  and \code{\link{summary.GPArotation}} and are displayed automatically
  when printing or summarising a \code{GPArotation} object. They can also
  be called directly using the triple-colon operator, for example
  \code{GPArotation:::calc_AUC(x)}.
}
\usage{
## Not exported -- access via GPArotation:::calc_AUC(x) etc.
## calc_AUC(x, digits = 3L)
## calc_FSI(x, digits = 3L)
## calc_simplicity(x, digits = 3L)
## calc_hyperplane(x, cutoff = 0.1, digits = 3L)
## calc_fitstats(x)
}
\arguments{
  \item{x}{a \code{GPArotation} object, a \code{factanal} object, or a
    factor loading matrix. For oblique rotations all measures are computed
    from the pattern matrix, not the structure matrix.}
  \item{digits}{integer. Number of decimal places for rounding. Default 3.}
  \item{cutoff}{numeric. Half-width of the hyperplane band for
    \code{calc_hyperplane}. Loadings with absolute value below
    \code{cutoff} are counted as hyperplane loadings. Default 0.1.
    Common choices are 0.1 (strict) and 0.15 (lenient). Results should
    always be reported with the cutoff value used.}
}
\details{
  All four measures are criterion-free --- they do not depend on how the
  rotation was obtained --- making them useful for comparing simple
  structure quality across different rotation methods applied to the same
  unrotated solution.

  \subsection{AUC (Liu et al., 2023)}{
    For each factor \eqn{j} the squared loadings are sorted in descending
    order and their cumulative proportions of total factor variance are
    plotted against their rank. AUC is the area under this curve:
    \deqn{AUC_j = \frac{1}{p} \sum_{i=1}^{p}
          \frac{\sum_{r=1}^{i} l_{(r)j}^2}{\sum_{r=1}^{p} l_{(r)j}^2}}
    \eqn{AUC = 1} indicates perfect simple structure (one non-zero
    loading); \eqn{AUC = 0.5} indicates no simple structure (equal
    loadings). Adjusted AUC subtracts 0.5 so that 0 corresponds to
    no simple structure.
  }

  \subsection{FSI (Lorenzo-Seva, 2003)}{
    Based on the kurtosis of the squared loadings within each factor:
    \deqn{FSI_j = \frac{p \sum l_{ij}^4 - \left(\sum l_{ij}^2\right)^2}
                 {(p-1)\left(\sum l_{ij}^2\right)^2}}
    \eqn{FSI = 0} indicates no simple structure; \eqn{FSI = 1} indicates
    perfect simple structure. More sensitive to dominant loadings than AUC.
  }

  \subsection{Overall solution measures}{
    \strong{Hoffman (1978):}
    \deqn{H = 1 - \frac{\sum_j \sum_i l_{ij}^2 (1 - l_{ij}^2)}{p \cdot k \cdot 0.25}}
    Measures how far loadings are from 0.5. \eqn{H = 1} when all loadings
    are 0 or 1; \eqn{H = 0.25} when all loadings equal 0.5.

    \strong{Gini (Kaiser, 1974):}
    Based on the Gini coefficient of the absolute loadings within each
    factor, averaged across factors. Higher values indicate greater
    concentration of variance on fewer indicators per factor.

    \strong{Bentler (1977):}
    Ratio of between-factor to within-factor variance of squared loadings.
    Values near 1 indicate clean simple structure.
  }

  \subsection{Hyperplane count (Mair, 2018)}{
    For each factor \eqn{j}:
    \deqn{HP_j = \sum_{i=1}^{p} \mathbf{1}(|l_{ij}| < c)}
    where \eqn{c} is the hyperplane cutoff. The theoretical maximum is
    \eqn{p(k-1)}, achieved when each indicator loads on exactly one factor.
    The percentage of the theoretical maximum provides a scale-free summary
    of hyperplane density.
  }

  AUC and FSI are per-factor measures shown in \code{print.GPArotation}.
  All four measures plus their summaries are shown in
  \code{summary.GPArotation}.
}
\examples{
data("CCAI", package = "GPArotation")
fa.un <- factanal(factors = 3, covmat = CCAI_R, n.obs = 461,
                  rotation = "none")

res.tan <- tandemI(fa.un)
res.ob  <- oblimin(fa.un)

# Simple structure measures shown automatically in print
# Orthogonal: SS loadings, Proportion Var, Cumulative Var, AUC, FSI
print(res.tan)

# Oblique: SS loadings, AUC, FSI, Phi
print(res.ob)

# Summary adds overall measures: Hoffman, Gini, Bentler, hyperplane
summary(res.tan)
summary(res.ob)

# Direct access for custom analyses
GPArotation:::calc_AUC(res.ob)
GPArotation:::calc_FSI(res.ob)
GPArotation:::calc_simplicity(res.ob)
GPArotation:::calc_hyperplane(res.ob)

# Stricter hyperplane cutoff
GPArotation:::calc_hyperplane(res.ob, cutoff = 0.05)

# Compare simple structure across rotation methods
auc.tan <- GPArotation:::calc_AUC(res.tan)$AUC_mean
auc.ob  <- GPArotation:::calc_AUC(res.ob)$AUC_mean
fsi.tan <- GPArotation:::calc_FSI(res.tan)$FSI_mean
fsi.ob  <- GPArotation:::calc_FSI(res.ob)$FSI_mean

cat(paste0(formatC("Tandem I", width = 10, flag = "-"),
           "  AUC: ", round(auc.tan, 3),
           "  FSI: ", round(fsi.tan, 3), "\n"))
cat(paste0(formatC("Oblimin", width = 10, flag = "-"),
           "  AUC: ", round(auc.ob, 3),
           "  FSI: ", round(fsi.ob, 3), "\n"))           
}
\seealso{
  \code{\link{print.GPArotation}},
  \code{\link{summary.GPArotation}}
}
\references{
  Bentler, P.M. (1977) Factor simplicity index and transformations.
    \emph{Psychometrika}, \bold{42}(2), 277--295.
    \doi{10.1007/BF02294054}

  Hoffman, P.J. (1978) The paramorphic representation of clinical judgment.
    \emph{Psychological Bulletin}, \bold{55}(2), 116--131.
    \doi{10.1037/h0047807}

  Kaiser, H.F. (1974) An index of factorial simplicity.
    \emph{Psychometrika}, \bold{39}(1), 31--36.
    \doi{10.1007/BF02291575}

  Liu, X., Wallin, G., Chen, Y., and Moustaki, I. (2023). Rotation to
    sparse loadings using \eqn{L^p}{Lp} losses and related inference
    problems. \emph{Psychometrika}, \bold{88}(2), 527--553.
    \doi{10.1007/s11336-023-09911-y}

  Lorenzo-Seva, U. (2003) A factor simplicity index.
    \emph{Psychometrika}, \bold{68}(1), 49--60.
    \doi{10.1007/BF02296652}

  Mair, P. (2018). \emph{Modern Psychometrics with R}.
    Springer. \doi{10.1007/978-3-319-93177-7}
}
\author{Coen A. Bernaards.}
\keyword{internal}
\concept{rotation}
\keyword{multivariate}

\encoding{UTF-8}
\name{Harman}
\docType{data}
\alias{Harman}
\alias{Harman8}
\title{Harman's Eight Physical Variables}
\description{
  Unrotated factor loading matrix extracted by the centroid method
  for Harman's 8 physical variables, a classic dataset in factor analysis.
}
\usage{
  data(Harman)
}
\details{
  \code{Harman8} is a \eqn{8 \times 2}{8 x 2} matrix of unrotated
  factor loadings extracted by the centroid method (Table 14.3 in
  Harman, 1976) from the correlation matrix of 8 physical measurements
  taken on 305 girls (Table 2.3 in Harman, 1976). The centroid method
  was a standard extraction procedure before iterative methods became
  computationally feasible. The two columns represent unrotated centroid
  factors (\code{CF1} and \code{CF2}). Row names are the 8 variable
  names.

  The covariance matrix for these 8 physical variables is available
  in base R (see \code{\link[datasets]{Harman23.cor}}).
}
\format{
  \code{Harman8}: a \eqn{8 \times 2}{8 x 2} numeric matrix of
  unrotated centroid factor loadings. Row names are variable
  names; column names are \code{CF1} and \code{CF2}.
}
\source{
  Harman, H.H. (1976). \emph{Modern Factor Analysis} (3rd ed.).
  University of Chicago Press.
}
\seealso{
  \code{\link[datasets]{Harman23.cor}},
  \code{\link{GPForth}},
  \code{\link{rotations}},
  \code{\link{Thurstone}},
  \code{\link{WansbeekMeijer}},
  \code{\link{CCAI}}
}
\examples{
  data(Harman, package = "GPArotation")

  # Centroid unrotated loadings from Harman (1976) Table 14.3
  print(Harman8)

  # Rotate the centroid solution
  quartimax(Harman8)
  oblimin(Harman8, randomStarts = 100)
}
\keyword{datasets}

\name{00.GPArotation.Intro}
\alias{00.GPArotation.Intro}
\docType{package}

\title{GPA Rotation for Factor Analysis}

\description{See \code{\link{GPArotation-package}} ( in the help system use
   package?GPArotation or ?"GPArotation-package") for an overview.
}
\keyword{package}

\encoding{UTF-8}
\name{echelon}
\alias{echelon}
\title{Echelon Rotation}
\usage{
    echelon(L, reference = seq(NCOL(L)), ...)
}
\arguments{
    \item{L}{a factor loading matrix.}
    \item{reference}{integer vector indicating which rows of the loading
         matrix are used to determine the rotation transformation.
         Default uses the first \eqn{k}{k} rows. If the submatrix
         indicated by \code{reference} is singular, a different choice
         of rows must be supplied.}
    \item{...}{additional arguments discarded.}
}
\value{
  A \code{GPArotation} object which is a list with elements:
  \item{loadings}{The rotated loadings matrix.}
  \item{Th}{The rotation matrix.}
  \item{method}{A string indicating the rotation method (\code{"echelon"}).}
  \item{orthogonal}{Always \code{TRUE} (an orthogonal solution is assumed;
       \eqn{\Phi}{Phi} is the identity matrix).}
  \item{convergence}{Always \code{TRUE} (the optimization is not iterative).}
}
\description{
  Rotates a factor loading matrix to an echelon parameterization.
}
\details{
  The loading matrix is rotated so that the \eqn{k}{k} rows indicated by
  \code{reference} form the Cholesky factorization given by
  \code{t(chol(L[reference,] \%*\% t(L[reference,])))}.
  This defines the rotation transformation, which is then applied to all
  rows to give the rotated loading matrix.

  The optimization is not iterative and does not use the gradient projection
  algorithm. The function can be used directly or passed to factor analysis
  functions like \code{\link[stats]{factanal}} via the \code{rotation}
  argument.

  This parameterization has several useful properties:

  \enumerate{
    \item It can be useful for comparison with published results in this
          parameterization.
    \item Standard errors are more straightforward to compute because the
          solution corresponds to an unconstrained optimization.
    \item Models with \eqn{k}{k} and \eqn{k+1}{k+1} factors are nested,
          making it straightforward to test the \eqn{k}{k}-factor model
          versus the \eqn{(k+1)}{k+1}-factor model. In particular, the
          Wald test and LM test can be used in addition to the LR test.
          The test of a \eqn{k}{k}-factor model versus a
          \eqn{(k+1)}{k+1}-factor model is a joint test of whether all
          free parameters (loadings) in the \eqn{(k+1)}{k+1}st column
          are zero.
    \item For some purposes, only the subspace spanned by the factors
          matters, not the specific parameterization within this subspace.
    \item Back-predicted indicators (the explained portion of the
          indicators) do not depend on the rotation method. Combined with
          the greater ease of obtaining correct standard errors, this
          allows easier and more accurate prediction standard errors.
    \item This parameterization and its standard errors can be used to
          detect identification problems (McDonald, 1999, pp. 181--182).
  }

  One use of echelon rotation is obtaining good starting values for
  subsequent rotation, though it may seem counterintuitive to rotate
  towards this solution afterwards.
}
\examples{
  data("WansbeekMeijer", package = "GPArotation")
  fa.unrotated <- factanal(factors = 2, covmat = NetherlandsTV,
                           rotation = "none")

  # Direct call
  fa.ech <- echelon(fa.unrotated$loadings)

  # Equivalent via factanal rotation argument
  fa.ech2 <- factanal(factors = 2, covmat = NetherlandsTV,
                      rotation = "echelon")

  # Compare unrotated, echelon, and factanal echelon loadings
  cbind(loadings(fa.unrotated), loadings(fa.ech), loadings(fa.ech2))

  # Echelon rotation with a different reference set
  fa.ech3 <- echelon(fa.unrotated$loadings, reference = 6:7)
  cbind(loadings(fa.unrotated), loadings(fa.ech), loadings(fa.ech3))
}
\seealso{
  \code{\link{eiv}},
  \code{\link{rotations}},
  \code{\link{GPForth}},
  \code{\link{GPFoblq}}
}
\references{
  McDonald, R.P. (1999). \emph{Test Theory: A Unified Treatment}.
    Erlbaum.

  Wansbeek, T. and Meijer, E. (2000). \emph{Measurement Error and
    Latent Variables in Econometrics}. North-Holland.
}
\author{Erik Meijer and Paul Gilbert.}
\concept{rotation}
\keyword{multivariate}

\encoding{UTF-8}
\name{eiv}
\alias{eiv}

\title{Errors-in-Variables Rotation}
\usage{
    eiv(L, identity = seq(NCOL(L)), ...)
}
\arguments{
    \item{L}{a factor loading matrix.}
    \item{identity}{integer vector indicating which rows of the loading
         matrix should form an identity matrix. Default uses the first
         \eqn{k}{k} rows. If inverting the submatrix indicated by
         \code{identity} fails, a different choice of rows must be
         supplied.}
    \item{...}{additional arguments discarded.}
}
\value{
  A \code{GPArotation} object which is a list with elements:
  \item{loadings}{The rotated loadings matrix.}
  \item{Th}{The rotation matrix.}
  \item{method}{A string indicating the rotation method (\code{"eiv"}).}
  \item{orthogonal}{Always \code{FALSE} (oblique rotation).}
  \item{convergence}{Always \code{TRUE} (the optimization is not iterative).}
  \item{Phi}{The covariance matrix of the rotated factors.}
}
\description{
  Rotates a factor loading matrix to an errors-in-variables representation.
}
\details{
  The loading matrix is rotated so that the \eqn{k}{k} rows indicated by
  \code{identity} form an identity matrix, with the remaining
  \eqn{M-k}{M-k} rows as free parameters. \eqn{\Phi}{Phi} is also free.

  The optimization is not iterative and does not use the gradient projection
  algorithm. The function can be used directly or passed to factor analysis
  functions like \code{\link[stats]{factanal}} via the \code{rotation}
  argument.

  Viewed as a rotation method it is oblique, with an explicit solution.
  Given an initial loadings matrix \eqn{L}{L} partitioned as
  \eqn{L = (L_1^T, L_2^T)^T}{L = rbind(L1, L2)}, the rotated loadings
  matrix is \eqn{(I, (L_2 L_1^{-1})^T)^T}{rbind(I, L2 \%*\% solve(L1))}
  and \eqn{\Phi = L_1 L_1^T}{Phi = L1 \%*\% t(L1)}, where \eqn{I}{I} is
  the \eqn{k \times k}{k x k} identity matrix. It is assumed that
  \eqn{\Phi = I}{Phi = I} for the initial loadings matrix.

  Not all authors consider this representation to be a rotation in the
  strict sense.

  This parameterization has several useful properties:

  \enumerate{
    \item It can be useful for comparison with published results in this
          parameterization.
    \item Standard errors are more straightforward to compute because the
          solution corresponds to an unconstrained optimization.
    \item One may have prior knowledge about which reference variables load
          on only one factor without imposing restrictive constraints on
          other loadings --- in this sense it has similarities to CFA.
    \item For some purposes, only the subspace spanned by the factors
          matters, not the specific parameterization within this subspace.
    \item Back-predicted indicators (the explained portion of the
          indicators) do not depend on the rotation method. Combined with
          the greater ease of obtaining correct standard errors, this
          allows easier and more accurate prediction standard errors.
  }

  One use of this parameterization is obtaining good starting values for
  subsequent rotation, though it may seem counterintuitive to rotate
  towards this solution afterwards.
}
\examples{
  data("WansbeekMeijer", package = "GPArotation")
  fa.unrotated <- factanal(factors = 2, covmat = NetherlandsTV,
                           rotation = "none")

  # Direct call
  fa.eiv <- eiv(fa.unrotated$loadings)

  # Equivalent via factanal rotation argument
  fa.eiv2 <- factanal(factors = 2, covmat = NetherlandsTV,
                      rotation = "eiv")

  # Compare unrotated, eiv, and factanal eiv loadings
  cbind(loadings(fa.unrotated), loadings(fa.eiv), loadings(fa.eiv2))

  # Eiv rotation with a different identity set
  fa.eiv3 <- eiv(fa.unrotated$loadings, identity = 6:7)
  cbind(loadings(fa.unrotated), loadings(fa.eiv), loadings(fa.eiv3))
}
\seealso{
  \code{\link{echelon}},
  \code{\link{rotations}},
  \code{\link{GPForth}},
  \code{\link{GPFoblq}}
}
\references{

  \enc{Hägglund}{Haggland}, G. (1982). Factor analysis by instrumental
    variables methods. \emph{Psychometrika}, \bold{47}, 209--222.

  Lewin-Koh, S.C. and Amemiya, Y. (2003). Heteroscedastic factor analysis.
    \emph{Biometrika}, \bold{90}, 85--97.

  Wansbeek, T. and Meijer, E. (2000). \emph{Measurement Error and
    Latent Variables in Econometrics}. North-Holland.
      }

\author{Erik Meijer and Paul Gilbert.}
\concept{rotation}
\keyword{multivariate}

\name{GPArotation-package}
\alias{GPArotation-package}
\alias{GPArotation.Intro}
\docType{package}

\title{GPA Rotation for Factor Analysis}

\description{GPArotation implements Gradient Projection Algorithms and several
rotation objective functions for factor analysis.
}

\details{
\tabular{ll}{
Package: \tab GPArotation\cr
Depends: \tab R (>= 2.0.0)\cr
License: \tab GPL Version 2.\cr
URL: \tab https://optimizer.r-forge.r-project.org/GPArotation_www/\cr}

The main optimization functions are
\code{\link{GPForth}} and \code{\link{GPFoblq}}. Rotation objectives include
\code{\link{oblimin}} and many others.
}

\author{Coen A. Bernaards and Robert I. Jennrich
        with some R modifications by Paul Gilbert.

Code is modified from original source \file{splusfunctions.net} available at 
  \url{https://optimizer.r-forge.r-project.org/GPArotation_www/}.

}
\references{
   The software reference is

   Bernaards, C.A. and Jennrich, R.I. (2005) Gradient Projection Algorithms 
     and Software for Arbitrary Rotation Criteria in Factor Analysis. 
     \emph{Educational and Psychological Measurement}, \bold{65}, 676--696.

   Theory of gradient projection algorithms may be found in:

   Jennrich, R.I. (2001). A simple general procedure for orthogonal rotation.
       \emph{Psychometrika}, \bold{66}, 289--306.

   Jennrich, R.I. (2002). A simple general method for oblique rotation. 
       \emph{Psychometrika}, \bold{67}, 7--19.
}
\keyword{ package }

\seealso{
   \code{\link{rotations}},
   \code{\link{GPForth}},
   \code{\link{GPFoblq}},
   \code{\link[stats]{factanal}} 
}

\name{plot2fOrthComparison}
\alias{plot2fOrthComparison}
\title{Algorithm Comparison for Two-Factor Orthogonal Rotation}
\description{
  Produces a \eqn{2 \times 3} panel figure comparing the \code{"legacy"},
  \code{"bb"}, and \code{"cayley"} optimization algorithms on a two-factor
  orthogonal rotation. The top row shows the criterion landscape and the
  algorithm trajectory for each algorithm; the bottom row shows the
  corresponding factor migration paths. An optional endgame inset zooms
  into the convergence region near the global minimum.
}
\usage{
plot2fOrthComparison(A, method = "quartimax", items = NULL,
                     nang = 6000, maxit = 500, magnify = FALSE, 
                     inset_pos = "auto", randomStart = FALSE, \dots)
}
\arguments{
  \item{A}{An unrotated loading matrix with exactly two columns, or a
    \code{factanal} object.}
  \item{method}{Character string naming the rotation criterion. Must be
    an orthogonal criterion available in \pkg{GPArotation}, for example
    \code{"quartimax"}, \code{"Varimax"}, or \code{"geominT"}.
    Default is \code{"quartimax"}.}
  \item{items}{Integer vector specifying which variables to include in
    the factor migration path plots. Default \code{NULL} includes all
    variables.}
  \item{nang}{Number of angles at which to evaluate the criterion landscape.
    Default \code{6000}.}
  \item{maxit}{Maximum number of iterations for each algorithm.
    Default \code{500}.}
  \item{magnify}{Logical. If \code{TRUE}, an endgame inset is added to
    each landscape panel, zooming into the convergence region near the
    global minimum. Default \code{FALSE}.}
  \item{inset_pos}{Placement of the inset. Options are "topleft", 
    "bottomleft", and the default = "auto"}
  \item{randomStart}{Initiate with random start matrix. Default = FALSE}
  \item{\dots}{Additional arguments passed to the rotation function,
    for example \code{normalize = TRUE}.}
}
\value{
  Invisibly returns a list with components \code{legacy}, \code{bb},
  and \code{cayley}, each a \code{"GPArotation"} object from the
  corresponding algorithm run.
}
\note{
  This function requires a two-factor loading matrix. The criterion
  landscape visualization is only defined for two-factor orthogonal
  rotations, where all rotation matrices are parameterized by a single
  angle \eqn{\theta \in [0, 2\pi)}.
}
\seealso{
  \code{\link{plot.GPArotation}}, \code{\link{GPForth}},
  \code{\link{rotations}}
}
\examples{
data("Harman", package = "GPArotation")
plot2fOrthComparison(Harman8, method = "quartimax", magnify = TRUE)

plot2fOrthComparison(Harman8, method = "tandemII", magnify = FALSE)
}
\keyword{hplot}

\encoding{UTF-8}
\name{rotations}
\alias{rotations}
\alias{oblimin}
\alias{quartimin}
\alias{targetT}
\alias{targetQ}
\alias{pstT}
\alias{pstQ}
\alias{oblimax}
\alias{entropy}
\alias{quartimax}
\alias{Varimax}
\alias{simplimax}
\alias{bentlerT}
\alias{bentlerQ}
\alias{tandemI}
\alias{tandemII}
\alias{geominT}
\alias{geominQ}
\alias{bigeominT}
\alias{bigeominQ}
\alias{cfT}
\alias{cfQ}
\alias{equamax}
\alias{parsimax}
\alias{infomaxT}
\alias{infomaxQ}
\alias{mccammon}
\alias{varimin}
\alias{bifactorT}
\alias{bifactorQ}
\alias{lpT}
\alias{lpQ}

\title{Rotations Functions Using Gradient Projection Algorithms}
\usage{
    oblimin(A, Tmat=diag(ncol(A)), gam=0, normalize=FALSE, randomStarts=0, ...)
    quartimin(A, Tmat=diag(ncol(A)), normalize=FALSE, randomStarts=0, ...)
    targetT(A=NULL, Tmat=diag(ncol(A)), Target=NULL, normalize=FALSE, eps=1e-5, 
        maxit=1000, randomStarts=0, L=NULL, ...)
    targetQ(A=NULL, Tmat=diag(ncol(A)), Target=NULL, normalize=FALSE, eps=1e-5, 
        maxit=1000, randomStarts=0, L=NULL, ...)
    pstT(A=NULL, Tmat=diag(ncol(A)), W=NULL, Target=NULL, normalize=FALSE, eps=1e-5, 
        maxit=1000, randomStarts=0, L=NULL, ...)
    pstQ(A=NULL, Tmat=diag(ncol(A)), W=NULL, Target=NULL, normalize=FALSE, eps=1e-5, 
        maxit=1000, randomStarts=0, L=NULL, ...)
    oblimax(A, Tmat=diag(ncol(A)), normalize=FALSE, randomStarts=0, ...)
    entropy(A, Tmat=diag(ncol(A)), normalize=FALSE, randomStarts=0, ...)
    quartimax(A, Tmat=diag(ncol(A)), normalize=FALSE, randomStarts=0, ...)
    Varimax(A, Tmat=diag(ncol(A)), normalize=FALSE, randomStarts=0, ...)
    simplimax(A, Tmat=diag(ncol(A)), k=nrow(A), normalize=FALSE, randomStarts=0, ...)
    bentlerT(A, Tmat=diag(ncol(A)), normalize=FALSE, randomStarts=0, ...)
    bentlerQ(A, Tmat=diag(ncol(A)), normalize=FALSE, randomStarts=0, ...)
    tandemI(A, Tmat=diag(ncol(A)), normalize=FALSE, randomStarts=0, ...)
    tandemII(A, Tmat=diag(ncol(A)), normalize=FALSE, randomStarts=0, ...)
    geominT(A, Tmat=diag(ncol(A)), delta=0.01, normalize=FALSE, randomStarts=0, ...)
    geominQ(A, Tmat=diag(ncol(A)), delta=0.01, normalize=FALSE, randomStarts=0, ...)
    bigeominT(A, Tmat=diag(ncol(A)), delta=0.01, normalize=FALSE, randomStarts=0, ...)
    bigeominQ(A, Tmat=diag(ncol(A)), delta=0.01, normalize=FALSE, randomStarts=0, ...)
    cfT(A, Tmat=diag(ncol(A)), kappa=0, normalize=FALSE, randomStarts=0, ...)
    cfQ(A, Tmat=diag(ncol(A)), kappa=0, normalize=FALSE, randomStarts=0, ...)
    equamax(A, Tmat=diag(ncol(A)), kappa=ncol(A)/(2*nrow(A)), normalize=FALSE,
        randomStarts=0, ...)
    parsimax(A, Tmat=diag(ncol(A)), kappa=(ncol(A)-1)/(ncol(A)+nrow(A)-2), 
        normalize=FALSE, randomStarts=0, ...)
    infomaxT(A, Tmat=diag(ncol(A)), normalize=FALSE, randomStarts=0, ...)
    infomaxQ(A, Tmat=diag(ncol(A)), normalize=FALSE, randomStarts=0, ...)
    mccammon(A, Tmat=diag(ncol(A)), normalize=FALSE, randomStarts=0, ...)
    varimin(A, Tmat=diag(ncol(A)), normalize=FALSE, randomStarts=0, ...)
    bifactorT(A, Tmat=diag(ncol(A)), normalize=FALSE, randomStarts=0, ...)
    bifactorQ(A, Tmat=diag(ncol(A)), normalize=FALSE, randomStarts=0, ...)
    lpT(A, Tmat=diag(ncol(A)), p=1, normalize=FALSE, eps=1e-05, maxit=1000, 
            randomStarts=0, gpaiter=5) 
    lpQ(A, Tmat=diag(ncol(A)), p=1, normalize=FALSE, eps=1e-05, maxit=1000, 
        randomStarts=0, gpaiter=5)
    }
    
\arguments{
    \item{A}{an initial loadings matrix to be rotated.}
    \item{Tmat}{initial rotation matrix.}
    \item{gam}{Obliqueness parameter (\eqn{\gamma} in the mathematical definition).
          0=Quartimin, .5=Biquartimin, 1=Covarimin.}
    \item{Target}{rotation target for objective calculation.}
    \item{W}{weighting of each element in target.}
    \item{k}{number of close to zero loadings.}
    \item{delta}{constant added to \eqn{\Lambda^2}{Lambda^2} in the objective calculation.}
    \item{kappa}{see details.}
    \item{normalize}{parameter passed to optimization routine (GPForth or GPFoblq).}
    \item{eps}{convergence tolerance passed to \code{GPForth} or \code{GPFoblq} via \code{...}.
         Convergence is assumed when the norm of the gradient is smaller than \code{eps}.
         Default is \code{1e-5}.}
    \item{maxit}{maximum number of iterations passed to \code{GPForth} or \code{GPFoblq} 
         via \code{...}. Default is \code{1000}.}
    \item{...}{additional arguments passed to \code{GPForth} or \code{GPFoblq}, 
         including \code{eps} and \code{maxit}.}
    \item{randomStarts}{parameter passed to optimization routine (GPFRSorth or GPFRSoblq).}
    \item{L}{provided for backward compatibility in target rotations only. Use A going forward.}
    \item{p}{Component-wise \eqn{L^p}{Lp}, where 0 < p \eqn{=<}{=<} 1. }
	  \item{gpaiter}{Maximum iterations for GPA rotation loop in \eqn{L^p}{Lp} rotation.}
}

\value{
  A \code{GPArotation} object which is a list with elements:
  \item{loadings}{The rotated loadings matrix, one column per factor.
       If random starts were requested, this is the solution with the
       lowest criterion value.}
  \item{Th}{The rotation matrix, satisfying
       \code{loadings \%*\% t(Th) = A} for orthogonal rotation and
       \code{loadings = A \%*\% solve(t(Th))} for oblique rotation.}
  \item{Table}{A matrix recording the iteration history: iteration
       number, criterion value, log10 of the gradient norm, and step
       size (alpha).}
  \item{method}{A string indicating the rotation criterion.}
  \item{orthogonal}{A logical indicating if the rotation is orthogonal.}
  \item{convergence}{A logical indicating if convergence was obtained.}
  \item{Phi}{\code{t(Th) \%*\% Th}, the covariance matrix of the rotated
       factors. Omitted (\code{NULL}) for orthogonal rotations.}
  \item{Gq}{The gradient of the criterion at the rotated loadings.}
  \item{randStartChar}{A named vector summarising random start results:
       \code{randomStarts}, \code{Converged}, \code{atMinimum},
       \code{localMins}. Only present when \code{randomStarts > 1}.}
}
\description{
   Optimize factor loading rotation objective.
}

\details{
These functions optimize a rotation objective. They can be used directly or the
function name can be passed to factor analysis functions like \code{factanal}.
Several of the function names end in T or Q, which indicates if they are
orthogonal or oblique rotations (using \code{GPFRSorth} or \code{GPFRSoblq}
respectively). The gradient projection algorithms are described in
Bernaards and Jennrich (2005).
    \tabular{lll}{
 \code{oblimin}   \tab oblique    \tab oblimin family; \code{gam} controls obliqueness \cr
 \code{quartimin} \tab oblique    \tab oblimin with \code{gam = 0} \cr
 \code{targetT}   \tab orthogonal \tab rotation towards a target matrix \cr
 \code{targetQ}   \tab oblique    \tab rotation towards a target matrix \cr
 \code{pstT}      \tab orthogonal \tab partially specified target rotation \cr
 \code{pstQ}      \tab oblique    \tab partially specified target rotation \cr
 \code{oblimax}   \tab oblique    \tab maximizes overall kurtosis of loadings \cr
 \code{entropy}   \tab orthogonal \tab minimizes entropy of squared loadings \cr
 \code{quartimax} \tab orthogonal \tab maximizes variance of squared loadings within variables \cr
 \code{Varimax}   \tab orthogonal \tab maximizes variance of squared loadings within factors \cr
 \code{simplimax} \tab oblique    \tab minimizes the k smallest squared loadings \cr
 \code{bentlerT}  \tab orthogonal \tab invariant pattern simplicity \cr
 \code{bentlerQ}  \tab oblique    \tab invariant pattern simplicity \cr
 \code{tandemI}   \tab orthogonal \tab factors share high loadings on same variables \cr
 \code{tandemII}  \tab orthogonal \tab factors do not share high loadings on same variables \cr
 \code{geominT}   \tab orthogonal \tab minimizes geometric mean of squared loadings \cr
 \code{geominQ}   \tab oblique    \tab minimizes geometric mean of squared loadings \cr
 \code{bigeominT} \tab orthogonal \tab geomin with a general factor in column 1 \cr
 \code{bigeominQ} \tab oblique    \tab geomin with a general factor in column 1 \cr
 \code{cfT}       \tab orthogonal \tab Crawford-Ferguson family; \code{kappa} controls complexity \cr
 \code{cfQ}       \tab oblique    \tab Crawford-Ferguson family; \code{kappa} controls complexity \cr
 \code{equamax}   \tab orthogonal \tab Crawford-Ferguson with \code{kappa = m/(2p)} \cr
 \code{parsimax}  \tab orthogonal \tab Crawford-Ferguson with \code{kappa = (m-1)/(p+m-2)} \cr
 \code{infomaxT}  \tab orthogonal \tab infomax information criterion \cr
 \code{infomaxQ}  \tab oblique    \tab infomax information criterion \cr
 \code{mccammon}  \tab orthogonal \tab minimizes entropy ratio across factors \cr
 \code{varimin}   \tab orthogonal \tab minimizes variance of squared loadings within factors \cr
 \code{bifactorT} \tab orthogonal \tab bifactor; general factor in column 1 \cr
 \code{bifactorQ} \tab oblique    \tab biquartimin; general factor in column 1 \cr
 \code{lpT}       \tab orthogonal \tab \eqn{L^p}{Lp} sparsity rotation \cr
 \code{lpQ}       \tab oblique    \tab \eqn{L^p}{Lp} sparsity rotation \cr
 }
 
 The \code{Varimax} implementation in the list uses the gradient projection algorithm 
 applied to \code{vgQ.varimax}. This implementation is different that the 
 \code{varimax} rotation defined in the \code{stats} package. Additionally, 
 \code{varimax} does Kaiser normalization by default whereas 
 \code{GPArotation::Varimax} does not.
 
 The argument \code{kappa} parameterizes the family for the Crawford-Ferguson 
 method. If \code{m} is the number of factors and \code{p} is the number of 
 indicators then \code{kappa} values having special names are \eqn{0=}{0=}Quartimax, 
 \eqn{1/p=}{1/p=}Varimax, \eqn{m/(2*p)=}{m/(2*p)=}Equamax, 
 \eqn{(m-1)/(p+m-2)=}{(m-1)/(p+m-2)=}Parsimax, \eqn{1=}{1=}Factor parsimony.
 
 Bifactor rotations, bifactorT and bifactorQ are called bifactor and
 biquartimin in Jennrich and Bentler (2011). For a
  comparison of exploratory bifactor analysis algorithms including
  those implemented here, see Garcia-Garzon, Abad and Garrido (2021).
      
 The argument \code{p} is needed for \eqn{L^p}{Lp} rotation. See 
 \link{Lp rotation} for details on the rotation method.
 
 }  
  \examples{
  # For extended examples see the vignettes:
  # vignette("GPA1guide",    package = "GPArotation")
  # vignette("GPA2local",    package = "GPArotation")
  # vignette("GPA3bifactor", package = "GPArotation")

  # --- Accessing rotated loadings ---
  data("Harman", package = "GPArotation") # 8 physical variables
  qHarman <- quartimax(Harman8)
  loadings(qHarman)                              # via extractor (recommended)
  qHarman$loadings                               # via direct list access
  all.equal(loadings(qHarman), qHarman$loadings) # identical

  # --- Rotating factanal loadings ---
  data("WansbeekMeijer", package = "GPArotation") # Netherlands TV viewership
  fa.unrotated <- factanal(factors = 2, covmat = NetherlandsTV,
                           normalize = TRUE, rotation = "none")
  quartimax(loadings(fa.unrotated), normalize = TRUE)
  geominQ(loadings(fa.unrotated), normalize = TRUE, randomStarts = 100)

  # --- Passing rotation to factanal ---
  # CCAI:Climate-Friendly Purchasing Choices domain of the Climate Change Action Inventory
  data("CCAI", package = "GPArotation") 
  factanal(factors = 3, covmat = CCAI_R, n.obs = 461, rotation = "infomaxT")
  factanal(factors = 3, covmat = CCAI_R, n.obs = 461, rotation = "infomaxT",
           control = list(rotate = list(normalize = TRUE, eps = 1e-6)))
           
  # --- Target rotation ---
  # Orthogonal target rotation of two varimax rotated matrices 
  # towards each other. Data from Fischer and Fontaine (2010).
  # See vignette("GPA1guide", package = "GPArotation") for further analyses.
  trBritain <- matrix(c(.783, -.163, .811, .202, .724, .209, .850, .064,
                       -.031, .592, -.028, .723, .388, .434, .141, .808,
                       .215, .709), byrow = TRUE, ncol = 2)

  trGermany <- matrix(c(.778, -.066, .875, .081, .751, .079, .739, .092,
                       .195, .574, -.030, .807, -.135, .717, .125, .738,
                       .060, .691), byrow = TRUE, ncol = 2)
  trx <- targetT(trGermany, Target = trBritain)
  round(trx$loadings - trBritain, 3)  # difference from target

  # --- Partially specified target rotation ---
  # See vignette("GPA1guide", package = "GPArotation") for full context.
  # Unrotated loadings matrix A and partially specified target SPA
  # NA entries in SPA are unspecified --- rotation is free there
  # Numeric entries are the target values the rotation aims towards
  A <- matrix(c(.664, .688, .492, .837, .705, .82, .661, .457, .765, .322,
                .248, .304, -0.291, -0.314, -0.377, .397, .294, .428,
                -0.075, .192, .224, .037, .155, -.104, .077, -.488, .009), ncol = 3)
  SPA <- matrix(c(rep(NA, 6), .7, .0, .7, rep(0, 3), rep(NA, 7),
                  0, 0, NA, 0, rep(NA, 4)), ncol = 3)
  comparison <- cbind(round(A, 3), rep(NA, nrow(A)), SPA)
  colnames(comparison) <- c("A.F1", "A.F2", "A.F3", "|", "T.F1", "T.F2", "T.F3")
  cat("Unrotated loadings (A) and partially specified target (SPA):\n")
  print(comparison, na.print = "NA")
  targetT(A, Target = SPA)

  # --- Random starts ---
  # CCAI Climate-Friendly Purchasing Choices domain, 14 items, 3 oblique factors.
  # High factor intercorrelations make oblimin the natural choice.
  # Note: factanal uses MLE extraction; results differ somewhat from
  # PCA-based extraction used in Bi and Barchard (2024).
  data("CCAI", package = "GPArotation")
  fa.unrotated <- factanal(factors = 3, covmat = CCAI_R, n.obs = 461, rotation = "none")
  oblimin(loadings(fa.unrotated), Tmat = Random.Start(3))  # single random start
  oblimin(loadings(fa.unrotated), randomStarts = 1)        # equivalent
  oblimin(loadings(fa.unrotated), randomStarts = 100)      # multiple starts

  # Directly via factanal call 
  factanal(factors = 3, covmat = CCAI_R, n.obs = 461, rotation = "oblimin",
      control = list(rotate = list(normalize = TRUE, gam = -0.1, randomStarts = 100)))

  # --- Assessing local minima ---
  # For detailed investigation of local minima across all random starts
  # see vignette("GPA2local", package = "GPArotation").
  data(Thurstone, package = "GPArotation")
  infomaxQ(box26, normalize = TRUE, randomStarts = 150)
  geominQ(box26,  normalize = TRUE, randomStarts = 150)
}
\seealso{
   \code{\link[stats]{factanal}}, 
   \code{\link{GPFRSorth}}, 
   \code{\link{GPFRSoblq}}, 
   \code{\link{vgQ}},
   \code{\link{Harman8}}, 
   \code{\link{NetherlandsTV}}, 
   \code{\link{CCAI}}, 
   \code{\link{box26}} 
}
\references{
  Bernaards, C.A. and Jennrich, R.I. (2005) Gradient Projection Algorithms
    and Software for Arbitrary Rotation Criteria in Factor Analysis.
    \emph{Educational and Psychological Measurement}, \bold{65}, 676--696.
    doi: 10.1177/0013164404272507
  
  Bi, Y. and Barchard, K.A. (2024). Purchasing choices that reduce
  climate change: An exploratory factor analysis.
  \emph{Spectra Undergraduate Research Journal}, \bold{3}(2), 8--14.
  doi: 10.9741/2766-7227.1028.
    
  Fischer, R., & Fontaine, J. (2010). Methods for investigating structural 
    equivalence. In D. Matsumoto & F. van de Vijver (Eds.), 
    Cross-Cultural Research Methods in Psychology (pp. 179--215). 
    Cambridge University Press. doi: 10.1017/CBO9780511779381.010

  Garcia-Garzon, E., Abad, F.J. and Garrido, L.E. (2021). On omega
    hierarchical estimation: A comparison of exploratory bi-factor
    analysis algorithms. \emph{Multivariate Behavioral Research},
    \bold{56}(1), 101--119.
    doi: 10.1080/00273171.2020.1736977

  Jennrich, R.I. and Bentler, P.M. (2011). Exploratory bi-factor analysis.
    \emph{Psychometrika}, \bold{76}(4), 537--549.
    doi: 10.1007/s11336-011-9218-4

For references to individual rotation criteria see
  \code{vignette("GPA1guide", package = "GPArotation")}.  } 
\author{Coen A. Bernaards and Robert I. Jennrich
        with some R modifications by Paul Gilbert.}
\concept{rotation}
\keyword{multivariate}

\encoding{UTF-8}
\name{CCAI}
\docType{data}
\alias{CCAI}
\alias{CCAI_R}
\alias{CCAI_pattern}
\alias{CCAI_Phi}
\title{The CCAI Climate-Friendly Purchasing Choices Domain}
\description{
  Correlation matrix, factor pattern matrix, and factor intercorrelations
  for the Climate Change Action Inventory (CCAI) (Barchard et al., 2021)
  Climate-Friendly Purchasing Choices domain, analyzed in Bi and Barchard
  (2024). The scale measures the frequency with which individuals make
  purchasing choices aimed at reducing climate change. Data were collected
  from 500 United States MTurk workers. After 15 climate change deniers
  and 24 multivariate outliers were removed, 461 participants remained.
  Climate change deniers were excluded because the scale measures behaviors
  intended to reduce climate change --- including individuals who do not
  believe climate change exists would be inconsistent with the measure's
  intent.

  The Climate Change Action Inventory contains eight domains; the
  Climate-Friendly Purchasing Choices domain analyzed here is one of them.
}
\usage{
data(CCAI, package = "GPArotation")
}
\format{
  Three objects are loaded by \code{data(CCAI)}:
  \itemize{
    \item \code{CCAI_R}: a \eqn{14 \times 14}{14 x 14} numeric
      correlation matrix. Row and column names are item labels
      \code{CCAI1} through \code{CCAI14}, ordered by factor
      to match \code{CCAI_pattern}.
    \item \code{CCAI_pattern}: a \eqn{14 \times 3}{14 x 3} numeric
      matrix of factor pattern coefficients. Row names are item labels
      \code{CCAI1} through \code{CCAI14}, ordered by factor to
      facilitate interpretation. Column names are factor labels
      \code{SustainableOptions}, \code{CollectiveAction}, and
      \code{AvoidBuyingNew}.
    \item \code{CCAI_Phi}: a \eqn{3 \times 3}{3 x 3} numeric matrix
      of factor intercorrelations. Row and column names are the three
      factor labels.
  }
}
\details{
  The CCAI Climate-Friendly Purchasing Choices domain consists of 14 items.
  Items used a nine-point frequency scale ranging from 1 (less than once a
  year) to 9 (at least 14 times a week). For full details on study design,
  data collection, analysis, and interpretation see Bi and Barchard (2024).

  \code{CCAI_R} is the observed \eqn{14 \times 14}{14 x 14} correlation
  matrix for the 14 items of the Climate-Friendly Purchasing Choices domain.
  The correlation matrix was kindly provided by the authors and has not been
  published separately.

  \code{CCAI_pattern} is a \eqn{14 \times 3}{14 x 3} factor pattern matrix
  from principal components extraction followed by direct oblimin rotation,
  reported in Table 2 of Bi and Barchard (2024) (the Table is labeled
  ``Factor Structure'' in the paper but contains pattern coefficients).
  The three factors are: Choosing Sustainable Options (F1), Supporting
  Collective Action (F2), and Avoiding Buying New (F3).

  \tabular{clrrr}{
  Item   \tab Description                                           \tab F1  \tab F2  \tab F3  \cr
  CCAI8  \tab Choose products with less impact on climate change    \tab .95 \tab .00 \tab -.02 \cr
  CCAI6  \tab Choose products with less packaging                   \tab .91 \tab -.04 \tab .01 \cr
  CCAI7  \tab Choose products made locally                          \tab .89 \tab -.09 \tab .07 \cr
  CCAI11 \tab Encourage others to choose climate-friendly products  \tab .56 \tab .40 \tab .05 \cr
  CCAI12 \tab Problem solve to reduce impact of purchases           \tab .52 \tab .44 \tab .04 \cr
  CCAI10 \tab Encourage others to buy less                          \tab .41 \tab .44 \tab .12 \cr
  CCAI14 \tab Give time/money to orgs reducing purchase impact      \tab -.01 \tab .97 \tab -.02 \cr
  CCAI13 \tab Give time/money to orgs reducing purchases            \tab .02 \tab .93 \tab .01 \cr
  CCAI5  \tab Donate to charity rather than buying a gift           \tab -.02 \tab .78 \tab .20 \cr
  CCAI2  \tab Use borrowed/rented/digital rather than buying        \tab -.02 \tab -.18 \tab .90 \cr
  CCAI4  \tab Buy used rather than new                              \tab .00 \tab .15 \tab .76 \cr
  CCAI1  \tab Repair rather than buying replacements                \tab .03 \tab .06 \tab .76 \cr
  CCAI3  \tab Use borrowed/rented tools rather than buying          \tab .10 \tab .21 \tab .62 \cr
  CCAI9  \tab Donate or sell old possessions                        \tab .22 \tab .27 \tab .46 \cr
  }

  \code{CCAI_Phi} is a \eqn{3 \times 3}{3 x 3} factor intercorrelation
  matrix. All three intercorrelations exceed 0.50.

  \tabular{lrrr}{
                               \tab F1   \tab F2   \tab F3   \cr
  Choosing Sustainable Options \tab 1.00 \tab 0.59 \tab 0.59 \cr
  Supporting Collective Action \tab 0.59 \tab 1.00 \tab 0.53 \cr
  Avoiding Buying New          \tab 0.59 \tab 0.53 \tab 1.00 \cr
  }

  \subsection{The Tandem Paradigm on Correlated Data}{
    The \code{CCAI} Purchasing Choices domain illustrates the Tandem
    criteria on highly correlated factors. Because the underlying
    dimensions are strongly intercorrelated (\eqn{\phi_{ij} > 0.50}),
    \strong{TandemI} pulls shared variance into a single dominant general
    factor (approximately 59 percent of total variance), alerting the
    researcher to a unified behavioral core.

    \strong{TandemII} forces an orthogonal simple structure, but the
    90-degree constraint creates an artificial compromise when true
    dimensions are correlated --- smearing loadings across factors.
    Comparing TandemII to oblique \strong{Oblimin} using the package
    summary diagnostics (\code{calc_AUC}, \code{calc_hyperplane})
    provides empirical evidence that an oblique structure is
    conceptually superior for these data.
  }

  See \code{vignette("GPA3bifactor", package = "GPArotation")} for a
  bifactor analysis of these data.
}
\note{
  The raw data are publicly available on the Open Science Framework.
  As the data are subject to a license, users should consult the OSF
  page for terms of use before using the raw data directly:
  \url{https://osf.io/h38yb/overview}.
}
\seealso{
  \code{\link{rotations}},
  \code{\link{bifactorT}},
  \code{\link[base]{eigen}},
  \code{\link[stats]{factanal}},
  \code{\link{Harman}},
  \code{\link{Thurstone}},
  \code{\link{WansbeekMeijer}},
  \code{\link{GriffithMulaik}}  
}
\references{
  Barchard, K.A., Okagawa, K., Hoffman, C.K., and Odents, O. (2021).
  \emph{Climate Change Action Inventory.} Unpublished psychological
  test. Available from kim.barchard@unlv.edu.

  Bi, Y. and Barchard, K.A. (2024). Purchasing choices that reduce
  climate change: An exploratory factor analysis.
  \emph{Spectra Undergraduate Research Journal}, \bold{3}(2), 8--14.
  \doi{10.9741/2766-7227.1028}.
}
\examples{
  data(CCAI, package = "GPArotation")

  # Observed correlation matrix
  round(CCAI_R, 2)

  # Published pattern matrix and factor intercorrelations
  round(CCAI_pattern, 2)
  round(CCAI_Phi, 2)

  # Reproduce published analysis: PCA extraction via eigendecomposition
  # followed by oblimin rotation. No additional packages required.
  # Gives the same result as psych::principal used by Bi and Barchard (2024).
  ev          <- eigen(CCAI_R)
  k           <- 3
  L_unrotated <- ev$vectors[, 1:k] \%*\% diag(sqrt(ev$values[1:k]))
  rownames(L_unrotated) <- colnames(CCAI_R)
  res.ob <- oblimin(L_unrotated, randomStarts = 100)

  # Sort and extract loadings for comparison
  res_sorted <- GPArotation:::.sortGPALoadings(res.ob)
  L_repro    <- unclass(res_sorted$loadings)

  # Compare reproduced vs published side by side, alternating by factor
  comparison <- cbind(round(L_repro[, 1], 2), round(CCAI_pattern[, 1], 2),
                      round(L_repro[, 2], 2), round(CCAI_pattern[, 2], 2),
                      round(L_repro[, 3], 2), round(CCAI_pattern[, 3], 2))
  colnames(comparison) <- c("F1.repro", "F1.pub",
                            "F2.repro", "F2.pub",
                            "F3.repro", "F3.pub")
  print(comparison)

  # --- Tandem criteria ---
  # Stage 1: TandemI reveals factor redundancy structure.
  # One dominant general factor (SS = 8.3, 59\% variance) suggests
  # highly correlated underlying constructs.
  res.t1 <- tandemI(L_unrotated, randomStarts = 100)
  print(res.t1)

  # Stage 2: TandemII for simple structure (orthogonal constraint).
  res.t2 <- tandemII(L_unrotated, randomStarts = 100)
  summary(res.t2)

  # For heavily correlated constructs, oblique rotation achieves
  # substantially cleaner simple structure than TandemII.
  # Compare AUC and hyperplane counts across methods.
  cat("           AUC    Hyperplane\n")
  cat("TandemII  ",
      round(GPArotation:::calc_AUC(res.t2)$AUC_mean, 3), "  ",
      sum(abs(unclass(res.t2$loadings)) < 0.1), "loadings\n")
  cat("Oblimin   ",
      round(GPArotation:::calc_AUC(res.ob)$AUC_mean, 3), "  ",
      sum(abs(unclass(res.ob$loadings)) < 0.1), "loadings\n")

  # Orthogonal bifactor rotation using MLE extraction
  fa.un <- factanal(factors = 3, covmat = CCAI_R, n.obs = 461,
                    rotation = "none")
  bif <- bifactorT(fa.un)
  print(bif, sortLoadings = FALSE, digits = 3, cutoff = 0.05)
}
\keyword{datasets}

\encoding{UTF-8}
\name{Random.Start}
\alias{Random.Start}
\title{Random Starting Matrix for Factor Rotation}
\usage{
    Random.Start(k = 2L)
}
\arguments{
    \item{k}{a positive integer specifying the dimension of the square
         orthogonal matrix to generate (default 2).}
}
\description{
  Generates a random orthogonal matrix for use as an initial rotation
  matrix (\code{Tmat}) in gradient projection rotation functions.
}
\value{
  A \eqn{k \times k}{k x k} orthogonal matrix drawn uniformly from the
  Haar measure on the orthogonal group O(k). Columns have unit length
  and are mutually orthogonal.
}
\details{
  The function generates a random orthogonal matrix using QR decomposition
  of a matrix of standard normal variates, with a sign correction applied
  to the diagonal of R to ensure uniform sampling from the Haar measure.
  This follows the approach of Stewart (1980) and Mezzadri (2007).

  The naive approach of \code{qr.Q(qr(matrix(rnorm(k*k), k)))} does not
  guarantee uniform sampling from the Haar measure. The sign correction
  \code{Q \%*\% diag(sign(diag(R)))} is required to achieve this. This
  was updated in GPArotation 2024.2-1 following a suggestion by Yves
  Rosseel.

  For oblique rotation a random starting transformation matrix can be
  generated by normalizing the columns of a random matrix:
  \code{X \%*\% diag(1/sqrt(diag(crossprod(X))))} where
  \code{X <- matrix(rnorm(k*k), k)}.
}
\examples{
  # Generate a 5 x 5 random orthogonal matrix
  Random.Start(5)

  # Verify orthogonality: t(Q) %*% Q should be the identity matrix
  Q <- Random.Start(4)
  round(t(Q) \%*\% Q, 10)

  # Use as starting matrix for rotation
  data("Thurstone", package = "GPArotation")
  simplimax(box26, Tmat = Random.Start(3))
}
\seealso{
  \code{\link{GPFRSorth}},
  \code{\link{GPFRSoblq}},
  \code{\link{GPForth}},
  \code{\link{GPFoblq}},
  \code{\link{rotations}}
}
\references{
  Stewart, G.W. (1980). The efficient generation of random orthogonal
    matrices with an application to condition estimators.
    \emph{SIAM Journal on Numerical Analysis}, \bold{17}(3), 403--409.
    doi: 10.1137/0717034

  Mezzadri, F. (2007). How to generate random matrices from the classical
    compact groups. \emph{Notices of the American Mathematical Society},
    \bold{54}(5), 592--604. arXiv:math-ph/0609050
}
\author{Coen A. Bernaards and Robert I. Jennrich
        with some R modifications by Paul Gilbert.
        Updated following a suggestion by Yves Rosseel.}
\concept{rotation}
\keyword{multivariate}

\encoding{UTF-8}
\name{GriffithMulaik}
\docType{data}
\alias{GriffithMulaik}
\title{Griffith and Mulaik Interpersonal Personality Traits}
\description{
  Correlation matrix for 24 bipolar seven-point trait-adjective scales
  representing six hypothetical factors from the interpersonal domain
  of personality. Data from an unpublished senior thesis conducted under
  the supervision of Stanley Mulaik at the School of Psychology,
  Georgia Institute of Technology, 1998.
}
\usage{
data(GriffithMulaik, package = "GPArotation")
}
\format{
  A \eqn{24 \times 24}{24 x 24} numeric correlation matrix with
  \code{n.obs = 523}. Row and column names are abbreviated variable
  labels for 24 interpersonal trait scales:
  \code{ALLOW}, \code{OUTGO}, \code{NONTALK}, \code{PERMISS},
  \code{FAIR}, \code{UNFRIEND}, \code{FORCELSS}, \code{RESTRAIN},
  \code{NONDOMIN}, \code{UNLOVING}, \code{NONCNFRM}, \code{COMPLINT},
  \code{IMPARTL}, \code{LEADER}, \code{UNKIND}, \code{UNCNVENT},
  \code{DIRECTNG}, \code{PROHIBTV}, \code{UNGREGAR}, \code{JUST},
  \code{NEUTRAL}, \code{SOCIABLE}, \code{UNAFFECT}, \code{REBELLOS}.
}
\details{
  Participants were 523 Georgia Tech introductory psychology students
  who each rated three stereotypical persons selected at random from a
  large list of stereotypes on 52 bipolar seven-point trait-adjective
  scales. The 24 variables analyzed here represent six hypothetical
  factors from the interpersonal domain of personality: four synonym
  trait items per factor.

  The study was intended to test Mulaik's theory of personality trait
  factors. Six simple structure factors were obtained as expected,
  providing tentative support for the theory.

  The correlation matrix is reproduced as Table 8.5 in Mulaik (2018).
  The data are used in that chapter to illustrate maximum likelihood
  factor analysis with a reduced correlation matrix scree plot.
  
  The six-factor oblimin solution obtained by \code{GPArotation}
  closely replicates the Direct Oblimin solution reported in
  Mulaik (2018) Table 8.6, with minor differences attributable
  to factor ordering and sign conventions.
}
\seealso{
  \code{\link{rotations}},
  \code{\link{Harman}},
  \code{\link{Thurstone}},
  \code{\link{CCAI}}
}
\references{
  Griffith, D. and Mulaik, S.A. (1998).
  \emph{Personality trait factors from the interpersonal domain.}
  Poster presented at the Society for Multivariate Experimental
  Psychology (SMEP).

  Mulaik, S.A. (2018). Fundamentals of common factor analysis.
  In P. Irwing, T. Booth, and D.J. Hughes (Eds.),
  \emph{The Wiley Handbook of Psychometric Testing}
  (pp. 211--252). Wiley.
}
\examples{
  data(GriffithMulaik, package = "GPArotation")

  ## Scree plot comparing raw versus reduced correlation matrix
  ## (Mulaik, 2018, recommends reduced matrix for ML factor analysis)
  fa.un <- factanal(factors = 6, covmat = GriffithMulaik,
                    n.obs = 523, rotation = "none")
  res.un <- quartimax(fa.un)

  ## Six factor oblimin rotation
  res.ob <- oblimin(fa.un, randomStarts = 100)
  summary(res.ob)

  ## Compare simple structure across rotation methods
  res.vm <- Varimax(fa.un)
  res.gm <- geominQ(fa.un, randomStarts = 100)

  cat(sprintf("\%-12s  AUC = \%.3f  Hyperplane = \%.1f pct\n",
              "Varimax",  GPArotation:::calc_AUC(res.vm)$AUC_mean,
              GPArotation:::calc_hyperplane(res.vm)$HP_pct))
  cat(sprintf("\%-12s  AUC = \%.3f  Hyperplane = \%.1f pct\n",
              "Oblimin",  GPArotation:::calc_AUC(res.ob)$AUC_mean,
              GPArotation:::calc_hyperplane(res.ob)$HP_pct))
  cat(sprintf("\%-12s  AUC = \%.3f  Hyperplane = \%.1f pct\n",
              "GeominQ",  GPArotation:::calc_AUC(res.gm)$AUC_mean,
              GPArotation:::calc_hyperplane(res.gm)$HP_pct))

  ## --- Theory-guided rotation via partially specified target ---
  ## Mulaik (2018) Table 8.7 specifies a CFA hypothesis matrix:
  ## * = free parameter (NA), o = constrained to zero (0)
  GM_target <- matrix(c(
  ## F1  F2  F3  F4  F5  F6
    NA,  0,  0,  0, NA,  0,  ## ALLOW
    NA, NA,  0,  0, NA,  0,  ## OUTGO
     0, NA,  0,  0, NA,  0,  ## NONTALK
    NA,  0,  0,  0,  0,  0,  ## PERMISS
     0,  0, NA, NA,  0,  0,  ## FAIR
    NA, NA, NA, NA,  0,  0,  ## UNFRIEND
    NA, NA,  0, NA, NA,  0,  ## FORCELSS
    NA, NA,  0,  0, NA,  0,  ## RESTRAIN
    NA, NA,  0, NA, NA,  0,  ## NONDOMIN
     0,  0,  0, NA,  0,  0,  ## UNLOVING
     0,  0,  0,  0,  0, NA,  ## NONCNFRM
    NA,  0,  0,  0,  0, NA,  ## COMPLINT
     0, NA, NA,  0, NA,  0,  ## IMPARTL
     0,  0,  0,  0, NA,  0,  ## LEADER
     0, NA, NA, NA,  0,  0,  ## UNKIND
     0,  0,  0,  0,  0, NA,  ## UNCNVENT
     0,  0,  0, NA, NA,  0,  ## DIRECTNG
    NA,  0,  0,  0, NA, NA,  ## PROHIBTV
     0, NA,  0,  0,  0,  0,  ## UNGREGAR
     0,  0, NA, NA,  0,  0,  ## JUST
    NA, NA, NA, NA, NA,  0,  ## NEUTRAL
    NA, NA,  0, NA, NA, NA,  ## SOCIABLE
     0, NA,  0, NA,  0,  0,  ## UNAFFECT
     0, NA,  0, NA,  0, NA   ## REBELLOS
  ), nrow = 24, ncol = 6, byrow = TRUE)
  rownames(GM_target) <- rownames(GriffithMulaik)
  colnames(GM_target) <- paste0("F", 1:6)

  ## W: 1 = constrained to zero, 0 = free
  GM_target_pst <- ifelse(is.na(GM_target), 0, GM_target)
  GM_W          <- ifelse(is.na(GM_target), 0, 1)

  ## Oblique PST closely replicates Mulaik (2018) Table 8.8 CFA solution
  res.pstQ <- pstQ(fa.un, Target = GM_target_pst, W = GM_W)
  print(res.pstQ)

  ## Compare data-driven vs theory-guided rotation
  cat(sprintf("\%-10s  AUC = \%.3f  Hyperplane = \%.1f pct\n",
              "PST-Q",   GPArotation:::calc_AUC(res.pstQ)$AUC_mean,
              GPArotation:::calc_hyperplane(res.pstQ)$HP_pct))
  cat(sprintf("\%-10s  AUC = \%.3f  Hyperplane = \%.1f pct\n",
              "Oblimin", GPArotation:::calc_AUC(res.ob)$AUC_mean,
              GPArotation:::calc_hyperplane(res.ob)$HP_pct))
}
\keyword{datasets}

\encoding{UTF-8}
\name{print.GPArotation}
\alias{print.GPArotation}
\alias{summary.GPArotation}
\alias{print.summary.GPArotation}
\alias{.sortGPALoadings}
\title{Print and Summary Methods for GPArotation Class Objects}
\usage{
    \method{print}{GPArotation}(x, digits=3, sortLoadings=TRUE, rotateMat=FALSE, Table=FALSE, ...)
    \method{summary}{GPArotation}(object, digits=3, Structure=TRUE, ...)
    \method{print}{summary.GPArotation}(x, ...)
}
\arguments{
    \item{x}{a \code{GPArotation} or \code{summary.GPArotation} object to print.}
    \item{object}{a \code{GPArotation} object to summarize.}
    \item{digits}{precision of printed numbers.}
    \item{sortLoadings}{logical; if \code{TRUE} (default) factors are sorted
         by descending variance explained and factor signs are adjusted so
         that the sum of loadings per factor is positive. Adapted from
         \code{factanal} sorting conventions. Use \code{sortLoadings = FALSE}
         to display the raw unsorted solution, for example when the factor
         order is meaningful as in bifactor rotation.}
    \item{rotateMat}{logical; if \code{TRUE} the rotation matrix is displayed.}
    \item{Table}{logical; if \code{TRUE} the iteration table is displayed.}
    \item{Structure}{logical; if \code{TRUE} (default) the structure matrix
         (\code{loadings \%*\% Phi}) is displayed for oblique rotations in
         \code{summary}.}
    \item{...}{further arguments passed to other methods.}
}
\value{
  \code{print.GPArotation} returns the sorted \code{GPArotation}
  object invisibly when \code{sortLoadings = TRUE}, or the unsorted
  object when \code{sortLoadings = FALSE}.
  \code{summary.GPArotation} returns a \code{summary.GPArotation} object
  with sorted loadings and, for oblique rotations, the structure matrix.
  \code{print.summary.GPArotation} returns the object invisibly.
}
\description{
Print and summary methods for objects returned by \code{GPFRSorth},
\code{GPFRSoblq}, \code{GPForth}, or \code{GPFoblq}.

Both \code{print.GPArotation} and \code{summary.GPArotation} apply
consistent factor sorting and sign correction via the internal helper
\code{.sortGPALoadings} when \code{sortLoadings = TRUE}. Factors are
ordered by descending variance explained and signs are adjusted so that
the sum of loadings per factor is positive. This convention matches
that used by \code{\link[stats]{factanal}}.

For oblique rotations, \code{summary.GPArotation} displays both the
pattern matrix (regression coefficients of items on factors, controlling
for factor intercorrelations) and the structure matrix
(\code{loadings \%*\% Phi}, correlations between items and factors)
when \code{Structure = TRUE}. The two matrices coincide for orthogonal
rotations where \eqn{\Phi = I}{Phi = I}.

Output includes contributions of factors via \code{SS loadings}
(sum of squared loadings); see Harman (1976), sections 2.4 and 12.4.
}
\details{
Factor sorting and sign correction are applied consistently in both
\code{print} and \code{summary} via the internal function
\code{.sortGPALoadings}, adapted from \code{factanal} sorting
conventions (R Core Team). This ensures that the pattern matrix shown
by \code{summary} is consistent with the loadings shown by
\code{print}.

The \code{digits} argument controls the number of decimal places
shown in the loadings, structure matrix, and Phi.

For examples see \code{\link{GPFRSorth}} and the package vignettes:
\code{vignette("GPA1guide", package = "GPArotation")}.
}
\references{
  Harman, H.H. (1976). \emph{Modern Factor Analysis}.
    The University of Chicago Press.
}
\seealso{
  \code{\link{GPFRSorth}},
  \code{\link{GPForth}},
  \code{\link[stats]{factanal}},
  \code{\link[base]{summary}}
}
\examples{
  data(Harman, package = "GPArotation")
  res <- oblimin(Harman8, normalize = TRUE, randomStarts = 100)

  # Print sorted loadings (default)
  print(res)

  # Print unsorted loadings
  print(res, sortLoadings = FALSE)

  # Summary with pattern and structure matrices
  summary(res, Structure = TRUE)

  # Summary without structure matrix
  summary(res, Structure = FALSE)

  # Print with iteration table
  print(res, Table = TRUE)
}
\concept{rotation}
\keyword{multivariate}

\encoding{UTF-8}
\name{00.GPArotation}
\alias{GPArotation}
\alias{GPArotation-package}
\alias{GPArotation.Intro}
\docType{package}
\title{Gradient Projection Algorithms for Factor Rotation}
\description{GPA Rotation for Factor Analysis

The GPArotation package contains functions for the rotation of factor loadings
matrices. The functions implement Gradient Projection (GP) algorithms for orthogonal
and oblique rotation. Additionally, a number of rotation criteria are provided.
The GP algorithms minimize the rotation criterion function and provide the
corresponding rotation matrix. For oblique rotation, the covariance/correlation matrix
of the factors is also provided. The rotation criteria implemented in this package
are described in Bernaards and Jennrich (2005) in addition to a number of others. 
Theory of the GP algorithm is described in Jennrich (2001, 2002).

Vignettes are provided covering general usage, local minima
diagnostics, bifactor rotation and reliability, and a visual walkthrough
of the gradient projection algorithm.
Access them via \code{browseVignettes("GPArotation")}.

\tabular{ll}{
Package: \tab GPArotation\cr
Depends: \tab R (>= 3.5.0)\cr
License: \tab GPL Version 2.\cr
}

Index of functions:

Rotations using gradient projection algorithms\cr
\tabular{ll}{
\code{\link{oblimin}}    \tab Oblimin rotation \cr
\code{\link{quartimin}}  \tab Quartimin rotation \cr
\code{\link{targetT}}    \tab Orthogonal target rotation \cr
\code{\link{targetQ}}    \tab Oblique target rotation \cr
\code{\link{pstT}}       \tab Orthogonal partially specified target rotation \cr
\code{\link{pstQ}}       \tab Oblique partially specified target rotation \cr
\code{\link{oblimax}}    \tab Oblimax rotation \cr
\code{\link{entropy}}    \tab Minimum entropy rotation \cr
\code{\link{quartimax}}  \tab Quartimax rotation \cr
\code{\link{Varimax}}    \tab Varimax rotation \cr
\code{\link{simplimax}}  \tab Simplimax rotation \cr
\code{\link{bentlerT}}   \tab Orthogonal Bentler invariant pattern simplicity rotation \cr
\code{\link{bentlerQ}}   \tab Oblique Bentler invariant pattern simplicity rotation \cr
\code{\link{tandemI}}    \tab Tandem criteria principle I rotation \cr
\code{\link{tandemII}}   \tab Tandem criteria principle II rotation \cr
\code{\link{geominT}}    \tab Orthogonal Geomin rotation \cr
\code{\link{geominQ}}    \tab Oblique Geomin rotation \cr
\code{\link{bigeominT}}  \tab Orthogonal Bi-Geomin rotation \cr
\code{\link{bigeominQ}}  \tab Oblique Bi-Geomin rotation \cr
\code{\link{cfT}}        \tab Orthogonal Crawford-Ferguson family rotation \cr
\code{\link{cfQ}}        \tab Oblique Crawford-Ferguson family rotation \cr
\code{\link{equamax}}    \tab Equamax rotation \cr
\code{\link{parsimax}}   \tab Parsimax rotation \cr
\code{\link{infomaxT}}   \tab Orthogonal Infomax rotation \cr
\code{\link{infomaxQ}}   \tab Oblique Infomax rotation \cr
\code{\link{mccammon}}   \tab McCammon minimum entropy ratio rotation \cr
\code{\link{varimin}}    \tab Varimin rotation \cr
\code{\link{bifactorT}}  \tab Orthogonal bifactor rotation \cr
\code{\link{bifactorQ}}  \tab Oblique bifactor rotation \cr
\code{\link{lpT}}        \tab Orthogonal \eqn{L^p}{Lp} rotation \cr
\code{\link{lpQ}}        \tab Oblique \eqn{L^p}{Lp} rotation \cr
}

Core gradient projection algorithms\cr
\tabular{ll}{
\code{\link{GPForth}} \tab Orthogonal rotation function \cr
\code{\link{GPFoblq}} \tab Oblique rotation function \cr
}

Random-start wrappers and internal engine\cr
\tabular{ll}{
\code{\link{GPFRSorth}}  \tab Random-start wrapper for orthogonal rotation \cr
\code{\link{GPFRSoblq}}  \tab Random-start wrapper for oblique rotation \cr
}

2D orthogonal trajectory plot \cr
\tabular{ll}{
\code{\link{plot2fOrthComparison}}		\tab 2D trajectory plot for 2 factor orthogonal rotation\cr
}

S3 methods\cr
\tabular{ll}{
\code{\link{print.GPArotation}}     \tab Print rotated solution with simple structure measures \cr
\code{\link{summary.GPArotation}}   \tab Summary with pattern, structure, and diagnostics \cr
\code{\link{plot.GPArotation}}      \tab Plot: salient, pairs, profile, vector, target,
                                         heatmap, trajectory, diagnostics, residuals \cr
\code{residuals.GPArotation}        \tab Observed minus modeled residual of correlation matrix \cr
\code{\link[stats]{update}} 		\tab update [The R Stats Package] \cr
}

Data sets\cr
\tabular{ll}{
\code{\link{Harman8}}       \tab Harman's 8 physical variables; centroid loadings \cr
\code{\link{NetherlandsTV}} \tab Wansbeek and Meijer Netherlands TV viewership; correlation matrix \cr
\code{\link{box26}}         \tab Thurstone's 26 box variables; unrotated factor loadings \cr
\code{\link{CCAI}}          \tab CCAI Climate-Friendly Purchasing Choices domain;
                                 correlation matrix, pattern matrix, and factor intercorrelations \cr
\code{\link{GriffithMulaik}} \tab Griffith and Mulaik interpersonal personality
                                   traits; 24-variable correlation matrix \cr
}

Other rotations not using gradient projection algorithms\cr
\tabular{ll}{
\code{\link{eiv}}              \tab Errors-in-variables rotation \cr
\code{\link{echelon}}          \tab Echelon rotation \cr
\code{\link[stats]{varimax}}   \tab varimax [The R Stats Package] \cr
\code{\link[stats]{promax}}    \tab promax [The R Stats Package] \cr
}

Legacy gradient projection algorithms (code unchanged since 2008)\cr
\tabular{ll}{
\code{GPForth.legacy} \tab Orthogonal rotation, original implementation (not exported) \cr
\code{GPFoblq.legacy} \tab Oblique rotation, original implementation (not exported) \cr
}

Utility functions\cr
\tabular{ll}{
\code{\link{Random.Start}}        \tab Random starting matrix for factor rotation \cr
\code{\link{GPForth.lp}}          \tab Single-start \eqn{L^p}{Lp} orthogonal rotation \cr
\code{\link{GPFoblq.lp}}          \tab Single-start \eqn{L^p}{Lp} oblique rotation \cr
}

Utility functions (not exported)\cr
\tabular{ll}{
\code{.GPA_RS_engine}    		  \tab Internal random-start engine  \cr
\code{.sortGPALoadings}           \tab Sort and sign-correct factors  \cr
\code{\link{NormalizingWeight}}   \tab Normalizing weights utility \cr
\code{\link{calc_AUC}}            \tab AUC simple structure measure  \cr
\code{\link{calc_FSI}}            \tab Factor Simplicity Index  \cr
\code{\link{calc_simplicity}}     \tab Hoffman, Gini, Bentler simplicity indices  \cr
\code{\link{calc_hyperplane}}     \tab Hyperplane count  \cr
\code{\link{calc_fitstats}}		  \tab ML fit statistics: RMSEA, SRMR  \cr
\code{plot_trajectory}			  \tab Helper function for plot2fOrthComparison  \cr
\code{plot_gpa_diagnostics}	      \tab Helper function for plot2fOrthComparison  \cr
\code{plot_algorithm_comparison}  \tab Helper function for plot2fOrthComparison  \cr
\code{.reconstruct_trajectory}    \tab Helper function for plot2fOrthComparison  \cr
\code{plotRotationLandscape}      \tab Helper function for plot2fOrthComparison  \cr
\code{plot_landscape_trajectory}  \tab Helper function for plot2fOrthComparison  \cr
}

Value, gradient, rotation criterion functions (not exported)\cr
\tabular{ll}{
\code{\link{vgQ.oblimin}}   \tab Oblimin \cr
\code{\link{vgQ.quartimin}} \tab Quartimin \cr
\code{\link{vgQ.target}}    \tab Target \cr
\code{\link{vgQ.pst}}       \tab Partially specified target \cr
\code{\link{vgQ.oblimax}}   \tab Oblimax \cr
\code{\link{vgQ.entropy}}   \tab Minimum entropy \cr
\code{\link{vgQ.quartimax}} \tab Quartimax \cr
\code{\link{vgQ.varimax}}   \tab Varimax \cr
\code{\link{vgQ.simplimax}} \tab Simplimax \cr
\code{\link{vgQ.bentler}}   \tab Bentler invariant pattern simplicity \cr
\code{\link{vgQ.tandemI}}   \tab Tandem criteria principle I \cr
\code{\link{vgQ.tandemII}}  \tab Tandem criteria principle II \cr
\code{\link{vgQ.geomin}}    \tab Geomin \cr
\code{\link{vgQ.bigeomin}}  \tab Bi-Geomin \cr
\code{\link{vgQ.cf}}        \tab Crawford-Ferguson family \cr
\code{\link{vgQ.infomax}}   \tab Infomax \cr
\code{\link{vgQ.mccammon}}  \tab McCammon minimum entropy ratio \cr
\code{\link{vgQ.varimin}}   \tab Varimin \cr
\code{\link{vgQ.bifactor}}  \tab Bifactor \cr
\code{\link{vgQ.lp.wls}}    \tab Weighted least squares for \eqn{L^p}{Lp} rotation \cr
}

Vignettes\cr
\tabular{ll}{
\code{GPA1guide}    \tab Gradient Projection Factor Rotation (main guide) \cr
\code{GPA2local}    \tab Assessing Local Minima in Factor Rotation \cr
\code{GPA3bifactor} \tab Bifactor Rotation and Reliability Coefficients \cr
\code{GPA4fitstats} \tab Factor Model Fit and Simple Structure Diagnostics \cr
}

}

\author{Coen A. Bernaards and Robert I. Jennrich
        with some R modifications by Paul Gilbert.
}

\references{
   The software reference is:

   Bernaards, C.A. and Jennrich, R.I. (2005). Gradient projection algorithms
     and software for arbitrary rotation criteria in factor analysis.
     \emph{Educational and Psychological Measurement}, \bold{65}, 676--696.
     \doi{10.1177/0013164404272507}

   Theory of gradient projection algorithms:

   Jennrich, R.I. (2001). A simple general procedure for orthogonal rotation.
     \emph{Psychometrika}, \bold{66}, 289--306.
     \doi{10.1007/BF02294840}

   Jennrich, R.I. (2002). A simple general method for oblique rotation.
     \emph{Psychometrika}, \bold{67}, 7--19.
     \doi{10.1007/BF02294706}

   A clear and accessible introduction to gradient projection algorithms
   for factor rotation is provided in:

   Mansolf, M. and Reise, S.P. (2016). Exploratory bifactor analysis:
     The Schmid-Leiman orthogonalization and Jennrich-Bentler analytic
     rotations. \emph{Multivariate Behavioral Research}, \bold{51}(5),
     698--717.
     \doi{10.1080/00273171.2016.1215898}

   Barzilai-Borwein step size:

   Barzilai, J. and Borwein, J.M. (1988). Two-point step size gradient
     methods. \emph{IMA Journal of Numerical Analysis}, \bold{8}, 141--148.
     \doi{10.1093/imanum/8.1.141}

   Cayley transform retraction:

   Wen, Z. and Yin, W. (2013). A feasible method for optimization with
     orthogonality constraints. \emph{Mathematical Programming},
     \bold{142}, 397--434.
     \doi{10.1007/s10107-012-0584-1}

   Non-monotone line search:

   Grippo, L., Lampariello, F., and Lucidi, S. (1986).  A nonmonotone 
     line search technique for Newton's method. 
     \emph{SIAM Journal on Numerical Analysis}, \bold{23}(4), 707--716.
     \doi{10.1137/0723046}

   Zhang, H. and Hager, W.W. (2004). A nonmonotone line search technique
     and its application to unconstrained optimization.
     \emph{SIAM Journal on Optimization}, \bold{14}(4), 1043--1056.
     \doi{10.1137/S1052623403428208}

   Simple structure measures:

  Liu, X., Wallin, G., Chen, Y., and Moustaki, I. (2023). Rotation to
    sparse loadings using \eqn{L^p}{Lp} losses and related inference
    problems. \emph{Psychometrika}, \bold{88}(2), 527--553.
    \doi{10.1007/s11336-023-09911-y}

   Lorenzo-Seva, U. (2003). A factor simplicity index.
     \emph{Psychometrika}, \bold{68}(1), 49--60.
     \doi{10.1007/BF02296652}

}

\keyword{package}

\seealso{
   \code{\link{GPFRSorth}},
   \code{\link{GPFRSoblq}},
   \code{\link{rotations}},
   \code{\link{vgQ}},
   \code{\link{simple_structure}},
   \code{\link{plot.GPArotation}}
   \code{browseVignettes("GPArotation")}
}

\name{plot.GPArotation}
\alias{plot.GPArotation}
\title{Plot Method for GPArotation Objects}
\description{
  Produces visual summaries of rotated factor matrices, including an APA-style 
  salient table-style, a heatmap, a tripartite pairs plot of factor geometry, 
  a macro-level profile bar chart, a target rotation schema, 
  a rotation trajectory view, or a set of diagnostics.
}
\usage{
  \method{plot}{GPArotation}(x, type = c("salient", "pairs", "profile", "vector", 
     "target", "heatmap", "trajectory", "diagnostics", "residuals"), 
     hide_hyperplane = FALSE, salient = 0.40, cutoff = .1, R = NULL, 
     factors = c(1,2), items = NULL, main = NULL, \dots)
     
  }
\arguments{
  \item{x}{An object of class \code{"GPArotation"}.}
 
  \item{type}{A character string indicating the type of plot. Must be one 
    of \code{"salient"}, \code{"pairs"}, or \code{"profile"}, \code{"vector"}, 
    \code{"target"}, \code{"heatmap"}, \code{"trajectory"}, \code{"diagnostics"},
    \code{residuals}.}

  \item{hide_hyperplane}{Boolean. Hide values below cutoff. Default is FALSE.}
  
  \item{salient}{Numeric. Absolute values equal to or above this are highlighted 
  as salient. Default is 0.40.}
 
  \item{cutoff}{Numeric. Absolute values strictly below this are considered 
    noise/hyperplane. Default is 0.1.}

  \item{R}{Correlation matrix. If not provided taken from GPArotation object. 
      Default is NULL.}

  \item{factors}{Two factors used in trajectory plot. Default = c(1,2).}

  \item{items}{items plotted in the trajectory plot. All items are 
     plotted by default. Default = NULL.}

  \item{main}{main title where applicable. Default = NULL.}
  
  \item{\dots}{Additional graphical parameters passed to base plotting functions.}
}
\value{
  Invisibly returns the original \code{x} object.
}
\author{
  Your Name
}
\examples{
  \dontrun{
    # Assuming 'res' is a fitted GPArotation object:
    plot(res, type = "salient")
    plot(res, type = "pairs")
    plot(res, type = "profile")
  }
}

\encoding{UTF-8}
\name{lp}
\alias{Lp rotation}
\alias{GPForth.lp}
\alias{GPFoblq.lp}

\title{\eqn{L^p}{Lp} Rotation}
\usage{
GPForth.lp(A, Tmat = diag(ncol(A)), p = 1, normalize = FALSE, eps = 1e-05,
           maxit = 10000, gpaiter = 5)
GPFoblq.lp(A, Tmat = diag(ncol(A)), p = 1, normalize = FALSE, eps = 1e-05,
           maxit = 10000, gpaiter = 5)
}

\arguments{
    \item{A}{Initial factor loadings matrix to be rotated.}
    \item{Tmat}{Initial rotation matrix.}
    \item{p}{Component-wise \eqn{L^p}{Lp} where 0 < p \eqn{\le}{<=} 1.}
    \item{normalize}{Not recommended for \eqn{L^p}{Lp} rotation.}
    \item{eps}{Convergence is assumed when the norm of the gradient is smaller
         than \code{eps}.}
    \item{maxit}{Maximum number of iterations allowed in the main loop.}
    \item{gpaiter}{Maximum iterations for the inner GPA rotation loop.
         The goal is to decrease the objective value, not fully optimize the
         inner loop. Warnings may appear but can be ignored if the main loop
         converges.}
}

\value{
  A \code{GPArotation} object, which is a list containing:
  \item{loadings}{Rotated loadings matrix, with one column per factor.}
  \item{Th}{Rotation matrix, satisfying \code{loadings \%*\% t(Th) = A}.}
  \item{Table}{Data frame recording iteration details: iteration count,
       objective value, and elapsed time.}
  \item{method}{String indicating the rotation objective function.}
  \item{orthogonal}{Logical indicating whether the rotation is orthogonal.}
  \item{convergence}{Logical indicating whether convergence was achieved.
       Convergence is assessed element-wise using \code{eps} as tolerance.}
  \item{Phi}{Covariance matrix of rotated factors, \code{t(Th) \%*\% Th}.}
}

\description{
  Performs \eqn{L^p}{Lp} rotation to obtain sparse factor loadings.
}

\details{
These functions optimize an \eqn{L^p}{Lp} rotation objective where
\code{0 < p <= 1}. A smaller value of \code{p} promotes greater sparsity
in the loading matrix but increases computational difficulty. For 
guidance on choosing \code{p}, see the Concluding Remarks in
Liu et al. (2023).

The user-facing wrapper functions \code{\link{lpT}} and \code{\link{lpQ}}
provide random start functionality on top of \code{GPForth.lp} and
\code{GPFoblq.lp} respectively, analogous to how \code{\link{GPFRSorth}}
and \code{\link{GPFRSoblq}} wrap \code{GPForth} and \code{GPFoblq} for
standard rotation criteria. For most users \code{lpT} and \code{lpQ}
are the recommended entry points.

Since the \eqn{L^p}{Lp} objective is nonsmooth, a different optimization
method is required compared to smooth rotation criteria. \code{GPForth.lp}
and \code{GPFoblq.lp} replace \code{GPForth} and \code{GPFoblq} for
orthogonal and oblique \eqn{L^p}{Lp} rotations, respectively.

The optimization uses an iterative reweighted least squares (IRLS) approach.
The nonsmooth objective is approximated by a smooth weighted least squares
function in the outer loop, which is then optimized using GPA in the inner
loop (\code{gpaiter} controls the maximum inner iterations).

Normalization is not recommended for \eqn{L^p}{Lp} rotation and may
produce unexpected results.

}
\examples{
  data("WansbeekMeijer", package = "GPArotation")
  fa.unrotated <- factanal(factors = 2, covmat = NetherlandsTV, rotation = "none")
  options(warn = -1)

  # Orthogonal rotation — single start
  fa.lpT1 <- GPForth.lp(loadings(fa.unrotated), p = 1)

  # Orthogonal rotation — 10 random starts
  fa.lpT <- lpT(loadings(fa.unrotated), Tmat = Random.Start(2), p = 1,
                randomStarts = 10)
  print(fa.lpT, digits = 5, sortLoadings = FALSE, Table = TRUE, rotateMat = TRUE)

  # Oblique rotation — single start
  fa.lpQ1 <- GPFoblq.lp(loadings(fa.unrotated), p = 1)

  # Oblique rotation — 10 random starts
  fa.lpQ <- lpQ(loadings(fa.unrotated), p = 1, randomStarts = 10)
  summary(fa.lpQ, Structure = TRUE)

  # Compare Lp (p=1), Lp (p=0.5), and Geomin oblique rotations
  set.seed(1020)
  fa.lpQ1   <- lpQ(loadings(fa.unrotated), p = 1,   randomStarts = 10)
  fa.lpQ0.5 <- lpQ(loadings(fa.unrotated), p = 0.5, randomStarts = 10)
  fa.geo    <- geominQ(loadings(fa.unrotated),       randomStarts = 10)

  # With factor ordering using internal sortGPALoadings helper
  res <- round(cbind(GPArotation:::.sortGPALoadings(fa.lpQ1)$loadings,
                     GPArotation:::.sortGPALoadings(fa.lpQ0.5)$loadings,
                     GPArotation:::.sortGPALoadings(fa.geo)$loadings), 3)
  print(c("oblique --  Lp p=1        Lp p=0.5       Geomin"))
  print(res)

  # Without factor ordering
  res <- round(cbind(fa.lpQ1$loadings, fa.lpQ0.5$loadings, fa.geo$loadings), 3)
  print(c("oblique --  Lp p=1        Lp p=0.5       Geomin"))
  print(res)

  options(warn = 0)
}

\seealso{
  \code{\link{lpT}},
  \code{\link{lpQ}},
  \code{\link{vgQ.lp.wls}}
}

\references{
  Liu, X., Wallin, G., Chen, Y., and Moustaki, I. (2023). Rotation to
    sparse loadings using \eqn{L^p}{Lp} losses and related inference
    problems. \emph{Psychometrika}, \bold{88}(2), 527--553.
    doi: 10.1007/s11336-023-09911-y
}

\author{Xinyi Liu, with minor modifications for GPArotation by C. Bernaards.}

\concept{rotation}
\keyword{multivariate}

\encoding{UTF-8}
\name{WansbeekMeijer}
\docType{data}
\alias{WansbeekMeijer}
\alias{NetherlandsTV}
\title{Netherlands Television Viewership Data}
\description{
  Correlation matrix for Netherlands television viewership, used as an
  example dataset for factor rotation. From Wansbeek and Meijer (2000),
  page 171.
}
\usage{
  data(WansbeekMeijer)
}
\details{
  \code{NetherlandsTV} is a list with components \code{$cov}
  (a \eqn{7 \times 7}{7 x 7} correlation matrix for 7 television
  viewership variables measured in the Netherlands) and \code{$n.obs}
  (sample size, 2154). Use \code{cov2cor(NetherlandsTV\$cov)} to obtain
  the correlation matrix, or pass \code{NetherlandsTV} directly to
  \code{\link[stats]{factanal}} which handles the list structure
  automatically. It is used throughout the \code{GPArotation}
  documentation and vignettes as an example dataset for oblique rotation
  with 2 or 3 factors.
}
\format{
  \code{NetherlandsTV} is a list with components:
  \itemize{
    \item \code{$cov}: a \eqn{7 \times 7}{7 x 7} numeric correlation matrix
    \item \code{$n.obs}: sample size (2154)
  }
}
\source{
  Wansbeek, T. and Meijer, E. (2000). \emph{Measurement Error and
  Latent Variables in Econometrics}. North-Holland.
}
\seealso{
  \code{\link{GPForth}},
  \code{\link{rotations}},
  \code{\link{Thurstone}},
  \code{\link{Harman}},
  \code{\link{CCAI}}
}
\examples{
  data(WansbeekMeijer, package = "GPArotation")

  # Correlation matrix
  round(cov2cor(NetherlandsTV$cov), 2)

  # factanal picks up n.obs automatically from the list
  factanal(factors = 2, covmat = NetherlandsTV, rotation = "none")

  # Two-step oblique rotation
  fa.unrotated <- factanal(factors = 3, covmat = NetherlandsTV,
                           rotation = "none")
  oblimin(loadings(fa.unrotated), randomStarts = 100)
}
\keyword{datasets}

\encoding{UTF-8}
\name{NormalizingWeight}
\alias{NormalizingWeight}
\title{Normalizing Weights for Factor Rotation}
\usage{
    NormalizingWeight(A, normalize=FALSE)
}
\arguments{
  \item{normalize}{controls normalization. One of:
    \itemize{
      \item \code{FALSE} (default) --- no normalization
      \item \code{TRUE} or \code{"Kaiser"} --- Kaiser (Horst) normalization:
        rows of \code{A} are scaled to unit norm before rotation and
        rescaled after
      \item \code{"CM"} --- Cureton-Mulaik normalization: rows are
        weighted by a function of their angular distance from the
        equal-loading point (Cureton and Mulaik, 1975)
      \item a numeric vector of length \code{nrow(A)} --- custom weights
      \item a function taking \code{A} and returning a weight vector
    }
  }
}
\value{
  A numeric vector of normalizing weights. This function is not exported
  from the NAMESPACE and is only called internally by the gradient projection
  rotation functions. See \code{\link{GPFRSorth}} for details on the
  \code{normalize} argument.
}
\description{
  Internal utility function that computes normalizing weights for factor
  loading matrices prior to rotation. Called by \code{GPForth},
  \code{GPFoblq}, and the random-start wrappers.
}
\details{
  \code{NormalizingWeight} is not exported from the NAMESPACE and is
  called internally by \code{GPForth}, \code{GPFoblq}, and the
  random-start wrapper functions. For a full description of the
  \code{normalize} argument and its options, see \code{\link{GPFRSorth}}.

  The choice of normalization method can affect the rotation solution
  and its interpretation. For a detailed investigation of the effects
  of normalization on factor rotations, see Nguyen and Waller (2023).
}
\examples{
  data("CCAI", package = "GPArotation")
  # Kaiser normalization
  factanal(factors = 3, covmat = CCAI_R, n.obs = 461, rotation = "oblimin",
           control = list(rotate = list(normalize = TRUE)))

  data(Harman, package = "GPArotation")
  # two ways to do Kaiser normalization
  quartimin(Harman8, normalize = TRUE, randomStarts = 100)
  quartimin(Harman8, normalize = "Kaiser", randomStarts = 100)
  # Cureton-Mulaik normalization
  quartimin(Harman8, normalize = "CM", randomStarts = 100)

}
\references{
  Cureton, E.E. and Mulaik, S.A. (1975). The weighted varimax rotation
    and the promax rotation. \emph{Psychometrika}, \bold{40}(2), 183--195.
    \doi{10.1007/BF02291565}
    
  Nguyen, H.V. and Waller, N.G. (2023). Local minima and factor rotations
    in exploratory factor analysis. \emph{Psychological Methods},
    \bold{28}(5), 1122--1141.
    \doi{10.1037/met0000467}
}
\seealso{
  \code{\link{GPFRSorth}},
  \code{\link{GPForth}}
}
\keyword{internal}

\encoding{UTF-8}
\name{vgQ}
\alias{vgQ}
\alias{vgQ.oblimin}
\alias{vgQ.quartimin}
\alias{vgQ.target}
\alias{vgQ.pst}
\alias{vgQ.oblimax}
\alias{vgQ.entropy}
\alias{vgQ.quartimax}
\alias{vgQ.varimax}
\alias{vgQ.simplimax}
\alias{vgQ.bentler}
\alias{vgQ.tandemI}
\alias{vgQ.tandemII}
\alias{vgQ.geomin}
\alias{vgQ.bigeomin}
\alias{vgQ.cf}
\alias{vgQ.infomax}
\alias{vgQ.mccammon}
\alias{vgQ.bifactor}
\alias{vgQ.varimin}
\alias{vgQ.lp.wls}

\title{Rotation Criteria: Value and Gradient Functions}
\usage{
    vgQ.oblimin(L, gam=0)
    vgQ.quartimin(L)
    vgQ.target(L, Target=NULL)
    vgQ.pst(L, W=NULL, Target=NULL)
    vgQ.oblimax(L)
    vgQ.entropy(L)
    vgQ.quartimax(L)
    vgQ.varimax(L)
    vgQ.simplimax(L, k=nrow(L))
    vgQ.bentler(L)
    vgQ.tandemI(L)
    vgQ.tandemII(L)
    vgQ.geomin(L, delta=0.01)
    vgQ.bigeomin(L, delta=0.01)
    vgQ.cf(L, kappa=0)
    vgQ.infomax(L)
    vgQ.mccammon(L)
    vgQ.varimin(L)
    vgQ.bifactor(L)
    vgQ.lp.wls(L, W)
}
\arguments{
    \item{L}{a factor loading matrix.}
    \item{gam}{0 = Quartimin, 0.5 = Biquartimin, 1 = Covarimin.}
    \item{Target}{rotation target matrix for objective calculation.}
    \item{W}{weight matrix; weighting of each element in target rotation
         (\code{vgQ.pst}) or in iterative reweighted least squares
         (\code{vgQ.lp.wls}).}
    \item{k}{number of near-zero loadings to target.}
    \item{delta}{small constant added to \eqn{\Lambda^2}{Lambda^2} for
         numerical stability in the objective calculation.}
    \item{kappa}{complexity weight for the Crawford-Ferguson family; see
         \code{\link{cfT}} for details.}
}

\value{A list with:
  \item{f}{The value of the rotation criterion at \code{L}.}
  \item{Gq}{The gradient of the criterion at \code{L}.}
  \item{Method}{A string indicating the criterion name.}
}

\description{
  Rotation criterion functions that compute the value and gradient of the
  rotation objective Q. Not exported from the package NAMESPACE.
}

\details{
  The \code{vgQ.*} functions are called internally by the gradient projection
  optimization routines and would typically not be used directly. They can be
  inspected using \code{:::}, for example:
  \code{GPArotation:::vgQ.oblimin}. The gradient projection algorithms are
  described in Bernaards and Jennrich (2005).

  The \code{T} or \code{Q} suffix on rotation function names should be omitted
  for the corresponding \code{vgQ.*} function. For example,
  \code{GPArotation:::vgQ.target} is the criterion used by both
  \code{targetT} and \code{targetQ}.

\tabular{lll}{
\code{vgQ.oblimin}   \tab orthogonal or oblique \tab oblimin family \cr
\code{vgQ.quartimin} \tab oblique               \tab oblimin with \code{gam = 0} \cr
\code{vgQ.target}    \tab orthogonal or oblique \tab target rotation \cr
\code{vgQ.pst}       \tab orthogonal or oblique \tab partially specified target rotation \cr
\code{vgQ.oblimax}   \tab oblique               \tab maximizes overall kurtosis of loadings \cr
\code{vgQ.entropy}   \tab orthogonal            \tab minimum entropy \cr
\code{vgQ.quartimax} \tab orthogonal            \tab maximizes variance of squared loadings within variables \cr
\code{vgQ.varimax}   \tab orthogonal            \tab maximizes variance of squared loadings within factors \cr
\code{vgQ.simplimax} \tab oblique               \tab minimizes the k smallest squared loadings \cr
\code{vgQ.bentler}   \tab orthogonal or oblique \tab Bentler invariant pattern simplicity \cr
\code{vgQ.tandemI}   \tab orthogonal            \tab Tandem principle I criterion \cr
\code{vgQ.tandemII}  \tab orthogonal            \tab Tandem principle II criterion \cr
\code{vgQ.geomin}    \tab orthogonal or oblique \tab minimizes geometric mean of squared loadings \cr
\code{vgQ.bigeomin}  \tab orthogonal or oblique \tab geomin with a general factor in column 1 \cr
\code{vgQ.cf}        \tab orthogonal or oblique \tab Crawford-Ferguson family \cr
\code{vgQ.infomax}   \tab orthogonal or oblique \tab infomax information criterion \cr
\code{vgQ.mccammon}  \tab orthogonal            \tab McCammon minimum entropy ratio \cr
\code{vgQ.varimin}   \tab orthogonal            \tab varimin criterion \cr
\code{vgQ.bifactor}  \tab orthogonal or oblique \tab bifactor/biquartimin rotation \cr
\code{vgQ.lp.wls}    \tab orthogonal or oblique \tab iterative reweighted least squares
                                                      for \eqn{L^p}{Lp} rotation \cr
                                                      }

  See \code{\link{rotations}} for details on rotation arguments.

  New rotation criteria can be added by defining a function named
  \code{vgQ.newmethod}. The function takes \code{L} as its first argument,
  plus any additional arguments, and must return a list with elements
  \code{f}, \code{Gq}, and \code{Method}.

  Gradient projection \emph{without} derivatives can be performed using the
  \code{GPArotateDF} package; type
  \code{vignette("GPArotateDF", package = "GPArotation")} at the command line.
}

\examples{
  GPArotation:::vgQ.oblimin
}

\seealso{
   \code{\link{rotations}},
   \code{\link{GPFRSorth}}
}

\references{
  Bernaards, C.A. and Jennrich, R.I. (2005) Gradient Projection Algorithms
    and Software for Arbitrary Rotation Criteria in Factor Analysis.
    \emph{Educational and Psychological Measurement}, \bold{65}, 676--696.
    doi: 10.1177/0013164404272507
    }

\author{Coen A. Bernaards and Robert I. Jennrich
        with some R modifications by Paul Gilbert.}
\concept{rotation}
\keyword{multivariate}

\encoding{UTF-8}
\name{GPA}
\alias{GPFRSorth}
\alias{GPFRSoblq}
\alias{GPForth}
\alias{GPFoblq}
\alias{GPForth.legacy}
\alias{GPFoblq.legacy}
\alias{.GPA_RS_engine}

\title{Core Algorithms and Random-Start Wrappers}
\usage{
    GPForth(A, Tmat=diag(ncol(A)), normalize=FALSE, eps=1e-5, maxit=2000, 
       method="varimax", methodArgs=NULL, algorithm = "bb", fwindow = 10)
    GPFoblq(A, Tmat=diag(ncol(A)), normalize=FALSE, eps=1e-5, maxit=2000, 
       method="quartimin", methodArgs=NULL, algorithm = "bb", fwindow = 10)

    GPFRSorth(A, Tmat=NULL, normalize=FALSE, eps=1e-5, maxit=2000, method="varimax", 
       methodArgs=NULL, randomStarts=0, algorithm = "bb", fwindow = 10, ...)
    GPFRSoblq(A, Tmat=NULL, normalize=FALSE, eps=1e-5, maxit=2000, method="quartimin", 
       methodArgs=NULL, randomStarts=0, algorithm = "bb", fwindow = 10, ...)
    }
\arguments{
    \item{A}{initial factor loadings matrix for which the rotation 
             criterion is to be optimized.}
    \item{Tmat}{initial rotation matrix.}
    \item{normalize}{see details.}
    \item{eps}{convergence is assumed when the norm of the gradient is smaller
         than \code{eps}.}
    \item{maxit}{maximum number of iterations allowed in the main loop.}
    \item{method}{rotation objective criterion.}
    \item{methodArgs}{a list of additional arguments passed to the rotation objective.}
    \item{randomStarts}{number of random starts (\code{GPFRSorth} and 
         \code{GPFRSoblq} only).}
    \item{algorithm}{Line-search strategy used to calculate step size alpha.}
    \item{fwindow}{integer. Number of previous criterion values used in the 
         non-monotone line search.}
    \item{...}{additional arguments passed to \code{GPForth} or \code{GPFoblq},
         such as \code{eps} and \code{maxit} when calling via \code{GPFRSorth} 
         or \code{GPFRSoblq}.}
}
\value{
  A \code{GPArotation} object which is a list with elements:
  \item{loadings}{The rotated loadings matrix, one column per factor.
       If random starts were requested, this is the solution with the
       lowest criterion value.}
  \item{Th}{The rotation matrix, satisfying
       \code{loadings \%*\% t(Th) = A} for orthogonal rotation and
       \code{loadings = A \%*\% solve(t(Th))} for oblique rotation.}
  \item{Table}{A matrix recording the iteration history: iteration
       number, criterion value, log10 of the gradient norm, and step
       size (alpha).}
  \item{method}{A string indicating the rotation criterion.}
  \item{orthogonal}{A logical indicating if the rotation is orthogonal.}
  \item{convergence}{A logical indicating if convergence was obtained.}
  \item{Phi}{\code{t(Th) \%*\% Th}, the covariance matrix of the rotated
       factors. Omitted (\code{NULL}) for orthogonal rotations.}
  \item{Gq}{The gradient of the criterion at the rotated loadings.}
  \item{randStartChar}{A named vector summarising random start results:
       \code{randomStarts}, \code{Converged}, \code{atMinimum},
       \code{localMins}. Only present when \code{randomStarts > 1}.}
}
\description{
  Gradient projection rotation optimization routines for orthogonal and
  oblique factor rotation. These functions can be used directly to rotate
  a loadings matrix, or indirectly through a rotation objective passed to
  a factor estimation routine such as \code{\link[stats]{factanal}}.
}\details{
The \code{GPFRSorth} and \code{GPFRSoblq} functions serve as the primary user 
interfaces for orthogonal and oblique rotations, respectively. They act as wrappers 
for the core GP algorithms (\code{GPForth} for orthogonal rotation and \code{GPFoblq} 
for oblique rotation), extending them with the ability to perform multiple random starts. 
Any additional arguments provided to these wrappers are passed directly down to the 
underlying GP algorithms. While the wrappers are generally recommended, the core 
functions \code{GPForth} and \code{GPFoblq} can also be invoked directly.

All of these functions require an initial loadings matrix, \code{A}, which fixes 
the equivalence class over which the optimization is performed. This matrix must be 
the solution to an orthogonal factor analysis problem, such as one obtained from 
\code{factanal} or another factor estimation routine.

Mathematically, a general rotation of a matrix \code{A} is defined as 
\code{A \%*\% solve(t(Th))}. In the case of orthogonal rotation, 
the initial rotation matrix \code{Tmat} is orthonormal, which simplifies the 
rotation formula to \code{A \%*\% Th}. In all scenarios, the final rotation 
matrix \code{Th} is computed by the GP rotation algorithm.

An accessible introduction to gradient projection algorithms for
factor rotation is provided in Mansolf and Reise (2016).

\subsection{The \code{normalize} argument}{
The \code{normalize} argument specifies whether and how the loadings matrix 
should be normalized prior to rotation, and subsequently denormalized after rotation.

\itemize{
  \item If \code{FALSE} (the default), no normalization is performed.
  \item If \code{TRUE}, Kaiser normalization is applied so that the 
  squared row entries of the normalized matrix \code{A} sum to 1.0. 
  This procedure is sometimes referred to as Horst normalization.
  \item If provided as a \strong{vector} (which must have a length equal to 
  the number of indicators, i.e., the number of rows in \code{A}), 
  the columns of \code{A} are divided by this vector before rotation and multiplied by it afterward.
  \item If provided as a \strong{function}, it can be used to apply a 
  custom normalization scheme. The function must take \code{A} as an 
  argument and return a vector, which is then applied in the same manner 
  as the vector input described above. See \code{\link{NormalizingWeight}} 
  for an example implementing Cureton-Mulaik normalization.
}

For a detailed investigation into how normalization affects factor rotations, 
including its potential impact on the qualitative interpretation of loadings, see Nguyen and Waller (2022).
}

\subsection{The \code{method} argument}{
The \code{method} argument takes a string specifying the rotation objective function. 
By default, oblique rotations use \code{"quartimin"}, while orthogonal rotations 
default to \code{"varimax"}. The package supports a comprehensive suite of rotation 
objectives: \code{"oblimin"}, \code{"quartimin"}, \code{"target"}, \code{"pst"}, 
\code{"oblimax"}, \code{"entropy"}, \code{"quartimax"}, \code{"Varimax"}, 
\code{"simplimax"}, \code{"bentler"}, \code{"tandemI"}, \code{"tandemII"}, 
\code{"geomin"}, \code{"cf"}, \code{"infomax"}, \code{"mccammon"}, \code{"bifactor"}, 
\code{"lp"}, and \code{"varimin"}.

Internally, this string is prefixed with \code{"vgQ."} to invoke the actual 
calculation function (see \code{\link{vgQ}} for underlying mathematical details). 
It is important to note that several rotation criteria --- specifically \code{"oblimin"}, 
\code{"target"}, \code{"pst"}, \code{"simplimax"}, \code{"geomin"}, \code{"cf"}, and 
\code{"lp"} --- require one or more supplementary arguments. These additional arguments 
can be seamlessly passed via the \code{methodArgs} list in the wrapper functions. 
Default values and direct usage examples for these arguments can be found in the 
\code{\link{rotations}} documentation.
}

\subsection{The \code{randomStarts} argument}{
Because factor rotation criteria frequently suffer from local minima, 
trying multiple starting configurations can help identify a superior solution. 
The \code{randomStarts} argument, available exclusively in the \code{GPFRSorth} 
and \code{GPFRSoblq} wrappers, facilitates this robust search approach.

\itemize{
  \item By default, \code{randomStarts = 0}, which defaults to using the 
  identity matrix as the initial rotation matrix \code{Tmat}. The initial 
  rotation matrix \code{Tmat} can also be set by the user.
  \item Setting \code{randomStarts = 1} initializes \code{Tmat} with a single random matrix.
  \item Setting \code{randomStarts > 1} attempts multiple random starts and 
  returns the rotated loadings matrix that achieved the lowest criterion value 
  \code{f} across all attempts. Note that this returned solution is technically 
  still a local minimum, and is not guaranteed to be the global minimum. 
  Users are encouraged to review the random start diagnostics detailed in the package examples.
}

Under the hood, an internal, unexported engine named \code{.GPA_RS_engine} 
safely manages the random start loop, tracks convergence diagnostics, and handles 
factor correlation matrix naming.

While the core algorithms \code{GPForth} and \code{GPFoblq} do not support the 
\code{randomStarts} argument directly, users can manually supply a single random 
initial rotation matrix to them using \code{Tmat = \link{Random.Start}(ncol(A))}.
}

\subsection{The \code{algorithm} argument}{
The \code{algorithm} argument controls how the step size \eqn{\alpha} is initialized
at each iteration of the gradient projection loop. Three options are available:

\describe{
  \item{\code{"legacy"}}{The original Bernaards and Jennrich (2005) heuristic:
    \eqn{\alpha} is doubled at the start of each iteration and halved until the
    Armijo sufficient decrease condition is satisfied. Simple and reliable,
    particularly for smooth orthogonal criteria. This is the default for all
    orthogonal rotations and for target and PST rotations.}

  \item{\code{"bb"}}{Barzilai-Borwein (BB) step size initialization. Uses the
    ratio of successive changes in the rotation matrix and projected gradient to
    estimate a good starting step size automatically:
    \deqn{\alpha_{BB} = \frac{\|\Delta T\|^2}{|\langle \Delta T, \Delta G_p \rangle|}}
    where \eqn{\Delta T = T_k - T_{k-1}} and \eqn{\Delta G_p = G_{p,k} - G_{p,k-1}}
    are the changes in the rotation matrix and projected gradient between consecutive
    iterations. The BB estimate is then used as the starting point for the Armijo
    line search. Benchmarking shows meaningful speedups for oblique rotation criteria
    on larger matrices (50 variables or more) with many random starts, particularly
    for non-smooth criteria such as \code{simplimax}. This is the default for all
    oblique rotations except target and PST.}

  \item{\code{"cayley"}}{The Cayley transform retraction for orthogonal rotations
    (Wen and Yin, 2013).
    Instead of projecting back onto the orthogonal manifold via SVD, the Cayley
    transform produces an exactly orthogonal update in closed form:
    \deqn{T_{k+1} = \left(I + \frac{\alpha}{2} W\right)^{-1}
                    \left(I - \frac{\alpha}{2} W\right) T_k}
    where \eqn{W = G_p T_k^T - T_k G_p^T} is a skew-symmetric matrix derived
    from the projected gradient. The Cayley transform avoids the SVD decomposition
    at each iteration and can be faster for larger orthogonal problems. Available
    for orthogonal rotations only.}
}

\itemize{
  \item \code{algorithm = "bb"} with \code{fwindow = 10} is the default
    for all rotation criteria.
  \item \code{algorithm = "cayley"} is available for orthogonal rotations
    and can be specified explicitly by the user.
  \item \code{algorithm = "legacy"} is available for exact reproducibility
    of pre-2027 results.
}

Users can override the default by passing \code{algorithm} explicitly to any
rotation function or directly to \code{GPForth} and \code{GPFoblq}.
}

\subsection{The \code{fwindow} argument}{
The \code{fwindow} argument controls the width of the non-monotone line search
window (Zhang and Hager, 2004). At each iteration, the Armijo sufficient decrease
condition is evaluated against the maximum criterion value over the last
\code{fwindow} iterations rather than just the immediately preceding value:

\deqn{f(T_{k+1}) < \max_{j \in W_k} f(T_j) - 0.5\, s^2\, \alpha}

where \eqn{W_k} is the window of the last \code{fwindow} iterates and \eqn{s}
is the projected gradient norm.

\itemize{
  \item \code{fwindow = 1} (default for \code{"legacy"}) reduces to the standard
    monotone Armijo condition --- the criterion must decrease at every iteration.
  \item \code{fwindow > 1} allows temporary increases in the criterion value,
    which can help the algorithm escape flat regions and shallow local minima.
    This is particularly beneficial in combination with \code{algorithm = "bb"}.
  \item \code{fwindow = 10} is the default for \code{algorithm = "bb"} and
    \code{algorithm = "cayley"}.
}

Benchmarking across matrix sizes and rotation criteria suggests that
\code{fwindow = 10} with \code{algorithm = "bb"} provides the best balance
of speed and solution quality for oblique rotations on larger models.
For orthogonal rotations with \code{algorithm = "legacy"}, \code{fwindow = 1}
is recommended as larger windows can slow convergence.
}

\subsection{Legacy functions}{
The original implementations authored by Bernaards and Jennrich (2005) have been 
retained as \code{GPForth.legacy} and \code{GPFoblq.legacy}. These functions are 
kept purely for historical reference and backward compatibility for reproducibility. 
They are not exported into the package namespace, meaning they must be explicitly 
invoked using the triple-colon operator:
\preformatted{
GPArotation:::GPForth.legacy(A, method = "varimax")
GPArotation:::GPFoblq.legacy(A, method = "quartimin")
}
The results generated by these legacy functions should be numerically identical 
to those produced by the current implementations when \code{algorithm = "legacy"}
and \code{fwindow = 1} are used.
}
}

\seealso{
   \code{\link{rotations}},
   \code{\link{Random.Start}},
   \code{\link[stats]{factanal}},
   \code{\link{Harman8}},
   \code{\link{box26}},
   \code{\link{CCAI}},
   \code{\link{NetherlandsTV}}
}
\examples{
  # --- Basic rotation calls ---
  data(Harman, package = "GPArotation")           # 8 physical variables
  quartimax(Harman8)                              # direct rotation call
  GPFRSorth(Harman8, method = "quartimax")        # equivalent via wrapper
  GPFRSoblq(Harman8, method = "quartimin", normalize = TRUE)
  loadings(quartimin(Harman8, normalize = TRUE))  # extract loadings directly

  # --- Passing criterion arguments via methodArgs ---
  # Crawford-Ferguson family: kappa selects the criterion.
  # For box26: p = 26 variables, m = 3 factors.
  # Equamax:  kappa = m / (2 * p) = 3 / 52
  # Parsimax: kappa = (m - 1) / (p + m - 2) = 2 / 27
  data(Thurstone, package = "GPArotation") # 26 variable box problem 
  GPFRSoblq(box26, method = "cf", methodArgs = list(kappa = 3/52))  # Equamax
  GPFRSoblq(box26, method = "cf", methodArgs = list(kappa = 2/27))  # Parsimax
  
  # --- Two-step vs single-step factanal for oblique rotation ---
  #
  # The recommended approach for oblique rotation is the two-step procedure:
  # (1) obtain unrotated loadings from factanal, then
  # (2) rotate separately using GPArotation.
  # This gives full control over the rotation, including random starts.
  #
  # Prior to R 4.5.1, the single-step approach (rotation inside factanal)
  # had a bug in factor reordering after oblique rotation. 
  # This was fixed by the R core team in R 4.5.1.

  data("WansbeekMeijer", package = "GPArotation")

  # Step 1: unrotated 3-factor solution
  fa.unrotated <- factanal(factors = 3, covmat = NetherlandsTV,
                           normalize = TRUE, rotation = "none")

  # Step 2: oblique Crawford-Ferguson rotation with kappa = 0.3
  # (non-standard kappa, not corresponding to any named special case)
  set.seed(44)
  fa.cf <- cfQ(fa.unrotated, kappa = 0.3, normalize = TRUE, randomStarts = 100)
  fa.cf

  # Single-step via factanal - correct in R >= 4.5.1
  if (getRversion() >= "4.5.1") {
    set.seed(44)
    fa.factanal <- factanal(factors = 3, covmat = NetherlandsTV, rotation = "cfQ",
      control = list(rotate = list(normalize = TRUE, kappa = 0.3, randomStarts = 100)))
    # The two approaches should agree after sorting
    fa.sorted <- print(fa.cf, sortLoadings = TRUE)
    cat("Maximum difference in loadings between two-step and single-step:\n")
    print(max(abs(abs(fa.sorted$loadings) - abs(fa.factanal$loadings))))
  } else {
    cat("Single-step factanal oblique rotation requires R >= 4.5.1.\n")
    cat("Use the two-step procedure above for correct results.\n")
  }

  # --- Displaying rotation output ---
  origdigits <- options("digits")
  data("CCAI", package = "GPArotation")
  fa.unrotated <- factanal(factors = 3, covmat = CCAI_R, n.obs = 461, rotation = "none")
  res <- oblimin(fa.unrotated, gam = -0.5, randomStarts = 20)
                                    # gam = -0.5: more orthogonal than quartimin
  res                               # default print
  print(res)                        # equivalent to above
  print(res, Table = TRUE)          # include iteration table
  print(res, rotateMat = TRUE)      # include rotating matrix
  print(res, digits = 2)            # rounded to 2 decimal places
  summary(res)                      # pattern and structure matrices for oblique rotation
  summary(res, Structure = FALSE)   # pattern matrix only
  options(digits = origdigits$digits)
    
  # --- Random start diagnostics ---
  # When randomStarts > 1, the output includes randStartChar which summarizes
  # the random start results:
  #   randomStarts : number of random starts attempted
  #   Converged    : number of starts that converged
  #   atMinimum    : number of starts at the same lowest minimum
  #   localMins    : number of distinct local minima found
  data(Thurstone, package = "GPArotation")
  res <- GPFRSoblq(box26, method = "geomin", normalize = TRUE, randomStarts = 50)
  res$randStartChar

  # --- Factor ordering ---
  # Raw GPArotation output is unsorted — factors may appear in any order
  # depending on the starting matrix. Use print() to obtain sorted loadings.
  # Once sorted, repeated calls to print() are stable.
  set.seed(334)
  xusl <- quartimin(Harman8, normalize = TRUE, randomStarts = 100)

  loadings(xusl)                                        # unsorted raw output
  max(abs(print(xusl)$loadings - xusl$loadings)) == 0  # FALSE: print() reorders
  xsl <- print(xusl)                                    # capture sorted result
  max(abs(print(xsl)$loadings - xsl$loadings)) == 0    # TRUE: already sorted
  
  # --- Normalization ---
  # Kaiser normalization
  data("CCAI", package = "GPArotation")
  fa.unrotated <- factanal(factors = 3, covmat = CCAI_R, n.obs = 461, rotation = "none")
  oblimin(fa.unrotated, normalize = TRUE, randomStarts = 100)
    data("CCAI", package = "GPArotation")
  res.u <- factanal(covmat = CCAI_R, factors = 4, rotation = "none",
                    n.obs = 461)

  # TandemI --- non-smooth orthogonal, BB and Cayley both faster
  res.t1l <- tandemI(res.u, algorithm = "legacy", fwindow =  1, maxit = 10000)
  res.t1b <- tandemI(res.u, algorithm = "bb",     fwindow = 10)
  res.t1c <- tandemI(res.u, algorithm = "cayley", fwindow = 10)
  cat("TandemI   legacy:", nrow(res.t1l$Table) - 1, "iterations\n")
  cat("TandemI   bb:    ", nrow(res.t1b$Table) - 1, "iterations",
      " max diff:", format(max(abs(res.t1l$loadings - res.t1b$loadings)),
                           scientific = TRUE, digits = 2), "\n")
  cat("TandemI   cayley:", nrow(res.t1c$Table) - 1, "iterations",
      " max diff:", format(max(abs(res.t1l$loadings - res.t1c$loadings)),
                           scientific = TRUE, digits = 2), "\n")

  # Entropy --- orthogonal, BB and Cayley both faster
  res.t2l <- entropy(res.u, algorithm = "legacy", fwindow =  1, maxit = 10000)
  res.t2b <- entropy(res.u, algorithm = "bb",     fwindow = 10)
  res.t2c <- entropy(res.u, algorithm = "cayley", fwindow = 10)
  cat("Entropy   legacy:", nrow(res.t2l$Table) - 1, "iterations\n")
  cat("Entropy   bb:    ", nrow(res.t2b$Table) - 1, "iterations",
      " max diff:", format(max(abs(res.t2l$loadings - res.t2b$loadings)),
                           scientific = TRUE, digits = 2), "\n")
  cat("Entropy   cayley:", nrow(res.t2c$Table) - 1, "iterations",
      " max diff:", format(max(abs(res.t2l$loadings - res.t2c$loadings)),
                           scientific = TRUE, digits = 2), "\n")

  # Oblimin --- smooth oblique, BB faster, same solution
  res.ol <- oblimin(res.u, algorithm = "legacy", fwindow =  1, maxit = 10000)
  res.ob <- oblimin(res.u, algorithm = "bb",     fwindow = 10)
  cat("Oblimin   legacy:", nrow(res.ol$Table) - 1, "iterations\n")
  cat("Oblimin   bb:    ", nrow(res.ob$Table) - 1, "iterations",
      " max diff:", format(max(abs(res.ol$loadings - res.ob$loadings)),
                           scientific = TRUE, digits = 2), "\n")

  # Simplimax --- non-smooth oblique with many local minima.
  # BB finds a substantially better solution in fewer iterations.
  # Difference in loadings reflects different local minima, not noise.
  res.sl <- simplimax(res.u, algorithm = "legacy", fwindow =  1, maxit = 10000)
  res.sb <- simplimax(res.u, algorithm = "bb",     fwindow = 10)
  cat("Simplimax legacy:", nrow(res.sl$Table) - 1, "iterations",
      " f =", round(res.sl$Table[nrow(res.sl$Table), 2], 6), "\n")
  cat("Simplimax bb:    ", nrow(res.sb$Table) - 1, "iterations",
      " f =", round(res.sb$Table[nrow(res.sb$Table), 2], 6), "\n")
  cat("BB found a lower criterion value --- better solution quality.\n")
}
\author{Coen A. Bernaards and Robert I. Jennrich
        with some R modifications by Paul Gilbert}
\references{
Barzilai, J. and Borwein, J.M. (1988) Two-point step size gradient
    methods. \emph{IMA Journal of Numerical Analysis}, \bold{8}, 141--148.
    \doi{10.1093/imanum/8.1.141}

Bernaards, C.A. and Jennrich, R.I. (2005) Gradient Projection Algorithms
    and Software for Arbitrary Rotation Criteria in Factor Analysis.
    \emph{Educational and Psychological Measurement}, \bold{65}, 676--696.
    \doi{10.1177/0013164404272507}

Jennrich, R.I. (2001) A simple general procedure for orthogonal rotation.
    \emph{Psychometrika}, \bold{66}, 289--306.
    \doi{10.1007/BF02294840}

Jennrich, R.I. (2002) A simple general method for oblique rotation.
    \emph{Psychometrika}, \bold{67}, 7--19.
    \doi{10.1007/BF02294706}

Mansolf, M. and Reise, S.P. (2016) Exploratory Bifactor Analysis:
    The Schmid-Leiman Orthogonalization and Jennrich-Bentler Analytic Rotations.
    \emph{Multivariate Behavioral Research}, \bold{51}(5), 698--717.
    \doi{10.1080/00273171.2016.1215898}

Nguyen, H.V. and Waller, N.G. (2023) Local minima and factor rotations
    in exploratory factor analysis. \emph{Psychological Methods},
    \bold{28}(5), 1122--1141.
    \doi{10.1037/met0000467}
    
Wen, Z. and Yin, W. (2013) A feasible method for optimization with
  orthogonality constraints. \emph{Mathematical Programming}, \bold{142},
  397--434. \doi{10.1007/s10107-012-0584-1}

Zhang, H. and Hager, W.W. (2004) A nonmonotone line search technique
  and its application to unconstrained optimization.
  \emph{SIAM Journal on Optimization}, \bold{14}(4), 1043--1056.
  \doi{10.1137/S1052623403428208}

   }
\concept{rotation}
\keyword{multivariate}

\encoding{UTF-8}
\name{Thurstone}
\docType{data}
\alias{Thurstone}
\alias{box26}
\title{Thurstone Box Data}
\description{
  Factor loading matrix derived from Thurstone's box data.
}
\usage{
  data(Thurstone)
}
\details{
  Loading the data makes the following object available:

  \code{box26} is a \eqn{26 \times 3}{26 x 3} unrotated factor loading
  matrix for 26 variables measured on a set of boxes. It appears
  frequently in the rotation literature as a dataset for
  comparing rotation criteria, particularly for assessing local minima.

  The matrix is suitable as input to \code{GPForth}, \code{GPFoblq},
  and all rotation wrapper functions in \code{GPArotation}. For a richer
  collection of factor analysis datasets, see the \code{psych} package.
}
\format{
  A numeric matrix:
  \itemize{
    \item \code{box26}: 26 x 3 matrix of unrotated factor loadings.
  }
}
\source{
  Thurstone, L.L. (1947). \emph{Multiple Factor Analysis}.
  University of Chicago Press.
}
\seealso{
  \code{\link{GPForth}},
  \code{\link{rotations}},
  \code{\link{Harman}},
  \code{\link{CCAI}},
  \code{\link{WansbeekMeijer}},
  \code{\link{GriffithMulaik}}  
}
\keyword{datasets}
